#!/usr/bin/env python
# -*- coding: utf8 -*-

from __future__ import print_function
from __future__ import unicode_literals

import collections
import os
import sys
import time
import datetime
import socket
import Queue
import multiprocessing
import logging
import logging.handlers
import traceback
import signal
import threading
import uuid
import argparse
import setproctitle

import yaml

import driverng
import driverng.protocol
import driverng.transport
from driverng.protocol.message_pb2 import MCAMessage, MonitoringMessage
from driverng.protocol.mca import MCAProtocol
from driverng.transport.base import TransportException

if getattr(sys, "frozen", False):
    from driverng.version import VERSION
else:
    import pkg_resources
    VERSION = {
        "Version": pkg_resources.require("driverng")[0].version,
    }

LOG_FORMAT = "%(asctime)-15s %(processName)s[pid=%(process)d] %(message)s"
LOG_DIR = None

logger = logging.getLogger("driverng")
log_formatter = logging.Formatter(LOG_FORMAT)


class Connection(multiprocessing.Process):

    def __init__(self, config):
        self.rqueue = multiprocessing.Queue()
        self.wqueue = multiprocessing.Queue()
        self.status = MonitoringMessage.STATUS_STARTED
        self.config = config
        multiprocessing.Process.__init__(self, name=config.name)
        self.logger = None
        self.unstable_event = multiprocessing.Event()
        self.ack_queue = {}

    @staticmethod
    def check_parent():
        me = os.getpid()
        while True:
            if os.getppid() == 1:
                os.kill(me, signal.SIGKILL)
            time.sleep(1)

    def run_driver(self):
        transport = None
        while True:
            try:
                if setproctitle:
                    setproctitle.setproctitle("driver [%s]" % (self.name,))
                proto = driverng.protocol.get(logger=self.logger, **self.config.protocol)
                transport = driverng.transport.get(logger=self.logger, **self.config.transport)
                proto.connect(transport)
                proto.start(self.rqueue, self.wqueue, background=False)
            except SystemExit:
                self.logger.info("Connection stopped")
                break
            except Exception as e:
                if isinstance(e, TransportException):
                    self.unstable_event.set()
                    self.logger.error(
                        "TransportException: %s restart in 3 seconds", e)
                    if transport is not None:
                        transport.close()
                    time.sleep(3)
                else:
                    for line in traceback.format_exc().decode("ISO-8859-15").splitlines():
                        self.logger.error(line)
                    raise

    def run(self):
        global LOG_DIR
        signal.signal(signal.SIGTERM, signal.SIG_DFL)
        self.logger = logging.getLogger(self.name)
        self.logger.setLevel(logging.DEBUG)
        if LOG_DIR is not None:
            log_handler = logging.handlers.TimedRotatingFileHandler(
                os.path.join(LOG_DIR, "%s.log" % self.name),
                when="midnight",
                interval=1,
                backupCount=30,
            )
        else:
            log_handler = logging.StreamHandler()
        log_handler.setFormatter(log_formatter)
        self.logger.addHandler(log_handler)
        t = threading.Thread(target=self.check_parent)
        t.daemon = True
        t.start()
        self.run_driver()


class MCAClientError(Exception):
    pass


class MCAClient(MCAProtocol):
    managed_connections = {}

    def __init__(self, addr, node_name):
        self.node_name = node_name
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        super(MCAClient, self).__init__(sock, logger)
        try:
            sock.connect(addr)
        except socket.error:
            self.handle_traceback()
            try:
                sock.close()
            except socket.error:
                self.handle_traceback()
            raise
        sock.setblocking(0)

        self.last_status = datetime.datetime.now()
        self.sig_hup = False
        signal.signal(signal.SIGHUP, self.reload_config)
        signal.signal(signal.SIGTERM, self.stop)

        if setproctitle:
            setproctitle.setproctitle("driver [manager]")

        self.configure()
        for cnx in MCAClient.managed_connections.values():
            for msg in cnx.ack_queue.values():
                self.send_message(msg)

    def stop(self, signum, frame):
        self.global_exit = True
        for connection in MCAClient.managed_connections.itervalues():
            logger.info(
                "Try to stop connection %s", connection.name)
            connection.terminate()
            connection.join()

    def reload_config(self, signum, frame):
        logger.info("SIGHUP recevied, force reloading")
        self.sig_hup = True

    def force_check(self):
        self.last_status -= datetime.timedelta(minutes=1)

    def configure(self):
        logger.info("Send config request")
        msg = MCAMessage()
        msg.type = MCAMessage.CONFIG_REQUEST
        msg.config_request.hostname = self.node_name
        msg.config_request.version = VERSION["Version"]
        self.send_message(msg)

    def do_something(self):

        if self.sig_hup:
            self.sig_hup = False
            self.configure()

        for id_connection, connection in MCAClient.managed_connections.items():

            if connection.status == MonitoringMessage.STATUS_STOPPED:
                continue

            if not connection.is_alive():
                logger.warning(
                    "Connection %s (pid=%d) died",
                    connection.name,
                    connection.pid,
                )
                connection.status = MonitoringMessage.STATUS_STOPPED
                self.force_check()
                continue

            if connection.unstable_event.is_set():
                connection.unstable_event.clear()
                if connection.status != MonitoringMessage.STATUS_UNSTABLE:
                    connection.status = MonitoringMessage.STATUS_UNSTABLE
                    self.force_check()
                    logger.warning(
                        "Connection %s (pid=%d) is unstable",
                        connection.name,
                        connection.pid,
                    )

            while True:
                try:
                    data = connection.rqueue.get_nowait()
                except Queue.Empty:
                    break

                msg = None

                if data["data"]:
                    msg = MCAMessage()
                    msg.type = MCAMessage.DATA
                    msg.data.id = id_connection
                    msg.data.data = data["data"]
                    msg.data.uuid = uuid.uuid4().hex
                    if data["filename"] is not None:
                        meta = msg.data.meta.add()
                        meta.key = "filename"
                        meta.value = data["filename"]
                    connection.ack_queue[msg.data.uuid] = msg
                elif data["uuid"]:  # ack
                    msg = MCAMessage()
                    msg.type = MCAMessage.ACKDATA
                    msg.ackdata.uuid = data["uuid"]

                if msg is not None:
                    if connection.status == MonitoringMessage.STATUS_UNSTABLE:
                        logger.info(
                            "Connection %s (pid=%s) become stable",
                            connection.name,
                            connection.pid,
                        )
                        connection.status = MonitoringMessage.STATUS_STARTED
                        self.force_check()
                    self.send_message(msg)

        now = datetime.datetime.now()
        if now - self.last_status > datetime.timedelta(minutes=1):
            msg = self.get_status_message()
            self.last_status = now
            self.send_message(msg)

    def handle_message(self, msg):
        type_func_map = {
            MCAMessage.CONFIG_REPLY: self.handle_config_reply_message,
            MCAMessage.DATA: self.handle_data_message,
            MCAMessage.ACKDATA: self.handle_ackdata_message,
        }

        try:
            func = type_func_map[msg.type]
        except KeyError:
            raise MCAClientError("Unknown message type %d" % (msg.type,))

        func(msg)

    def handle_config_reply_message(self, msg):
        logger.info("Configuring connections")
        active_connections = []
        if msg.config.version:
            logger.info("Server version: %s", msg.config.version)
        config = driverng.Config.from_legacy(msg)
        config.set_defaults()
        config.validate()
        for conn_config in config.connections:
            active_connections.append(conn_config.id)

            if conn_config.id in MCAClient.managed_connections:
                connection = MCAClient.managed_connections[conn_config.id]
                if connection.is_alive() and conn_config == connection.config:
                    connection.status = MonitoringMessage.STATUS_STARTED
                    logger.info("Connection %s, no changes", connection.name)
                    continue
                else:
                    if connection.is_alive():
                        logger.info(
                            "Connection %s killed (for update)",
                            connection.name,
                        )
                        connection.terminate()
                        connection.join()

            connection = Connection(conn_config)
            MCAClient.managed_connections[conn_config.id] = connection
            logger.info("Connection %s started", connection.name)
            connection.start()

        for pk, connection in MCAClient.managed_connections.items():
            if pk not in active_connections:
                logger.info(
                    "Connection %s killed (inactive)",
                    connection.name,
                )
                if connection.is_alive():
                    connection.terminate()
                    connection.join()
                MCAClient.managed_connections[pk].status = MonitoringMessage.STATUS_STOPPED

        self.force_check()

    @staticmethod
    def handle_data_message(msg):
        if msg.data.data_bytes != b'':
            data = msg.data.data_bytes
        else:
            data = msg.data.data
        MCAClient.managed_connections[msg.data.id].wqueue.put({
            "data": data,
            "uuid": msg.data.uuid,
            "meta": dict((m.key, m.value) for m in msg.data.meta),
        })

    @staticmethod
    def handle_ackdata_message(msg):
        if msg.ackdata.id:
            try:
                del MCAClient.managed_connections[msg.ackdata.id].ack_queue[msg.ack_queue.uuid]
            except KeyError:
                logger.warning("Unknown uuid %s", uuid)
        else:
            # LEGACY
            for cnx in MCAClient.managed_connections.values():
                try:
                    del cnx.ack_queue[msg.ackdata.uuid]
                    break
                except KeyError:
                    pass

    @staticmethod
    def get_status_message():
        msg = MCAMessage()
        msg.type = MCAMessage.MONITORING
        for pk, connection in MCAClient.managed_connections.items():
            status = msg.monitoring.status.add()
            status.id = pk
            status.status = connection.status

            if connection.status == MonitoringMessage.STATUS_STOPPED:
                # garbage
                del MCAClient.managed_connections[pk]

        return msg


class Manager(object):

    def __init__(self, config_path):

        # connection name -> Connection process
        self.connections = {}

        # connection -> [message]
        self.waiting = {}

        # connection -> uuid -> [message]
        self.pending = {}

        self.sig_hup = False
        signal.signal(signal.SIGHUP, self.reload)
        self.config_path = config_path
        self.config = None
        if not self.load_config():
            sys.exit(1)

        if setproctitle:
            setproctitle.setproctitle("driver [manager]")
        super(Manager, self).__init__()

    def reload(self, signum, frame):
        self.sig_hup = True

    def load_config(self):
        logger.info("loading config in %s", self.config_path)
        try:
            config = yaml.safe_load(open(self.config_path).read())
            config = driverng.Config(config)
            config.set_defaults()
            config.validate()
        except Exception:
            logger.exception("Invalid config")
            return False
        else:
            self.config = config
            return True

    def apply_config(self):
        new = dict([
            (c.name, c) for c in self.config.connections
            if c.get("enabled", True)
        ])
        for name, connection in self.connections.items():
            if (
                name not in new or
                new[name] != connection.config
            ):
                if connection.is_alive():
                    if name not in new:
                        logger.info(
                            "Connection %s (pid=%d) killed (removed or disabled)",
                            name, connection.pid)
                    else:
                        logger.info(
                            "Connection %s (pid=%d) killed (config update)",
                            name, connection.pid)
                    connection.terminate()
                    connection.join()
                    del self.connections[connection.name]

        for name, config in new.items():
            if name not in self.connections:
                self.connections[name] = Connection(config)
                self.connections[name].start()
                logger.info(
                    "Connection %s (pid=%d) started",
                    name, self.connections[name].pid)

    def run(self):
        self.apply_config()
        while True:
            if self.sig_hup:
                self.load_config()
                self.apply_config()
                self.sig_hup = False
            for connection in self.connections.values():
                if not connection.is_alive():
                    logger.warning(
                        "Connection %s (pid=%d) died",
                        connection.name, connection.pid,
                    )
                    del self.connections[connection.name]
                    break

                while True:
                    try:
                        data = connection.rqueue.get_nowait()
                    except Queue.Empty:
                        break

                    if data["uuid"]:
                        del self.pending[connection.name][data["uuid"]]

                    if data["data"]:
                        if connection.config.forward_to:
                            self.waiting.setdefault(
                                connection.config.forward_to, collections.deque(),
                            ).append(data["data"])

                if (
                    not self.pending.get(connection.name) and
                    self.waiting.get(connection.name)
                ):
                    try:
                        message = self.waiting[connection.name].popleft()
                    except IndexError:
                        pass
                    else:
                        uid = uuid.uuid4().hex
                        connection.wqueue.put({
                            "data": message,
                            "uuid": uid,
                            "meta": {},
                        })
                        self.pending.setdefault(
                            connection.name, {},
                        ).setdefault(
                            uid, collections.deque(),
                        ).append(message)

            time.sleep(1)


class Driver(object):
    def __init__(self, args):
        self.args = args
        self.global_exit = False
        signal.signal(signal.SIGTERM, self.stop)

    def stop(self, signum, frame):
        self.global_exit = True

    def run(self):
        if self.args.config:
            self.run_from_config()
        else:
            self.run_with_mca()

    def run_from_config(self):
        Manager(self.args.config).run()

    def run_with_mca(self):
        node_name = self.args.node_name
        try:
            server, port = self.args.server.split(":")
            port = int(port)
        except ValueError:
            server = self.args.server
            port = 6666
        mca = None
        was_connected = True
        while not self.global_exit:
            try:
                mca = MCAClient((server, port), node_name)
                logger.info("Connected to MCA")
                was_connected = True
                mca.run_protocol()
            except Exception:
                pass

            if setproctitle:
                setproctitle.setproctitle("driver [manager]: disconnected")

            if mca is not None and mca.global_exit:
                self.global_exit = True
            if was_connected:
                logger.info("Disconnected from MCA, "
                            "try reconnect in 3 seconds")
                was_connected = False
            time.sleep(3)

    @staticmethod
    def start_driver():
        logger.setLevel(logging.DEBUG)
        hostname = socket.gethostname()
        parser = argparse.ArgumentParser(description="Driver NG")
        parser.add_argument(
            "--server",
            default="mca1",
            help="Server IP host:port or host (default port 6666)",
        )
        parser.add_argument(
            "--node-name",
            default=hostname,
            help="Node name",
        )
        parser.add_argument(
            "--config",
            help="Config file",
        )
        parser.add_argument(
            "--log-dir",
            help="Log directory (default stdout)",
        )
        parser.add_argument(
            "--daemon",
            action="store_true",
            help="DEPRECATED. Same as --log-dir ~/logs",
        )
        parser.add_argument(
            "--version",
            action="store_true",
            help="Show version",
        )
        args = parser.parse_args()

        if not args.log_dir and args.daemon:
            args.log_dir = os.path.expanduser("~/logs")

        if args.version:
            print("Driver NG")
            print("\n".join(["%s: %s" % (k, v) for k, v in VERSION.items()]))
            sys.exit(0)

        if args.log_dir:
            global LOG_DIR
            LOG_DIR = args.log_dir
            log_handler = logging.handlers.TimedRotatingFileHandler(
                os.path.join(args.log_dir, "driver.log"),
                when="midnight",
                backupCount=30,
            )
        else:
            log_handler = logging.StreamHandler()

        log_handler.setFormatter(log_formatter)
        logger.addHandler(log_handler)
        Driver(args).run()


def main():
    Driver.start_driver()


if __name__ == "__main__":
    main()
