# -*- coding: utf8 -*-

from __future__ import unicode_literals

import logging

from driverng.astm.parser import RecursiveList
from driverng.utils import get_date, format_date

logger = logging.getLogger("driverng.astm")


class HprimForeignContainer(list):
    def __init__(self, type_segment, cls):
        self.cls = cls
        self.type_segment = type_segment
        list.__init__(self)

    def add(self, *args, **kwargs):
        kwargs.setdefault("type_segment", self.type_segment)
        segment = self.cls(*args, **kwargs)
        self.append(segment)
        return segment


class HprimForeign(object):

    def __init__(self, type_segment, relname, one2one=False, required=False):
        self.type_segment = type_segment
        self.relname = relname
        self.required = required
        self.one2one = one2one
        self.cls = None
        super(HprimForeign, self).__init__()

    def copy(self):
        """Permet d'hériter d'un Segment avec une foreign field
        Le field est reinstancié et peut donc avoir un attribut cls
        différent de la classe mère

        Par exemple: Le field commentaires de la classe Segment
        """
        return HprimForeign(
            self.type_segment,
            self.relname,
            one2one=self.one2one,
            required=self.required,
        )

    def set_segment(self, cls):
        self.cls = cls

    def get_default(self):
        if self.one2one:
            return None
        else:
            return HprimForeignContainer(self.type_segment, self.cls)


class AbstractHprimField(object):

    def __init__(self, position, **kwargs):
        """`obligatoire`: Champ obligatoire
        `consigne`: Consigné (à retourner à l'identique s'il a été
        `necessary`: Envoie du champs necessaire meme vide
        renseigné par l'émetteur
        """
        self.obligatoire = kwargs.pop("obligatoire", False)
        self.consigne = kwargs.pop("consigne", False)
        self._default = kwargs.pop("default", None)
        self.necessary = kwargs.pop("necessary", False)
        repetiteur = kwargs.pop("repetiteur", False)

        if isinstance(position, int):
            self.position = (position,)
        else:
            self.position = position

        if not repetiteur:
            self.position = self.position[:1] + (1,) + self.position[1:]

        super(AbstractHprimField, self).__init__()

    def get_default(self):
        raise NotImplementedError

    def to_hprim(self, value):
        raise NotImplementedError

    def to_python(self, value):
        raise NotImplementedError


class BaseHprimField(AbstractHprimField):
    """Champ Hprim de base."""

    def get_default(self):
        return self._default

    def to_hprim(self, value):

        if value is None:
            return ""

        return value

    def to_python(self, value):
        return value


class HprimString(BaseHprimField):
    """Type hprim ST."""

    def __init__(self, *position, **kwargs):
        self.max_length = kwargs.pop("max_length", None)
        self.trunc = kwargs.pop("trunc", False)
        kwargs.setdefault("default", "")
        super(HprimString, self).__init__(*position, **kwargs)

    def to_hprim(self, value):
        value = "%s" % (super(HprimString, self).to_hprim(value),)
        if self.max_length and len(value) > self.max_length:
            if self.trunc:
                logger.info(
                    "trunc '%s' to '%s'",
                    value, value[:self.max_length])
                value = value[:self.max_length]
            else:
                logger.info(
                    "Length of '%s' > %d",
                    value, self.max_length)

        return value

    def to_python(self, value):
        while isinstance(value, list):
            value = value[0]
        return value


class HprimFloat(BaseHprimField):

    def to_python(self, value):

        try:
            value = float(value)
        except ValueError:
            value = None

        return value


class HprimDate(BaseHprimField):
    """Type hprim TS."""

    def __init__(self, *position, **kwargs):
        self.fmt_date = kwargs.pop("fmt_date", None)
        super(HprimDate, self).__init__(*position, **kwargs)

    def to_hprim(self, value):
        return format_date(value, fmt=self.fmt_date)

    def to_python(self, value):
        return get_date(value, fmt=self.fmt_date)


class HprimListContainner(list):

    def __init__(self, *args, **kwargs):
        self.basefield = kwargs["basefield"]
        list.__init__(self, *args)

    def add(self):
        obj = self.basefield.get_default()
        self.append(obj)
        return obj


class HprimList(BaseHprimField):

    def __init__(self, *position, **kwargs):
        self.basefield = kwargs.pop("basefield", HprimString(0))
        kwargs["repetiteur"] = True
        super(HprimList, self).__init__(*position, **kwargs)

    def get_default(self):
        return HprimListContainner(basefield=self.basefield)

    def to_hprim(self, value):
        if value is not None:
            rlist = RecursiveList([self.basefield.to_hprim(e) for e in value])
        else:
            rlist = RecursiveList()
        return rlist

    def to_python(self, value):
        if not value:
            return HprimListContainner([], basefield=self.basefield)
        if not isinstance(value, RecursiveList):
            value = RecursiveList([value])
        return HprimListContainner([
            self.basefield.to_python(e) for e in value
        ], basefield=self.basefield)


class HprimMultipleContainer(object):
    pass


class HprimMultiple(BaseHprimField):

    def __init__(self, *position, **kwargs):

        self.fields = {}

        for name, field in kwargs.items():
            if isinstance(field, BaseHprimField):
                field.position = field.position[:1] + field.position[2:]
                self.fields[name] = field
                del kwargs[name]

        super(HprimMultiple, self).__init__(*position, **kwargs)

    def get_default(self):
        obj = HprimMultipleContainer()

        for name, field in self.fields.items():
            setattr(obj, name, field.get_default())

        return obj

    def to_hprim(self, value):
        r = RecursiveList()
        for name, field in self.fields.items():
            hprim_value = field.to_hprim(getattr(value, name))
            if (
                hprim_value not in ("", None) and not (
                    isinstance(hprim_value, RecursiveList) and
                    not hprim_value
                )
            ):
                r.set(hprim_value, *field.position)
        return r

    def to_python(self, value):
        obj = HprimMultipleContainer()
        for name, field in self.fields.items():
            if not isinstance(value, RecursiveList):
                value = RecursiveList([value])
            setattr(obj, name, field.to_python(value.get(*field.position)))
        return obj


class HprimMultipleList(BaseHprimField):

    def __init__(self, *position, **kwargs):
        self.basefield = kwargs.pop("basefield", HprimString(0))
        super(HprimMultipleList, self).__init__(*position, **kwargs)

    def get_default(self):
        return RecursiveList()

    def to_hprim(self, value):
        r = RecursiveList()
        for position, val in enumerate(value, 1):
            r.set(val, position)
        return RecursiveList(r)

    def to_python(self, value):
        if isinstance(value, list):
            return value
        elif value:
            return [value]
        else:
            return []


class HprimPN(HprimMultiple):

    def __init__(self, *position, **kwargs):
        kwargs.update({
            "nom": HprimString(1),
            "prenom": HprimString(2),
            "prenom2": HprimString(3),
            "sobriquet": HprimString(4),
            "civilite": HprimString(5),
            "diplome": HprimString(6),
        })

        super(HprimPN, self).__init__(*position, **kwargs)


class HprimCNA(HprimMultiple):

    def __init__(self, *position, **kwargs):

        kwargs.update({
            "code": HprimString(1),
            "nom": HprimPN(2),
            # Type de code
            # L: Local
            # A: Adeli
            # R: RPPS
            "type": HprimString(3),
        })

        super(HprimCNA, self).__init__(*position, **kwargs)


class HprimAD(HprimMultiple):

    def __init__(self, *position, **kwargs):
        kwargs.update({
            "ligne1": HprimString(1),
            "ligne2": HprimString(2),
            "ville": HprimString(3),
            "departement": HprimString(4),
            "code_postal": HprimString(5),
            "pays": HprimString(6),
        })

        super(HprimAD, self).__init__(*position, **kwargs)


class HprimCE(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "code1": HprimString(1),
            "label1": HprimString(2),
            "codification1": HprimString(3),
            "code2": HprimString(4),
            "label2": HprimString(5),
            "codification2": HprimString(6),
        })
        super(HprimCE, self).__init__(*position, **kwargs)
