# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm.segment import Segment
from driverng.astm.parser import RecursiveList
from driverng.exceptions import ParsingError, ValidationError
from driverng import Message, MessageV2


class MetaAstmMessage(type):
    """Validation du Message / Check des foreigns."""
    def __new__(cls, name, bases, attrs):
        new_class = super(
            MetaAstmMessage, cls).__new__(cls, name, bases, attrs)

        # Merge des SEGMENT_CLASS_MAP
        for base in bases:
            if hasattr(base, "SEGMENT_CLASS_MAP"):
                # TODO: il y a forcement un moyen plus propre que hasattr()
                # pylint: disable-msg=C0103
                new_class.SEGMENT_CLASS_MAP = dict(
                    base.SEGMENT_CLASS_MAP.items() +
                    new_class.SEGMENT_CLASS_MAP.items())

        # Mapping des foreign
        for segment_class in new_class.SEGMENT_CLASS_MAP.values():
            for field in segment_class._meta.foreign_fields.values():
                if field.type_segment not in new_class.SEGMENT_CLASS_MAP:
                    raise ValidationError(
                        "Segment '%s' is not mapped" % (field.type_segment,))
                field.set_segment(
                    new_class.SEGMENT_CLASS_MAP[field.type_segment])
        return new_class


class BaseAstmMessage(object):

    __metaclass__ = MetaAstmMessage

    SEGMENT_CLASS_MAP = {"C": Segment}
    DEFAULT_SEGMENT_CLASS = Segment
    # Faut-il splitter le segment avec des Addendum ?
    ADDENDUM_SEGMENT_SIZE = None
    DEFAULT_SEPARATORS = "|^~\&"
    FIRST_SEP_INDEX = 1
    SEGMENT_ENDING = "\r\n"

    def parse_segment(self, line, separators):
        """Parse la chaine `line` suivant les separateurs `separators`
        Renvoie un segment.
        """
        seg_type = line.split(separators[0], 1)[0]

        if seg_type in self.SEGMENT_CLASS_MAP:
            segment = self.SEGMENT_CLASS_MAP[seg_type]()
        else:
            segment = self.DEFAULT_SEGMENT_CLASS()

        splitted = RecursiveList.split(line, separators)
        segment.populate(splitted)

        return segment

    def parse(self, data):
        """Parse le message `data` et renvoie le message."""
        if isinstance(data, unicode):
            # Splitlines should only be done using 'str'
            # because unicode splitlines will split on much more
            # than '\n' or '\r\n'.
            # See https://docs.python.org/2/library/stdtypes.html#unicode.splitlines
            lines = [l.decode("utf-8") for l in data.encode("utf-8").splitlines() if l]
        else:
            lines = [l for l in data.splitlines() if l]

        # Extraction des separateurs
        try:
            sep = lines[0][self.FIRST_SEP_INDEX]
            split_header = lines[0].split(sep, 2)
            separators = "%s%s" % (sep, split_header[1])
            # ATTENTION:
            # En ASTM le caractère d'échappement est le dernier
            # caractère des séparateurs. Il échappe donc le caractère juste
            # après qui est un délimiteur de champs.
            # Le champ qui contient les séparateurs n'est pas parsable.
            # On a pas ce problème en Hprim car le caractère d'échappement
            # est suivi par le séparateurs de sous-sous-champs.
            # HACK: On supprime les séparateurs avant de parser.
            lines[0] = "%s%c%c%s" % (
                split_header[0],
                sep, sep,
                split_header[2] if len(split_header) > 2 else "",
            )
        except Exception as exc:
            raise ParsingError("%s" % (exc,))

        message, last_segment = None, None

        i = 0
        last_line = ''  # noqa
        while i < len(lines):
            line = lines[i]

            if self.ADDENDUM_SEGMENT_SIZE and i + 1 < len(lines):
                while (
                    i < len(lines) - 1 and
                    lines[i + 1][:2] == "A%c" % (separators[0],)
                ):
                    line += lines[i + 1][2:]
                    i += 1

            i += 1
            try:
                segment = self.parse_segment(line, separators)
                message, last_segment = self.add_segment(
                    segment, message, last_segment)
            except ParsingError as exc:
                exc.detail = "parsing error near line {} (offending line: {})".format(i, line)
                raise
            last_line = line  # noqa
        return message

    def add_segment(self, segment, message, last_segment):
        if segment.type_segment not in self.SEGMENT_CLASS_MAP.keys():
            raise ParsingError(
                "Unknown segment type '%s'" % (segment.type_segment,))

        if message is None:
            # Le point d'entrée doit être segment qui n'a pas de parent
            for other_segment in self.SEGMENT_CLASS_MAP.values():
                # pylint: disable-msg=W0212
                for field in other_segment._meta.foreign_fields.values():
                    if field.type_segment == segment.type_segment:
                        raise ParsingError(
                            "Segment '%s' is not allowed here" % (
                                segment.type_segment,
                            ))

            message = segment
        else:
            # On a un point d'entrée, il faut trouver la feuille qui
            # correspond a notre segment. C'est à dire le premier
            # segment de l'arbre qui accepte comme fils notre segment
            if not Segment.attach_segment(segment, last_segment):
                raise ParsingError("Segment '%s' is not allowed here" %
                                   (segment.type_segment,))
        last_segment = segment

        return message, last_segment

    def to_string(self, message, separators=None, **kwargs):
        if separators is None:
            separators = self.DEFAULT_SEPARATORS
        lines = [s.to_string(
            separators=separators) for s in message.get_segments()]

        if self.ADDENDUM_SEGMENT_SIZE:
            new_lines = []
            for line in lines:
                new_lines.append(line[:self.ADDENDUM_SEGMENT_SIZE])
                line = line[self.ADDENDUM_SEGMENT_SIZE:]
                while len(line):
                    new_lines.append("A%c%s" % (
                        separators[0],
                        line[:self.ADDENDUM_SEGMENT_SIZE],
                    ))
                    line = line[self.ADDENDUM_SEGMENT_SIZE:]
            lines = new_lines
        return self.SEGMENT_ENDING.join(lines)


class AstmMessage(BaseAstmMessage, Message):
    pass


class AstmMessageV2(BaseAstmMessage, MessageV2):
    pass
