# -*- coding: utf8 -*-

from __future__ import unicode_literals


class AstmParser(object):
    def __init__(self, separators):
        self.separators = separators
        self.escape_map = self.get_escape_map()
        super(AstmParser, self).__init__()

    def get_separators(self):
        raise NotImplementedError

    def get_escape_map(self):
        raise NotImplementedError

    def escape(self, string):
        for k, v in self.escape_map.items():
            string = string.replace(v, k)
        return string

    def unescape(self, string):
        for k, v in self.escape_map.items():
            string = string.replace(k, v)
        return string

    @staticmethod
    def get(separators):
        if len(separators) == 4:
            return AstmParser1394(separators)
        elif len(separators) == 5:
            return AstmParser1238(separators)
        else:
            raise NotImplementedError


class AstmParser1394(AstmParser):

    def get_separators(self):
        return (
            self.separators[0],  # Field
            self.separators[1],  # Repetition
            self.separators[2],  # Component
        )

    def get_escape_map(self):
        escape = self.separators[3]
        e_map = {}
        for k, sep in (
            ("F", self.separators[0]),
            ("S", self.separators[2]),
            ("R", self.separators[1]),
        ):
            e_map["%s%s%s" % (
                escape, k, escape)] = sep
        return e_map

    def escape(self, string):
        string = string.replace(
            self.separators[3],
            "{0}E{0}".format(self.separators[3]),
        )
        return super(AstmParser1394, self).escape(string)

    def unescape(self, string):
        string = super(AstmParser1394, self).unescape(string)
        return string.replace(
            "{0}E{0}".format(self.separators[3]),
            self.separators[3],
        )


class AstmParser1238(AstmParser1394):
    def get_separators(self):
        return (
            self.separators[0],  # Field
            self.separators[2],  # Repetition
            self.separators[1],  # Component
            self.separators[4],  # Subcomponent
        )
        # TODO:
        # H start highlighting
        # N normal text (end highlighting)
        # Dnnn decimal code value of an 8 bit character
        # Xdddd hexadecimal data
        # Zdddd locally defined escape sequence

    def get_escape_map(self):
        escape = self.separators[3]
        e_map = super(AstmParser1238, self).get_escape_map()
        e_map["%sT%s" % (escape, escape)] = self.separators[4]
        return e_map


class RecursiveList(list):
    """Classe qui représente des listes récursives de chaines.
    Toutes les listes sont infinie et l'élément vide est la chaine vide.
    """
    DEFAULT_SEPARATORS = "|~^\&"

    def __init__(self, value=None):
        """Initialise toute sous liste en RecursiveList
        TODO: voir si python ne permet pas de faire tout ça plus simplement
        """
        if value is None:
            value = []

        # La liste des positions qui sont "safe", c'est à dire la liste des
        # elements qu'il ne faut pas échapper.
        self._safe = []

        if isinstance(value, list) and len(value):
            for e in value:
                if isinstance(e, list):
                    self.append(RecursiveList(e))
                else:
                    self.append(e)
        else:
            list.__init__(self, value)

    def get(self, *pos):
        """Retourne la valeur de la liste ou sous liste à la position `pos`."""

        try:
            data = self[pos[0] - 1]
        except IndexError:
            data = ""

        if len(pos) > 1:
            if isinstance(data, basestring):
                data = RecursiveList([data])
            data = data.get(*pos[1:])

        return data

    def set(self, value, *pos, **kwargs):
        """Met la valeur `value` dans la RecursiveList à la position `pos`."""

        if kwargs.get("safe", False) and len(pos) == 1:
            self._safe.append(pos[0])

        # Rajoute tous les "" manquants
        if len(self) < pos[0]:
            self.extend([""] * (pos[0] - len(self)))

        if len(pos) > 1:
            if not isinstance(self[pos[0] - 1], RecursiveList):
                self[pos[0] - 1] = RecursiveList()
            self[pos[0] - 1].set(value, *pos[1:], **kwargs)
        else:
            if isinstance(value, (list, tuple)):
                value = RecursiveList(value)
            self[pos[0] - 1] = value

    @staticmethod
    def split(line, separators=None):
        """Renvoie une RecursiveList construite avec la chaine `line`
        et suivant les séparateurs `separators`.
        """

        if separators is None:
            separators = RecursiveList.DEFAULT_SEPARATORS

        astm = AstmParser.get(separators)
        rlist = RecursiveList.rsplit(line, *astm.get_separators())
        rlist.apply(astm.unescape)
        return rlist

    def apply(self, func):
        for i, e in enumerate(self):
            if isinstance(e, RecursiveList):
                e.apply(func)
            else:
                self[i] = func(e)

    @staticmethod
    def rsplit(line, *separators):

        substrings = RecursiveList.splitline(line, separators[0])

        if len(separators) == 1:
            return substrings

        for index, strings in enumerate(substrings):

            if not len(strings):
                continue

            substrings[index] = RecursiveList.rsplit(
                strings, *separators[1:])

            if (
                len(substrings[index]) == 1 and
                isinstance(substrings[index][0], basestring)
            ):
                substrings[index] = substrings[index][0]

        return substrings

    @staticmethod
    def splitline(line, sep):

        result, curline = RecursiveList(), ""

        for index, c in enumerate(line):
            if c == sep:
                result.append(curline)
                curline = ""
            else:
                curline += c

        if curline:
            result.append(curline)

        return result

    def join(self, separators=None):
        """Renvoie la chaine correspondante à la RecursiveList suivant
        les séparateurs `separators`.
        """

        if separators is None:
            separators = RecursiveList.DEFAULT_SEPARATORS

        astm = AstmParser.get(separators)
        return self.rjoin(astm.escape, astm.get_separators())

    def rjoin(self, escape, separators, offset=0):
        elements = []
        for position, element in enumerate(self, 1):

            if isinstance(element, RecursiveList):
                string = element.rjoin(escape, separators, offset + 1)
            else:
                string = "%s" % (element,)

                if position not in self._safe:
                    string = escape(string)

            elements.append(string)

        return separators[offset].join(elements)
