# -*- coding: utf8 -*-

from __future__ import unicode_literals

import driverng.exceptions as exceptions
from driverng.astm.parser import RecursiveList
from driverng.astm.fields import (
    HprimString,
    BaseHprimField,
    HprimForeign,
    HprimForeignContainer,
)


class Options(object):
    """Options du Segment.
    Accessible via l'attribut _meta
    """
    def __init__(self, fields, foreign_fields):
        self.fields = fields
        self.foreign_fields = foreign_fields


class BaseSegment(type):
    """Attache les fields et les options à la classe du Segment."""

    def __new__(cls, name, bases, attrs):
        fields = {}
        foreign_fields = {}
        for base in bases:
            if isinstance(base, BaseSegment):
                for k, v in base._meta.fields.iteritems():
                    fields[k] = v
                for k, v in base._meta.foreign_fields.iteritems():
                    # On reinstancie un nouveau field
                    foreign_fields[k] = v.copy()
        for k, v in attrs.items():
            if isinstance(v, BaseHprimField):
                fields[k] = v
                del attrs[k]
            if isinstance(v, HprimForeign):
                foreign_fields[k] = v
                del attrs[k]
        new_class = super(BaseSegment, cls).__new__(cls, name, bases, attrs)
        meta = Options(fields, foreign_fields)
        new_class._meta = meta
        return new_class


class AbstractSegment(object):
    """Représente un segment."""

    __metaclass__ = BaseSegment
    _meta = None

    # Permet de remplir le segment jusqu'au champ FILL_SEGMENT
    FILL_SEGMENT = None
    # Permet de rajouter un séparateur en fin de ligne
    END_LINE_WITH_SEPARATOR = False
    # Permet de remplir le rang du segment
    RANG_SEGMENT = True

    # Définit l'ordre des segments fils quand on genère le message
    SORT_CHILD_SEGMENT = ["commentaires"]

    type_segment = HprimString(1, obligatoire=True, max_length=3)
    commentaires = HprimForeign("C", "parent")

    def __init__(self, **kwargs):
        self._parent = None
        self._raw = {}

        for name, field in self._meta.fields.iteritems():
            setattr(self, name, kwargs.get(name, field.get_default()))

        for name, field in self._meta.foreign_fields.iteritems():
            setattr(self, name, field.get_default())

        super(AbstractSegment, self).__init__()

    @staticmethod
    def attach_segment(segment, parent):
        if parent is None:
            return False

        for name, field in parent._meta.foreign_fields.items():
            if (
                field.type_segment == segment.type_segment and
                segment.type_segment != parent.type_segment
            ):
                segment._parent = parent
                setattr(segment, field.relname, parent)
                if field.one2one:
                    # /!\ Mapping direct vers le segment
                    setattr(parent, name, segment)
                else:
                    getattr(parent, name).append(segment)
                return True

        return AbstractSegment.attach_segment(segment, parent._parent)

    def set_raw(self, attr, value):
        try:
            position = self._meta.fields[attr].position
        except KeyError:
            raise AttributeError("'%s' has not attribute '%s'" % (self, attr))
        self.set_raw_position(value, *position)

    def set_raw_position(self, value, *position):
        self._raw[position] = value

    def populate(self, rlist):
        """Initialise un Segment à partir d'une RecursiveList."""

        for name, field in self._meta.fields.iteritems():
            hprim_value = rlist.get(*field.position)
            try:
                value = field.to_python(hprim_value)
            except Exception as e:
                raise exceptions.ParsingError(
                    "Unhandled exception while parsing %s.%s at position %s with value %s.\n%s: %s" % (
                        self.__class__.__name__, name, field.position, hprim_value, e.__class__.__name__, e
                    ))
            setattr(self, name, value)
            setattr(self, "_raw_%s" % (name,), hprim_value)

    def get_rlist(self):
        """Renvoie une rlist remplie avec les datas."""
        rlist = RecursiveList()
        for name, field in sorted(self._meta.fields.iteritems()):

            if isinstance(field, BaseHprimField):
                value = field.to_hprim(getattr(self, name))

                if (
                    (value not in ("", None) or field.necessary) and not (
                        isinstance(value, RecursiveList) and not value
                    )
                ):
                    rlist.set(value, *field.position)
        for position, value in self._raw.iteritems():
            rlist.set(value, *position, safe=True)
        return rlist

    def to_string(self, separators=None):
        splitted = self.get_rlist()
        if self.END_LINE_WITH_SEPARATOR:
            splitted.append(RecursiveList(['']))
        if self.FILL_SEGMENT is not None and len(splitted) < self.FILL_SEGMENT:
            splitted.extend(RecursiveList(
                (self.FILL_SEGMENT - len(splitted)) * ['']))
        return splitted.join(separators=separators)

    def get_segments(self):
        segments = [self]
        extras = []
        childs = sorted(set(["commentaires"] + self._meta.foreign_fields.keys()))

        def sort_childs(x, y):
            key = self.SORT_CHILD_SEGMENT
            l = childs

            if x in key and y in key:
                return key.index(x) - key.index(y)
            elif x in key:
                return -1
            elif y in key:
                return 1
            else:
                return l.index(x) - l.index(y)

        childs = sorted(childs, cmp=sort_childs)

        for name in childs:
            obj = getattr(self, name)
            if isinstance(obj, HprimForeignContainer):
                if self.RANG_SEGMENT:
                    for n, seg in enumerate(obj, 1):
                        seg.rang_segment = n
                        segments.extend(seg.get_segments())
            elif obj is not None:
                # one2one
                if self.RANG_SEGMENT:
                    obj.rang_segment = 1
                if name in self.SORT_CHILD_SEGMENT:
                    segments.extend(obj.get_segments())
                else:
                    # Il n'est pas listé dans le tri explicite
                    # on le rajoute à la fin
                    extras.extend(obj.get_segments())

        segments.extend(extras)
        return segments


class Segment(AbstractSegment):
    rang_segment = HprimString(2, obligatoire=True, max_length=4)


class NoRangSegment(AbstractSegment):
    RANG_SEGMENT = False


class HeaderSegment(Segment):

    def to_string(self, separators=None):
        if separators is None:
            separators = RecursiveList.DEFAULT_SEPARATORS

        self.rang_segment = ""

        return "%s%s%s" % (
            self.type_segment,
            separators,
            super(HeaderSegment, self).to_string(separators)[
                len(self.type_segment) + 1:],
        )
