# -*- coding: utf8 -*-
from __future__ import unicode_literals

import json
import logging

import driverng

import jsonschema
import jsonschema_model

logger = logging.getLogger("driverng")


class ConfigError(Exception):
    pass


CONNECTION_SCHEMA = {
    "name": "Connection",
    "type": "object",
    "required": ["name", "transport", "protocol"],
    "properties": {
        "name": {"type": "string"},
        "enabled": {"type": "boolean"},
        "format": {
            "type": "object",
            "properties": {
                "type": {"type": "string"},
            },
            "required": ["type"],
        },
        "protocol": {
            "type": "object",
            "properties": {
                "type": {"type": "string"},
            },
            "required": ["type"],
        },
        "transport": {
            "type": "object",
            "properties": {
                "type": {"type": "string"},
            },
            "required": ["type"],
        },
        "forward_to": {"type": "string"},
    },
}

CONFIG_SCHEMA = {
    "name": "Config",
    "type": "object",
    "properties": {
        "connections": {
            "type": "array",
            "items": CONNECTION_SCHEMA,
        },
    },
}


_DEFAULTS_PROTOCOLS = {
    "stago": "astm",
    "mpl": "astm",
    "sysmex_suit": "astm",
    "sysmex_suit_plus": "astm",
    "advia": "advia2120",
    "ctlink": "astm",
    "isys": "astm",
    "advia1200": "astm",
    "advia1800": "astm",
    "piano": "astm",
    "piano_sil": "astm",
    "hemofax": "astm",
    "ruby": "astm",
    "sapphire": "astm",
    "ca560": "acknack",
    "arch2000": "astm",
    "versacell": "astm",
    "centaur": "astm",
    "eurolab": "void",
    "vitros": "astm",
    "zenit": "mllp",
    "dimension": "dimension",
    "vscube": "vscube",
    "pentra120": "abx",
    "tosoh": "tosoh",
    "vista": "astm",
    "cd3200": "cd3200",
    "immulite": "astm",
    "alifax": "alifax",
    "acltop": "astm",
    "tango": "astm",
    "dm": "dm_astm",
    "dm_extended": "dm_astm",
    "cs2x00i": "astm",
    "vidas": "bci",
    "scanlab": "void",
    "ssa": "void",
    "ca660": "astm",
    "ais": "astm",
    "liaison": "astm",
    "phoresys": "phoresys",
    "m2000": "astm",
    "immunocap": "astm",
    "swing": "void",
    "rapidlab": "rapidlab",
    "sp1000i": "noastm",
    "lxl": "astm",
    "acl9000": "astm",
    "hprim": "void",
    "bacexpress": "void",
    "dxh": "astm",
    "variant": "astm",
    "autovue": "void",
    "kone": "astm",
    "vitros_kermit": "kermit",
    "ha8180": "astm",
    "biorad": "void",
    "streamlab": "astm",
    "poch": "noastm",
    "lh": "lh",
    "kalilab": "void",
    "vidasxml": "void",
    "bcsxp": "astm",
    "vitek": "bci",
    "bcsxp_bctmode": "astm",
    "ha81": "ha81",
    "immucor": "void",
    "protis": "void",
    "gentiane": "void",
    "elecsys": "astm",
    "previisolaxml": "void",
    "vt": "acknack",
    "itm": "void",
    "vt50": "acknack",
    "kryptor": "astm",
    "qwalys": "astm",
    "evolis": "astm",
    "b221": "astm",
    "iq200xml": "astm",
    "cobas": "astm",
    "au680": "au680",
    "diana": "astm",
    "erytra": "astm",
    "cla1": "void",
    "abl": "astm",
    "helena": "astm",
    "mylaxml": "astm",
    "pvs": "pvs",
    "bnpros": "astm",
    "bn2_serhost": "bn2_serhost",
    "bn2_autohost": "bn2_autohost",
    "qiagen": "mllp",
    "remisol": "astm",
    "meqnet": "astm",
    "etimax": "astm",
    "quantalizer": "astm",
    "genexpert": "astm",
    "genexpert_v46": "astm",
    "engen": "astm",
    "sedimax": "sedimax",
    "bc6800": "astm",
    "interqc": "void",
    "advia_chimie_xpt": "astm",
    "centaur_xpt": "astm",
    "byg_astm": "astm",
    "dms": "astm",
    "dms_tracking": "astm",
    "advia_centaur_xpt": "astm",
    "biorad_urt2": "astm",
    "scanbac": "astm",
    "sir": "void",
    "dxi": "astm",
    "starexp_astm": "astm",
    "clarilab": "void",
    "alert": "mllp",
    "bd_max": "astm",
    "gem4000": "astm",
    "kx21": "acknack",
    "vt_syslab": "vt_syslab",
    "easylink": "astm",
    "c111": "astm",
    "starrsed": "acknack",
    "sorting_drive": "void",
    "valab": "valab",
    "kpcr_astm": "astm",
    "advia560": "mllp",
    "eurolinescan": "void",
    "kpcr_ad": "void",
    "suit_uf500": "astm",
    "wasp": "astm",
    "au800": "astm",
    "adagioxml": "void",
    "tecan": "void",
    "walkaway": "walkaway",
    "nimbus": "void",
    "bep2000": "astm",
    "virclia": "astm",
    "bactalert": "astm",
    "biorad_d10": "astm",
    "cytovision": "void",
    "bactec9000": "astm",
    "elite_astm": "astm",
    "filmarray": "void",
    "helios": "dm_astm",
    "sofia": "astm",
    "wasplab": "astm",
    "gemweb": "astm",
    "blotrix_recomscan": "void",
    "hprim_facturation": "void",
    "clinitek": "mllp",
    "ihcom_astm": "astm",
    "atellica": "atellica",
    "hemohub": "astm",
    "dr_dot": "void",
    "sorting_drive_file1": "void",
    "poccelerator": "hl7poc",
}

_DEFAULTS_KWARGS = {
    # Le max_car_bloc est géré par le format...
    "advia1200": {"end_payload_with_cr": False, "max_car_bloc": None},
    "advia1800": {"end_payload_with_cr": False, "max_car_bloc": None},
    "versacell": {"max_car_bloc": None},
    # SUIT plus has no limitation in the string length.
    # SUIT plus do not uses "ETB"
    "sysmex_suit_plus": {"max_car_bloc": None},
    "acltop": {"mode_bloc": True},
    "ha8180": {"mode_etb": True},
    "bcsxp_bctmode": {"end_payload_with_cr": False},
    "vt": {"ack": False},
    "iq200xml": {"mode_iq200": True, "end_payload_with_cr": False},
    "abl": {"mode_etb": True},
    "genexpert": {"mode_bloc": True},
    "genexpert_v46": {"mode_bloc": True},
    "bc6800": {"mode_etb": True, "checksum_ignore_end": True},
    "advia_chimie_xpt": {"mode_etb": True},
    "centaur_xpt": {"mode_etb": True},
    "starrsed": {"checksum_type": "xor_starrsed"},
    "gemweb": {"mode_etb": True},
}


def get_legacy_protocol_kwargs(options):
    type_serv = int(options["type_serv"])
    kwargs = {}
    if type_serv != 5:  # non fichier
        for b_opt in ("mode_bloc", "mode_query", "is_automate"):
            if b_opt in options:
                kwargs[b_opt] = True

        if "REPERTOIRE_FTP_ADVIA2120" in options and options["format_automate"] == "advia":
            kwargs["repertoire_ftp"] = options["REPERTOIRE_FTP_ADVIA2120"]

    kwargs["encoding"] = options.get("encoding", "ISO-8859-15")
    return kwargs


def get_legacy_transport_kwargs(options):
    type_serv = int(options["type_serv"])
    kwargs = {}

    if type_serv == 0:
        # serial
        kwargs["type"] = "serial"
        if options["nom_port"][0] != "/":
            kwargs["port"] = "/dev/%s" % options["nom_port"]
        else:
            kwargs["port"] = options["nom_port"]

        kwargs["baudrate"] = int(options["baud"])
        if options["parite"].lower() == "true":
            kwargs["parity"] = "O"

        kwargs["bytesize"] = int(options["n_bit"])
        kwargs["stopbits"] = int(options["stop_bit"])
        if options["xon_in"].lower() == "true":
            kwargs["xonxoff"] = True
        else:
            kwargs["xonxoff"] = False
    elif type_serv == 6:
        kwargs["type"] = "tcp"
        if options["parite"].lower() == "true":
            kwargs["mode"] = "server"
        else:
            kwargs["mode"] = "client"
        host, port = options["nom_port"].split(":")
        port = int(port)
        kwargs["host"] = host
        kwargs["port"] = port
    elif type_serv == 5:  # Fichier
        kwargs["type"] = "file"
        kwargs["input_dir"] = options["nom_port"]
        if "fic_in" in options:
            kwargs["output_dir"], kwargs["fmt"] = options["fic_in"].rsplit("/", 1)
        elif "output_dir" in options:
            kwargs["output_dir"], kwargs["fmt"] = options["output_dir"], None
        kwargs["glob"] = options["fic_out"]
    else:
        raise NotImplementedError("transport type %s not implemented" % (
            type_serv,
        ))
    return kwargs


class Config(jsonschema_model.model_factory(CONFIG_SCHEMA)):

    def set_defaults(self):
        for connection in self.connections:
            connection.format.setdefault("type", "")

            if (
                connection.format.type != "" and
                "encoding" not in connection.protocol
            ):
                connection.protocol.encoding = driverng.get(connection.format.type).ENCODING

            if (
                connection.transport.type == "file" or
                connection.transport.type == "ftp" and (
                    connection.format.type == "cla1" or
                    connection.format.type == "eurolinescan" or
                    connection.format.type == "nimbus"
                )
            ):
                connection.transport.setdefault("append", True)
            if connection.transport.type in ("file", "ftp", "samba"):
                connection.protocol.setdefault("type", "void")
                if connection.format.type == "qwalys":
                    connection.protocol.setdefault("multipart_astm", True)
            else:
                default_protocol = _DEFAULTS_PROTOCOLS.get(connection.format.type)
                if not connection.protocol.type:
                    if not default_protocol:
                        raise ConfigError("Cannot auto-detect protocol for format %s" % (
                            connection.format.type,
                        ))
                    else:
                        connection.protocol.type = default_protocol

                if connection.protocol.type == default_protocol:
                    for key, value in _DEFAULTS_KWARGS.get(connection.format.type, {}).items():
                        connection.protocol.setdefault(key, value)

    def validate(self):
        jsonschema.validate(self, CONFIG_SCHEMA)
        names = [c["name"] for c in self.connections]
        for connection in self.connections:

            # Transport/Protocol validation
            if connection.transport.type in ("file", "ftp", "samba"):
                if (
                    connection.protocol.type is not None and
                    connection.protocol.type != "void"
                ):
                    raise ConfigError("Cannot use protocol %s with transport %s" % (
                        connection.protocol.type,
                        connection.transport.type,
                    ))

            # Ensure all forwarded connection exists
            if (
                connection.forward_to and
                connection.forward_to not in names
            ):
                raise ConfigError("Connection %s doesn't exists" % (connection.forward_to),)
        return True

    @staticmethod
    def from_legacy(msg):
        config = Config()
        if msg.config.connections:
            config.connections = json.loads(msg.config.connections)
        else:
            for connection in msg.config.config:
                try:
                    options = {}
                    for opt in connection.options:
                        options[opt.key] = opt.value
                    protocol = get_legacy_protocol_kwargs(options)
                    transport = get_legacy_transport_kwargs(options)
                    format_type = options["format_automate"]
                    name = options["label"]
                except Exception:
                    logger.exception("Unhandled exception")
                else:
                    c = config.connections.add(
                        id=connection.id,
                        name=name,
                        transport=transport,
                        protocol=protocol,
                    )
                    c.format.type = format_type
        return config
