# -*- coding: utf8 -*-

from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class AblSegmentH(HeaderSegment):

    sender_type = HprimString((5, 1), default="MCA")
    sender_name = HprimString((5, 2))
    comment = HprimString(11)
    version = HprimString(13, default='1')
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class AblSegmentP(Segment):

    patient_lab_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    age = HprimString((15, 1))
    age_unit = HprimString((15, 2))
    gestation_age = HprimString((16, 1))
    gestation_age_unit = HprimString((16, 2))
    gestation_weight = HprimString((16, 3))
    gestation_weight_unit = HprimString((16, 4))
    height = HprimString((17, 1))
    height_unit = HprimString((17, 2))
    weight = HprimString((18, 1))
    weight_unit = HprimString((18, 2))
    location = HprimString(26)
    orders = HprimForeign("O", "patient")


class AblSegmentQ(Segment):
    patient_id = HprimString((3, 1))
    sample_id = HprimString((3, 2))
    location = HprimString((11, 1))
    patient_dept = HprimString((11, 2))
    status_code = HprimString(13)


class AblSegmentO(Segment):

    specimen_id = HprimString(3)
    # QC #: QC
    # Cal #: Calibration
    # Sample #: Patient
    sample_type = HprimString((4, 1))
    sample_no = HprimString((4, 2))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))

    sample_time = HprimDate(8)

    specimen_type = HprimString((16, 1))
    specimen_sample = HprimString((16, 2))
    physician = HprimString(17)
    # F: Final
    # C: Corrected
    report_type = HprimString(27)

    results = HprimForeign("R", "order")


class AblSegmentR(Segment):

    test = HprimString((3, 4))
    type = HprimString((3, 5))
    value = HprimString(4)
    units = HprimString(5)

    # L: Below low normal range
    # H: Above high normal range
    # LL: Below low critical range
    # HH: Above high critical range
    # <: under range
    # >: over range
    # N: Normal value
    abnormal_flag = HprimList(7)

    # F: Final
    # R: retransmission
    # C: Correction
    status = HprimString(9)
    operator = HprimString(11)
    date_start = HprimDate(12)
    date_completed = HprimDate(13)


class AblSegmentC(Segment):

    # I: Clinical instrument system
    # L: Computer system
    source = HprimString(3)
    code = HprimString((4, 1))
    libelle = HprimString((4, 2))
    # I: Instrument flag comments
    # G: Générals comments
    c_type = HprimString(5)


class AblSegmentM(Segment):

    command = HprimString(3)


class AblSegmentL(Segment):

    # N: Normal termination
    # I: No info available
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "abl",
        "label": "ABL 7XX, 8XX",
    }
    SEGMENT_CLASS_MAP = {
        "H": AblSegmentH,
        "P": AblSegmentP,
        "O": AblSegmentO,
        "Q": AblSegmentQ,
        "C": AblSegmentC,
        "R": AblSegmentR,
        "M": AblSegmentM,
        "L": AblSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "qc_prefix_lot",
            label="Préfixer le lot de QC par le champ O.16.1",
            default=False),
    ]

    def __init__(self, qc_prefix_lot=False, **kwargs):
        self.qc_prefix_lot = qc_prefix_lot
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = AblSegmentH(type_segment="H", **kwargs)
        message.end = AblSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])

        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seg_p = msg.patients.add(
                patient_lab_id=patient.lab_patient_id or patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            resultats = set()
            for sample, action in patient.samples:
                for physician in sample.physicians:
                    if physician.type == "prescriber":
                        prescriber = physician.code
                        break
                else:
                    prescriber = None
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    sample_time=sample.request_date,
                    physician=prescriber,
                    specimen_type=sample.specimen,
                )
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code
                for result in sample.results:
                    if not result.anterior:
                        resultats.add("%s:%s" % (result.code, result.value))
            if resultats:
                seg_p.location = "%s(%s)" % (
                    patient.location,
                    " ".join(sorted(resultats)),
                )
            else:
                seg_p.location = patient.location

        if msg.patients:
            return msg
        elif context["query"]:
            seg_p = msg.patients.add()
            for seg_q in context["query"].queries:
                seg_p.orders.add(specimen_id=seg_q.sample_id)
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.sample_id)
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.sample_type == "Sample #":
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.specimen_id,
                    )
                elif seg_o.sample_type == "QC #":

                    if self.qc_prefix_lot:
                        lot = "%s%s" % (seg_o.specimen_type, seg_o.specimen_sample)
                    else:
                        lot = seg_o.specimen_sample

                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=lot,
                        control_lot=lot,
                    )
                else:
                    continue

                date = None
                operator = None

                for seg_r in seg_o.results:
                    if sample.type == "patient" and seg_r.status not in ("F", "C"):
                        continue
                    if seg_r.date_start:
                        date = seg_r.date_start

                    flags = set(seg_r.abnormal_flag[:])
                    for seg_c in seg_r.commentaires:
                        if seg_c.source == 'I' and seg_c.c_type == 'I' and seg_c.code:
                            flags.add(seg_c.code)

                    result = sample.results.add(
                        code=seg_r.test,
                        value=seg_r.value,
                        date=date,
                        comments=[
                            seg_c.code for seg_c in seg_r.commentaires
                            if seg_c.source == "G"
                        ],
                        flags=sorted(flags) if sample.type == "patient" else [],
                    )
                    if seg_r.operator:
                        operator = seg_r.operator
                        result.operator = operator

                if sample.type == "patient":
                    sample.results.add(
                        code="PRELEV",
                        value=seg_o.specimen_type,
                        date=date,
                    )
                    if date is not None:
                        sample.results.add(
                            code="HEURE",
                            value=date.strftime("%H:%M"),
                            date=date,
                        )
                    if seg_o.tests and seg_o.tests[0].code:
                        sample.results.add(
                            code="ORIGPRL",
                            value=seg_o.tests[0].code,
                            date=date,
                        )
                    if operator:
                        sample.results.add(
                            code="OPE",
                            value=operator,
                            date=date,
                        )

        return worklist
