# -*- coding: utf8 -*-

from __future__ import unicode_literals

from unidecode import unidecode

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class Acl9000SegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10, default="ACL9000")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Acl9000SegmentQ(Segment):
    starting_range_id = HprimString((3, 2))
    request_code = HprimString(11)


class Acl9000SegmentP(Segment):
    FILL_SEGMENT = 35

    # patient_name = HprimString(6, max_length=30)
    last_name = HprimString((6, 1), max_length=20)
    first_name = HprimString((6, 2), max_length=10)
    birth_date = HprimDate(8, fmt_date="%Y%m%d")
    # M or m/F or f/U or u
    sex = HprimString(9)
    location = HprimString(26)
    orders = HprimForeign("O", "patient")


class Acl9000SegmentO(Segment):
    FILL_SEGMENT = 35

    specimen_id = HprimString(3)
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))
    priority = HprimString(6)
    action_code = HprimString(12)
    # Vérifier que la présence du sépérateur ^ n'est pas obligatoire
    # dans le champ specimen_descriptor
    specimen_descriptor = HprimString(16)
    ordering_physician = HprimString(17)
    report_type = HprimString(27, default="O")
    results = HprimForeign("R", "order")


class Acl9000SegmentC(Segment):
    source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))
    comment_type = HprimString(5)


class Acl9000SegmentR(Segment):
    test_id = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    result_status = HprimString(9)
    date_completed = HprimDate(13)


class Acl9000SegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "acl9000",
        "label": "Acl9000",
    }
    ENCODING = "cp437"
    SEGMENT_CLASS_MAP = {
        "H": Acl9000SegmentH,
        "Q": Acl9000SegmentQ,
        "P": Acl9000SegmentP,
        "O": Acl9000SegmentO,
        "R": Acl9000SegmentR,
        "C": Acl9000SegmentC,
        "Q": Acl9000SegmentQ,
        "L": Acl9000SegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    OPTIONS = [
        AstmMessageV2.Option("qc_prefix", label="Préfixe QC"),
    ]

    def __init__(self, qc_prefix=None, **kwargs):
        self.qc_prefix = qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = Acl9000SegmentH(type_segment="H", **kwargs)
        message.end = Acl9000SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            action={"del": worklist.IGNORE},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = message.patients.add(
                # FIXME: A supprimer quand le calcul du checksum se
                # fera après encodage
                last_name=unidecode(patient.last_name),
                first_name=unidecode(patient.first_name),
                birth_date=patient.birthdate,
                sex=patient.sex,
            )
            for sample, priority in patient.samples:
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        priority=priority,
                    )
                    for physician in sample.physicians:
                        if physician.type == "prescriber":
                            seg_o.ordering_physician = physician.code
                            break
                    t = seg_o.universal_test_id.add()
                    t.code = order.code

        if message.patients or context["query"]:
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id.strip(),
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                sample_id = seg_o.specimen_id.strip()

                if seg_o.action_code == "Q" or (
                    self.qc_prefix and
                    sample_id.startswith(self.qc_prefix)
                ):
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=sample_id,
                        control_lot=sample_id,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=sample_id,
                    )

                for seg_r in seg_o.results:
                    if sample.type == "patient" in "F" not in seg_r.result_status:
                        continue
                    comments = []
                    for seg_c in seg_r.commentaires:
                        comments.append(seg_c.comment_text)

                    # le code reponse est la concatenation
                    # du test avec l'unite
                    if seg_r.units:
                        test = "%s%s" % (seg_r.test_id, seg_r.units)
                    else:
                        test = seg_r.test_id

                    sample.results.add(
                        code=test,
                        value=seg_r.value,
                        comments=comments,
                        date=seg_r.date_completed,
                    )
        return worklist
