# -*- coding: utf8 -*-
from __future__ import unicode_literals

import xml.etree.cElementTree as ET
import datetime
from collections import OrderedDict

from driverng import Message
from driverng import utils


class Driver(Message):
    SPEC = {
        "code": "adagioxml",
        "label": "Adagio XML",
    }
    ENCODING = "utf8"

    OPTIONS = [
        Message.Option(
            "atb_order_classtcode",
            label=u"Ordonner les antibiotiques selon la classe de texte codé"),
    ]

    def __init__(self, atb_order_classtcode=None, **kwargs):
        self.atb_order_classtcode = atb_order_classtcode
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, 'utf-8'))

    def get_message_type(self, message):
        if message.find("query"):
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for query in message.findall("query"):
            for sample in query.findall("sample"):
                queries.append(query.findtext("sample"))
        return queries

    def get_order_message(self, worklist, context):
        dat = ET.Element("data")
        patients = ET.SubElement(dat, "patients")
        empty = True
        for sample, _ in worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seq = 1
            patient = ET.SubElement(patients, "patient")
            patient.set("id", "1")
            ET.SubElement(patient, "patient_id").text = sample["lab_patient_id"]
            name = ET.SubElement(patient, "name")
            ET.SubElement(name, "lastname").text = sample["last_name"]
            ET.SubElement(name, "firstname").text = sample["first_name"]
            if sample["birthdate"]:
                ET.SubElement(
                    patient,
                    "birthdate"
                ).text = utils.format_date(sample["birthdate"], fmt="%Y%m%d")
            if sample["sex"] == "F":
                ET.SubElement(
                    patient, "sex").text = "1"
            elif sample["sex"] == "M":
                ET.SubElement(
                    patient, "sex").text = "2"
            samples = ET.SubElement(patient, "samples")
            tube_sample = ET.SubElement(samples, "sample")
            tube_sample.set("id", str(seq))
            for order in sample["orders"]:
                ET.SubElement(tube_sample, "sample_code").text = sample["sample_id"]
                ET.SubElement(tube_sample, "sample_type_code").text = order["code"]
                ET.SubElement(
                    tube_sample,
                    "exam_date"
                ).text = utils.format_date(sample["request_date"], fmt="%Y%m%d")
                empty = False
            sample_prescriber = ET.SubElement(tube_sample, "sample_prescriber")
            ET.SubElement(sample_prescriber, "sample_prescriber_code").text = sample["physician"]
            ET.SubElement(sample_prescriber, "sample_prescriber_lastname").text = sample["physician_name"]

            # Envoi du code du service transmetteur
            if sample["service"]:
                hospitalizations = ET.SubElement(tube_sample, "hospitalizations")
                hospitalizations.set("nb", "1")
                hospitalization = ET.SubElement(hospitalizations, "hospitalization")
                hospitalization.set("id", "1")
                wards = ET.SubElement(hospitalization, "wards")
                wards.set("nb", "1")
                ward = ET.SubElement(wards, "ward")
                ward.set("id", "1")
                ET.SubElement(ward, "ward_code").text = sample["service"]

            examens = ET.SubElement(tube_sample, "analysiss")
            examen = ET.SubElement(examens, "analysis")
            for result in sample["results"]:
                id_germe = ""
                rank_germ = ""
                try:
                    id_germe = result["resu_json"]["germes"][0].get("Id")
                    if isinstance(id_germe, list):
                        id_germe = ""
                except KeyError:
                    id_germe = ""

                try:
                    rank_germ = result["resu_json"]["germes"][0].get("rang")
                except KeyError:
                    rank_germ = ""
                if result['value'].startswith('G_') or id_germe:
                    germ = ET.SubElement(examen, "germ")
                    if result['value'].startswith('G_'):
                        ET.SubElement(germ, "germ_code").text = result['value'][2:]
                    elif id_germe:
                        ET.SubElement(germ, "germ_code").text = id_germe
                    if rank_germ:
                        ET.SubElement(germ, "germ_rank").text = rank_germ

            seq += 1
        if not empty:
            return dat

    def get_result_dict(self, message):
        if not message.find("patients"):
            return {}
        samples_results = {}
        patients = message.find("patients")
        for patient in patients.findall("patient"):
            samples = patient.find("samples")
            for sample in samples.findall("sample"):
                sample_id = sample.findtext("sample_code")
                sample_results = samples_results.setdefault(sample_id, {})
                test = sample.findtext("sample_type_code")
                germe = "G_%s" % (
                    sample.findtext("./analysiss/analysis/germ/germ_code").upper(),
                )
                result = {
                    "atb": [],
                    "seq": sample.findtext("./analysiss/analysis/germ/germ_rank"),
                    "test": test,
                    "date": datetime.datetime.strptime(
                        sample.findtext("./analysiss/analysis/exam_validation_date"),
                        "%Y%m%d"
                    ),
                    "resu": germe,
                }
                antibiotiques = OrderedDict()
                antibiogramme = False
                for antibiotic in sample.findall(
                    "./analysiss/analysis/results/result"
                ):
                    if antibiotic.findtext("antibiotic_code") is None:
                        continue
                    code = "A_%s" % (antibiotic.findtext("antibiotic_code"),)
                    antibio = antibiotiques.setdefault(code.upper(), {
                        'ANTIBIO': code, 'SENSI': None, 'CMI': None
                    })
                    if antibiotic.findtext("estimated_MIC"):
                        antibio['CMI'] = antibiotic.findtext("estimated_MIC")
                    if antibiotic.findtext("final_RIS"):
                        antibio['SENSI'] = antibiotic.findtext("final_RIS")
                    listeAntibiotiques = []
                    for antibio in antibiotiques.itervalues():
                        listeAntibiotiques.append(tuple([
                            (key, antibio[key]) for key in sorted(antibio.iterkeys())
                        ]))
                        antibiogramme = True
                if antibiogramme:
                    for antibio in listeAntibiotiques:
                        if dict(antibio)['SENSI'] is None and dict(antibio)['CMI'] is None:
                            pos = listeAntibiotiques.index(antibio)
                            listeAntibiotiques.pop(pos)
                    result["atb"] = listeAntibiotiques

                if not result["atb"]:
                    # c'est une identification de germes
                    result["test"] = "IDENT_" + result["test"]
                    result["resu"] = {
                        "type_resultat": "identification_germes",
                        "germes": [],
                    }
                    result["seq"] = "0"
                    if result["test"] in sample_results:
                        result["resu"]["germes"] = (
                            sample_results[result["test"]]["resu"].get("germes", [])
                        )
                    result["resu"]["germes"].append({
                        "id": germe,
                        "rang": int(sample.findtext("./analysiss/analysis/germ/germ_rank")),
                    })
                sample_results[result["test"]] = result

            results = {}
            for sample_id, data in samples_results.iteritems():
                results.setdefault(sample_id, {"resultats": []})["resultats"].extend(
                    data.values())
        return results
