# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import (
    Message,
)
from driverng.fixed import (
    FixedLengthFormat,
    String,
    Date,
    Multiple,
)
from driverng.exceptions import ParsingError

FRAME_SIZE_512, FRAME_SIZE_256 = list(range(2))
FRAME_SIZE = FRAME_SIZE_512
MAX_SIZE_ORDER = {
    # First block, second block
    FRAME_SIZE_512: (105, 118),
    FRAME_SIZE_256: (41, 54),
}.get(FRAME_SIZE, None)
MAX_SIZE_RESULT = {
    # First block second block
    FRAME_SIZE_512: (21, 21),
    FRAME_SIZE_256: (8, 8),
}.get(FRAME_SIZE, None)


class Advia1200SingleMessage(FixedLengthFormat):

    @classmethod
    def parse_block(cls, block):
        raise NotImplementedError

    @staticmethod
    def splitlist(ilist, sizes):
        # Coupe la liste ilist
        # En liste de listes d'au maximum
        # sizes[0] pour le premier elements
        # sizes[1] pour les autres
        if len(ilist) > sizes[0]:
            rlist = [ilist[:sizes[0]], ilist[sizes[0]:]]
            while len(rlist[-1]) > sizes[1]:
                rlist = (
                    rlist[:-1] +
                    rlist[-1][:sizes[1]] +
                    rlist[-1][sizes[1]:]
                )
        else:
            rlist = [ilist]

        return rlist


class Advia1200Query(Advia1200SingleMessage):
    FMT_DESCRIPTION = [
        "Q",
        1,
        String("blocks", 2, right_justified=True, empty="0", default="01"),
        String("n_block", 2, right_justified=True, empty="0", default="01"),
        String("n_inquiry", 2, default="99"),     # 99
        # 0 => Arbitrary sample ID, Barcode ID, Sequential ID
        # 1 => Sample Tray TT No. and cup position No
        # 2 => Rack No. and number indicating position in rack
        String("sid_class", 1, default="1"),
        Multiple("samples", [
            String("sample_id", 13),
        ]),
        1,
    ]

    @classmethod
    def parse_block(cls, block):
        # Le format est le même sur toutes les lignes du block
        # On renvoie simplement plusieurs Advia1200Query
        messages = []

        for line in block:
            obj = cls()
            obj.parse(line)
            messages.append(obj)

        return messages


class Advia1200Order(Advia1200SingleMessage):
    FMT_DESCRIPTION_HEADER = [
        1,
        String("blocks", 2, right_justified=True, empty="0"),  # 01 to 07
        String("n_block", 2, right_justified=True, empty="0"),  # 01 to 07
        String("_n_item_per_block", 3,
               right_justified=True, empty="0"),    # 999
    ]

    FMT_DESCRIPTION_FOOTER = [
        # Appelé sample classification dans la doc
        # N => Général sample
        # I => Interruption sample
        # C => Control sample
        # S => STAT sample
        String("order_classification", 1),
        # 0 => New request (if workorder already exists, it's overwritten
        # 1 => Item addition, rerun.
        # 2 => No request.
        # 3 => Delete tests (Seulement pour advia 1650 & 1800)
        String("registration_data", 1),
        # Le tube
        String("sid", 13),
        String("position_number", 7),
        # Nom patient OU Nom du lot
        String("comment1", 16, truncable=True),
        # Prenom patient OU ref_lot/date expiration lot
        String("comment2", 16, truncable=True),
        # M => Male
        # F => Female
        # When not used, set to M
        String("sex", 1),
        String("age", 3, right_justified=True),
        Date("blood_sampling_date", 8, empty="0", fmt_date="%Y%m%d"),
        String("dilution_coefficient", 4, right_justified=True),
        # 1 => Serum
        # 2 => Urine
        # When not used, be sure to input 1
        String("sample_classification", 1),
        # 1 to 9, when not used 1
        String("container_classification", 1),
    ]

    FMT_ORDER = [
        String("code", 3, right_justified=True),
        # Sample volume and dilution determinad via the
        # analytical parameters
        # (chemistry) window. One byte "M" or "D" or "U"
        # Normally set to "M"
        # M => Analytical conditions value
        # U = 1 and D = 2 for Advia 1200 condition
        # TODO: Comprendre ce truc
        String("condition", 1, default="M"),
    ]

    FMT_DESCRIPTION = (
        ["O"] +
        FMT_DESCRIPTION_HEADER +
        FMT_DESCRIPTION_FOOTER + [
            Multiple("orders", FMT_ORDER),
            1,
        ]
    )

    FMT_DESCRIPTION_SECOND_BLOCK = (
        ["O"] +
        FMT_DESCRIPTION_HEADER + [
            2,
            String("sid", 13),
            7,
            Multiple("orders", FMT_ORDER),
            1,
        ]
    )

    COPY_SECONDS_ATTRS = ("sid")
    MAX_SIZE = MAX_SIZE_ORDER

    @classmethod
    def parse_block(cls, block):
        obj = cls()
        obj.parse(block[0])
        for line in block[1:]:
            # Second to last block
            second = FixedLengthFormat(
                fmt_description=cls.FMT_DESCRIPTION_SECOND_BLOCK)
            second.parse(line)
            for attr in cls.COPY_SECONDS_ATTRS:
                first_value = getattr(obj, attr)
                second_value = getattr(second, attr)
                if first_value != second_value:
                    raise ParsingError(
                        "Multiple block attr '%s' mismatch '%s' != '%s'" % (
                            attr, first_value, second_value))

            obj.orders.extend(second.orders)

        return [obj]

    def to_string(self):
        if self.MAX_SIZE is not None:
            orders = self.splitlist(self.orders, self.MAX_SIZE)
        else:
            orders = [self.orders]

        blocks = []

        for i, order in enumerate(orders):
            if i == 0:
                orig_orders = self.orders
                self.orders = order
                self._n_item_per_block = len(order)
                self.blocks = len(orders)
                self.n_block = "1"
                blocks.append(super(Advia1200Order, self).to_string())
                self.orders = orig_orders
            else:
                # second to last block
                obj = FixedLengthFormat(
                    fmt_description=self.FMT_DESCRIPTION_SECOND_BLOCK,
                    blocks=len(orders),
                    n_block="%s" % (i + 1),
                    _n_item_per_block=len(order),
                )
                obj.orders = order
                for attr in self.COPY_SECONDS_ATTRS:
                    setattr(obj, attr, getattr(self, attr))
                blocks.append(obj.to_string())

        return "\r\n".join(blocks)


ADVIA1200_FLAGS = {
    "judgment": {
        "C": "Calibration not possible",
        "H": "Abnormal value limit",
        "L": "Abnormal value limit",
        "h": "Normal value limit",
        "l": "Normal value limit",
    },
    "status": {
        "S": "Safety error",  # Plusieurs causes possibles
        "s": "Insufficient sample",
        "r": "Insufficient reagent",
        "t": "Insufficient diluted liquid",
        "u": "Light absorption limit",
        "d": "Light absorption limit",
        "U": "Light absorption limit",
        "D": "Light absorption limit",
        "P": "Prozone",
        "*": "Dispersion abnormality",
        "n": "Abnormal number of effective points",
        "N": "Cell-blank abnormality",
        "?": "Overflow",
    },
    "rerun": {
        "R": "Rerun",
    }
}


class Advia1200Result(Advia1200Order):

    FMT_RESULT = Advia1200Order.FMT_ORDER + [
        # Result value
        String("value", 8, right_justified=True),
        # Flags on 3 bytes, judgment, status and rerun each 1 byte
        String("judgment", 1),
        String("status", 1),
        String("rerun", 1),
        # E will be sent in this position when the operator
        # manually enters the measurement value
        String("mode", 1),
        # User code sur 3 bytes
        String("user", 3),
    ]
    FMT_DESCRIPTION_HEADER = Advia1200Order.FMT_DESCRIPTION_HEADER + [
        # Deux champs dans la doc (date et time), mais ils sont alignés
        Date("inspection_date", 14, fmt_date="%Y%m%d%H%M%S"),
    ]

    FMT_DESCRIPTION = (
        ["R"] +
        FMT_DESCRIPTION_HEADER +
        Advia1200Order.FMT_DESCRIPTION_FOOTER + [
            Multiple("orders", FMT_RESULT),
            1,
        ]
    )
    FMT_DESCRIPTION_SECOND_BLOCK = (
        ["R"] +
        FMT_DESCRIPTION_HEADER + [
            String("order_classification", 1),
            String("registration_data", 1),
            String("sid", 13),
            7,
            Multiple("orders", FMT_RESULT),
            1,
        ]
    )

    COPY_SECONDS_ATTRS = ("inspection_date",
                          "order_classification", "registration_data", "sid")
    MAX_SIZE = MAX_SIZE_RESULT


class MultipleAdvia1200Message(list):

    MAP = {
        "Q": Advia1200Query,
        "O": Advia1200Order,
        "R": Advia1200Result,
    }

    def add(self, message_type, *args, **kwargs):
        obj = self.MAP[message_type](*args, **kwargs)
        self.append(obj)
        return obj

    @staticmethod
    def create():
        return MultipleAdvia1200Message()

    @staticmethod
    def parse(data):
        obj = MultipleAdvia1200Message()

        block = []

        for line in data.splitlines():
            # suivant la "frame size" (256 ou 512 bytes) on peut recevoir
            # un seul message sur plusieurs lignes. Mais ce n'est pas une
            # simple continuation de la ligne, il y a un header commun
            # Exemple:
            # R 0301...
            # R 0302...
            # R 0303...
            # On empile et dès que blocks == n_block on parse le message
            block.append(line)
            if line[2:][:2] == line[4:][:2]:
                obj.extend(obj.MAP[line[0]].parse_block(block))
                block = []

        if block:
            raise ParsingError("Remaining data '%s'" % (
                block,
            ))

        return obj

    def to_string(self):
        return "\r\n".join([m.to_string() for m in self])


class DriverAdvia1200(Message):
    SPEC = {
        "code": "advia1200",
        "label": "Advia 1200",
    }
    MANAGE_DELETED_TESTS = False

    DEFAULT_MESSAGE_KWARGS = {
        "order_classification": "N",
        "dilution_coefficient": "1.0",
        "container_classification": "1",
    }

    def create(self):
        return MultipleAdvia1200Message.create()

    def get_message_type(self, message):
        for msg in message:
            if isinstance(msg, Advia1200Query):
                return self.QUERY
        return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        return MultipleAdvia1200Message.parse(data)

    def get_queries(self, message):
        queries = []
        for msg in message:
            if isinstance(msg, Advia1200Query):
                queries.extend([s.sample_id for s in msg.samples])
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create()
        tubes_ok = []
        actions = {"add": "0", "new": "0"}
        if self.MANAGE_DELETED_TESTS:
            actions["del"] = "3"
        else:
            actions["del"] = worklist.IGNORE

        for sample, action in worklist.get_worklist(
            unique_together=["action"],
            action=actions,
        ):
            tubes_ok.append(sample["sample_id"])
            # 1 => Blood serum
            # 2 => Urine
            # When not used, enter 1
            if sample["specimen"] in ("1", "2"):
                sample_classification = sample["specimen"]
            else:
                sample_classification = "1"

            if sample["birthdate"] and context["now"]:
                age = "%d" % (
                    (context["now"] - sample["birthdate"]).days / 365,
                )
            else:
                age = None

            request = msg.add(
                "O",
                sid=sample["sample_id"],
                comment1=sample["last_name"],
                comment2=sample["first_name"],
                age=age,
                # When not used set to M
                sex="F" if sample["sex"] == "F" else "M",
                blood_sampling_date=sample["collection_date"],
                sample_classification=sample_classification,
                **self.DEFAULT_MESSAGE_KWARGS
            )
            rd = set()
            for order in sample["orders"]:
                rd.add({
                    "add": "1",
                    "new": "0",
                    "del": "3"
                }.get(order["action"]))
                request.orders.add(code=order["code"])
            request.registration_data = max(rd or set([action]))

        if context["query"]:
            for sample_id in self.get_queries(context["query"]):
                if sample_id not in tubes_ok:
                    # On a pas de workorder pour ce tube mais il faut quand
                    # même répondre avec registration_data à 2 (No request)
                    msg.add(
                        "O",
                        sid=sample_id,
                        registration_data="2",
                        sex="M",
                        sample_classification="1",
                        **self.DEFAULT_MESSAGE_KWARGS
                    )

        if len(msg):
            return msg

    def get_result_dict(self, message):
        results = {}
        for result_message in message:

            if result_message.order_classification not in ("N", "I"):
                continue

            if result_message.sid not in results:
                results[result_message.sid] = {"resultats": []}

            for resu in result_message.orders:
                alarmes = []
                for attr in ("judgment", "rerun", "status"):
                    alarme = getattr(resu, attr)
                    if alarme:
                        alarmes.append(alarme)

                results[result_message.sid]["resultats"].append({
                    "test": resu.code,
                    "resu": resu.value,
                    "date": result_message.inspection_date,
                    "alarmes": alarmes,
                    "type_prlv": result_message.sample_classification,
                })
        return results

    def get_qc_result_list(self, message):
        result = []

        for result_message in message:

            if result_message.order_classification != "C":
                continue

            ref_lot = result_message.comment2.split("/", 1)[0].strip()

            for resu in result_message.orders:
                if resu.value and resu.value.strip():
                    result.append({
                        "lot": ref_lot,
                        "test": resu.code,
                        "resu": resu.value,
                        "date": result_message.inspection_date,
                        "type_prlv": result_message.sample_classification,
                    })

        return result
