# -*- coding: utf8 -*-
from __future__ import unicode_literals

import base64
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimForeign,
)


class Advia560SegmentMSH(HeaderSegment):
    sending_application = HprimString((3, 1), default="ADVIA560", required=True)
    sending_universal_id = HprimString((3, 2), default="XYZ_ID", required=True)
    date = HprimDate(7)
    message_type = HprimMultiple(
        9,
        m_type=HprimString(1, default="ORU"),
        m_code=HprimString(2, default="R01"),
    )
    message_control_id = HprimString(10)
    processing_id = HprimString(11, required=True, default="P")
    version_id = HprimString(12, required=True, default="2.5")
    sequence_number = HprimString(13)

    orders = HprimForeign("OBR", "header")


class Advia560SegmentOBR(Segment):
    entity_identifier = HprimString((4, 1))
    namespace_id = HprimString((4, 2))
    universal_service_identifier = HprimString(5)
    results = HprimForeign("OBX", "order")


class Advia560SegmentOBX(Segment):
    # TX
    value_type = HprimString(3)
    observation_identifier = HprimString(4)
    observation_sub_id = HprimString(5)
    # Result of the analysis
    value = HprimString(6)
    units_identifier = HprimString((7, 1))
    units_text = HprimString((7, 2))
    reference_range = HprimString(8)
    abnormal_flags = HprimString(9)
    probability = HprimString(10)
    nature_abnormal_test = HprimString(11)
    observation_result_status = HprimString(12)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "advia560",
        "label": "Advia360/560 Monodirectional"
    }

    SEGMENT_CLASS_MAP = {
        "MSH": Advia560SegmentMSH,
        "OBR": Advia560SegmentOBR,
        "OBX": Advia560SegmentOBX,
    }
    DEFAULT_SEPARATORS = "|&~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return Advia560SegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def decode_message(self, message):
        if message.message_type.m_type != "ORU":
            raise NotImplementedError

        worklist = Worklist(context="result")
        for seg_obr in message.orders:
            sample = worklist.get_or_create(
                sample_id=seg_obr.entity_identifier,
                type="patient",
            )
            for seg_obx in seg_obr.results:
                if seg_obx.value_type != 'ED':
                    sample.results.add(
                        code=seg_obx.observation_identifier,
                        value=seg_obx.value.replace(",", "."),
                        date=message.date,
                    )
                else:
                    sample.results.add(
                        code=seg_obx.observation_identifier,
                        value="GRAPHE",
                        graph={
                            "graph_type": "binary",
                            "format": "png",
                            "data": base64.b64decode(seg_obx.value),
                        }
                    )
        return worklist
