# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class AdviaXPTSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class AdviaXPTSegmentP(Segment):
    patient_id = HprimString(3)
    laboratory_id = HprimString(4)
    last_name = HprimString((6, 1), max_length=20, trunc=True)
    first_name = HprimString((6, 2), max_length=16, trunc=True)
    middle_name = HprimString((6, 3))
    date_birth = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician_id = HprimString((14, 1))
    attending_physician_name = HprimString((14, 2))
    species = HprimString(15)
    patient_status = HprimString(25)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class AdviaXPTSegmentO(Segment):
    sample_id = HprimString((3, 1))
    rack = HprimString((3, 2))
    position = HprimString((3, 3))
    universal_test_id = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            test=HprimString(4),
            dilution=HprimString(5),
            dilution_ratio=HprimString(6),
            replicates=HprimString(7),
        ))

    priority = HprimString(6)
    order_date = HprimDate(7)
    action_code = HprimString(12)
    specimen_descriptor_type = HprimString((16, 1))
    specimen_descriptor_source = HprimString((16, 2))
    report_type = HprimString(26, default="O")

    results = HprimForeign("R", "order")
    control = HprimForeign("M", "order")


class AdviaXPTSegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        code=HprimString(4),
        dilution_protocol=HprimString(5),
        dilution_coeff=HprimString(6),
        replicate_number=HprimString(7),
        # ALGC - Allergen code
        # CLSS - Allergy class
        # COFF - Cut off
        # DOSE - Result concentration value
        # INDX - Result index value
        # INTR - Result interpretation
        # RLU - Value used in result calculation
        result_aspect=HprimString(8),
        # Contains the Primary Reagent lot number that generated the test
        reagent_lot=HprimString(9),
        # Contains the Ancillary reagent lot number that generated the test result
        ancillary_reagent_lot=HprimString(10),
        # Contains the diluent lot number that generated the test result
        diluent_lot_number=HprimString(11),
    )
    result = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)
    flags = HprimList(7)
    result_status = HprimString(9)
    operator = HprimString((11, 1))
    validator = HprimString((11, 2))
    date_completed = HprimDate(13)
    instrument = HprimString(14)


class AdviaXPTSegmentC(Segment):
    source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))
    comment_type = HprimString(5)


class AdviaXPTSegmentM(Segment):
    manufacturer_order = HprimMultiple(
        3,
        manufacturer=HprimString(1),
        instrument=HprimString(2),
        record_version=HprimString(3),
        record_type=HprimString(4),
    )
    control_name = HprimString(4)
    control_lot = HprimString(5)


class AdviaXPTSegmentQ(Segment):
    starting_range_id = HprimMultiple(
        3,
        patient_id=HprimString(1),
        sample_id=HprimString(2),
        rack=HprimString(3),
        position=HprimString(4),
    )
    ending_range_id = HprimMultiple(
        4,
        patient_id=HprimString(1),
        sample_id=HprimString(2),
        rack=HprimString(3),
        position=HprimString(4),
    )
    universal_test_id = HprimMultiple(
        5,
        test_id=HprimString(1),
        test_name=HprimString(2),
        test_type=HprimString(3),
        manufacturer_code=HprimString(4),
    )
    nature_request = HprimString(6)
    beginning_request_date = HprimDate(7)
    ending_request_date = HprimDate(8)
    request_status_code = HprimString(13)


class AdviaXPTSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class DriverAdviaXPT(AstmMessageV2):
    SPEC = {
        "code": "advia_chimie_xpt",
        "label": "Advia Chimie XPT"
    }
    SEGMENT_CLASS_MAP = {
        "H": AdviaXPTSegmentH,
        "P": AdviaXPTSegmentP,
        "O": AdviaXPTSegmentO,
        "R": AdviaXPTSegmentR,
        "C": AdviaXPTSegmentC,
        "M": AdviaXPTSegmentM,
        "Q": AdviaXPTSegmentQ,
        "L": AdviaXPTSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = AdviaXPTSegmentH(type_segment="H", **kwargs)
        message.end = AdviaXPTSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", "A": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                laboratory_id=patient.lab_patient_id,
                first_name=patient.first_name,
                last_name=patient.last_name,
                date_birth=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    order_date=sample.request_date,
                    specimen_descriptor_type=sample.specimen,
                )
                for physician in sample.physicians:
                    seg_p.attending_physician_id = physician.code
                seg_p.location = sample.location
                for order in sample.orders:
                    t = seg_o.universal_test_id.add()
                    t.test = order["code"]
        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            msg.end.terminaison_code = "I"
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id.sample_id
                )
                if seg_q.starting_range_id.rack and seg_q.starting_range_id.position:
                    sample.rack = "%s-%s" % (
                        seg_q.starting_range_id.rack,
                        seg_q.starting_range_id.position,
                    )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q":
                        for seg_m in seg_o.control:
                            lot = seg_m.control_lot
                        sample = worklist.get_or_create(
                            control_lot=lot,
                            sample_id=lot,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id,
                            type="patient"
                        )
                    if seg_o.rack and seg_o.position:
                        sample.rack = "%s-%s" % (
                            seg_o.rack,
                            seg_o.position,
                        )
                    for seg_r in seg_o.results:
                        # Ne pas prendre en compte les résultats partiels avec
                        # le champ R.9 contenant le statut "P"
                        if seg_r.result_status == "P":
                            continue
                        alarmes = []
                        if seg_r.flags:
                            alarmes = seg_r.flags
                        for seg_c in seg_r.commentaires:
                            # Traitement uniquement des commentaires de type I
                            if seg_c.comment_type == "I":
                                alarmes.append(seg_c.comment_code)
                        if seg_r.universal_test_id.dilution_protocol or seg_r.universal_test_id.dilution_coeff:
                            dilution = "%s-%s" % (
                                seg_r.universal_test_id.dilution_protocol,
                                seg_r.universal_test_id.dilution_coeff,
                            )
                            alarmes.append(dilution)
                        test = "%s%s" % (
                            seg_r.universal_test_id.code,
                            seg_r.universal_test_id.result_aspect,
                        )
                        result = sample.results.add(
                            code=test,
                            value=seg_r.result,
                            flags=alarmes,
                            date=seg_r.date_completed,
                        )
                        if seg_r.universal_test_id.reagent_lot:
                            result.lots.add(
                                type="reagent",
                                lot=seg_r.universal_test_id.reagent_lot,
                            )
        return worklist
