# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class AlertSegmentMSH(HeaderSegment):
    sender_id = HprimString(3)
    sending_facility = HprimString(4, default="CDP")
    receiver_id = HprimString(5)
    date = HprimDate(7)
    # OML^O21: Order
    # ORL^O22: ACK
    # OUL^R24: Result
    message_type = HprimMultiple(
        9,
        m_type=HprimString(1),
        m_code=HprimString(2),
        m_name=HprimString(3),
    )
    message_control_id = HprimString(10)
    processing_id = HprimString(11, default="P")
    version_id = HprimString(12, default="2.5")

    patients = HprimForeign("PID", "header")
    acknowledgment = HprimForeign("MSA", "header", one2one=True)

    SORT_CHILD_SEGMENT = ["acknowledgment", "patients"]


class AlertSegmentMSA(Segment):
    # AA: Application accepted
    # AR: Application rejected
    code = HprimString(2)
    control_id = HprimString(3)
    # MSA.2 = AA: "OK"
    # MSA.2 = AR: Error message
    text = HprimString(4)

    def to_string(self, separators=None):
        # code
        self.rang_segment = self.code
        return super(AlertSegmentMSA, self).to_string(separators=separators)


class AlertSegmentPID(Segment):

    # IPP
    patient_id = HprimString((4, 1))

    # GAP
    patient_assigned_id = HprimString((4, 4))

    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    visit = HprimForeign("PV1", "patient", one2one=True)
    orders = HprimForeign("ORC", "patient")

    SORT_CHILD_SEGMENT = ["visit", "orders"]


class AlertSegmentPV1(Segment):

    # Nméro de séjour GAP
    visit_id = HprimString(20)


class AlertSegmentORC(Segment):

    # NW: New order
    # SC: Prélèvement
    # CA: Annulation d'analyse
    # SC: Status changed
    order_control = HprimString(2)

    # Alert ORDER ID (ID_ANALYSIS_REQ_DET)
    placer_order_id = HprimString((3, 1))
    placer_order_namespace_id = HprimString((3, 2))

    # Clarilab GROUP ID (Numéro de dossier)
    filler_order_number = HprimString(4)

    # Alert REQUEST ID (ID_ANALYSIS_REQ)
    placer_group_number = HprimString(5)

    # order_control SC
    # IP: In process
    # A: Some, but not all, results available
    # CM: Order completed
    order_status = HprimString(6)

    # Si order_control NW: Date de la demande
    # Si order_control SC: Date du prélèvement
    # Si order_control CA: Date de l'annulation
    date = HprimDate(10)

    # Code ADELI du médecin prescripteur
    order_provider_id = HprimString((13, 1))

    # ADELI
    order_provider_assigned_id = HprimString((13, 9))

    orders = HprimForeign("OBR", "common_order")

    def to_string(self, separators=None):
        # order control
        self.rang_segment = self.order_control
        return super(AlertSegmentORC, self).to_string(separators=separators)


class AlertSegmentOBR(Segment):

    placer_order_id = HprimString((3, 1))
    placer_order_namespace_id = HprimString((3, 2))

    # Code de l'analyse
    universal_service_id = HprimString((5, 1))

    # Nom de l'analyse
    universal_service_text = HprimString((5, 2))

    # "CLARILAB"
    universal_service_coding_name = HprimString((5, 3))

    # Identifiant de l'infirmière qui a fait le recueil
    collector_identifier = HprimString((11, 2))

    # S: Urgent
    # R: Routine
    priority = HprimString((28, 6))

    comments = HprimForeign("NTE", "order")
    specimen = HprimForeign("SPM", "order", one2one=True)
    results = HprimForeign("OBX", "order")

    SORT_CHILD_SEGMENT = ["comments", "specimen", "results"]


class AlertSegmentNTE(Segment):

    # Notes du medecin
    comment = HprimString(4)
    # "OD"
    # "RC" (Resultat)
    comment_type = HprimString(5)


class AlertSegmentSPM(Segment):

    sample_id = HprimString(3)

    # Date planifiée pour le prélèvement
    collection_date = HprimDate(18)

    # order_control CA: Raison de l'annulation
    specimen_reject_reason = HprimString(22)


class AlertSegmentOBX(Segment):
    MAX_VALUE_SUBFIELD = 10
    # NM: Numérique
    # ST: String
    # CE: Coded entry
    value_type = HprimString(3)

    # Code de question
    observation_id = HprimString((4, 1))

    # Texte de question
    observation_text = HprimString((4, 2))

    # "ALERT"
    observation_coding_name = HprimString((4, 3))

    value = HprimMultiple(
        6,
        **dict(
            ("line%d" % (i,), HprimString(i))
            for i in range(1, MAX_VALUE_SUBFIELD + 1))
    )
    observation_unit = HprimString(7)
    reference_range = HprimString(8)
    abnormal_flags = HprimList(9)

    # I: Specimen in lab, result pending
    # P: Résultat non définitif
    # F: Final
    observation_status = HprimString(12)

    observation_date = HprimDate(15)

    # Identifiant du laboratoire
    producer_id = HprimString(16)

    # Identifiant de l'analyseur
    equipment_instance_id = HprimString(19)

    responsible_observer = HprimString(17)


class Driver(AstmMessageV2):

    SPEC = {
        "code": "alert",
        "label": "Alert PFH",
    }

    SEGMENT_CLASS_MAP = {
        "MSH": AlertSegmentMSH,
        "MSA": AlertSegmentMSA,
        "PID": AlertSegmentPID,
        "PV1": AlertSegmentPV1,
        "ORC": AlertSegmentORC,
        "OBR": AlertSegmentOBR,
        "NTE": AlertSegmentNTE,
        "SPM": AlertSegmentSPM,
        "OBX": AlertSegmentOBX,
    }
    FIRST_SEP_INDEX = 3
    DEFAULT_SEPARATORS = "|^~\\&"
    ENCODING = "utf-8"

    def create(self, **kwargs):
        return AlertSegmentMSH(type_segment="MSH", **kwargs)

    def encode_worklist(self, worklist, context):
        message = self.create(
            sender_id=worklist.sender_id or "CLARILAB",
            receiver_id=worklist.receiver_id or "ALERT",
            date=context["now"],
            message_control_id=context.get("control_id", int(time.time() * 1000)),
        )

        if worklist.context == "order":
            message.message_type.m_type = "ORL"
            message.message_type.m_code = "O22"
            message.message_type.m_name = "ORL_O22"
            message.acknowledgment = self.SEGMENT_CLASS_MAP["MSA"](
                type_segment="MSA",
                code="AA",
                text="OK",
            )
        elif worklist.context == "result":
            message.message_type.m_type = "OUL"
            message.message_type.m_code = "R24"
        else:
            raise NotImplementedError

        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            if worklist.context == "result":
                seg_p.visit = self.SEGMENT_CLASS_MAP["PV1"](
                    type_segment="PV1",
                    visit_id=patient.alternate_patient_id,
                )
            for sample, _, _ in patient.samples:
                if (
                    (worklist.context == "order" and not sample.orders) or
                    (worklist.context == "result" and not sample.results)
                ):
                    continue

                seg_spm = self.SEGMENT_CLASS_MAP["SPM"](
                    type_segment="SPM",
                    sample_id=sample.sample_id,
                    collection_date=sample.collection_date,
                )

                if worklist.context == "order":
                    for order in sample.orders:
                        seg_orc = seg_p.orders.add(
                            placer_order_id=order.order_id,
                            filler_order_number=sample.request_id,
                            placer_group_number=sample.receiver_request_id,
                            date=sample.request_date,
                            order_control="OK",
                            order_status="IP",
                        )
                        seg_o = seg_orc.orders.add(
                            universal_service_id=order.code,
                        )
                        seg_o.specimen = seg_spm
                elif worklist.context == "result":
                    codes = {}
                    for result in sample.results:
                        codes.setdefault((result.order_id, result.sub_code), []).append(result)
                    for (order_id, code), results in sorted(codes.items()):
                        seg_orc = seg_p.orders.add(
                            placer_order_id=order_id,
                            filler_order_number=sample.request_id,
                            placer_group_number=sample.receiver_request_id,
                            date=sample.request_date,
                            order_control="SC",
                            order_status="CM",
                        )
                        seg_o = seg_orc.orders.add(
                            universal_service_id=code,
                        )
                        seg_o.specimen = seg_spm
                        for result in results:
                            seg_r = seg_o.results.add(
                                value_type=result.type,
                                observation_id=result.code,
                                observation_status=result.status or "F",
                                observation_date=result.date,
                                equipment_instance_id=result.instrument,
                            )
                            if isinstance(result.value, (list, tuple)):
                                for i, value in enumerate(result.value, 1):
                                    if i > seg_r.MAX_VALUE_SUBFIELD:
                                        raise RuntimeError(
                                            "Cannot encode %s: too much fields" % (result.value,))
                                    setattr(seg_r.value, "line%d" % (i,), value)
                            else:
                                seg_r.value.line1 = result.value
        return message

    def decode_message(self, message):
        if (
            message.message_type.m_type,
            message.message_type.m_code,
        ) != ("OML", "O21"):
            raise NotImplementedError

        worklist = Worklist(
            context="order",
            sender_id=message.sender_id,
            receiver_id=message.receiver_id,
        )
        for seg_p in message.patients:
            for seg_orc in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_orc.placer_group_number,
                    request_id=seg_orc.placer_group_number,
                )
                if seg_orc.order_control == "NW":
                    sample.request_date = seg_orc.date

                sample.alternate_patient_id = seg_p.visit.visit_id
                if seg_orc.order_provider_id:
                    sample.physicians.get_or_create(
                        type="attending",
                        code=seg_orc.order_provider_id,
                        codification="A",  # Adeli
                    )
                sample.title = {
                    "F": "Mme",
                    "M": "Mr",
                }.get(seg_p.sex, "")
                for attr in (
                    "last_name", "first_name", "birthdate",
                    "maiden_name", "patient_id",
                ):
                    sample[attr] = getattr(seg_p, attr)
                sample.sex = seg_p.sex if seg_p.sex in ("M", "F") else "U"
                for seg_o in seg_orc.orders:
                    if seg_o.specimen is not None:
                        sample.collection_date = seg_o.specimen.collection_date

                    if seg_orc.order_control == "CA":
                        action = "del"
                    else:
                        action = "new"

                    sample.orders.get_or_create(
                        code=seg_o.universal_service_id,
                        priority=seg_o.priority,
                        action=action,
                        order_id=seg_orc.placer_order_id,
                    )
        return worklist
