# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Message
from driverng.fixed import FixedLengthFormat, String


class AlifaxBase(FixedLengthFormat):
    QC = False
    RESULT = False


class AlifaxQuery(AlifaxBase):
    FMT_DESCRIPTION = [
        "Q",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("result", 4),
    ]


class AlifaxOrder(AlifaxBase):
    FMT_DESCRIPTION = [
        "T",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("result", 4),
    ]


class AlifaxResult(AlifaxBase):
    FMT_DESCRIPTION = [
        "R",
        String("workstation_number", 2),
        String("patient_id", 15),
        String("rack", 2),
        String("position", 2),
        String("cycle", 2),
        String("result", 4),
    ]
    RESULT = True


class AlifaxQCResult(AlifaxBase):
    FMT_DESCRIPTION = [
        "R",
        String("workstation_number", 2),
        String("latex_tube", 9),
        # 1, 2 or 3
        String("level", 1),
        String("expected_value", 3),
        2,
        String("field2", 2),
        String("field3", 2),
        String("field4", 2),
        String("detected_value", 4),
    ]
    QC = True


class Driver(Message):
    SPEC = {
        "code": "alifax",
        "label": "Alifax",
    }
    TYPE_MESSAGE_MAP = {
        "T": AlifaxOrder,
        "R": AlifaxResult,
        "Q": AlifaxQuery,
        "RQC": AlifaxQCResult,
    }

    def create(self, message_type, **kwargs):
        return self.TYPE_MESSAGE_MAP[message_type](**kwargs)

    def get_message_type(self, message):
        if isinstance(message, AlifaxQuery):
            return self.QUERY
        else:
            return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        message_type = data[0]
        if data[0] == "R" and data[3:][:5] == "LATEX":
            message_type = "RQC"
        else:
            message_type = data[0]
        obj = self.TYPE_MESSAGE_MAP[message_type]()
        obj.parse(data)
        return obj

    def get_queries(self, message):
        return [message.patient_id]

    def get_order_message(self, worklist, context):
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) > 1:
            # FIXME
            raise RuntimeError("Multi-sample message not supported")
        elif len(samples) == 1:
            sample, _ = samples[0]
            return self.create(
                "T",
                workstation_number="01",
                patient_id=sample["sample_id"],
                result="0001",
            )
        elif context["query"]:
            return self.create(
                "T",
                patient_id=context["query"].patient_id,
                result="0000",
            )

    def get_result_dict(self, message):
        if message.RESULT:
            return {message.patient_id: {
                "resultats": [{
                    "test": "VS1",
                    "resu": message.result,
                }],
            }}
        else:
            return {}

    def get_qc_result_list(self, message):
        if message.QC:
            date = datetime.datetime.now()
            date = date.replace(second=0, microsecond=0)
            return [{
                "lot": message.latex_tube + message.level,
                "test": "VS1",
                "resu": message.detected_value,
                "date": date,
            }]
        else:
            return []
