# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HprimList,
    HeaderSegment,
    HprimString,
    HprimDate,
    HprimForeign,
)

logger = logging.getLogger("driverng.atellica")


class AtellicaSegmentMSH(HeaderSegment):
    sending_application = HprimString(3, default="LIS_ID")
    sending_facility = HprimString(4, default="\"\"")
    receiving_application = HprimString(5, default="UIW_LIS")
    receiving_facility = HprimString(6, default="\"\"")
    date = HprimDate(7)
    message_type = HprimString((9, 1), default="OML")
    message_code = HprimString((9, 2), default="033")
    message_label = HprimString((9, 3), default="OML_033")
    control_id = HprimString(10)
    processing_id = HprimString(11)
    version_id = HprimString(12)
    accept_type = HprimString(15)
    application_type = HprimString(16)
    character_set = HprimString(18)
    message_profile = HprimString((21, 1), default="LAB-28")
    message_profile_code = HprimString((21, 2), default="IHE")

    patients = HprimForeign("PID", "header")
    orders = HprimForeign("OBR", "header")
    queries = HprimForeign("QPD", "header")
    errors = HprimForeign("ERR", "header")


class AtellicaSegmentERR(Segment):
    code_error = HprimString((4, 1))
    label_error = HprimString((4, 2))
    flag_error = HprimString((4, 3))
    severity = HprimString(5)
    application_code_error = HprimString((6, 1))
    label_application_error = HprimString((6, 2))
    flag_application_error = HprimString((6, 3))
    user_message = HprimString(9)


class AtellicaSegmentNTE(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class AtellicaSegmentPID(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    visits = HprimForeign("PV1", "patient")
    orders = HprimForeign("OBR", "patient")
    infos = HprimForeign("NTE", "patient")
    specimens = HprimForeign("SPM", "patient")


class AtellicaSegmentPV1(Segment):
    patient_class = HprimString(3)
    location = HprimString((4, 1))
    attending_doctor = HprimString((8, 1))

    orders = HprimForeign("OBR", "patient")
    specimens = HprimForeign("SPM", "patient")


class AtellicaSegmentOBR(Segment):
    awos_id = HprimString(3)
    lis_test_code = HprimString((5, 1))
    lis_test_name = HprimString((5, 2))
    lis_test_loinc = HprimString((5, 3))
    action_code = HprimString(12)
    system_identifier = HprimString((47, 1))
    module_idenfifier = HprimString((47, 2))

    results = HprimForeign("OBX", "order")
    infos_tests = HprimForeign("TCD", "order")
    common_order = HprimForeign("ORC", "order")


class AtellicaSegmentSPM(Segment):
    specimen_id = HprimString(3)
    specimen_type = HprimString((5, 1))
    specimen_label = HprimString((5, 2))
    specimen_role = HprimString(12)
    specimen_collection_date = HprimDate(18)
    specimen_received_date = HprimDate(19)
    container_type = HprimString(28)

    orders = HprimForeign("OBR", "patient")
    infos_samples = HprimForeign("SAC", "patient")
    sample_priority = HprimForeign("TQ1", "patient")

    def to_string(self, separators=None):
        self.rang_segment = "1"
        return super(AtellicaSegmentSPM, self).to_string(separators=separators)


class AtellicaSegmentTCD(Segment):
    auto_dilution = HprimList(3)
    auto_dilution_type = HprimString(12)

    orders = HprimForeign("OBR", "patient")


class AtellicaSegmentTQ1(Segment):
    priority = HprimString((10, 1))

    orders = HprimForeign("OBR", "patient")

    # Pas de rang de segment pour le segment TQ1
    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(AtellicaSegmentTQ1, self).to_string(separators=separators)


class AtellicaSegmentORC(Segment):
    order_control_code = HprimString(2)
    placer_order = HprimString(3)
    placer_group_number = HprimString(5)
    order_status = HprimString(6)
    parent = HprimString(9)

    sample_priority = HprimForeign("TQ1", "patient")


class AtellicaSegmentSID(Segment):
    sample_barcode = HprimString(4)
    carrier_identifier = HprimString(11)
    position_identifier = HprimString(12)
    tray_identifier = HprimString(14)
    position_idenfifier = HprimString(15)
    dilution_factor = HprimString(30)


class AtellicaSegmentSAC(Segment):
    sample_barcode = HprimString(4)
    tray_number = HprimString(11)
    position = HprimString(12)
    dilution_factor = HprimString(30)

    sample_priority = HprimForeign("TQ1", "patient")
    orders = HprimForeign("OBR", "patient")

    # Pas de rang de segment pour le segment SAC
    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(AtellicaSegmentSAC, self).to_string(separators=separators)


class AtellicaSegmentINV(Segment):
    control_name = HprimString((2, 1))
    control = HprimString((2, 2))
    instrument = HprimString((2, 3))
    test = HprimString((3, 1))
    test_info = HprimString((3, 2))
    test_code_info = HprimString((3, 3))
    reagent_code = HprimString((5, 1))
    reagent_type = HprimString((5, 2))
    reagent_lot = HprimString((5, 3))
    info = HprimString(17)


class AtellicaSegmentOBX(Segment):
    value_type = HprimString(3)
    observation_identifier = HprimString((4, 1))
    observation_sub_id = HprimString(5)
    value = HprimString(6)
    units = HprimString(7)
    reference_range = HprimString(8)
    interpretation_code = HprimString((9, 1))
    interpretation_text = HprimString((9, 2))
    name_coding_system = HprimString((9, 3))
    result_status = HprimString(12)
    observer = HprimString(17)
    analyzer = HprimString((19, 1))
    identifier_2 = HprimString((19, 2))
    identifier_3 = HprimString((19, 3))
    module = HprimString((19, 4))
    date = HprimDate(20)
    observation_type = HprimString(30)

    infos_tests = HprimForeign("TCD", "order")
    infos = HprimForeign("NTE", "result")
    inventory = HprimForeign("INV", "result")


class AtellicaSegmentQPD(Segment):
    barcode = HprimString((4, 1))
    carrier_identifier = HprimString(5)
    carrier_position = HprimString(6)
    tray_id = HprimString(7)
    tray_position = HprimString(8)

    queries_infos = HprimForeign("RCP", "queries")


class AtellicaSegmentRCP(Segment):
    query_priority = HprimString(2)
    response_modality_flag = HprimString((4, 1))
    response_modality_label = HprimString((4, 2))
    response_modality_code = HprimString((4, 3))


class Driver(AstmMessageV2):
    SPEC = {
        "code": "atellica",
        "label": "Atellica Siemens"
    }
    SEGMENT_CLASS_MAP = {
        "MSH": AtellicaSegmentMSH,
        "NTE": AtellicaSegmentNTE,
        "PID": AtellicaSegmentPID,
        "PV1": AtellicaSegmentPV1,
        "OBR": AtellicaSegmentOBR,
        "OBX": AtellicaSegmentOBX,
        "SPM": AtellicaSegmentSPM,
        "TCD": AtellicaSegmentTCD,
        "TQ1": AtellicaSegmentTQ1,
        "ORC": AtellicaSegmentORC,
        "SID": AtellicaSegmentSID,
        "SAC": AtellicaSegmentSAC,
        "INV": AtellicaSegmentINV,
        "QPD": AtellicaSegmentQPD,
        "RCP": AtellicaSegmentRCP,
        "ERR": AtellicaSegmentERR,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return AtellicaSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                priority={"A": "A", "S": "S", worklist.ELSE: "R"},
                action={"del": worklist.IGNORE, worklist.ELSE: "G"},
        ):
            seg_pid = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex or "U"
            )
            if patient.location:
                seg_pv1 = seg_pid.visits.add(location=patient.location)
            else:
                seg_pv1 = seg_pid
            for sample, action, priority in patient.samples:
                for order in sample["orders"]:
                    seg_spm = seg_pv1.specimens.add(
                        specimen_id=sample.sample_id,
                        specimen_type=sample.specimen,
                        specimen_role="P",
                        specimen_collection_date=sample.collection_date,
                        specimen_received_date=sample.request_date,
                    )
                    seg_sac = seg_spm.infos_samples.add(
                        sample_barcode=sample.sample_id
                    )
                    seg_tq1 = seg_sac.sample_priority.add(
                        priority=priority
                    )
                    if "," in order.code:
                        list_order_module = order.code.split(",")
                        order_code = list_order_module[0]
                        module = list_order_module[1]
                    else:
                        order_code = order.code
                        module = ""

                    seg_tq1.orders.add(
                        lis_test_code=order_code,
                        action_code=action,
                        module_idenfifier=module
                    )
        if len(msg.patients):
            return msg
        elif context["query"]:
            for sample_id in [seg_qpd.barcode for seg_qpd in context["query"].queries]:
                seg_pid = msg.patients.add()
                seg_spm = seg_pid.specimens.add(specimen_id=sample_id)
                seg_sac = seg_spm.infos_samples.add(sample_barcode=sample_id)
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_qpd in message.queries:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_qpd.barcode,
                )
                if seg_qpd.tray_id and seg_qpd.position:
                    sample.rack = "{}-{}".format(
                        seg_qpd.tray_id,
                        seg_qpd.tray_position
                    )
        else:
            worklist = Worklist(context="result")
            for patient in message.patients:
                if patient.visits:
                    seg_pv1 = patient.visits[0]
                else:
                    seg_pv1 = patient
                control = False
                for seg_spm in seg_pv1.specimens:
                    if seg_spm.specimen_role == "Q":
                        sample = worklist.get_or_create(
                            sample_id=seg_spm.specimen_id,
                            type="control"
                        )
                        control = True
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_spm.specimen_id,
                            type="patient"
                        )
                    for seg_sac in seg_spm.infos_samples:
                        if seg_sac.tray_number and seg_sac.position:
                            sample.rack = "{}-{}".format(
                                seg_sac.tray_number,
                                seg_sac.position
                            )
                        for seg_obr in seg_sac.orders:
                            if not control:
                                sample = worklist.get_or_create(
                                    sample_id=seg_sac.sample_barcode,
                                    type="patient",
                                )
                            for seg_obx in seg_obr.results:
                                if control:
                                    for seg_inv in seg_obx.inventory:
                                        if seg_inv.control == "Control":
                                            sample.control_lot = seg_inv.control_name
                                if seg_obx.interpretation_code == "\"\"":
                                    flag = ""
                                elif seg_obx.interpretation_code:
                                    flag = seg_obx.interpretation_code
                                else:
                                    flag = ""
                                if seg_sac.dilution_factor:
                                    dilution = seg_sac.dilution_factor
                                else:
                                    dilution = ""
                                comments = []
                                for seg_nte in seg_obx.commentaires:
                                    comments.append(seg_nte.comment_text)
                                result = sample.results.add(
                                    code=seg_obx.observation_identifier,
                                    value=seg_obx.value,
                                    flags=flag,
                                    comments=comments,
                                    date=seg_obx.date,
                                    operator=seg_obx.observer,
                                    instrument=seg_obx.analyzer,
                                    dilution=dilution,
                                    unit=seg_obx.units
                                )
                                for seg_inv in seg_obx.inventory:
                                    if seg_inv.control_name == "PREAG":
                                        result.lots.add(
                                            type="reagent",
                                            lot=seg_inv.reagent_lot,
                                        )
        return worklist
