# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)
from driverng.astm.parser import RecursiveList
from driverng.utils import num_years, num_month, num_days
import logging
logger = logging.getLogger("driverng.drivers.Au800")


class Au800SegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    sender_device = HprimString((5, 2))
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    infos = HprimForeign("M", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Au800SegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    midle_initial = HprimString((6, 3))
    birthdate = HprimDate((8, 1), fmt_date="%Y%m%d")
    age = HprimString((8, 2))
    unit_age = HprimString((8, 3), default='Y')
    # M/F/U
    sex = HprimString(9)
    physician = HprimString(14)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class Au800SegmentO(Segment):
    sample_id = HprimString(3)
    test_id = HprimList(5, basefield=HprimMultiple(
        0,
        test_name=HprimString(3),
        replicate=HprimString(4, default='1'),
    ))
    # R/S
    priority = HprimString(6)
    collection_date = HprimDate(8)
    date_collection_end = HprimDate(9)
    volume = HprimString((10, 1))
    volume_unit = HprimString((10, 2))
    preleveur = HprimString(11)
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    user_field1 = HprimString((19, 1), default='1')
    dilution_factor = HprimString((19, 2))
    report_type = HprimString(26)


class Au800SegmentM(Segment):
    record_id = HprimString(3)


class Au800SegmentM101(Segment):
    record_id = HprimString(3)
    sample_id = HprimString(4)
    date = HprimDate(5)
    instrument = HprimString(6)


class Au800SegmentM103(Segment):
    record_id = HprimString(3)
    sample_id = HprimString((4, 1))
    volume_initial = HprimString((4, 2))
    volume_remaining = HprimString((4, 3))
    alicot_id = HprimString((5, 1))
    alicot_volume = HprimString((5, 2))
    status = HprimString(6)
    error = HprimString(7)
    date = HprimDate(8)
    track_line = HprimDate(9)
    instrument = HprimString(10)
    unit = HprimString(11)
    tray_position = HprimString(12)
    tray_location = HprimString(13)
    tray_id = HprimString((15, 1))
    station_name = HprimString((15, 2))
    tray_status = HprimString(16)


class Au800SegmentC(Segment):
    text = HprimString(4)


class Au800SegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    request_status = HprimString(13)  # always O


class Au800SegmentL(Segment):
    terminator = HprimString(3, default='N')


class Driver(AstmMessageV2):
    SPEC = {
        "code": "au800",
        "label": "Beckman AU800",
    }
    SEGMENT_CLASS_MAP = {
        "H": Au800SegmentH,
        "P": Au800SegmentP,
        "O": Au800SegmentO,
        "Q": Au800SegmentQ,
        "C": Au800SegmentC,
        "M": Au800SegmentM,
        "L": Au800SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = Au800SegmentH(type_segment="H", **kwargs)
        message.end = Au800SegmentL(type_segment="L")
        return message

    def parse_segment(self, line, separators):
        segment = super(Driver, self).parse_segment(line, separators)
        if segment.type_segment == 'M':
            if segment.record_id not in ('101', '103'):
                logger.info("segment M , dont le record id = %s n'est pas traité" % (segment.record_id, ))
            else:
                if segment.record_id == '103':
                    segM = Au800SegmentM103()
                else:
                    segM = Au800SegmentM101()
                splitted = RecursiveList.split(line, separators)
                segM.populate(splitted)
                return segM

        return segment

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))

        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", worklist.ELSE: "N"},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            age = num_years(patient["birthdate"], patient["collection_date"])
            unit_age = "Y"
            if age in (None, 0):
                age = num_month(patient["birthdate"],
                                patient["collection_date"])
                unit_age = "M"
            if age in (None, 0):
                age = num_days(patient["birthdate"],
                               patient["collection_date"])
                unit_age = "D"
            if age in (None, 0):
                age = unit_age = None
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                age=age,
                unit_age=unit_age,
                physician=patient.physicians[0].code,
                location=patient.location,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    priority=priority,
                    collection_date=sample.collection_date,
                    action_code=action,
                    specimen_type=sample.specimen,
                )
                for order in sample.orders:
                    t = seg_o.test_id.add()
                    t.test_name = order.code

        if len(message.patients):
            return message
        elif context["query"] and context["query"].queries:
            for seg_q in context["query"].queries:
                message.queries.add(
                    specimen_id=seg_q.specimen_id,
                    request_status="Z",
                )
            return message

    def decode_message(self, message):

        module_dict = {
            ('81', '1'): 'Input/Output',
            ('82', '1'): 'Output 1',
            ('82', '2'): 'Output 2',
            ('83', '1'): 'Decapper',
            ('84', '1'): 'Centrifuge',
            ('85', '1'): 'Aliquot (AutoMate 800)',
            ('86', '1'): 'Labeler (AutoMate 800)',
            ('88', '1'): 'Level Sense',
            ('90', '1'): 'Conveyor (AutoMate 600)',
        }

        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id.strip())
            return worklist

        worklist = Worklist(context="result")
        for seg_m in message.infos:
            if seg_m.record_id in ('101', '103'):
                sample_id = seg_m.sample_id.split('.')
                sample_index = 0
                if len(sample_id) > 1:
                    sample_index = sample_id[1]
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=sample_id[0],
                )
                if seg_m.record_id == '101':
                    result = sample.results.add(
                        code='TRACKING',
                        value=seg_m.record_id,
                        date=seg_m.date,
                        instrument=seg_m.instrument,
                        date_plateau=seg_m.date
                    )
                if seg_m.record_id == '103':
                    commentaires = []
                    module_name = module_dict.get((seg_m.instrument, seg_m.unit), '%s_%s' % (seg_m.unit, seg_m.tray_id))
                    sample.rack = "%s %s %s-%s-%s" % (
                        seg_m.station_name or 'WSTN',
                        module_name,
                        seg_m.tray_id,
                        seg_m.tray_location,
                        seg_m.tray_position,
                    )

                    commentaires.append("Module %s" % (module_name))
                    commentaires.append("Rack %s-%s-%s" % (
                        seg_m.tray_id,
                        seg_m.tray_location,
                        seg_m.tray_position,
                    ))
                    if seg_m.volume_initial:
                        commentaires.append("Volume %s -> %s" % (seg_m.volume_initial, seg_m.volume_remaining, ))
                    if sample_index:
                        commentaires.append("Index %s" % (sample_index))
                    if seg_m.alicot_id:
                        commentaires.append("Alicot %s %s %s" % (seg_m.alicot_id, seg_m.status, seg_m.error))
                    result = sample.results.add(
                        code='TRACKING',
                        value="%s" % (seg_m.station_name),
                        date=seg_m.date,
                        instrument=seg_m.instrument,
                        comments=commentaires,
                    )
                    if seg_m.error and seg_m.error != '0':
                        result.flags.append(seg_m.error)

        return worklist
