# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import format_date
from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class AutovueSegmentH(HeaderSegment):
    sender_name = HprimMultiple(
        5,
        sender_id=HprimString(1, default="AUTOVUE"),
        name_software=HprimString(2),
        version_software=HprimString(3),
        instrument_id=HprimString(4),
    )
    processing = HprimString(12)
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class AutovueSegmentP(Segment):
    # pour conserver la compatibilité avec l'ancienne version de l'autovue
    # a voir si on peut s'en passer lors de la qualif
    # Le numéro était formé à partir de la date de la demande
    # et du numéro de l'échantillon
    patient_id = HprimString(3)
    patient_nid = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    mother_maiden = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d%H%M%S", default="???")
    # M/F/U
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class AutovueSegmentO(Segment):
    specimen_id = HprimString(3)
    test = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(1),
    ))
    # S/N
    priority = HprimString(6)
    request_date = HprimDate(7, fmt_date="%Y%m%d")
    # C: Cancel
    # A: Add
    # N: New
    action_code = HprimString(12)
    sample_type = HprimString(16, default="CENTBLOOD")
    date_validation_profile = HprimString(23)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class AutovueSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    report_type = HprimString(13)


class AutovueSegmentR(Segment):
    name_analysis = HprimString(3)
    result = HprimString(4)
    flag = HprimList(7)
    # F: Final result
    # R: Repeat result
    # X: Test canceled or in error
    status = HprimString(9)
    operator = HprimString(11)
    date = HprimDate(13)
    instrument_id = HprimString(14)

    infos = HprimForeign("M", "result")


class AutovueSegmentM(Segment):
    test_name = HprimString(3)
    cassette = HprimMultiple(
        4,
        analyse=HprimString(1),
        # no du puit
        number_well=HprimString(2),
        number_cassette=HprimString(3),
        lot_number=HprimString(4),
        expiration_date=HprimDate(5),
        # nom du fichier d'extension bmp
        filename_bmp=HprimString(6),
        # nom du fichier d'extension jpg sur configuration Vision
        filename_jpg=HprimString(7),
    )
    reagent = HprimMultiple(
        5,
        reagent_name=HprimString(1),
        lot_number=HprimString(2),
        expiration_date=HprimDate(3),
    )
    # Interprétation du puit
    final_result = HprimString((6, 1))
    # Manual correction flag:
    # M: Manual correction
    # A: Automatic
    correction_flag = HprimString((6, 2))
    read_result = HprimString((6, 3))
    operator_id = HprimString((6, 4))


class AutovueSegmentL(Segment):
    terminaison = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "autovue",
        "label": "Autovue/Innova/Vision (ASTM Etendu)",
    }
    SEGMENT_CLASS_MAP = {
        "H": AutovueSegmentH,
        "P": AutovueSegmentP,
        "O": AutovueSegmentO,
        "Q": AutovueSegmentQ,
        "R": AutovueSegmentR,
        "M": AutovueSegmentM,
        "L": AutovueSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessage.BooleanOption(
            "traite_code_operator",
            label="Traitement du code opérateur dans l'alarme"),
        AstmMessage.BooleanOption(
            "recuperer_alarmes",
            label="Récupérer les alarmes des champs R.7 et M.6.1"),
    ]

    def __init__(self, traite_code_operator=False, recuperer_alarmes=False, **kwargs):
        self.traite_code_operator = traite_code_operator
        self.recuperer_alarmes = recuperer_alarmes
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = AutovueSegmentH(type_segment="H", **kwargs)
        message.end = AutovueSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.sample_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={
                "del": "C",
                "add": "A",
                "new": "N",
            },
            priority={
                "A": "S",
                "S": "S",
                worklist.ELSE: "N"
            }
        ):
            seg_p = msg.patients.add(
                patient_nid=patient["patient_id"],
                last_name=patient["last_name"],
                first_name=patient["first_name"],
                mother_maiden=patient["maiden_name"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    specimen_id=sample["sample_id"],
                    priority=priority,
                    request_date=sample["collection_date"],
                    action_code=action,
                )
                # no patient formaté avec la date et le numéro d'échantillon
                # champ obligatoire pour l'integration de l'identité du patient
                seg_p.patient_id = "%s%s" % (
                    format_date(sample["collection_date"], fmt="%Y%m%d"),
                    sample["sample_id"],
                )
                for order in sample["orders"]:
                    test = seg_o.test.add()
                    test.code = order["code"]

        if context["query"] or msg.patients:
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                for seg_r in seg_o.results:
                    tube = seg_o.specimen_id
                    alarmes = []
                    # Flags alarmes présents dans le champ R.7
                    if self.recuperer_alarmes and seg_r.flag:
                        alarmes = seg_r.flag
                    # Code erreur présent dans le champ M.6.1
                    for seg_m in seg_r.infos:
                        if self.recuperer_alarmes and seg_m.final_result:
                            alarmes.append(seg_m.final_result)
                    if self.traite_code_operator and seg_r.operator:
                        alarmes.append(seg_r.operator)
                    lots_reactifs_temp = {}
                    lots_reactifs = []
                    for seg_m in seg_r.infos:
                        reagent = seg_m.reagent
                        if not reagent.reagent_name and not reagent.lot_number:
                            continue
                        if (reagent.reagent_name, reagent.lot_number) not in lots_reactifs_temp:
                            lots_reactifs_temp[(
                                reagent.reagent_name,
                                reagent.lot_number
                            )] = reagent.expiration_date

                        lots_reactifs = [
                            {
                                "ref": "{}_{}".format(lot[0], lot[1]),
                                "nom": "{}_{}".format(lot[0], lot[1]),
                                "peremption": expiration_date
                            }
                            for (lot, expiration_date) in iter(sorted(lots_reactifs_temp.iteritems()))
                        ]

                    alarms = sorted(set(alarmes))
                    # Gestion du cas particulier du Phenotype
                    # voir si on maintient
                    # concatener le code test du phenotype avec le no
                    # exemple Pheno_1 ou Pheno_2
                    if seg_r.name_analysis == "Pheno":
                        results.setdefault(
                            tube,
                            {"resultats": []})["resultats"].append(
                            {
                                "test": seg_r.name_analysis,
                                "resu": seg_r.result,
                                "alarmes": alarms,
                                "date": seg_r.date,
                                "lots_reactifs": lots_reactifs,
                                "operator": seg_r.operator,
                            })
                        for i, value in enumerate(seg_r.result):
                            test = "%s_%d" % (seg_r.name_analysis, i + 1)
                            results.setdefault(
                                tube,
                                {"resultats": []})["resultats"].append(
                                {
                                    "test": test,
                                    "resu": value,
                                    "alarmes": alarms,
                                    "date": seg_r.date,
                                    "lots_reactifs": lots_reactifs,
                                    "operator": seg_r.operator,
                                })
                    else:
                        results.setdefault(
                            tube,
                            {"resultats": []})["resultats"].append(
                            {
                                "test": seg_r.name_analysis,
                                "resu": seg_r.result,
                                "alarmes": alarms,
                                "date": seg_r.date,
                                "lots_reactifs": lots_reactifs,
                                "operator": seg_r.operator,
                            })
                    for seg_m in seg_r.infos:
                        # Dans le cas du réactif Anti-D, les champs R.7 et
                        # M.3.1 sont identiques pour les deux Anti-D la
                        # différence se fait sur le numéro du puit (M.4.2)
                        if seg_m.test_name == "Anti-D":
                            test = "%s_%s_%s" % (
                                seg_r.name_analysis,
                                seg_m.test_name,
                                seg_m.cassette.number_well,
                            )
                        else:
                            test = "%s_%s" % (
                                seg_r.name_analysis, seg_m.test_name,
                            )
                        if seg_m.correction_flag == "M" and seg_m.read_result:
                            resu = "%s^%s^%s^%s^%s^%s^%s^%s" % (
                                seg_m.cassette.analyse,
                                seg_m.cassette.number_well,
                                seg_m.cassette.number_cassette,
                                seg_m.cassette.lot_number,
                                seg_m.cassette.expiration_date,
                                seg_m.cassette.filename_jpg or seg_m.cassette.filename_bmp,
                                seg_m.final_result,
                                seg_m.read_result,
                            )
                        else:
                            resu = "%s^%s^%s^%s^%s^%s^%s" % (
                                seg_m.cassette.analyse,
                                seg_m.cassette.number_well,
                                seg_m.cassette.number_cassette,
                                seg_m.cassette.lot_number,
                                seg_m.cassette.expiration_date,
                                seg_m.cassette.filename_jpg or seg_m.cassette.filename_bmp,
                                seg_m.final_result,
                            )
                        results[tube]["resultats"].append({
                            "test": test,
                            "resu": resu,
                            "date": seg_r.date,
                        })
        return results
