# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class B221SegmentH(HeaderSegment):
    sender_name = HprimMultiple(
        5,
        instrument_id=HprimString(1),
        manufacturer_name=HprimString(2),
        instrument_type=HprimString(3),
        software_version=HprimString(4),
        protocol_version=HprimString(5),
        serial_number=HprimString(6),
        address_ip=HprimString(7),
    )
    receiver_id = HprimString(10)
    # M = measurement results
    # QC = quality control results
    # SR^REAL = calibration results
    # LSU^U12 = maintenance data
    # PQ = patient information query
    message_type = HprimString((11, 1))
    message_instruction = HprimString((11, 2))
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1394-97")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    segments_m = HprimForeign("M", "header")
    end = HprimForeign("L", "header", required=True)


class B221SegmentP(Segment):
    laboratory_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    mother_maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    attending_physic_id = HprimString(14)
    patient_height = HprimMultiple(
        17,
        format_value=HprimString(1),
        unit=HprimString(2),
    )
    patient_weight = HprimMultiple(
        18,
        format_value=HprimString(1),
        unit=HprimString(2),
    )
    orders = HprimForeign("O", "patient")


class B221SegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        account=HprimString(1),
        barcode=HprimString(2),
    )
    instrument_specimen_id = HprimMultiple(
        4,
        order_id=HprimString(1),
        measurement=HprimString(2),
        sample_container=HprimString(6),
    )
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))
    priority = HprimString(6)
    requested_date = HprimDate(7)
    action_code = HprimString(12)
    specimen_descriptor = HprimMultiple(
        16,
        sample_type=HprimString(1),
        blood_type=HprimString(2),
        puncture_site=HprimString(3),
    )
    ordering_physician = HprimString(17)
    report_type = HprimString(26)
    results = HprimForeign("R", "order")


class B221SegmentR(Segment):
    test = HprimMultiple(
        3,
        test_name=HprimString(4),
        result_type=HprimString(7),
        result_id=HprimString(8),
    )
    value = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimList(6, basefield=HprimMultiple(
        0,
        lower_limit=HprimString(1),
        upper_limit=HprimString(2),
        limit_name=HprimString(3),
    ))
    flags = HprimList(7)
    nature_abnormality = HprimString(8)
    # F: Final
    result_status = HprimString(9)
    date = HprimDate(13)


class B221SegmentM(Segment):
    # les champs de la calibration ne sont pas décrits dans la doc
    record_type = HprimMultiple(
        3,
        field1=HprimString(1),
        field2=HprimString(2),
        field3=HprimString(3),
        field4=HprimString(4),
    )
    manufacturer_field = HprimMultiple(
        4,
        field1=HprimString(1),
        field2=HprimString(2),
    )
    value = HprimString(5)
    units = HprimString(6)
    field7 = HprimMultiple(
        7,
        field1=HprimString(1),
        field2=HprimString(2),
        field3=HprimString(3),
    )
    field8 = HprimMultiple(
        8,
        field1=HprimString(1),
        field2=HprimString(2),
    )


class B221SegmentQ(Segment):
    starting_range_id = HprimString((3, 2))
    request_status = HprimString(13)


class B221SegmentL(Segment):
    # N: normal terminaison
    # T: sender aborted
    # E: unknown system error
    # Q: error in last request for information
    # I: no information avalaible from last query
    # F: last request for information processed
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "b221",
        "label": "B221",
    }
    SEGMENT_CLASS_MAP = {
        "H": B221SegmentH,
        "P": B221SegmentP,
        "O": B221SegmentO,
        "Q": B221SegmentQ,
        "R": B221SegmentR,
        "M": B221SegmentM,
        "L": B221SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = B221SegmentH(type_segment="H", **kwargs)
        message.end = B221SegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.starting_range_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "A"},
        ):
            seg_p = msg.patients.add(
                laboratory_id=patient["patient_id"],
                last_name=patient["last_name"],
                first_name=patient["first_name"],
                mother_maiden_name=patient["maiden_name"],
                sex=patient["sex"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    requested_date=sample["collection_date"],
                    action_code=action,
                    ordering_physician=sample["physician"],
                    report_type="Q" if context["query"] else "O",
                )
                seg_o.specimen_id.barcode = sample["sample_id"]
                seg_p.attending_physicic_id = sample["physician"]

                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.code = order["code"]

        if msg.patients:
            return msg

    def get_result_dict(self, message):
        results = {}
        # exclusion des QC
        if message.message_type == "QC":
            return results
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                tube = seg_o.specimen_id.account
                date = None
                resultats = results.setdefault(
                    tube,
                    {"resultats": []},
                )["resultats"]
                for seg_r in seg_o.results:
                    # la date n'est présente que sur le premier segment R
                    if seg_r.date:
                        date = seg_r.date
                    resultats.append({
                        "test": seg_r.test.test_name,
                        "resu": seg_r.value,
                        "alarmes": seg_r.flags,
                        "date": date,
                    })
        return results

    def get_qc_result_list(self, message):
        results = []
        # exclusion des résultats hors QC
        if message.message_type != "QC":
            return results
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                for seg_r in seg_o.results:
                    # La date n'est présente que sur le premier résultat
                    if seg_r.date:
                        date = seg_r.date
                    results.append({
                        "lot": seg_o.specimen_descriptor.puncture_site,
                        "test": seg_r.test.test_name,
                        "resu": seg_r.value,
                        "date": date,
                    })
        return results
