# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging
from driverng.drivers.hprim_spec import (
    Driver as DriverHprim
)
from driverng.astm import AstmMessageV2

logger = logging.getLogger("driverng.hprim")


class Driver(DriverHprim):
    SPEC = {
        "code": "bacexpress",
        "label": "Hprim 2.1 (Bac Express)",
    }

    OPTIONS = DriverHprim.OPTIONS + [
        AstmMessageV2.IntegerOption(
            "version_protocole",
            label="Version du protocole",
            choices=[(1, u"1"), (2, u"2")],
            default=1),
    ]

    def __init__(self, version_protocole=False, **kwargs):
        self.version_protocole = version_protocole
        super(Driver, self).__init__(**kwargs)

    def encode_worklist(self, worklist, context):
        if worklist.context not in ("result", "order"):
            raise NotImplementedError("context %s" % (worklist.context,))

        msg = self.create(
            date=context["now"],
            type_message="ORU" if worklist.context == "result" else "ORM",
            nom_fichier=context.get("filename"),
        )
        msg.emetteur.code = worklist.sender_id or self.code_emetteur
        msg.emetteur.nom = worklist.sender_name or self.nom_emetteur
        msg.recepteur.code = worklist.receiver_id or self.code_recepteur
        msg.recepteur.nom = worklist.receiver_name or self.nom_recepteur

        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "N"},
        ):
            if worklist.context == "result" and patient.lab_patient_id:
                patient_demandeur = patient.lab_patient_id
            else:
                patient_demandeur = patient.patient_id

            seg_p = msg.patients.add(
                patient_demandeur=patient_demandeur,
                no_adm_cli=patient.alternate_patient_id,
                date_nai=patient.birthdate,
                sexe=patient.sex,
                nomjf=patient.maiden_name,
                date_entree=patient.admission_date,
                date_sortie=patient.admission_end_date,
                poids=patient.weight,
                taille=patient.height,
                rang_naissance=patient.birth_row,
            )
            seg_p.date_deces = patient.deathdate
            if self.option_ipp_executant:
                seg_p.patient_executant = patient_demandeur
            else:
                seg_p.patient_demandeur = patient_demandeur

            if patient.phone:
                if patient.phone == "T":
                    # Spécifique Clarilab
                    seg_p.set_raw("telephone", "~T")
                else:
                    seg_p.telephone = [p for p in patient.phone.split(";") if p]
            if patient.medications:
                seg_p.traitement_medical = patient.medications
            seg_p.nom.nom = patient.last_name
            seg_p.nom.prenom = patient.first_name
            seg_p.nom.civilite = patient.title
            seg_p.stat_adm = patient.admission_status
            seg_p.localisation.lit = patient.bed
            seg_p.localisation.chambre = patient.room
            seg_p.localisation.corr = patient.location
            seg_p.adresse.ville = patient.city
            seg_p.adresse.pays = patient.country
            seg_p.adresse.code_postal = patient.postal_code
            if patient.address:
                if "~" in patient.address:
                    seg_p.adresse.ligne1, seg_p.adresse.ligne2 = patient.address.split("~", 1)
                else:
                    seg_p.adresse.ligne1 = patient.address
            for physician in patient.physicians:
                if physician.type == "attending":
                    med = seg_p.medecins.add()
                    med.code = physician.code
                    med.nom.nom = physician.last_name
                    med.nom.prenom = physician.first_name
                    med.type = physician.codification
                    med.nom.civilite = physician.title
                    med.nom.sobriquet = physician.alias
            for sample, action, priority in patient.samples:
                if not msg.emetteur.code:
                    msg.emetteur.code = sample.lab
                seg_o = seg_p.orders.add(
                    tube_demandeur=sample.sample_id,
                    no_demandeur=sample.sample_id if self.option_envoi_sample_id else sample.request_id or sample.sample_id,
                    priorite=priority or "R",
                    action=action or "N",
                    date_demande=sample.request_date,
                    date_prel=sample.collection_date,
                    date_fin_prel=sample.collection_end_date,
                )
                if self.option_identification_demandeur:
                    seg_o.lib1_dem = sample.request_info
                if worklist.context == "result":
                    seg_o.tube_executant = sample.exe_sample_id or sample.sample_id
                    seg_o.no_executant = (
                        sample.exe_request_id or
                        sample.exe_sample_id or
                        sample.request_id or
                        sample.sample_id
                    )
                if sample.services or sample.physicians:
                    presc = seg_o.prescripteur.add()
                    for service in sample.services:
                        if service:
                            presc.code_service = service.code
                            presc.service.nom = service.name
                    for physician in sample.physicians:
                        if physician.type == "prescriber":
                            presc.code_medecin = physician.code
                            presc.medecin.nom = physician.last_name
                            if physician.phone:
                                seg_o.tel_prescrip = physician.phone.split(";")
                        elif physician.type == "sampler":
                            seg_o.preleveur.code = physician.code
                            seg_o.preleveur.nom.nom = physician.last_name
                            seg_o.preleveur.nom.prenom = physician.first_name
                            seg_o.preleveur.type = physician.codification
                            seg_o.preleveur.nom.civilite = physician.title
                            seg_o.preleveur.nom.sobriquet = physician.alias

                if sample.transcriptionist:
                    seg_o.ope_saisie.code = sample.transcriptionist

                combined_orders_results = []
                for o in sample.orders:
                    if o.code in [x["code"] for x in combined_orders_results]:
                        logger.info("code %s already present: do not add.", o.code)
                        continue
                    combined_orders_results.append({
                        "code": o.code,
                        "label": o.label,
                        "alt_code": o.alt_code,
                        "alt_label": o.alt_label,
                    })
                for r in sample.results:
                    if r.code in [x["code"] for x in combined_orders_results]:
                        logger.info("code %s already present: do not add.", r.code)
                        continue
                    combined_orders_results.append({
                        "code": r.code,
                        "label": r.label,
                        "alt_code": r.alt_code,
                        "alt_label": r.alt_label,
                    })
                for orderdict in combined_orders_results:
                    test = seg_o.prescription.add()
                    test.code1 = orderdict["code"]
                    if orderdict["label"]:
                        test.label1 = orderdict["label"]
                    if orderdict["alt_code"]:
                        test.code2 = orderdict["alt_code"]
                    if orderdict["alt_label"]:
                        test.label2 = orderdict["alt_label"]
                for result in sample.results:
                    if worklist.context == "order" and not result.anterior:
                        date = None
                    else:
                        date = result.date
                    valideur = None
                    if not self.option_envoi_technicien or not result.operator or result.validator != 'auto':
                        valideur = result.validator
                    else:
                        valideur = result.operator

                    def do_seg_r():
                        seg_r = seg_o.results.add(
                            type_resu=result.type,
                            code=result.code,
                            sous_test=result.sub_code,
                            date=date,
                            indic_norm=result.flags,
                            stat_resu=result.status,
                            analyseur=result.instrument,
                            valideur=valideur,
                            resultat_controle=result.resultat_controle,
                        )
                        seg_r.unite.code1 = result.unit
                        return seg_r

                    if isinstance(result.value, basestring):
                        values = [{"value": v} for v in result.value.split(";")]
                    elif isinstance(result.value, dict):
                        values = [result.value]
                    elif isinstance(result.value, list):
                        values = result.value
                    else:
                        raise NotImplementedError

                    seg_r = None
                    non_json_values = []
                    germe_list = []
                    for value in values:
                        if isinstance(value, dict):
                            json_v = value.get('json', {})
                            schema = value.get('schema')
                            if schema and json_v:
                                self.parse_resultat_json(value, seg_o, do_seg_r)
                            else:
                                non_json_values.append(value)
                            if "antibiogram" in value:
                                germe_list.append({
                                    "code_germe": value["value"],
                                    "code_batterie": result.code,
                                    "rang": result.sub_code,
                                    "antibiogram": value.get("antibiogram", [])
                                })
                        else:
                            non_json_values.append(value)

                    if non_json_values:
                        seg_r = do_seg_r()
                        for value in non_json_values:
                            if isinstance(value, dict) and value.get("antibiogram"):
                                seg_r.code2 = "GERMEATB"
                            resu = seg_r.resu.add()
                            resu.code1 = value["value"]
                            resu.label1 = value.get("label")
                            resu.codification1 = value.get("codification")
                            resu.code2 = result.nom_antibiogramme

                    for germe in germe_list:
                        for rang, line in enumerate(germe["antibiogram"], start=1):
                            seg_r = do_seg_r()
                            seg_r.code = germe["code_germe"]
                            if self.version_protocole == "2":
                                seg_r.code2 = "ANTIBIOTIC"
                            seg_r.sous_test = rang
                            antibio = seg_r.resu.add()
                            antibio.code1 = line["antibiotic"]
                            if len(line.get("antibiotic_texte_code", []) or []):
                                antibio.label1 = line["antibiotic_texte_code"][1]
                                antibio.codification1 = line["antibiotic_texte_code"][2]
                            mic = seg_r.resu.add()
                            mic.code1 = line["mic"]
                            seg_r.indic_norm = [line["sensitivity"]]
                            seg_r.secteur_tech.code_batterie = germe["code_batterie"]
                            seg_r.secteur_tech.text = germe["rang"]

                    if (
                        seg_r is not None and
                        result.type == "GC" and
                        all([not v.get("label") for v in values])
                    ):
                        # Graphe à pas fixe de type y^y^y Dans
                        # ce cas il faut renseigner les champs
                        # pas et origine
                        seg_r.gc_pas = "1"
                        seg_r.gc_origine = "0"

                    for comment in result.comments:
                        if seg_r is not None and result.sub_code is not None:
                            texte = "COMATB_{}:{}".format(result.sub_code, comment)
                            seg_r.commentaires.add(origine="L", texte=[texte])
                        else:
                            seg_r.commentaires.add(origine="L", texte=[comment])

        if worklist.context == "result":
            msg = self.clean_result_msg(msg)

        if msg.patients:
            return msg

    def parse_resultat_json(self, value, seg_o, do_seg_r):
        sub_code = 1
        seg_r = do_seg_r()
        seg_r.sous_test = sub_code
        seg_r.type_resu = 'CE'
        resu = seg_r.resu.add()
        resu.code1 = value["value"]
        resu.label1 = value.get("label")
        resu.codification1 = value.get("codification")
        json_v = value['json']
        schema = value['schema']
        main_properties = sorted(
            schema["properties"].items(),
            key=lambda i: i[1].get('propertyOrder', 9999)
        )
        for pname, pdata in main_properties:
            property_list = []
            rows = []
            if pdata["type"] == "array":
                property_list = pdata["items"]["properties"]
                rows = json_v[pname]
            elif pdata["type"] == "object":
                property_list = pdata["properties"]
                rows = [json_v.get(pname, None)]
            json_fields = sorted(
                property_list.items(),
                key=lambda i: i[1].get('propertyOrder', 9999)
            )
            for index, row in enumerate(rows, start=1):
                if row is None:
                    continue
                sub_code += 1
                seg_r = do_seg_r()
                seg_r.code2 = "COMPLEXE"
                seg_r.sous_test = sub_code
                seg_r.type_resu = 'CE'
                if self.version_protocole == "2":
                    seg_r.secteur_tech.code_batterie = pname

                if pdata["type"] == "array":
                    rang = seg_r.resu.add()
                    rang.code1 = index
                    rang.codification1 = "RANG"

                for field_name, field_properties in json_fields:
                    default_value = field_properties.get(
                        "format", {}
                    ).get('default', '')
                    if not isinstance(row, dict):
                        continue
                    value = row.get(field_name, default_value)
                    if not value:
                        value = default_value
                    resu = seg_r.resu.add()
                    value_tc = field_properties.get(
                        "format", {}
                    ).get("textecodes", {}).get(value)
                    if value_tc:
                        resu.code1 = value_tc[0]
                        resu.label1 = value_tc[1]
                    else:
                        resu.code1 = value
                    if value:
                        resu.codification1 = field_name

    def decode_obr(self, seg_obr, worklist, sample):
        results_by_code = {}
        for index, obx in enumerate(seg_obr.results):
            if obx.code2 == "GERMEATB":  # C'est un germe, un antibiogramme va suivre derrière
                result = sample.results.add(
                    type=obx.type_resu,
                    code=obx.code,
                    sub_code=obx.sous_test,
                    date=obx.date,
                    value=[{
                        "antibiogram": [],
                        "codification": obx.resu[0].codification1,
                        "label": obx.resu[0].label1,
                        "value": obx.resu[0].code1,
                    }],
                    anterior=False,
                    instrument=obx.analyseur,
                    status=obx.stat_resu,
                    unit=obx.unite.code1,
                    validator=obx.valideur,
                )
                results_by_code.setdefault(obx.code, []).append(result)
            elif obx.code2 == "ANTIBIOTIC":  # C'est un antibiotique, à rattacher au germe precedent
                try:
                    results_by_code[obx.secteur_tech.code_batterie][-1].value[0]["antibiogram"].append({
                        "antibiotic": obx.resu[0].code1,
                        "mic": obx.resu[1].code1,
                        "sensitivity": obx.indic_norm[0],
                        "antibiotic_texte_code": (
                            obx.resu[0].code1,
                            obx.resu[0].label1,
                            obx.resu[0].codification1,
                        )
                    })
                except (IndexError, KeyError):
                    logger.error("OBX mal formé, réception d'un antibiogram sans germe.")
            elif (
                obx.code2 != "COMPLEXE" and
                len(seg_obr.results) > (index + 1) and
                seg_obr.results[index + 1].code2 == "COMPLEXE"
            ):  # C'est le résultat qui déclenche un complexe
                if obx.code not in results_by_code:
                    result = sample.results.add(
                        type=obx.type_resu,
                        code=obx.code,
                        sub_code=obx.sous_test,
                        date=obx.date,
                        value=[{
                            "json": {
                                "resultat": obx.resu[0].code1,
                                "type_resultat": "identification_germes",
                            },
                            "codification": obx.resu[0].codification1,
                            "label": obx.resu[0].label1,
                            "value": obx.resu[0].code1,
                        }],
                        anterior=False,
                        instrument=obx.analyseur,
                        status=obx.stat_resu,
                        unit=obx.unite.code1,
                        validator=obx.valideur,
                    )
                    results_by_code.setdefault(obx.code, []).append(result)
            elif obx.code2 == "COMPLEXE":  # C'est un resultat de type json
                json = results_by_code[obx.code][-1].value[0]["json"]
                values = {}
                for value in obx.resu:
                    if value.codification1 == "RANG" or not value.codification1:
                        continue  # TODO trier les germes par rang
                    values[value.codification1] = value.code1
                json.setdefault(
                    obx.secteur_tech.code_batterie, []).append(values)
            else:
                self.add_result(worklist, sample, obx)

    def get_physiologie(self, seg_p):
        if seg_p.comment_dem1:
            return "{} {}".format(seg_p.comment_dem1, seg_p.comment_dem1_libelle)
        return ""

    def get_medecin_prescripteur(self, sample, seg_obr):
        for prescripteur in seg_obr.prescripteur:
            if (
                prescripteur.code_medecin and
                prescripteur.code_medecin not in [p.code for p in sample.physicians]
            ):
                sample.physicians.get_or_create(
                    code=prescripteur.code_medecin,
                    last_name=prescripteur.medecin.nom,
                    type="prescriber",
                    phone=";".join(seg_obr.tel_prescrip),
                )
