# -*- coding: utf8 -*-
from __future__ import unicode_literals


from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class BactAlertSegmentH(HeaderSegment):
    sender = HprimString(5, default="LIS")
    receiver = HprimString(10, default="BACT/ALERT")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BactAlertSegmentP(Segment):
    hospital_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    # Used as the default specimen location, if no location is explicitly given
    # by 9.4.28
    specimen_location = HprimString(26)
    orders = HprimForeign("O", "patient")


class BactAlertSegmentO(Segment):
    accession_number = HprimString(3)
    alternative_accession_number = HprimString(4)
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        test=HprimString(4),
    ))
    collection_date = HprimDate(8)
    # N: Creates a new accession or updates an existing accession, depending on
    # the rule for reusing accession numbers
    # A: Adds these tests or other data to the accession specified or creates a
    # new accession if it doesn't already exist
    # C: Deletes the accession and its bottles
    action_code = HprimString(12)
    specimen_type = HprimString(16)
    location = HprimString(28)

    results = HprimForeign("R", "order")


class BactAlertSegmentR(Segment):
    test_id = HprimString((3, 4))
    bottle_type = HprimString((3, 5))
    bottle_id = HprimString((3, 6))
    result = HprimString(4)
    result_status = HprimString(9)
    date_bottle_loaded = HprimDate(12)
    date_status_determined = HprimDate(13)
    instrument_cell_id = HprimString(14)


class BactAlertSegmentQ(Segment):
    specimen_id = HprimString(3)


class BactAlertSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bactalert",
        "label": "Bact Alert ASTM"
    }

    SEGMENT_CLASS_MAP = {
        "H": BactAlertSegmentH,
        "P": BactAlertSegmentP,
        "O": BactAlertSegmentO,
        "R": BactAlertSegmentR,
        "Q": BactAlertSegmentQ,
        "L": BactAlertSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "bact_inclure_preliminaire",
            label="Inclure les résultats préliminaires (flag P dans le champ R.9)"
        )
    ]

    def __init__(self, bact_inclure_preliminaire=False, **kwargs):
        self.bact_inclure_preliminaire = bact_inclure_preliminaire
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = BactAlertSegmentH(type_segment="H", **kwargs)
        message.end = BactAlertSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": "C", "add": "A", "new": "N"},
        ):
            seg_p = msg.patients.add(
                hospital_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    accession_number=sample.sample_id,
                    collection_date=sample.collection_date,
                    action_code=action,
                    specimen_type=sample.specimen
                )
                for order in sample.orders:
                    t = seg_o.universal_test_id.add()
                    t.test = order["code"]
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    for seg_r in seg_o.results:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.accession_number,
                            type="patient",
                        )
                        # Uniquement les résultats avec le statut F
                        if not self.bact_inclure_preliminaire and seg_r.result_status != "F":
                            continue
                        sample.results.add(
                            code="{}_{}".format(seg_r.test_id, seg_r.bottle_type),
                            value=seg_r.result,
                            date=seg_r.date_status_determined,
                        )
                        if seg_r.bottle_type:
                            sample.results.add(
                                code="BOTTLE_TYPE_{}".format(seg_r.bottle_type),
                                value=seg_r.bottle_type,
                                date=seg_r.date_status_determined,
                            )
                        if seg_r.bottle_id:
                            sample.results.add(
                                code="BOTTLE_ID_{}".format(seg_r.bottle_type),
                                value=seg_r.bottle_id,
                                date=seg_r.date_status_determined,
                            )
                        if seg_r.instrument_cell_id:
                            sample.results.add(
                                code="POS_BOTTLE_{}".format(seg_r.bottle_type),
                                value=seg_r.instrument_cell_id,
                                date=seg_r.date_status_determined,
                            )
                        if seg_r.date_bottle_loaded:
                            sample.results.add(
                                code="INTRODUCE_DATETIME_{}".format(seg_r.bottle_type),
                                value=seg_r.date_bottle_loaded.strftime("%Y/%m/%d %H:%M:%S"),
                            )
                        if seg_r.date_status_determined:
                            sample.results.add(
                                code="DETERMINED_DATETIME_{}".format(seg_r.bottle_type),
                                value=seg_r.date_status_determined.strftime("%Y/%m/%d %H:%M:%S"),
                            )
                        if seg_r.date_bottle_loaded and seg_r.date_status_determined:
                            delay = (seg_r.date_status_determined - seg_r.date_bottle_loaded).total_seconds() / 3600
                            sample.results.add(
                                code="TIME_TO_DETECTION_{}".format(seg_r.bottle_type),
                                value="{:.1f}".format(delay),
                            )
        return worklist
