# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class Bactec9000SegmentH(HeaderSegment):
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Bactec9000SegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    hospital_code = HprimString(33)

    orders = HprimForeign("O", "patient")


class Bactec9000SegmentO(Segment):
    access_number = HprimString(3)
    # Barcode is the vial sequence number. This is an optional item.
    barcode = HprimString((5, 4))
    spec_coll_date_time = HprimDate(8)
    coll_id = HprimString(11)
    receipt_date = HprimDate(15)
    spec_type = HprimString((16, 1))
    spec_source = HprimString((16, 2))
    physician_id = HprimString(17, necessary=True)

    results = HprimForeign("R", "order")


class Bactec9000SegmentR(Segment):
    # Numéro de séquence du flacon, c'est l'identifiant unique de chaque flacon
    # d'hémoculture
    barcode = HprimString((3, 4))
    # Code milieu
    media_type = HprimString((3, 5))
    result = HprimString(4)
    # P is the result status and will always be "P" for Preliminary
    result_status = HprimString(5)
    datetime_started = HprimDate(12)
    datetime_completed = HprimDate(13)
    vial_station_location = HprimString(14)


class Bactec9000SegmentC(Segment):
    open_text = HprimString(2)


class Bactec9000SegmentQ(Segment):
    access_number = HprimString((3, 2))
    status_code = HprimString(13)


class Bactec9000SegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bactec9000",
        "label": "Bactec9000 Becton Dickinson"
    }

    SEGMENT_CLASS_MAP = {
        "H": Bactec9000SegmentH,
        "P": Bactec9000SegmentP,
        "O": Bactec9000SegmentO,
        "R": Bactec9000SegmentR,
        "Q": Bactec9000SegmentQ,
        "L": Bactec9000SegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = Bactec9000SegmentH(type_segment="H", **kwargs)
        message.end = Bactec9000SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                hospital_code=patient.location,
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    access_number=sample.sample_id,
                    spec_coll_date_time=sample.collection_date,
                    receipt_date=sample.request_date,
                    # code du specimen ou type de flacon
                    spec_source=sample.specimen,
                )
                for physician in sample.physicians:
                    seg_o.physician_id = physician.code
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.access_number,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    seq = 1
                    for seg_r in seg_o.results:
                        instrument = ""
                        rack = ""
                        if "/" in seg_r.vial_station_location:
                            li = seg_r.vial_station_location.split("/")
                            instrument = li[0] if li[0] else ""
                            rack = li[1] if li[1] else ""
                        sample = worklist.get_or_create(
                            sample_id=seg_o.access_number,
                            rack=rack,
                            type="patient",
                        )

                        # Résultat de l'hémoculture
                        # INST_POSITIVE, INST_NEGATIVE, INST_ONGOING
                        sample.results.add(
                            code="HEMOC{}".format(seq),
                            value=seg_r.result,
                            date=seg_r.datetime_completed,
                            instrument=instrument
                        )
                        # Identifiant du flacon
                        sample.results.add(
                            code="ID_BOTTLE{}".format(seq),
                            value=seg_r.barcode,
                            date=seg_r.datetime_completed,
                            instrument=instrument
                        )
                        # Code du milieu de culture
                        sample.results.add(
                            code="MILIEU_CULTURE{}".format(seq),
                            value=seg_r.media_type,
                            date=seg_r.datetime_completed,
                            instrument=instrument
                        )
                        seq += 1

        return worklist
