# -*- coding: utf8 -*-

from __future__ import unicode_literals

import base64
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimMultipleList,
)


class Bc6800GraphContainer(dict):
    @property
    def type_fichier(self):
        return "bmp"

    def get_bytes(self):
        return self["data"]


class Bc6800SegmentH(HeaderSegment):
    message_id = HprimString(3)
    sender_name = HprimMultiple(
        5,
        manufacturer=HprimString(1, default="Mindray"),
        instrument_model=HprimString(2, default="BC-6800"),
        protocol_version=HprimString(3),
    )
    special_instructions = HprimString((11, 1), default="Worksheet request")
    special_instructions2 = HprimString((11, 2), default="00011")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Bc6800SegmentP(Segment):
    patient_id = HprimString(5)
    first_name = HprimString((6, 1))
    last_name = HprimString((6, 2))
    birthdate = HprimMultiple(
        8,
        date_birth=HprimDate(1, fmt_date="%Y%m%d%H%M%S"),
        age=HprimString(2),
        age_unit=HprimString(3),
    )
    sex = HprimString(9)
    admission_status = HprimString(25)
    location = HprimMultiple(
        26,
        inpatient_zone=HprimString(1),
        bed_no=HprimString(2),
    )

    orders = HprimForeign("O", "patient")


class Bc6800SegmentO(Segment):
    specimen_id = HprimString(3)
    request_date = HprimDate(7)
    collection_date = HprimDate(8)
    collector_id = HprimString(11)
    clinical_information = HprimString(14)
    date_specimen_received = HprimDate(15)
    specimen_descriptor = HprimMultiple(
        16,
        type=HprimString(1),
        source=HprimString(2),
    )
    ordered_physician = HprimString(17)
    user_field_number1 = HprimString(19)
    user_field_number2 = HprimString(20)
    date_result_reported = HprimDate(23)
    # F: Final results; not request response; fixed to be F
    # Q: has result for request
    # Y: no result for request
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class Bc6800SegmentR(Segment):
    universal_test = HprimMultiple(
        3,
        test_name=HprimString(2),
        test_type=HprimString(3),
        local_code=HprimString(4),
    )
    data = HprimString(4)
    units = HprimString(5)
    reference_range = HprimMultiple(
        6,
        lower_limit=HprimString(1),
        upper_limit=HprimString(2),
    )
    # H: higher than upper limit
    # L: lower than lower limit
    # E: result edited flag
    # e: result changed due to the manual editing of another parameter result based on which it is calculated
    # N: normal
    # A: abnormal
    # O: reagent expired
    # T: instrument overtemperature
    # C: Result corrected flag
    # V: out of linearity range
    abnormal_flags = HprimMultipleList(7)


class Bc6800SegmentQ(Segment):
    starting_range_id = HprimString(3)
    date = HprimDate(7)
    user_field_number1 = HprimString(11)


class Bc6800SegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bc6800",
        "label": "Bc6800 DMU"
    }
    SEGMENT_CLASS_MAP = {
        "H": Bc6800SegmentH,
        "P": Bc6800SegmentP,
        "O": Bc6800SegmentO,
        "R": Bc6800SegmentR,
        "Q": Bc6800SegmentQ,
        "L": Bc6800SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    ENCODING = "utf-8"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "ipp_mca", label="Transmettre l'IPP patient du middleware",
            default=False,
        ),
    ]

    def __init__(self, ipp_mca=False, **kwargs):
        self.ipp_mca = ipp_mca
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = Bc6800SegmentH(type_segment="H", **kwargs)
        message.end = Bc6800SegmentL(type_segment="L")
        return message

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.starting_range_id)
        return queries

    @staticmethod
    def get_profile_test(liste_codes):
        codes_tests = []
        for test in liste_codes:
            if '+' in test:
                codes_tests.extend(test.split('+'))
            else:
                codes_tests.append(test)
        profiles_test = sorted(list(set(
            [elt for elt in codes_tests if
             elt in ("CBC", "DIFF", "RET", "NRBC")])))
        if len(profiles_test) == 4:
            # Dans le cas ou on a les 4 il faut mettre NRBC à la fin
            profile_test = "CBC+DIFF+RET+NRBC"
        else:
            # Sinon par ordre alphabetique
            profile_test = "+".join(profiles_test)

        return profile_test

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        # renvoyer le numéro de sequence du message_id
        if context["query"]:
            msg.message_id = context["query"].message_id

        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id if self.ipp_mca else patient.lab_patient_id,
                first_name=patient.first_name,
                last_name=patient.last_name,
            )
            if patient.sex == "M":
                seg_p.sex = "Mascul."
            elif patient.sex == "F":
                seg_p.sex = "Féminin"

            seg_p.birthdate.date_birth = patient.birthdate
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    request_date=sample.request_date,
                    collection_date=sample.collection_date,
                    report_type="Q",
                )
                for physician in sample.physicians:
                    seg_o.ordered_physician = physician.code

                for data, test_name, local_code in (
                    ("A", "Take Mode", "08001"),
                    (sample.specimen, "Blood Mode", "08002"),
                    (self.get_profile_test([o.code for o in sample.orders]),
                        "Test Mode", "08003"),
                    ("", "Ref Group", "01002"),
                    ("", "Remark", "01001"),
                    ("", "Charge type", "01015"),
                    ("", "Patient type", "01016"),
                    ("BL", "Project Type", "05007"),
                    ("", "Custom patient Info 1", "01009"),
                    ("", "Custom patient Info 2", "01010"),
                    ("", "Custom patient Info 3", "01011")
                ):
                    seg_r = seg_o.results.add()
                    seg_r.data = data
                    seg_r.universal_test.test_name = test_name
                    seg_r.universal_test.local_code = local_code

        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id,
                )
        else:
            worklist = Worklist(context="result")
            # Résultats de QC
            if "QC" in message.special_instructions:
                for seg_o in message.orders:
                    lot = None
                    instrument_name = None
                    for seg_r in seg_o.results:
                        # Le numéro de lot se trouve dans le résultat du test
                        # Qc lot No
                        if (
                            seg_r.universal_test.test_name == "Qc lot No"
                        ):
                            lot = seg_r.data
                            sample = worklist.get_or_create(
                                control_lot=lot,
                                sample_id=lot,
                                type="control",
                            )
                        elif lot is None:
                            continue
                        if seg_r.universal_test.test_name in ("Instrument Name", "Analyzer"):
                            instrument_name = seg_r.data
                        sample.results.add(
                            code=seg_r.universal_test.test_name.replace(
                                " ", "").replace("‰", "%m"),
                            value=seg_r.data,
                            flags=[
                                flag for flag in seg_r.abnormal_flags if
                                flag not in ('', 'N')
                            ],
                            date=None,
                            instrument=instrument_name,
                        )

            # Résultats Patient
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    alarmes_hemato = []
                    instrument_name = None
                    lot = None
                    for seg_r in seg_o.results:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            type="patient",
                        )

                        # Récupération du rack
                        if seg_r.universal_test.test_name == "Shelf No":
                            rack = seg_r.data
                        if seg_r.universal_test.test_name == "Tube No":
                            position = seg_r.data
                            sample.rack = "%s-%s" % (
                                rack,
                                position,
                            )
                        date_resultat = seg_o.date_result_reported
                        # Récupération du nom de l'instrument
                        if seg_r.universal_test.test_name in ("Instrument Name", "Analyzer"):
                            instrument_name = seg_r.data

                        # Si le résultat est "T", il s'agit d'une alarme
                        # pathologique
                        # Exclure le flag Recheck flag des alarmes
                        # pathologiques
                        if (
                            seg_r.data == "T" and
                            seg_r.universal_test.test_name != "Recheck flag"
                        ):
                            alarmes_hemato.append(
                                seg_r.universal_test.test_name
                            )
                        elif "BMP" in seg_r.universal_test.test_name:
                            sample.results.add(
                                code=seg_r.universal_test.test_name.replace(" ", ""),
                                value="GRAPHE",
                                graph={
                                    "graph_type": "binary",
                                    "format": "bmp",
                                    "data": base64.b64decode(seg_r.data),
                                },
                            )
                        else:
                            # Eliminer les flags alarme vides
                            sample.results.add(
                                code=seg_r.universal_test.test_name.replace(
                                    " ", "").replace("‰", "%m"),
                                value=seg_r.data,
                                flags=[
                                    flag for flag in seg_r.abnormal_flags if
                                    flag not in ('', 'N')
                                ],
                                date=date_resultat or None,
                                instrument=instrument_name,
                            )

                    # Apres le traitement du dernier segment R, on ajoute le résultat de l'alarme
                    if alarmes_hemato:
                        sample.results.add(
                            code="ALARME",
                            value="",
                            comments=alarmes_hemato,
                            date=date_resultat or None,
                            instrument=instrument_name
                        )
        return worklist
