# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class BdMaxSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    version_number = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class BdMaxSegmentP(Segment):
    patient_id = HprimString(4, max_length=20, trunc=True)

    orders = HprimForeign("O", "patient")


class BdMaxSegmentO(Segment):
    accession_number = HprimString((3, 1))
    test_id = HprimString((5, 4))
    test_name = HprimString((16, 1))

    results = HprimForeign("R", "order")


class BdMaxSegmentR(Segment):
    code = HprimString((3, 4))
    # INST_POSITIVE
    # INST_NEGATIVE
    # CONFIRMED_POSITIVE
    test_status = HprimString(4)
    # P: Preliminary
    # F: Final
    result_status = HprimString(9)
    date_result = HprimDate(12)
    instrument_type = HprimString((14, 1))
    machine_instrument_number = HprimString((14, 4))
    location_information = HprimString((14, 5))
    test_name = HprimString((14, 8))
    confirmed_positive = HprimString((15, 2))


class BdMaxSegmentQ(Segment):
    patient_id_request = HprimString((3, 1))
    accession_request = HprimString((3, 2))
    test_status = HprimString((5, 2))
    starting_date = HprimDate(7)
    end_date = HprimDate(8)
    request_info_status = HprimString(13)


class BdMaxSegmentL(Segment):
    terminaison = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bd_max",
        "label": "BD MAX Becton Dickinson"
    }
    SEGMENT_CLASS_MAP = {
        "H": BdMaxSegmentH,
        "P": BdMaxSegmentP,
        "O": BdMaxSegmentO,
        "R": BdMaxSegmentR,
        "Q": BdMaxSegmentQ,
        "L": BdMaxSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "bd_max_envoi_prenom",
            label="Envoyer le prénom du patient dans le champ P.4 (attention \
                champ limité à 20 caractères maximum)",)
    ]

    def __init__(self, bd_max_envoi_prenom=False, **kwargs):
        self.envoi_prenom = bd_max_envoi_prenom
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = BdMaxSegmentH(type_segment="H", **kwargs)
        message.end = BdMaxSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            if self.envoi_prenom:
                patient_id = "{} {}".format(
                    patient.last_name,
                    patient.first_name,
                )
            else:
                patient_id = patient.last_name
            seg_p = msg.patients.add(
                patient_id='{:.20}'.format(patient_id)
            )
            for sample, action in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        accession_number=sample.sample_id,
                        test_id='BDMAX_AND',
                        test_name=order.code,
                    )
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.accession_request.strip(),
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.accession_number.strip(),
                    )
                    # Le BD MAX ne transmet pas le statut du résultat
                    for seg_r in seg_o.results:
                        sample.results.add(
                            code="%s%s" % (seg_r.code, seg_r.test_name,),
                            value=seg_r.test_status,
                            date=seg_r.date_result,
                        )
        return worklist
