# -*- coding: utf8 -*-
from __future__ import unicode_literals
import dateutil
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)
import logging
import traceback
logger = logging.getLogger("driverng.drivers.Bep2000")


class Bep2000SegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    receiver_id = HprimString((10, 1))
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Bep2000SegmentQ(Segment):
    sample_id = HprimString((3, 1))
    test_order = HprimString(5)
    report_type = HprimString(13)


class Bep2000SegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class Bep2000SegmentO(Segment):
    specimen_id = HprimString(3)
    test_id = HprimString((5, 4))
    dilution = HprimString((5, 5))
    collection_date = HprimDate(8)
    results = HprimForeign("R", "order")


class Bep2000SegmentR(Segment):
    test_id = HprimString((3, 4))
    test_type = HprimString((3, 5))
    value = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)


class Bep2000SegmentC(Segment):
    texte = HprimString((4, 1))
    lot = HprimString((4, 2))
    date = HprimString((4, 3))


class Bep2000SegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "bep2000_astm",
        "label": "Bep2000 Astm",
    }
    SEGMENT_CLASS_MAP = {
        "H": Bep2000SegmentH,
        "Q": Bep2000SegmentQ,
        "P": Bep2000SegmentP,
        "O": Bep2000SegmentO,
        "R": Bep2000SegmentR,
        "C": Bep2000SegmentC,
        "L": Bep2000SegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    def create(self, **kwargs):
        message = Bep2000SegmentH(type_segment="H", **kwargs)
        message.end = Bep2000SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            action={"del": worklist.IGNORE, worklist.ELSE: ""}
        ):
            seg_p = msg.patients.add(
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for samples in patient.samples:
                sample = samples[0]
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                    )
                    seg_o.collection_date = sample.collection_date
                    seg_p.patient_id = sample.sample_id
                    seg_o.test_id = order["code"]
                    seg_o.dilution = order["dilution"]
        if len(msg.patients):
            return msg
        elif context["query"]:
            for sample_id in [seg_q.sample_id for seg_q in context["query"].queries]:
                msg.patients.add(
                    patient_id=sample_id
                )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.specimen_id:
                        tube = seg_o.specimen_id
                    else:
                        tube = seg_p.patient_id
                    for seg_r in seg_o.results:
                        sample = worklist.get_or_create(
                            sample_id=tube,
                            type="patient",
                        )
                        # 1er commentaire = lot reactif, 6 et plus alarmes, les autres a ignorer
                        # a voir pour filtrer les alarmes
                        # alarmes = [seg_c.texte for seg_c in seg_r.commentaires[6:] if seg_c.texte]
                        result = sample.results.add(
                            code='%s_%s' % (seg_r.test_id, seg_r.test_type),
                            value=seg_r.value,
                            # flags=alarmes,
                            dilution=seg_o.dilution,
                        )
                        if seg_r.commentaires and seg_r.commentaires[0].texte:
                            c = seg_r.commentaires[0]
                            date = None
                            try:
                                date = dateutil.parser.parse(c.date, dayfirst=True)  # 'dd MM YYYY')
                                result.lots.add(
                                    type="reagent",
                                    name=c.texte,
                                    lot=c.lot,
                                    expiration=date,
                                )
                            except Exception:
                                logger.info('lot ko(%s %s %s) date:%s, %s', c.texte, c.lot, c.date, date, traceback.print_exc())
        return worklist
