# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist
from driverng import MessageV2
from driverng.utils import get_date, format_date

SEP = "|"


class String(object):

    def to_python(self, value):
        return value or None

    def to_string(self, value):
        return value or ""


class Date(object):

    def __init__(self, fmt):
        self.fmt = fmt
        super(Date, self).__init__()

    def to_python(self, value):
        return get_date(value, fmt=self.fmt)

    def to_string(self, value):
        return format_date(value, fmt=self.fmt)


class BioradResult(object):
    fields = (
        ("code", String()),
        ("instrument", String()),
        ("lot", String()),
        ("level", String()),
        ("_date", Date(fmt="%Y%m%d")),
        ("_time", Date(fmt="%H:%M")),
        ("value", String()),
        ("reagent", String()),
    )

    def __init__(self, **kwargs):
        self.date = kwargs.pop("date", None)
        for name, field in self.fields:
            setattr(self, name, kwargs.get(name, None))
        super(BioradResult, self).__init__()

    def parse(self, line):
        splitted = line.split(SEP)
        for i, (name, field) in enumerate(self.fields):
            setattr(self, name, field.to_python(splitted[i]))
        if self._date and self._time:
            self.date = datetime.datetime(
                *(self._date.timetuple()[:3] +
                    self._time.timetuple()[3:5]))
        return self

    def to_string(self):
        self._date = self._time = self.date
        return SEP.join([
            field.to_string(getattr(self, name))
            for name, field in self.fields])


class Biorad(list):

    def add(self, **kwargs):
        obj = BioradResult(**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            self.append(BioradResult().parse(line))
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "biorad",
        "label": "Biorad",
    }

    def create(self):
        return Biorad()

    def parse(self, data):
        return Biorad().parse(data)

    def to_string(self, message):
        return message.to_string()

    def encode_worklist(self, worklist, context):
        if worklist.context != "result":
            raise NotImplementedError("context %s" % (worklist.context,))

        msg = self.create()
        for sample in worklist.samples:
            if sample.type != "control":
                continue

            for result in sample.results:
                reagent = ""
                for r in result.lots:
                    if r.type == "reagent":
                        reagent = r.lot
                msg.add(
                    code=result.code,
                    instrument=result.instrument,
                    lot=sample.sample_id,
                    level=sample.control_level,
                    date=result.date,
                    value=result.value,
                    reagent=reagent,
                )

        if len(msg):
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample = worklist.get_or_create(
                type="control",
                sample_id=m.lot,
                control_lot=m.lot,
            )
            result = sample.results.add(
                code=m.code,
                date=m.date,
                value=m.value,
                instrument=m.instrument,
            )
            if m.reagent:
                result.lots.add(
                    type="reagent",
                    lot=m.reagent,
                )
        return worklist
