# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimForeign,
)


class BioradSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    software_version = HprimString((5, 2), default="1.00")
    date = HprimDate(14)
    orders = HprimForeign("O", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class BioradSegmentO(Segment):
    specimen_id = HprimString(3)
    results = HprimForeign("R", "order")


class BioradSegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        code=HprimString(4),
        slide_number=HprimString(5),
    )
    value = HprimString(4)
    units = HprimString(5)
    operator_identification = HprimString(11)
    date_completed = HprimDate(13, fmt_date="%Y%m%d%H%M%S")
    instrument_identification = HprimString(14)

    infos = HprimForeign("S", "result")


class BioradSegmentC(Segment):
    comment_text = HprimString(4)


class BioradSegmentS(Segment):
    quality_control = HprimMultiple(
        7,
        # Status du controle
        # I: Data was successfully imported
        # R: Import was attempted, but data record was Rejected by URT Importer
        # P: Problem occurred that prevented result(s) from being imported
        # E: Error or Exception occurred during import
        status=HprimString(1),
        # Rules that were triggered (if none fired, this component will be empty)
        # Invalid point
        # Règle appliquée
        invalid_point=HprimString(2),
        # 0: if Rule Violation caused result to not be accepted
        # 1: if result was accepted
        rules_violation=HprimString(3),
        # unique integer ID in the Unity Real Time database for this result
        mID=HprimString(4)
    )


class BioradSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "biorad_urt2",
        "label": "Biorad URT2",
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        # cette option est utilisée dans integration
        AstmMessageV2.BooleanOption(
            "ignore_flags_biorad_urt2",
            label="Ignorer les retours des flags URT",
            default=False),
    ]

    SEGMENT_CLASS_MAP = {
        "H": BioradSegmentH,
        "O": BioradSegmentO,
        "R": BioradSegmentR,
        "C": BioradSegmentC,
        "S": BioradSegmentS,
        "L": BioradSegmentL,
    }

    def parse_segment(self, line, separators):
        if line[:2] == "S%s" % (separators[0],):
            splitted = line.split(separators[0])
            splitted[6] = splitted[6].replace("\\", ";")
            line = separators[0].join(splitted)
        return super(Driver, self).parse_segment(line, separators)

    def create(self, **kwargs):
        message = BioradSegmentH(type_segment="H", **kwargs)
        message.end = BioradSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for sample in worklist.samples:
            seg_o = msg.orders.add(
                specimen_id=sample.sample_id,
            )
            # envoyer les résultats de QC a destination de l'urt
            for result in sample.results:
                seg_r = seg_o.results.add(
                    value=result.value,
                    date_completed=result.date,
                    instrument_identification=result.instrument,
                )
                seg_r.universal_test_id.code = result.code
        if len(msg.orders):
            return msg
        else:
            return None

    def decode_message(self, message):
        worklist = Worklist(context="result")
        # enregistrer tous les résultats de QC
        for seg_o in message.orders:
            sample_id = seg_o.specimen_id.strip()
            for seg_r in seg_o.results:
                for seg_s in seg_r.infos:
                    # Enregistrer uniquement la valeur de l'integer ID lorsque
                    # la donnée a été importée avec succés (avec le status I)
                    # et que le champ rules_violation est présent (S.7.3)
                    if seg_s.quality_control.status == "I" and seg_s.quality_control.rules_violation:
                        sample = worklist.get_or_create(
                            control_lot=sample_id,
                            sample_id=sample_id,
                            type="control",
                        )
                        if seg_s.quality_control.invalid_point:
                            flags = seg_s.quality_control.invalid_point.split(";")
                        else:
                            flags = []
                        sample.results.add(
                            code=seg_r.universal_test_id.code,
                            date=seg_r.date_completed,
                            value=seg_r.value,
                            unit=seg_r.units,
                            flags=flags,
                            instrument=seg_r.instrument_identification,
                            status=seg_s.quality_control.rules_violation,
                        )
        return worklist
