# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
from StringIO import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Date, Multiple


class BlotrixRecomscanBase(Splitted):
    SEPARATOR = ";"


class BlotrixRecomscanOrder(BlotrixRecomscanBase):
    FIELDS = (
        String("name_test_kit"),
        String("name_setcard"),
        String("lot_number"),
        String("number_setcard"),
        String("control_number"),
        String("number_strip_analysis"),
        String("type_strip"),
        String("ig_type"),
        String("connected_with_strip_number"),
        String("external_patient_id"),
        String("last_name"),
        String("first_name"),
        Date("date_of_birth", fmt_date="%d.%m.%Y"),
        String("sample_id"),
        String("date_analysis"),
        String("patient_id"),
        String("result_elisa_iga"),
        String("result_elisa_igg"),
        String("result_elisa_igm"),
        String("database_reference"),
    )


class BlotrixRecomscanResult(object):
    FIELDS = [
        String("order_no"),
        String("company"),
        String("type_of_test"),
        String("revision"),
        String("control"),
        String("ig_type"),
        String("charge"),
        String("strip_type"),
        String("cw"),
        String("sample_no"),
        String("external_patient_id"),
        String("last_name"),
        String("first_name"),
        String("birthday"),
        String("sex"),
        String("elisa_iga"),
        String("elisa_igg"),
        String("elisa_igm"),
        Date("date_request", fmt_date="%d.%m.%Y"),
        Date("date_analysis", fmt_date="%d.%m.%Y"),
        String("cutoff_tolerance"),
        String("result_short_form"),
        String("result"),
        String("result_joined"),
        String("antigens"),
        String("rev_antigens"),
        String("windows_user"),
        String("checksum"),
        Multiple("results", (
            String("band"),
        ), separator=";"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class BlotrixRecomscanMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: BlotrixRecomscanOrder,
        MessageV2.RESULT: BlotrixRecomscanResult
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(u';'), quotechar=str(u'"'))
        for line in reader:
            # Ne pas traiter la première ligne d'entête
            if line[0] == "No" and line[1] == "Company":
                continue
            if not len(line):
                continue
            if len(line) < 12:
                type_message = MessageV2.ORDER
            else:
                type_message = MessageV2.RESULT
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "blotrix_recomscan",
        "label": "Blotrix Recomscan",
    }

    OPTIONS = [
        MessageV2.BooleanOption(
            "ignore_resultat_absent",
            label="Ignorer les résultats non disponibles (n.a.)",
        )
    ]

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: BlotrixRecomscanOrder,
        MessageV2.RESULT: BlotrixRecomscanResult,
    }

    LABEL_RESULT = (
        'ReaktKontr-I', 'ReaktKontr%CO', 'ReaktKontr-A',
        'ReaktKontr-R', 'IgG-I', 'IgG%CO', 'IgG-A', 'IgG-R', 'IgM-I', 'IgM%CO',
        'IgM-A', 'IgM-R', 'CutOff-I', 'CutOff%CO', 'CutOff-A ', 'CutOff-R',
        'p100-I', 'p100%CO', 'p100-A', 'p100-R', 'VlsE-I', 'VlsE%CO',
        'VlsE-A', 'VlsE-R', 'p58-I', 'p58%CO', 'p58-A', 'p58-R',
        'p41-I', 'p41%CO', 'p41-A', 'p41-R', 'p39-I', 'p39%CO',
        'p39-A', 'p39-R', 'OspA-I', 'OspA%CO', 'OspA-A', 'OspA-R',
        'OspCBss-I', 'OspCBss%CO', 'OspCBss-A', 'OspCBss-R', 'OspCBaf-I', 'OspCBaf%CO', 'OspCBaf-A', 'OspCBaf-R',
        'OspCBga-I', 'OspCBga%CO', 'OspCBga-A', 'OspCBga-R', 'OspCBsp-I',
        'OspCBsp%CO', 'OspCBsp-A', 'OspCBsp-R', 'p18Bss-I', 'p18Bss%CO',
        'p18Bss-A', 'p18Bss-R', 'p18Baf-I', 'p18Baf%CO', 'p18Baf-A',
        'p18Baf-R', 'p18Bg1-I', 'p18Bg1%CO', 'p18Bg1-A', 'p18Bg1-R', 'p18Bg2-I',
        'p18Bg2%CO', 'p18Bg2-A', 'p18Bg2-R', 'p18Bsp-I', 'p18Bsp%CO',
        'p18Bsp-A', 'p18Bsp-R'
    )

    def __init__(self, ignore_resultat_absent=False, **kwargs):
        self.ignore_resultat_absent = ignore_resultat_absent
        return super(Driver, self).__init__(**kwargs)

    def create(self):
        return BlotrixRecomscanMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = BlotrixRecomscanMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) == 0:
            return None
        elif len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = samples[0][0]
            for order in sample.orders:
                liste_order_codes = order.code.split(",")
                if "," in order.code and len(liste_order_codes) == 3:
                    name_test_kit = "\"{}\"".format(liste_order_codes[0])
                    name_setcard = "\"{}\"".format(liste_order_codes[1])
                    ig_type = liste_order_codes[2]
                else:
                    name_test_kit = ""
                    name_setcard = ""
                    ig_type = order.code
                msg.add(
                    type_message=MessageV2.ORDER,
                    name_test_kit=name_test_kit,
                    name_setcard=name_setcard,
                    type_strip="\"{}\"".format("Patient"),
                    ig_type=ig_type,
                    external_patient_id="\"{}\"".format(sample.patient_id),
                    pid="\"{}\"".format(sample.patient_id),
                    last_name="\"{}\"".format(sample.last_name),
                    first_name="\"{}\"".format(sample.first_name),
                    sample_id="\"{}\"".format(sample.sample_id),
                    date_of_birth=sample.birthdate,
                    result_elisa_iga="0",
                    result_elisa_igg="0",
                    result_elisa_igm="0",
                )
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample = worklist.get_or_create(
                sample_id=m.sample_no,
                type="patient",
            )
            if m.result_short_form:
                sample.results.add(
                    code="{}_rShort".format(m.ig_type.strip()),
                    value=m.result_short_form.strip(),
                )
            if m.result:
                sample.results.add(
                    code="{}_Result".format(m.ig_type.strip()),
                    value=m.result.strip(),
                )
            if m.antigens and not self.ignore_resultat_absent and m.antigens == "n.a. ":
                sample.results.add(
                    code="{}_antigens".format(m.ig_type.strip()),
                    value=m.antigens.strip(),
                )

            # Multiple résultats BAND
            for i, result in enumerate(m.results):
                # Ignorer les résultats de type "n.a." mis par défaut quand pas
                # de résultat
                if self.ignore_resultat_absent and result == 'n.a. ':
                    continue
                sample.results.add(
                    code="{}_{}".format(m.ig_type.strip(), self.LABEL_RESULT[i]),
                    value=result.replace(",", ".").strip(),
                )
        return worklist
