# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class BygSegmentH(HeaderSegment):

    sender_id = HprimString((5, 1), default='MCA')
    receiver_id = HprimString((10, 1), default='EVM')
    processing_id = HprimString(12, default='P')
    version = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class BygSegmentP(Segment):

    patient_id = HprimString(3, max_length=11)
    last_name = HprimString((6, 1), max_length=26)
    first_name = HprimString((6, 2), max_length=26)
    nomjf = HprimString(7)
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sexe = HprimString(9)
    service = HprimString((26, 3))
    orders = HprimForeign("O", "patient")


class BygSegmentO(Segment):

    tube = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))

    # S: Stat
    # R: Routine
    priorite = HprimString(6, default='R')
    request_date = HprimDate(7)

    action_code = HprimString(12, default='N')
    report_type = HprimString(26, default='O')

    results = HprimForeign("R", "order")


class BygSegmentR(Segment):

    test = HprimString((3, 4))
    value = HprimString(4)
    # F: Final
    # X: Non fait
    status = HprimString(9)
    date = HprimDate(13)


class BygSegmentC(Segment):
    # C|1|L|XXXXXX

    commentaire = HprimString(4)


class BygSegmentL(Segment):

    code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "byg_astm",
        "label": "Byg EVM Astm",
    }
    SEGMENT_CLASS_MAP = {
        "H": BygSegmentH,
        "P": BygSegmentP,
        "O": BygSegmentO,
        "R": BygSegmentR,
        "C": BygSegmentC,
        "L": BygSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = BygSegmentH(type_segment="H", **kwargs)
        message.end = BygSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", worklist.ELSE: "N"},
            priority={"S": "S", "A": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                date_naissance=patient.birthdate,
                sexe=patient.sex,
                service=patient.location,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    tube=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    request_date=sample.request_date,
                    report_type="O",
                )
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code
                if action == "N":
                    for result in sample.results:
                        seg_o.results.add(
                            test=result.code,
                            value=result.value,
                            status="N",
                            date=result.date,
                        )

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.tube,
                    type="patient"
                )
                for seg_r in seg_o.results:
                    if seg_r.status != "F":
                        continue
                    commentaires = [seg_c.commentaire for seg_c in
                                    seg_r.commentaires]
                    sample.results.add(
                        code=seg_r.test,
                        date=seg_r.date,
                        value=seg_r.value,
                        comments=commentaires,
                    )
        return worklist
