# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class C111SegmentH(HeaderSegment):
    sender_name = HprimMultiple(
        5,
        name=HprimString(1),
        manufacturer=HprimString(2),
        instrument_type=HprimString(3),
        sw_version=HprimString(4),
        protocol_version=HprimString(5),
        serial_number=HprimString(6)
    )
    receiver_name = HprimString(10)
    comment = HprimString(11)
    processing_id = HprimString(12, default="P")
    version_no = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    infos = HprimForeign("M", "infos")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class C111SegmentP(Segment):
    laboratory_pat_id = HprimString(4)
    orders = HprimForeign("O", "patient")


class C111SegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        sample_id=HprimString(1),
        rack_id=HprimString(2),
        position=HprimString(3),
    )
    sample_id = HprimString((4, 1))
    rack_id = HprimString((4, 2))
    position = HprimString((4, 3))
    tray_id = HprimString((4, 4))
    tube_type = HprimString((4, 5))
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        test_id=HprimString(4),
        dilution=HprimString(5),
        pretreatment_type=HprimString(6),
        result_evaluation_type=HprimString(7)
    ))
    # R: Routine
    # S: STAT
    priority = HprimString(6)

    # A: Add tests to an existing order
    # C: Cancel
    # Q: Quality control result data
    # N: New order
    action_code = HprimString(12)
    date_results = HprimDate(23)

    # O: Order record
    # X: Results cannot be done / request cancelled
    # Z: No record of this patient/sample
    # F: Final results
    # Q: Query response
    report_types = HprimList(26)
    results = HprimForeign("R", "order")


class C111SegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimList(
        4,
        basefield=HprimMultiple(
            0,
            code=HprimString(1),
            text=HprimString(2),
        ),
    )
    comment_type = HprimString(5)


class C111SegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        test_id=HprimString(4),
        treatment_type=HprimString(5),
        pretreatment_type=HprimString(6),
        result_evaluation_type=HprimString(7)
    )
    data = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)
    # L: Less than reference range
    # H: Higher than reference range
    # <: Below absolute low scale of assay (test range)
    # >: Above absolute high scale of assay (test range)
    # N: Normal
    # A: Abnormal (Technical problems)

    abnormal_flags = HprimList(7)
    # F: Final result
    # C: Correction of previously transmitted result
    # X: Test cannot be completed
    # R: Repeat
    result_status = HprimString(9)
    operator_identification = HprimString(11)
    infos = HprimForeign("M", "result")


class C111SegmentM(Segment):
    roche_specific_record = HprimMultiple(
        3,
        type=HprimString(1),
        bmtag=HprimString(2),
        instrument=HprimString(3),
        version=HprimString(4)
    )
    test_number = HprimString(4)
    reagent_lot_number = HprimString(5)
    units = HprimString(6)
    test_container_description = HprimString(7)
    test_package_number = HprimString(8)
    calibration_method = HprimString(9)
    replication_number = HprimString(10)
    date = HprimDate(11)
    # A: Accepted
    # O: Calculated no flags
    # F: Calculated with flags
    # X: cannot be measured
    result_states = HprimString(12)
    curve_parameter = HprimString(14)
    calibrators = HprimString(15)
    calibrator_result = HprimString(16)


class C111SegmentQ(Segment):
    starting_range_id = HprimString((3, 2))
    universal_test_id = HprimString(5)
    request_status_code = HprimString(13)


class C111SegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "c111",
        "label": "Cobas C111",
    }
    SEGMENT_CLASS_MAP = {
        "H": C111SegmentH,
        "P": C111SegmentP,
        "O": C111SegmentO,
        "R": C111SegmentR,
        "C": C111SegmentC,
        "M": C111SegmentM,
        "Q": C111SegmentQ,
        "L": C111SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = C111SegmentH(type_segment="H", **kwargs)
        message.end = C111SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):
            seg_p = message.patients.add()
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    priority=priority,
                    action_code=action,
                )
                seg_o.specimen_id.sample_id = sample.sample_id
                seg_o.report_types.append("O")
                if context["query"]:
                    seg_o.report_types.append("Q")
                for order in sample.orders:
                    test = seg_o.universal_test_id.add()
                    test.test_id = order.code
                    test.dilution = order.dilution
        if message.patients:
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # Controle qualité
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        control_lot="%s-%s" % (
                            seg_o.specimen_id.rack_id,
                            seg_o.sample_id,
                        ),
                        sample_id="%s-%s" % (
                            seg_o.specimen_id.rack_id,
                            seg_o.sample_id,
                        ),
                        control_level=seg_o.sample_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id,
                        type="patient"
                    )
                if seg_o.rack_id and seg_o.position:
                    sample.rack = "%s-%s" % (seg_o.rack_id, seg_o.position)
                for seg_r in seg_o.results:
                    commentaires = []
                    alarmes = seg_r.abnormal_flags
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type == "I":
                            for comment in seg_c.comment_text:
                                alarmes.append(comment.code)
                                commentaires.append(comment.text)
                    result = sample.results.add(
                        code=seg_r.universal_test_id.test_id,
                        value=seg_r.data,
                        flags=alarmes,
                        comments=commentaires,
                        date=seg_o.date_results,
                    )
                    if sample.type == 'patient' and seg_r.operator_identification:
                        result.operator = seg_r.operator_identification
                    # Numéro de lot réactif
                    for seg_m in seg_r.infos:
                        if seg_m.reagent_lot_number:
                            result.lots.add(
                                type="reagent",
                                lot=seg_m.reagent_lot_number,
                            )
        return worklist
