# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Message
from driverng.utils import format_date
from driverng.exceptions import ProgrammingError, ParsingError


class CA560Segment(object):
    # ("Q" = fixed string, N = N spaces, attr cf TRAME_DICT)
    TRAME_DESC = ()
    # La longeur des elements de TRAME_DESC
    TRAME_END_OFFSET = None
    # {key: length}
    TRAME_DICT = {}
    # Liste des keys justifiés à droite
    # tous les autres sont justifiés à gauche.
    RIGHT_JUSTIFIED_ITEMS = ()
    # Liste des keys que l'on peut truncate
    TRUNCABLE_ITEMS = ()

    def __init__(self, **kwargs):
        for key in self.TRAME_DESC:
            if key in self.TRAME_DICT:
                setattr(self, key, kwargs.get(key, ""))
        super(CA560Segment, self).__init__()

    @classmethod
    def parse(cls, data):
        obj = cls()

        cur = data
        for key in cls.TRAME_DESC:

            if key in cls.TRAME_DICT:
                length = cls.TRAME_DICT[key]
                value = cur[:length].strip()
                setattr(obj, key, value)
                cur = cur[length:]
            elif isinstance(key, basestring):
                if cur[:len(key)] != key:
                    raise ParsingError("Key '%s' not found" % (key,))
                cur = cur[len(key):]
            elif isinstance(key, int):
                # Nombre d'espaces à sauter.
                # On verifie que ce sont bien des espaces
                if cur[:key] != key * " ":
                    raise ParsingError("No enough spaces here !")
                cur = cur[key:]
            else:
                raise ProgrammingError("Unknown key '%s'" % (key,))

        return obj

    def to_string(self):

        data = ""
        for key in self.TRAME_DESC:
            if key in self.TRAME_DICT:
                length = self.TRAME_DICT[key]
                value = "%s" % getattr(self, key)

                if key in self.TRUNCABLE_ITEMS:
                    value = value[:length]

                if len(value) > length:
                    raise ParsingError("Value '%s' for key '%s' "
                                       "is too long (> %d char) !" % (
                                           value,
                                           key,
                                           length,
                                       ))

                fmt = "%%%d.%ds" % (length, length)
                value = fmt % (value,)
                data += value
            elif isinstance(key, basestring):
                data += key
            elif isinstance(key, int):
                data += key * " "
            else:
                raise ProgrammingError("Unknown key '%s'" % (key,))

        return data


class CA560Data(object):

    def __init__(self, code=None, value=None, flag=None):
        self.code = code
        self.value = value
        self.flag = flag

    def to_string(self):
        return "%3.3s%5.5s%1.1s" % (
            self.code or "",
            self.value or "",
            self.flag or "",
        )


class _CA560Message(CA560Segment):
    TRAME_DESC = (
        "type_message", "tdc2", "21",
        "n_block", "total_blocks", "sample_distinction_code",
        "raw_date", "rack", "position",
        "sample_id", "id_information", "patient_name",
    )

    TRAME_END_OFFSET = 50

    TRAME_DICT = {
        # R (request ?): Query
        # S: Order
        # D: Data Result
        "type_message": 1,
        "tdc2": 1,  # 1 (normal data), 2 (data)
        "n_block": 2,
        "total_blocks": 2,
        # U: Routine
        # E: STAT
        # S: Standard curve
        # C: Quality control
        "sample_distinction_code": 1,
        "raw_date": 10,  # %y%m%d%H%M
        "rack": 4,  # ou 6 ??
        "position": 2,  # 01 - 10
        "sample_id": 13,  # ou 15 ??
        # M: Manual entry
        # A: Automatically assigned by the instrument
        # B: Read by ID barcode reader
        # C: Set by the host computer order information
        "id_information": 1,
        "patient_name": 11,  # ou 15 ??
    }
    TRUNCABLE_ITEMS = ("patient_name",)

    def __init__(self, **kwargs):
        # Liste de CA560Data
        self.data = []
        self.date = kwargs.pop("date", datetime.datetime.now())
        kwargs.setdefault("tdc2", "2")
        kwargs.setdefault("n_block", "01")
        kwargs.setdefault("total_blocks", "01")
        super(_CA560Message, self).__init__(**kwargs)

    def add_data(self, *args, **kwargs):
        ap = CA560Data(*args, **kwargs)
        self.data.append(ap)
        return ap

    @classmethod
    def parse(cls, data):
        obj = super(_CA560Message, cls).parse(data)

        if obj.raw_date:
            obj.date = datetime.datetime.strptime(obj.raw_date, "%d%m%y%H%M")

        n = obj.TRAME_END_OFFSET
        while len(data[n:][:9]) == 9:
            code = data[n:][:3].strip()
            value = data[n + 3:][:5].strip()
            flag = data[n + 8:][:1].strip()
            obj.add_data(code, value, flag)
            n += 9

        if data[n:]:
            raise ParsingError("Remaining data after parsing '%s'" % (
                data[n:],
            ))
        return obj

    def to_string(self):
        if self.date:
            self.raw_date = format_date(self.date, fmt="%d%m%y%H%M")

        data = super(_CA560Message, self).to_string()
        for ap in self.data:
            data += ap.to_string()

        return data


class Driver(Message):
    SPEC = {
        "code": "ca560",
        "label": "CA (500, 560, 1500, 7000)",
    }

    TYPE_MESSAGE_MAP = {
        Message.QUERY: "R",
        Message.RESULT: "D",
        Message.ORDER: "S",
    }

    def create(self, message_type, *args, **kwargs):
        try:
            type_message = self.TYPE_MESSAGE_MAP[message_type]
        except KeyError:
            raise ProgrammingError("Message type '%s' unknown" % (
                message_type,
            ))
        kwargs.setdefault("type_message", type_message)
        return _CA560Message(*args, **kwargs)

    def get_message_type(self, message):
        for m_type, m_type_code in self.TYPE_MESSAGE_MAP.items():
            if message.type_message == m_type_code:
                return m_type

        raise ProgrammingError("Unkwnon message type '%s'" % (
            message.m_type_code,
        ))

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        return _CA560Message.parse(data)

    def get_queries(self, message):
        if self.get_message_type(message) == self.QUERY:
            return [message.sample_id]
        else:
            return []

    def get_order_message(self, worklist, context):

        if len(worklist.samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        elif len(worklist.samples) == 1:
            sample = worklist.samples[0]
            msg = self.create(
                self.ORDER,
                date=context["now"],
                sample_distinction_code="U",
                sample_id=sample["sample_id"],
                patient_name=sample["last_name"],
                id_information="C",
            )

            if context["query"]:
                msg.rack = context["query"].rack
                msg.position = context["query"].position
                msg.tdc2 = context["query"].tdc2

            for order in sample["orders"]:
                if order["action"] == "del":
                    continue
                msg.add_data(code=order["code"])
            return msg
        elif context["query"]:
            return self.create(
                self.ORDER,
                sample_distinction_code="U",
                sample_id=context["query"].sample_id,
                id_information="C",
                rack=context["query"].rack,
                position=context["query"].position,
                tdc2=context["query"].tdc2,
            )

    def get_result_dict(self, message):

        if message.sample_distinction_code == "C":  # QC
            return {}

        if message.rack and message.position:
            rack = "%s-%s" % (message.rack, message.position)
        else:
            rack = None

        results = {}
        results[message.sample_id] = {"resultats": [], "rack": rack}

        for data in message.data:
            try:
                value = self.get_decimale(data.code, data.value)
            except ValueError:
                value = data.value
            results[message.sample_id]["resultats"].append({
                "test": data.code,
                "resu": value,
                "alarmes": [data.flag] if data.flag.strip() else [],
                "date": message.date,
            })

        return results

    @staticmethod
    def get_decimale(test, resu):
        # décimale du résultat selon le dernier caractère du parameter code
        # 1 Time: 1 décimale
        # 2 Percent/concentration: 1 décimale
        # 3 PT ratio: 2 décimales
        # 4 INR: 2 décimales
        # 5 dFbg en g/L: 3 décimales

        if test[-1] in ('1', '2', '3', '4', '5') and test != '272':
            multiplier = {"1": 1, "2": 1, "3": 2, "4": 2, "5": 3}[test[-1]]
            return unicode(float(resu) / (10 ** multiplier))
        elif test == '272':
            return unicode(float(resu) / 100)
        else:
            return resu

    def get_qc_result_list(self, message):

        # uniquement les QC
        if message.sample_distinction_code != "C":
            return []
        result = []
        for data in message.data:
            try:
                value = self.get_decimale(data.code, data.value)
            except ValueError:
                value = data.value
            result.append({
                "lot": message.sample_id,
                "test": data.code,
                "resu": value,
                "date": message.date,
            })

        return result
