# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
    HprimMultipleList,
)


class CA660SegmentH(HeaderSegment):

    sender_id = HprimMultiple(
        5,
        name=HprimString(1),
        software_version=HprimString(2),
        analyzer_serial_no=HprimString(3),
        interface_version=HprimString(4),
        logical_id_number=HprimString(5),
    )

    receiver_id = HprimString(10, default="CA-600")
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class CA660SegmentP(Segment):

    # Le segment P contient uniquement le numéro de séquence
    orders = HprimForeign("O", "patient")


class CA660SegmentQ(Segment):

    starting_range_id = HprimMultiple(
        3,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_id_no=HprimString(3),
        sample_id_attribute=HprimString(4),
    )

    tests = HprimList(5, basefield=HprimMultiple(
        0,
        test_code=HprimString(4),
        test_name=HprimString(5),
    ))
    request_day_limit = HprimString(6)
    request_date = HprimDate(7)


class CA660SegmentO(Segment):

    specimen_id = HprimMultiple(
        3,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_id_no=HprimString(3),
        sample_id_attribute=HprimString(4),
    )
    instrument_specimen_id = HprimMultiple(
        4,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_id_no=HprimString(3),
        sample_id_attribute=HprimString(4),
    )

    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution_ratio=HprimString(6, default="100"),
    ))

    # S : Stat
    # R : Routine
    priority = HprimString(6)
    order_date = HprimDate(7)
    # N : Normal
    # Q : QC
    # S : STD
    action_code = HprimString(12)

    results = HprimForeign("R", "order")


class CA660SegmentR(Segment):

    universal_test_id = HprimMultiple(
        3,
        code=HprimString(4),
        parameter=HprimString(5),
        dilution_ratio=HprimString(6),
        analysis_result_type=HprimString(7),
    )
    value = HprimString(4)
    units = HprimString(5)
    abnormal_flags = HprimMultipleList(7)
    date = HprimDate(13)


class CA660SegmentL(Segment):
    # N : Normal termination
    termination_code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "ca660",
        "label": "CA660 ASTM",
    }
    SEGMENT_CLASS_MAP = {
        "H": CA660SegmentH,
        "P": CA660SegmentP,
        "O": CA660SegmentO,
        "Q": CA660SegmentQ,
        "R": CA660SegmentR,
        "L": CA660SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = CA660SegmentH(type_segment="H", **kwargs)
        message.end = CA660SegmentL(type_segment="L")
        message.set_raw("sender_id", 'HostName^^^^')
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.starting_range_id.sample_id_no.strip())
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add()
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    priority=priority,
                    action_code=action,
                    order_date=sample["request_date"],
                )
                seg_o.specimen_id.sample_id_no = sample["sample_id"]
                if context["query"] is not None:
                    for seg_q in context["query"].queries:
                        if seg_q.starting_range_id.sample_id_no.strip() == sample["sample_id"]:
                            seg_o.specimen_id.sample_id_no = seg_q.starting_range_id.sample_id_no
                            for attr in (
                                "rack_no", "tube_position",
                                "sample_id_attribute",
                            ):
                                setattr(
                                    seg_o.specimen_id, attr, getattr(
                                        seg_q.starting_range_id, attr))
                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.code = order["code"]
                    t.dilution_ratio = "100"

        if len(msg.patients):
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                # exclusion des CQ
                if seg_o.action_code == "Q":
                    continue
                if seg_o.instrument_specimen_id.rack_no and seg_o.instrument_specimen_id.tube_position:
                    rack = "%s-%s" % (
                        seg_o.instrument_specimen_id.rack_no,
                        seg_o.instrument_specimen_id.tube_position,
                    )
                else:
                    rack = None
                tube = seg_o.instrument_specimen_id.sample_id_no.strip()
                results.setdefault(tube, {
                    "resultats": [],
                    "rack": rack,
                })
                for seg_r in seg_o.results:
                    results[tube]["resultats"].append({
                        "test": seg_r.universal_test_id.code,
                        "resu": seg_r.value,
                        "date": seg_r.date,
                        "alarmes": seg_r.abnormal_flags,
                    })

        return results

    def get_qc_result_list(self, message):
        results = []
        for seg_p in message.patients:

            for seg_o in seg_p.orders:
                # exclusion des resus non CQ
                if seg_o.action_code != "Q":
                    continue
                lot = seg_o.instrument_specimen_id.sample_id_no.strip()
                for seg_r in seg_o.results:
                    results.append({
                        "lot": lot,
                        "test": seg_r.universal_test_id.code,
                        "resu": seg_r.value,
                        "date": seg_r.date,
                    })
        return results
