# -*- coding: utf8 -*-

from __future__ import unicode_literals

import logging
import datetime
from StringIO import StringIO

from driverng.utils import get_date, format_date
from driverng import (
    Message,
    GraphContainer,
)
from driverng.fixed import (
    FixedLengthFormat,
    String as BaseString,
    Multiple,
)
from driverng.exceptions import ParsingError, ProgrammingError

from driverng.lazy import lazy


logger = logging.getLogger("driverng.drivers.cd3200")


class HistogramGraph(GraphContainer):
    def __init__(self, points, type_fichier="png"):
        self.points = points
        super(HistogramGraph, self).__init__(type_fichier=type_fichier)

    def get_bytes(self):
        pylab = lazy.pylab
        pylab.figure(figsize=(4, 4), edgecolor="r")
        pylab.plot(self.points)
        pylab.fill_between(
            list(range(len(self.points))), self.points, color="blue")
        bufimg = StringIO()
        pylab.savefig(bufimg, dpi=100)
        pylab.close()
        return bufimg.getvalue()


class String(BaseString):

    def __init__(self, name, length, quoted=True, minus=True, **kwargs):
        self.quoted = quoted
        self.offset = 3 if quoted else 1
        length += self.offset
        # Remplacer les espaces par des "-" ?
        self.minus = minus
        super(String, self).__init__(name, length, **kwargs)

    def unquote(self, value):
        if (
            (self.quoted and (value[0], value[-2:]) != ('"', '",')) or (
                not self.quoted and value[-1] != ','
            )
        ):
            raise ParsingError(
                "Unexpected unquoted string or missing comma '%s'" % (
                    value,))
        return value[1:-2] if self.quoted else value[:-1]

    def to_python(self, value):
        self.length -= self.offset
        value = super(String, self).to_python(self.unquote(value))
        self.length += self.offset
        if self.quoted and self.minus and value == len(value) * "-":
            return ""
        else:
            return value

    def quote(self, value):
        if value is not None:
            if self.quoted:
                return '"%s",' % (value,)
            else:
                return '%s,' % (value,)
        else:
            return None

    def to_string(self, value):
        if not value and self.quoted and self.minus:
            value = (self.length - self.offset) * "-"
        self.length -= self.offset
        value = self.quote(super(String, self).to_string(value))
        self.length += self.offset
        return value


class Date(String):

    def __init__(self, *args, **kwargs):
        self.fmt_date = kwargs.pop("fmt_date", "%m/%d/%Y")
        self.fmt_map = kwargs.pop("fmt_map", {10: "%m/%d/%Y"})
        self.default_string = kwargs.pop("default_string", "--/--/----")
        super(Date, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value == self.default_string:
            return None
        else:
            return get_date(super(Date, self).to_python(value),
                            fmt_map=self.fmt_map)

    def to_string(self, value):
        if not value:
            value = self.default_string
        else:
            value = format_date(value, fmt=self.fmt_date)
        return super(Date, self).to_string(value)


class Time(Date):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault("fmt_date", "%H:%M")
        kwargs.setdefault("fmt_map", {5: "%H:%M"})
        kwargs.setdefault("default_string", "--/--")
        super(Time, self).__init__(*args, **kwargs)


FMT_DESCRIPTION_SPECIMEN = [
    String("specimen_id", 12),
    String("specimen_name", 28, truncable=True),
    String("patient_id", 16),
    # M/F
    String("specimen_sex", 1),
    Date("specimen_birthdate", 10)
]

SPECIMEN_IDENTIFICATION_SEGMENT = [
    String("message_type", 3, minus=False),
    # CD3200C: CELL-DYN 3200CS
    # CD3200S: CELL-DYN 3200SL
    String("instrument_type", 7),
    String("serial_number", 12),
    String("sequence_number", 4, quoted=False),
    String("spare", 1, quoted=False),
    # 0: Patient
    # 1: QC sample
    # 2: Background
    # 3: Fragile WBC
    # 4: Calibrator
    # 5: Whole blood
    # 6: Latex
    # 7: Unused
    # 8: Resitant RBC
    String("specimen_type", 1, quoted=False),
] + FMT_DESCRIPTION_SPECIMEN + [
    String("doctor_name", 22, truncable=True),
    String("operator_id", 3),
    Date("specimen_date", 10),
    Time("specimen_time", 5),
    Date("collection_date", 10),
    Time("collection_time", 5),
    String("comment", 16),
]


class Cd3200BaseResult(FixedLengthFormat):
    # MESSAGE_TYPE
    # WB1: WBC Histogram message 1
    # WB2: WBC Histogram message 2
    # RBC: RBC Histogram message
    # PLT: PLT Histogram message
    # NOC: NOC Histogram message
    #    : (3 blanks) CBC Count data message
    # RTC: Retic count message
    FMT_DESCRIPTION = SPECIMEN_IDENTIFICATION_SEGMENT


class Cd3200HistogramResult(Cd3200BaseResult):
    FMT_DESCRIPTION = Cd3200BaseResult.FMT_DESCRIPTION + [
        String("scale_factor", 5, quoted=False),
        Multiple("channel_data", [
            String("data", 3, quoted=False),
        ])
    ]


class Cd3200CBCResult(Cd3200BaseResult):
    CODES = (
        "WBC", "NEU", "LYM", "MONO", "EOS",
        "BASO", "RBC", "HGB", "HCT", "MCV",
        "MCH", "MCHC", "RDW", "PLT", "MPV",
        "PCT", "PDW", "NEU%", "LYM%", "MONO%",
        "EOS%", "BASO%",
    )
    FLAGS = (
        "Moving Average", "DFLT", "BLAST", "Variant Lym",
        "DFLT Neutrophil Qualifier", "DFLT Eosinophil Qualifier",
        "DFLT Lymphocyte Qualifier", "IG", "BAND",
        "DFLT Monocyte Qualifier", "DFLT Basophil Qualifier",
        "IG/Band", "FWBC", "WBC", "Nucleated RBC", "DLTA",
        "NWBC", "RBC MORPH", "RRBC", "ATYPDEP",
        "Platelet Recount", "Spare RBC", "LRI", "URI",
    )
    FMT_DESCRIPTION = Cd3200BaseResult.FMT_DESCRIPTION + [
        String("value_%d" % (i,), 5, quoted=False) for i in range(len(CODES))
    ] + [
        String("flag_%d" % (i,), 1, default="0", quoted=False)
        for i in range(len(FLAGS))
    ] + [
        String("spare_%s" % (i,), 5, quoted=False, default="00000")
        for i in range(8)
    ] + [
        String("noc_concentration", 5, quoted=False),
        String("woc_concentration", 5, quoted=False),
        String("limit_set", 1, quoted=False),
        # 0: Open sample mode
        # 1: Closed sample mode
        String("sample_mode", 1, quoted=False),
        String("spare", 1, quoted=False),
        String("noc_flow_error_flag", 1, quoted=False),
        String("woc_flow_error_flag", 1, quoted=False),
        String("rbc_flow_error_flag", 1, quoted=False),
        String("sampling_error_aspiration_flag", 1, quoted=False),
    ]


class Cd3200RTCResult(Cd3200BaseResult):
    CODES = (
        "RETIC%", "RETIC_ABS", "RBC",
        "BACKGROUND", "IMMATURE_RETIC_FRACTION",
    )
    FMT_DESCRIPTION = Cd3200BaseResult.FMT_DESCRIPTION + [
        String("value_%s" % (i,), 5) for i in range(len(CODES))
    ] + [
        String("spare1_%s" % (i,), 5) for i in range(5)
    ] + [
        String("retic_flow_error_flag", 1, quoted=False),
        String("fragile_rbc_flag", 1, quoted=False),
        String("too_few_events_flag", 1, quoted=False),
        String("spare2", 1, quoted=False),
        # Excessive RBC loss
        String("erl_flag", 1, quoted=False),
        String("no_hematology_data_warning_flag", 1, quoted=False),
    ] + [
        String("spare3_%s" % (i,), 1, quoted=False) for i in range(4)
    ]


BLOCK_IDENTIFICATION_SEGMENT = [
    # MESSAGE TYPE
    # W1E: WBC Extended Histogram Message 1
    # W2E: WBC Extended Hisgogram Message 2
    # RBE: RBC Extended Histogram Message
    # PLE: PLT Extended Histogram Message
    # NOE: NOC Extended Histogram Message
    # RT0: Retic Extended 90-0 Degree Histogram Message
    # RT1: Retic Excessive 90-10 Degree Histogram Message
    # SWE: WBC Scatterplot Message
    # SRE: RBC Scatterplot Message
    # RTS: Retic Scatterplot Message
    String("block_message_type", 3),
    String("block_specimen_id", 12),
    # 0: Last block of the message
    # 1: More blocks follow
    String("continuation_flag", 1, quoted=False),
    # 1: Specimen identification block
    # 2: Histogram block
    # 3: Scatterplot block
    # 4: Cell type block
    String("block_type", 1, quoted=False),
    # 01 to 09
    String("block_number", 2, empty="0", quoted=False),
    String("data_length", 4, quoted=False),
]


class Cd3200SubBlock(FixedLengthFormat):
    FMT_DESCRIPTION = BLOCK_IDENTIFICATION_SEGMENT

    def __init__(self, *args, **kwargs):
        # TODO: HACK
        self._locked = False
        self.binary_data = ""
        super(Cd3200SubBlock, self).__init__(*args, **kwargs)

    def parse(self, data):
        self.binary_data = super(Cd3200SubBlock, self).parse(data)
        return ""

    def to_string(self):
        return "%s%s" % (
            super(Cd3200SubBlock, self).to_string(),
            self.binary_data,
        )


class Cd3200Extended(FixedLengthFormat):
    # Specimen identification block
    FMT_DESCRIPTION = (
        BLOCK_IDENTIFICATION_SEGMENT +
        SPECIMEN_IDENTIFICATION_SEGMENT + [
            String("scale_factor", 5, quoted=False),
        ]
    )

    def __init__(self, *args, **kwargs):
        # TODO: HACK
        self._locked = False
        self.blocks = []
        super(Cd3200Extended, self).__init__(*args, **kwargs)

    def parse(self, data):
        # Extended Histogram:
        # Specimen identification block followed by an
        # extended histogram block

        # Extended Scatterplot
        # Specimen identification block followed by one
        # or more scatterplot blocks followed by one or more cell
        # type block

        lines = data.splitlines()
        super(Cd3200Extended, self).parse(lines[0])
        for line in lines[1:]:
            block = Cd3200SubBlock()
            block.parse(line)
            self.blocks.append(block)
        return ""

    def to_string(self):
        return "\r\n".join([
            super(Cd3200Extended, self).to_string(),
        ] + [b.to_string() for b in self.blocks])


class Cd3200Order(FixedLengthFormat):
    FMT_DESCRIPTION = FMT_DESCRIPTION_SPECIMEN + [
        Date("collection_date", 10),
        Time("collection_time", 5),
        String("doctor_name", 22, truncable=True),
        # Valid value range from 0 to 4
        # 0 signifying default limit_set
        String("limit_set", 1, quoted=False, default="0"),
        # Valid value range from 0 to 4
        # 0 signifying default parameter_set
        String("parameter_set", 1, quoted=False, default="0"),
    ]


class Driver(Message):
    SPEC = {
        "code": "cd3200",
        "label": "Celldyn 3200",
    }

    MAP = {
        "   ": Cd3200CBCResult,
        "RTC": Cd3200RTCResult,
        "WB1": Cd3200HistogramResult,
        "WB2": Cd3200HistogramResult,
        "RBC": Cd3200HistogramResult,
        "PLT": Cd3200HistogramResult,
        "NOC": Cd3200HistogramResult,
        "W1E": Cd3200Extended,
        "W2E": Cd3200Extended,
        "RBE": Cd3200Extended,
        "PLE": Cd3200Extended,
        "NOE": Cd3200Extended,
        "RT0": Cd3200Extended,
        "RT1": Cd3200Extended,
        "SWE": Cd3200Extended,
        "SRE": Cd3200Extended,
        "RTS": Cd3200Extended,
    }

    def create(self, message_type=None, **kwargs):
        if message_type == Message.ORDER:
            return Cd3200Order(**kwargs)

        if message_type == "WBC":
            message_type = "   "
        try:
            klass = self.MAP[message_type]
        except KeyError:
            raise ProgrammingError("Unknown message type '%s'" % (
                message_type,))

        return klass(**kwargs)

    def get_message_type(self, message):
        return self.ORDER

    def parse(self, data):
        key = data[1:6]
        klass_map = dict(('%s",' % k, self.MAP[k]) for k in self.MAP.keys())
        try:
            klass = klass_map[key]
        except KeyError:
            klass = Cd3200Order

        o = klass()
        o.parse(data)
        return o

    def to_string(self, message):
        return message.to_string()

    def get_order_message(self, worklist, context):
        if len(worklist.samples) == 0:
            return
        elif len(worklist.samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = worklist.samples[0]
            for order in sample["orders"]:
                if order["action"] != "del":
                    return self.create(
                        message_type=self.ORDER,
                        specimen_id=sample["sample_id"],
                        specimen_name="%(last_name)s %(first_name)s" % sample,
                        patient_id=sample["patient_id"],
                        specimen_birthdate=sample["birthdate"],
                        collection_date=sample["collection_date"],
                        collection_time=sample["collection_date"],
                        doctor_name=sample["physician"],
                        specimen_sex="F" if sample["sex"] == "F" else "M",
                    )

    @staticmethod
    def get_date(message):
        date = message.specimen_date
        if date and message.specimen_time:
            date += datetime.timedelta(
                hours=message.specimen_time.hour,
                minutes=message.specimen_time.minute,
            )
        return date

    def get_result_dict(self, message):
        if message.specimen_type != "0":
            # N'est pas un résultat patient
            return {}

        results = {message.specimen_id: {"resultats": []}}
        resus = results[message.specimen_id]["resultats"]

        date = self.get_date(message)
        if message.comment:
            commentaires = [message.comment]
        else:
            commentaires = []

        if message.message_type in ("", "   ", "RTC"):
            for i, code in enumerate(message.CODES):
                resu = getattr(message, "value_%d" % (i,))
                resus.append({
                    "test": code,
                    "resu": resu,
                    "date": date,
                    "commentaires": commentaires,
                })
            if message.message_type in ("   ", ""):  # CBC
                alarmes_hemato = []
                for i, flag in enumerate(message.FLAGS):
                    if getattr(message, "flag_%d" % (i,)) == "1":
                        alarmes_hemato.append(flag)
                resus.append({
                    "test": "ALARME",
                    "resu": "",
                    "commentaires": alarmes_hemato,
                    "date": date,
                })
        elif message.message_type in ("WB1", "WB2", "RBC", "PLT", "NOC"):
            # Graphes
            points = [int(c.data) for c in message.channel_data]
            resus.append({
                "test": "GR_%s" % (message.message_type,),
                "graph": HistogramGraph(points),
                "date": date,
                "commentaires": commentaires,
            })
        else:
            logger.info(
                "Message type '%s' non traité",
                message.message_type)

        return results

    def get_qc_result_list(self, message):
        if message.specimen_type != "1":
            # N'est pas un résultat QC
            return []

        date = self.get_date(message)
        results = []
        if message.message_type in ("", "  ", "RTC"):
            for i, code in enumerate(message.CODES):
                resu = getattr(message, "value_%d" % (i,))
                results.append({
                    "test": code,
                    "resu": resu,
                    "date": date,
                    "lot": message.specimen_id,
                })
        else:
            logger.info(
                "Message type '%s' non traité",
                message.message_type)
        return results
