# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class CentaurSegmentH(HeaderSegment):

    emetteur = HprimString(5)
    recepteur = HprimString(10)
    type_message = HprimString(12, default="P")
    version = HprimString(13, default="1")

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")

    # Ceci est probablement un bug du centaur.
    # H|\^&|||ACCP1|||||Host||P|1
    # M|1|CCD^ACS:NG^V1^T|&X00&&X01&&X02&&X03&&X04&&X05&&X06&&X07&
    # &X08&&X09&&X0A&&X0B&&X0C&&X0D&&X0E&&X0F&&X10&&X11&&X12&&X13&
    # &X14&&X15&&X16&&X17&&X18&&X19&&X1A&&X1B&&X1C&&X1D&&X1E&&X1F&
    # !"#$%&E&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ
    # [&R&]&S&_`abcdefghijklmnopqrstuvwxyz{&F&}~&X7F&
    # L|1|N
    bug_centaur = HprimForeign("M", "header")

    end = HprimForeign("L", "header", one2one=True, required=True)


class CentaurSegmentP(Segment):

    # Patient ID
    no_patient = HprimString(3)

    # Nom du Patient
    nom = HprimString((6, 1), max_length=30)

    # Prenom du Patient
    prenom = HprimString((6, 2), max_length=30)

    # Nom de jeune fille
    nomjf = HprimString((6, 3), max_length=30)

    # date de naissance
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")

    # sexe  M/F/U
    sexe = HprimString(9)

    # code medecin prescripteur
    code_prescripteur = HprimString(15)

    # service
    service = HprimString(26)

    orders = HprimForeign("O", "patient")


class CentaurSegmentO(Segment):

    # numero de tube de l'echantillon dans le cas d'un resultat
    # reference du controle lors d'un CQ au format KNNNN ou NNN
    # est le lot du controle
    tube = HprimString((3, 1), max_length=16)

    # position du tube et rack
    position = HprimString((3, 2))
    rack = HprimString((3, 3))

    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution_condition=HprimString(5),
        coeff_dilution=HprimString(6),
    ))

    # urgence
    #  R : Routine   S : Stat (urgence)
    urgence = HprimString(6, max_length=1)

    # code action
    #  Q : Test QC
    #  C : Cancel Request
    #  vide sinon pour la demande
    code_action = HprimString(12, max_length=1)

    # type de resultat
    #  F : Final
    #  Q : Reponse a un Query
    #  X : test annule
    #  I : Instrument Pending (en cours d'analyse)
    report_type = HprimList(26)

    results = HprimForeign("R", "order")
    control = HprimForeign("M", "order")


class CentaurSegmentQ(Segment):

    # Starting Range ID
    #  Sample ID
    tube = HprimString((3, 2))

    # Ending Range ID
    #  Sample ID
    sample_id = HprimString((4, 2))

    # Universal Test ID : ALL (request all tests; no restrictions)
    test = HprimString(5, default="ALL")

    # request information
    # status code : O : request tests
    request = HprimString(13)


class CentaurSegmentR(Segment):

    test = HprimString((3, 4), max_length=8)
    coeff_dilution = HprimString((3, 6))
    # Result Aspects
    #  DOSE : Result concentration value
    #  INDX : Result index value
    #  INTR : Result interpretation (system)
    #  RLU - RLU : value used in result calculation
    aspect = HprimString((3, 8))

    # valeur pouvant etre numerique ou qualitative ou alphanumerique
    valeur = HprimString(4)

    unite = HprimString(5)

    # Result Abnormal Flags
    abnormal_flags = HprimList(7)

    # Result Status F : Final P : Preliminary
    status = HprimString(9)

    # date du resultat ou du QC
    date = HprimDate(13)


class CentaurSegmentC(Segment):

    comment_source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))
    comment_type = HprimString(5)


class CentaurSegmentM(Segment):

    control_name = HprimString(4)

    # lot du controle
    control_lot = HprimString(5)


class CentaurSegmentL(Segment):

    # code de terminaison
    # N terminaison normal (terminaison par defaut)
    # autre valeur possible :
    #  F = Finale
    #  T/R/Q/I
    code = HprimString(3, default="F")


class Driver(AstmMessage):
    SPEC = {
        "code": "centaur",
        "label": "Centaur CP",
    }
    SEGMENT_CLASS_MAP = {
        "H": CentaurSegmentH,
        "P": CentaurSegmentP,
        "O": CentaurSegmentO,
        "Q": CentaurSegmentQ,
        "M": CentaurSegmentM,
        "R": CentaurSegmentR,
        "C": CentaurSegmentC,
        "L": CentaurSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessage.Option("code_emetteur", label="Code emetteur", default="Host"),
        AstmMessage.Option("code_recepteur", label="Code recepteur", default="ACCP1"),
    ]

    def __init__(self,
                 code_emetteur="Host", code_recepteur="ACCP1",
                 **kwargs):
        self.code_emetteur = code_emetteur
        self.code_recepteur = code_recepteur
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = CentaurSegmentH(type_segment="H", **kwargs)
        message.end = CentaurSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.tube)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(
            emetteur=self.code_emetteur,
            recepteur=self.code_recepteur,
            date=context["now"]
        )

        for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                action={"del": "C", worklist.ELSE: ""},
                priority={"S": "S", worklist.ELSE: "R"},

        ):
            seg_p = msg.patients.add(
                no_patient=patient["patient_id"],
                nom=patient["last_name"],
                prenom=patient["first_name"],
                nomjf=patient["maiden_name"],
                date_naissance=patient["birthdate"],
                sex=patient["sex"],
                service=patient["location"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    tube=sample["sample_id"],
                    urgence=priority,
                    report_type=["O"],
                    code_action=action,
                )

                if context["query"]:
                    seg_o.report_type.append("Q")

                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.code = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            msg.end.code = "I"
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                # exclusion des CQ
                if seg_o.code_action == "Q":
                    continue

                # prise en compte du rack et de la position
                if seg_o.rack and seg_o.position:
                    rack = "%s-%s" % (seg_o.rack, seg_o.position)
                else:
                    rack = None

                results.setdefault(seg_o.tube, {"resultats": [], "rack": rack})
                for seg_r in seg_o.results:

                    # uniquement les resultats avec le status final F
                    if seg_r.status not in ("F", ""):
                        continue
                    if seg_r.abnormal_flags:
                        alarmes = seg_r.abnormal_flags[:]
                    else:
                        alarmes = []

                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type == "I":
                            alarmes.append(seg_c.comment_code)

                    # le code resultat est la concatenation du
                    # code test avec l'aspect du test (si présent)
                    if seg_r.aspect:
                        test = "%s%s" % (seg_r.test, seg_r.aspect)
                    else:
                        test = seg_r.test

                    results[seg_o.tube]["resultats"].append({
                        "test": test,
                        "resu": seg_r.valeur,
                        "date": seg_r.date,
                        "alarmes": alarmes,
                        "commentaires": commentaires,
                    })

        return results

    def get_qc_result_list(self, message):
        results = []
        for seg_p in message.patients:

            for seg_o in seg_p.orders:
                # exclusion des resus non CQ
                if seg_o.code_action != "Q":
                    continue

                for seg_m in seg_o.control:
                    lot = seg_m.control_lot

                for seg_r in seg_o.results:
                    if seg_r.aspect:
                        test = "%s%s" % (seg_r.test, seg_r.aspect)
                    else:
                        test = seg_r.test

                    results.append({
                        "lot": lot,
                        "test": test,
                        "resu": seg_r.valeur,
                        "date": seg_r.date,
                    })

        return results
