# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimDate,
    HprimForeign,
)

logger = logging.getLogger("driverng.clinitek")


class ClinitekSegmentMSH(HeaderSegment):
    sender = HprimString(3)
    date = HprimDate(7)
    message_type = HprimString((9, 1))
    control_id = HprimString(10)
    processing_id = HprimString(11)
    version = HprimString(12)

    patients = HprimForeign("PID", "header")
    orders = HprimForeign("OBR", "header")


class ClinitekSegmentPID(Segment):
    patient_identifier = HprimString(3)
    patient_name = HprimString(6)

    visits = HprimForeign("PV1", "patient")
    orders = HprimForeign("OBR", "patient")
    infos = HprimForeign("NTE", "patient")


class ClinitekSegmentNTE(Segment):
    comment_source = HprimString(3)
    comment_field = HprimString(4)
    comment_type = HprimString(5)

    visits = HprimForeign("PV1", "infos")


class ClinitekSegmentPV1(Segment):
    patient_identifier = HprimString(3)
    patient_name = HprimString(5)

    orders = HprimForeign("OBR", "patient")


class ClinitekSegmentOBR(Segment):
    filler_number = HprimString(4)
    universal_service_id = HprimString(5)
    specimen_source = HprimString((16, 1))
    # P: Patient
    # Q: QC
    specimen_role = HprimString((16, 7))
    reagent_lot_number = HprimString(21)
    reagent_lot_expiry_date = HprimDate(22)

    results = HprimForeign("OBX", "order")
    infos = HprimForeign("NTE", "order")


class ClinitekSegmentOBX(Segment):
    observation_identifier = HprimString(4)
    value = HprimString(6)
    units = HprimString(7)
    abnormal_flags = HprimString(9)
    result_status = HprimString(12)
    observer = HprimString(17)
    date = HprimDate(20)
    infos = HprimForeign("NTE", "result")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "clinitek",
        "label": "Clinitek Status Siemens",
    }
    SEGMENT_CLASS_MAP = {
        "MSH": ClinitekSegmentMSH,
        "PID": ClinitekSegmentPID,
        "PV1": ClinitekSegmentPV1,
        "NTE": ClinitekSegmentNTE,
        "OBR": ClinitekSegmentOBR,
        "OBX": ClinitekSegmentOBX,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def decode_message(self, message):
        if message.message_type != "ORU":
            raise NotImplementedError

        worklist = Worklist(context="result")

        # Résultats avec présence des segments PID et PV1
        if message.patients:
            for patient in message.patients:
                if patient.infos:
                    seg_pv1 = patient.infos[-1].visits
                else:
                    seg_pv1 = patient.visits
                for seg_obr in seg_pv1[0].orders:
                    if seg_obr.specimen_role == "P":
                        sample = worklist.get_or_create(
                            sample_id=patient.patient_name,
                            type="patient"
                        )
                    for seg_obx in seg_obr.results:
                        comments = []
                        for seg_nte in seg_obx.infos:
                            if ":" in seg_nte.comment_field:
                                comments.append(seg_nte.comment_field.split(":")[1])
                            else:
                                comments.append(seg_nte.comment_field)
                        sample.results.add(
                            code=seg_obx.observation_identifier,
                            value=seg_obx.value,
                            flags=[seg_obx.abnormal_flags] if seg_obx.abnormal_flags else [],
                            comments=comments,
                            operator=seg_obx.observer,
                            date=seg_obx.date,
                        )
        # Pour les CQ Pas de segments PID et PV1
        else:
            for seg_obr in message.orders:
                if seg_obr.specimen_role == "Q":
                    sample = worklist.get_or_create(
                        control_lot=seg_obr.universal_service_id,
                        sample_id=seg_obr.universal_service_id,
                        type="control"
                    )
                elif seg_obr.specimen_role == "P":
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.universal_service_id,
                        type="patient"
                    )
                for seg_obx in seg_obr.results:
                    sample.results.add(
                        code=seg_obx.observation_identifier,
                        value=seg_obx.value,
                        flags=[seg_obx.abnormal_flags] if seg_obx.abnormal_flags else [],
                        operator=seg_obx.observer,
                        date=seg_obx.date,
                    )
        return worklist
