# -*- coding: utf8 -*-
from __future__ import unicode_literals

import xml.etree.cElementTree as ET

from driverng import Message
from driverng.utils import get_date, format_date
from driverng.exceptions import ParsingError


class Driver(Message):
    SPEC = {
        "code": "cytovision",
        "label": "cytovision XML",
    }
    ENCODING = "utf8"

    def parse(self, data):
        # supp des caractéres indesirables en debut
        xml = data.encode("utf8")
        try:
            deb = xml.find('<?xml')
        except Exception:
            deb = 3

        if deb > 0:
            xml = xml[deb:]
        try:
            return ET.fromstring(xml)
        except Exception:
            raise ParsingError(xml[:10])

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message):
        header = '<?xml version="1.0" encoding="UTF-8"?>'
        return header + ET.tostring(message).decode()

    def setCasedetail(self, casedetailset, detail, valeur, ctrltype='Text'):
        casedetail = ET.SubElement(casedetailset, "CaseDetail")
        casedetail.set("title", detail)
        casedetail.set("editable", "True")
        casedetail.set("mandatory", "False")
        casedetail.set("ctrltype", ctrltype)
        casedetail.set("type", "system")
        casedetail.set("text", valeur)

    def get_order_message(self, worklist, context):
        root = ET.Element("CWS")
        n_order = 0
        for sample in worklist.samples:
            n_order += 1
            case = ET.SubElement(root, "Case")
            case.set("name", sample["sample_id"])
            casedetail = ET.SubElement(case, "CaseDetailsSet")
            self.setCasedetail(casedetail, "Date du dossier", format_date(sample["collection_date"], fmt="%d/%m/%Y"), ctrltype='Date')
            self.setCasedetail(casedetail, "Nom", sample["last_name"])
            self.setCasedetail(casedetail, "Nom de naissance", sample["maiden_name"])
            self.setCasedetail(casedetail, "Prenom", sample["first_name"])
            self.setCasedetail(casedetail, "Date de naissance", format_date(sample["birthdate"], fmt="%d/%m/%Y"), ctrltype='Date')
            order_codes = {}
            for rg, order in enumerate(sample["orders"]):
                # Les suppressions ne sont pas gérées
                if order["action"] == "del":
                    continue
                codes = order['code'].split(',')
                valeur = ''
                value_label = ''
                if len(codes) > 1:
                    ctype = codes[1]
                    if ctype == 'Number':
                        valeur = '0'
                else:
                    ctype = 'Text'
                try:
                    rang = int(codes[2])
                except Exception:
                    rang = rg + 1

                vals = [r for r in sample.results if not r.anterior and r.code == order['code']]
                if vals and vals[0]:
                    valeur = ctype == 'Text' and vals[0].value_label or vals[0].value
                    value_label = vals[0].value_label
                order_codes[codes[0]] = {'code': codes[0], 'valeur': valeur, 'value_label': value_label, 'ctrltype': ctype, 'rang': rang}
                n_order += 1
            for r in [r for r in sample.results if not r.anterior and r.code not in order_codes.keys()]:
                codes = r.code.split(',')
                if len(codes) > 1:
                    ctype = codes[1]
                else:
                    ctype = 'Text'
                try:
                    rang = int(codes[2])
                except Exception:
                    rang = 0
                order_codes[codes[0]] = {'code': codes[0], 'valeur': r.value, 'value_label': r.value_label, 'ctrltype': ctype, 'rang': rang}
                n_order += 1
            for key, order in sorted(order_codes.items(), key=lambda y: y[1]['rang']):
                value = order['valeur']
                if order['ctrltype'] == 'Date' and order['valeur']:
                    date = get_date(order['valeur'])
                    if date:
                        value = format_date(date, fmt="%d/%m/%Y")
                if order['ctrltype'] == 'Text' and order['value_label']:
                    value = order['value_label']
                self.setCasedetail(casedetail, key, valeur=value, ctrltype=order['ctrltype'])
        if n_order:
            return root

    def get_result_dict(self, message):
        results = {}
        cws = message.tag
        if cws == "CWS":
            case = message.find("Case")
            if case is None:
                return results
            sample_id = case.get('name')
            result = results.setdefault(
                sample_id, {"resultats": []})["resultats"]

            for detail in case.find('CaseDetailsSet').findall('CaseDetail'):
                test = detail.get('title').strip()
                if test not in ['Nom', 'Nom de naissance', 'Prenom', 'Date de naissance']:
                    value = detail.get('text').strip()
                    if value:
                        result.append({
                            "test": test,
                            "resu": value,
                        })
            flag = case.find('Flag')
            if flag is not None:
                resu = flag.get('title').strip()
            if resu:
                result.append({
                    "test": 'Flag',
                    "resu": resu,
                })

        return results
