# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class D10SegmentH(HeaderSegment):
    sender_id = HprimMultiple(
        5,
        system_name=HprimString(1),
        instrument_name=HprimString(2),
    )
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class D10SegmentP(Segment):
    orders = HprimForeign("O", "patient")


class D10SegmentQ(Segment):
    sample_id = HprimString((3, 2))
    test_order = HprimString((5, 4))
    begdate = HprimDate(7)
    enddate = HprimDate(8)


class D10SegmentO(Segment):

    sample_id = HprimString(3)
    instrument_id = HprimString(4)
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        test_id=HprimString(4),
    ))
    report = HprimString(28)
    results = HprimForeign("R", "order")


class D10SegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        peak_name=HprimString(4),
        component=HprimString(5),
    )
    value = HprimString(4)
    date_completed = HprimDate(13)


class D10SegmentL(Segment):
    code = HprimString(3, default="N")
# N - normal termination.
# F - last request for information processed.
# I - no information available from last query.
# E - unknown internal error
# Q - error in last request for information.


class Driver(AstmMessageV2):
    SPEC = {
        "code": "biorad_d10",
        "label": "Biorad / D10",
    }
    SEGMENT_CLASS_MAP = {
        "H": D10SegmentH,
        "P": D10SegmentP,
        "O": D10SegmentO,
        "Q": D10SegmentQ,
        "R": D10SegmentR,
        "L": D10SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option("qc_prefix", label="Préfixes QC (séparés par une virgule)"),
    ]

    def __init__(self, qc_prefix="", **kwargs):
        self.qc_prefix = [e for e in qc_prefix.split(",") if e]
        if not self.qc_prefix:
            self.qc_prefix = ["QC", "HC", "LC"]
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = D10SegmentH(type_segment="H", **kwargs)
        message.end = D10SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        # ignorer les suppressions car non gérées par le D10
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE},
        ):
            seg_p = message.patients.add()
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                )
                for order in sample.orders:
                    test = seg_o.universal_test_id.add()
                    test.test_id = order.code

        if message.patients:
            return message
        elif context["query"]:
            message.end.code = "I"
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id.strip(),
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                qc = False
                for p in self.qc_prefix:
                    if seg_o.sample_id.startswith(p):
                        qc = True
                if qc:
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=seg_o.sample_id,
                        control_lot=seg_o.sample_id,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.sample_id,
                    )
                for seg_r in seg_o.results:
                    sample.results.add(
                        code="%s%s" % (
                            seg_r.universal_test_id.peak_name,
                            seg_r.universal_test_id.component,
                        ),
                        value=seg_r.value,
                        date=seg_r.date_completed
                    )
        return worklist
