# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimList,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class DianaSegmentH(HeaderSegment):
    emetteur = HprimString(5)
    destinataire = HprimString(10)
    processing = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class DianaSegmentP(Segment):
    ref_patient = HprimString(3)
    patient_name = HprimString((6, 1))
    prenom = HprimString((6, 2))
    jeune_fille = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sexe = HprimString(9)
    orders = HprimForeign("O", "patient")


class DianaSegmentO(Segment):
    sample_id = HprimString(3)
    test_code = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))
    urgence = HprimString(6, default='R')
    action = HprimString(12, default='A')
    report_type = HprimString(26, default='O')
    results = HprimForeign("R", "order")


class DianaSegmentQ(Segment):
    specimen_id = HprimString((3, 1))
    tests = HprimString((5, 1), default="ALL")
    request_status = HprimString(13)


class DianaSegmentR(Segment):
    test = HprimString((3, 4))
    status = HprimString((3, 8))
    resu = HprimString(4)
    status = HprimList(9)
    date = HprimDate(13)


class DianaSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "diana",
        "label": "Diana",
    }
    SEGMENT_CLASS_MAP = {
        "H": DianaSegmentH,
        "P": DianaSegmentP,
        "Q": DianaSegmentQ,
        "O": DianaSegmentO,
        "R": DianaSegmentR,
        "L": DianaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = DianaSegmentH(type_segment="H", **kwargs)
        message.end = DianaSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.specimen_id.strip())
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
                unique_together=["action"],
                action={"del": worklist.IGNORE, worklist.ELSE: "A"},
        ):
            seg_p = msg.patients.add(
                ref_patient=patient["patient_id"],
                patient_name=patient["last_name"],
                prenom=patient["first_name"],
                jeune_fille=patient["maiden_name"],
                birthdate=patient["birthdate"],
                sexe=patient["sex"],
            )
            for sample, action in patient["samples"]:
                for order in sample["orders"]:
                    seg_o = seg_p.orders.add(
                        sample_id=sample["sample_id"],
                    )
                    t = seg_o.test_code.add()
                    t.code = order["code"]

        if msg.patients:
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                tube = seg_o.sample_id.strip()
                resultats = results.setdefault(
                    tube,
                    {"resultats": []},
                )["resultats"]
                for seg_r in seg_o.results:
                    resultats.append({
                        "test": seg_r.test,
                        "resu": seg_r.resu,
                        "date": seg_r.date,
                    })
        return results
