# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Message
from driverng.utils import get_date, format_date


class Field(object):

    def __init__(self, name, default=None):
        self.name = name
        self.default = default
        super(Field, self).__init__()

    def get_default(self):
        return self.default

    def to_python(self):
        raise NotImplementedError

    def to_string(self):
        raise NotImplementedError


class String(Field):
    def __init__(self, name, default="", max_length=1):
        self.max_length = max_length
        super(String, self).__init__(name, default=default)

    def to_python(self, splitted):
        return splitted[1:], splitted[0]

    def to_string(self, value):
        fmt = "%%.%ds" % (self.max_length,)
        return fmt % (value,)


class Date(Field):
    def __init__(self, name, default=None):
        self.name = name
        self.fmt = "%S%M%H%d%m%y"
        super(Date, self).__init__(name, default=default)

    def to_python(self, splitted):
        return splitted[1:], get_date(
            splitted[0],
            fmt=self.fmt,
        )

    def to_string(self, value):
        return format_date(value, fmt=self.fmt)


class MultipleContainer(list):

    def __init__(self, fields):
        self.fields = fields
        super(MultipleContainer, self).__init__()

    def add(self, **kwargs):
        obj = DimensionBase(fields=self.fields, **kwargs)
        self.append(obj)
        return obj


class Multiple(Field):
    def __init__(self, name, fields):
        self.fields = fields
        super(Multiple, self).__init__(name)

    def get_default(self):
        return MultipleContainer(self.fields)

    def to_python(self, splitted, number):
        values = MultipleContainer(self.fields)
        i = 0
        while i < number:
            i += 1
            obj = values.add()
            splitted = obj.parse(splitted=splitted)
        return splitted, values

    def to_string(self, values):
        return DimensionBase.SEPARATOR.join([v.to_string() for v in values])


class DimensionBase(object):
    SEPARATOR = "\x1c"  # <FS>
    FIELDS = ()

    def __init__(self, **kwargs):
        self._fields = kwargs.get("fields", self.FIELDS)
        for field in self._fields:
            setattr(self, field.name,
                    kwargs.get(field.name, field.get_default()))
        super(DimensionBase, self).__init__()

    def parse(self, data=None, splitted=None):
        if splitted is None:
            splitted = data.split(self.SEPARATOR)
        for field in self._fields:
            if isinstance(field, Multiple):
                number = getattr(self, "_%s" % (field.name,))
                splitted, value = field.to_python(splitted, int(number))
            else:
                splitted, value = field.to_python(splitted)
            setattr(self, field.name, value)
        return splitted

    def to_string(self):
        values = []
        for field in self._fields:
            value = getattr(self, field.name)
            if isinstance(field, Multiple):
                setattr(self, "_%s" % (field.name,), len(value))
        for field in self._fields:
            value = getattr(self, field.name)
            values.append(field.to_string(value))
        return self.SEPARATOR.join(values)


class Poll(DimensionBase):

    FIELDS = (
        # (name, max_length, default)
        String("type", default="P"),
        # This field is set by the user and allows each instrument to be
        # uniquely identified, distinguishing it from others in
        # multi-instrument configuration
        String("id", max_length=5, default="MCA"),
        # 1: Initial poll
        # 0: Conversational poll
        String("first_poll", max_length=1),
        # 1: Send a request
        # 0: Busy send no request
        String("request", max_length=1),
        # 0-99
        # This field was previously used with legacy Dimension products but is
        # no longer supported by Siemmens. For all instruments, this field is
        # set to 0
        String("carriers", max_length=2, default="0"),
    )


class Request(DimensionBase):

    FIELDS = (
        String("type", default="D"),
        # This field was previously used with legacy Dimension products, but is
        # no longer supported by Siemens. For all instrument this field should
        # be set to 0
        String("sample_carrier_id", default="0"),
        String("loadlist_id", max_length=2, default="0"),
        # A: Add sample
        # D: Delete
        String("transaction"),
        String("patient_id", max_length=27),
        String("sample", max_length=12),
        # 1 to 9 or W
        # W: Whole blood
        # 1: Serum
        # 2: Plasma
        # 3: Urine
        # 4: CSF
        # 5: Serum QC1
        # 6: Serum QC2
        # 7: Serum QC3
        # 8: Urine QC1
        # 9: Urine QC2
        # Serum QC3 is used for CSF QC
        String("sample_type"),
        String("location", max_length=6),
        # 0: Routine
        # 1: STAT
        # 2: ASAP
        # 3: QC
        # 4: XQC (Used for crossover QC)
        String("priority"),
        String("_cups"),
        Multiple("cups", (
            # 0 or "**"
            String("cup_position", max_length=2),
            # 1-100
            # A dilution of 0 is automatically converted to 1 by the instrument
            String("dilution", max_length=3, default="1"),
            # 1 to 36 (instrument limit)
            String("_tests", max_length=2),
            Multiple("tests", (
                String("test_name", max_length=5),
            )),
        )),
    )


class NoRequest(DimensionBase):
    FIELDS = (
        String("type", default="N"),
    )


class RequestAcceptance(DimensionBase):
    FIELDS = (
        String("type", default="M"),
        # A: Accept
        # R: Reject
        String("status"),
        # Empty: Accept
        # 1-9: Reject
        # 1: Request in process
        # 2: Result no longer available
        # 3: Sample carrier in use
        # 4: No memory to store request
        # 5: Error in test request
        # 6: Reserved
        # 7: Sample carrier full
        # 8: No known carriers
        # 9: Incorrect fluid type
        String("reason", max_length=2),
        # A or 0
        String("sample_carrier_id"),
        String("cups"),
        # 0-60
        String("sample_position", max_length=2),
    )


class Query(DimensionBase):
    FIELDS = (
        String("type", default="I"),
        String("sample_id", max_length=12),
    )


class ResultAcceptance(DimensionBase):
    FIELDS = (
        String("type", default="M"),
        # A: Accept
        # R: Reject
        String("status"),
        # Empty: Accept
        # 1: Not accepted by computer
        String("reason", max_length=2),
    )


class Result(DimensionBase):
    FIELDS = (
        String("type", default="R"),
        String("loadlist_id", max_length=2, default="0"),
        String("patient_id", max_length=27),
        String("sample", max_length=12),
        # 1 to 9 or W
        String("sample_type"),
        String("location", max_length=6),
        String("priority"),
        Date("date"),
        String("foo"),  # ???
        String("bar"),  # ???
        String("_tests", max_length=2),
        Multiple("tests", (
            String("test_name", max_length=5),
            String("test_result", max_length=10),
            String("units", max_length=10),
            String("error_code", max_length=2),
        )),
    )


class Calibration(DimensionBase):
    FIELDS = (
        String("type", default="C"),
        String("test_name", max_length=5),
        String("units", max_length=10),
        String("lot_number", max_length=10),
        String("calibrator", max_length=10),
        String("calibrator_lot_number", max_length=10),
        String("operator", max_length=10),
        Date("date"),
        String("slope", max_length=10),
        String("intercept", max_length=10),
        String("_coefficients"),
        Multiple("coefficients", (
            String("coefficient", max_length=10),
        )),
        String("_bottles"),
        Multiple("bottles", (
            String("bootle_value", max_length=10),
            String("_results"),
            Multiple("results", (
                String("result", max_length=10),
            )),
        )),
    )


class DimensionMessage(list):
    (DIMENSION_REQUEST, DIMENSION_NO_REQUEST, DIMENSION_QUERY,
        DIMENSION_RESULT, DIMENSION_CALIBRATION) = list(range(5))

    MESSAGE_TYPE_MAP = {
        DIMENSION_REQUEST: Request,
        DIMENSION_NO_REQUEST: NoRequest,
        DIMENSION_QUERY: Query,
        DIMENSION_RESULT: Result,
        DIMENSION_CALIBRATION: Calibration,
    }
    TYPE_MESSAGE_MAP = {
        "D": Request,
        "N": NoRequest,
        "I": Query,
        "R": Result,
        "C": Calibration,
    }

    def add(self, message_type, **kwargs):
        msg = self.MESSAGE_TYPE_MAP[message_type](**kwargs)
        self.append(msg)
        return msg

    def parse(self, data):
        for line in data.split("\n"):
            obj = self.TYPE_MESSAGE_MAP[line[0]]()
            obj.parse(line)
            self.append(obj)
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(Message):
    SPEC = {
        "code": "dimension",
        "label": "Dimension (AR, XL, RXL, XPAND, EXL)",
    }

    def create(self):
        return DimensionMessage()

    def get_message_type(self, message):
        for m in message:
            if isinstance(m, (Poll, Query)):
                return self.QUERY
        return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        return DimensionMessage().parse(data)

    def get_queries(self, message):
        queries = []
        for msg in message:
            if isinstance(msg, Query):
                queries.append(msg.sample_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create()
        for sample, action, priority in worklist.get_worklist(
            unique_together=["action", "priority"],
            action={"del": "D", worklist.ELSE: "A"},
            priority={"S": "1", "A": "2", worklist.ELSE: "0"},
        ):
            request = msg.add(
                msg.DIMENSION_REQUEST,
                transaction=action,
                sample_type=sample["specimen"],
                patient_id="%s %s" % (sample["last_name"], sample["first_name"]),
                sample=sample["sample_id"],
                priority=priority,
            )
            cup = request.cups.add()
            for order in sample["orders"]:
                cup.tests.add(test_name=order["code"])

        if len(msg):
            return msg
        elif context["query"]:
            msg.add(msg.DIMENSION_NO_REQUEST)
            return msg

    def get_result_dict(self, message):
        results = {}
        for msg in message:
            if msg.type != "R":
                return {}

            if msg.priority in ("3", "4"):  # QC
                return {}
            results.setdefault(msg.sample, {
                "resultats": [],
                "rack": msg.location,
            })

            for test in msg.tests:
                alarmes = []
                if test.error_code:
                    alarmes.append(test.error_code)
                results[msg.sample]["resultats"].append({
                    "test": test.test_name,
                    "resu": test.test_result,
                    "date": msg.date,
                    "alarmes": alarmes,
                })

        return results

    def get_qc_result_list(self, message):
        results = []
        for msg in message:
            if msg.type != "R":
                continue

            if msg.priority not in ("3", "4"):  # Non QC
                continue

            if msg.sample_type in ("5", "6", "7"):
                type_prlv = "1"  # Serum
            elif msg.sample_type in ("8", "9"):
                type_prlv = "3"  # Urine
            else:
                type_prlv = None
            for test in msg.tests:
                results.append({
                    "lot": msg.sample,
                    "test": test.test_name,
                    "resu": test.test_result,
                    "date": msg.date,
                    "type_prlv": type_prlv,
                })

        return results
