# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    HprimString,
)
from driverng.drivers.dm.base import DMDriverBase, DMSegmentH, DMSegmentQ, DMSegmentP, DMSegmentO, DMSegmentR, DMSegmentC, DMSegmentL


class DMEXTSegmentH(DMSegmentH):
    pass


class DMEXTSegmentP(DMSegmentP):

    nature_diagnoctic_code = HprimString(27, max_length=3, default='ICD')
    diagnoctic_code = HprimString(28, max_length=20)


class DMEXTSegmentO(DMSegmentO):

    hbg_count = HprimString((22, 3))
    hct_count = HprimString((22, 4))
    mcv_count = HprimString((22, 5))
    mch_count = HprimString((22, 6))
    mchc_count = HprimString((22, 7))
    platelet_count = HprimString((22, 8))

    neut_count = HprimString((22, 14))
    lymph_count = HprimString((22, 15))
    mono_count = HprimString((22, 16))
    eos_count = HprimString((22, 17))
    bas_count = HprimString((22, 18))
    nrbc_count = HprimString((22, 19))
    other_count = HprimString((22, 20))

    end_count = HprimString((22, 25), necessary=True)


class DMEXTDriver(DMDriverBase):
    SPEC = {
        "code": "dm_extended",
        "label": "Extended CellaVision DM",
    }
    SEGMENT_CLASS_MAP = {
        "H": DMEXTSegmentH,
        "P": DMEXTSegmentP,
        "O": DMEXTSegmentO,
        "Q": DMSegmentQ,
        "R": DMSegmentR,
        "C": DMSegmentC,
        "L": DMSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def __init__(self, nombre_cellules="100", **kwargs):
        self.code_cellules_map = {
            'WBC': {'field': 'wbc_count', 'format': '%.2f'},
            'RBC': {'field': 'rbc_concentration', 'format': '%.1f'},
            'HBG': {'field': 'hbg_count', 'format': '%.1f'},
            'HCT': {'field': 'hct_count', 'format': '%.1f'},
            'MCV': {'field': 'mcv_count', 'format': '%.1f'},
            'MCH': {'field': 'mch_count', 'format': '%.1f'},
            'MCHC': {'field': 'mchc_count', 'format': '%.1f'},
            'PLT': {'field': 'platelet_count', 'format': '%.1f'},
            'NEUT': {'field': 'neut_count', 'format': '%.1f'},
            'LYMPH': {'field': 'lymph_count', 'format': '%.1f'},
            'MONO': {'field': 'mono_count', 'format': '%.1f'},
            'EOS': {'field': 'eos_count', 'format': '%.1f'},
            'BAS': {'field': 'bas_count', 'format': '%.1f'},
            'NRBC': {'field': 'nrbc_count', 'format': '%.1f'},
            'OTHER': {'field': 'other_count', 'format': '%.1f'},
        }
        super(DMEXTDriver, self).__init__(nombre_cellules=nombre_cellules, **kwargs)

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        tubes_ok = []
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "A"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                pid=patient["patient_id"],
                last_name=patient["last_name"],
                first_name=patient["first_name"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
                location=patient["location"],
            )
            for sample, action, priority in patient["samples"]:
                tests = list(set([
                    o["code"] for o in sample["orders"]
                    if o["code"] in self.code_WBC_RBC_PLT
                ]))
                if not tests:
                    continue
                tubes_ok.append(sample["sample_id"])
                seg_o = seg_p.orders.add(
                    specimen_id=sample["sample_id"],
                    priority=priority,
                    collection_date=sample["collection_date"],
                    action_code=action,
                    clinical_information=sample["diagnostic"],
                    doctor_name=sample["physician"],
                    report_type="Q",
                )
                nb_cellules = self.nombre_cellules
                for result in sample["results"]:
                    try:
                        resuNum = float(result["value"])
                    except Exception:
                        resuNum = None
                    if result["code"] in self.code_cellules_map.keys():
                        if resuNum is not None:
                            setattr(
                                seg_o,
                                self.code_cellules_map[result['code']]['field'],
                                self.code_cellules_map[result['code']]['format'] % (float(result["value"]),)
                            )
                    elif result["code"] == "NB_CELLULES":
                        nb_cellules = result["value"]
                if tests:
                    tests = sorted(tests, key=self.code_WBC_RBC_PLT.index)
                    # Ici on envoie le nombre de cellules que si WBC est présent
                    for analysis, specific in (
                        ([self.code_WBC_RBC_PLT[0]], "%s^" % (nb_cellules,)),
                        ([self.code_WBC_RBC_PLT[1]], "^"),
                        ([self.code_WBC_RBC_PLT[2]], "^"),
                        (self.code_WBC_RBC_PLT[:2], "%s^\\^" % (nb_cellules,)),
                        (self.code_WBC_RBC_PLT[1:3], "^\\^"),
                        (self.code_WBC_RBC_PLT, "%s^\\^\\^" % (nb_cellules,)),
                    ):
                        if tests == analysis:
                            seg_o.set_raw("specific_parameters", specific)
                    code_tests = {
                        self.code_WBC_RBC_PLT[0]: 'WBC',
                        self.code_WBC_RBC_PLT[1]: 'RBC',
                        self.code_WBC_RBC_PLT[2]: 'PLT',
                    }
                    seg_o.set_raw(
                        "tests",
                        "\\".join(["^%s^^" % code_tests[code] for code in tests])
                    )

        if context["query"] is not None:
            for tube in self.get_queries(context["query"]):
                if tube not in tubes_ok:
                    seg_p = msg.patients.add()
                    seg_p.orders.add(
                        specimen_id=tube,
                        report_type="Y",
                    )

        if len(msg.patients):
            return msg
