# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimList,
)


class DmsTrackingSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    software_version = HprimString((5, 2))
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    infos = HprimForeign("M", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class DmsTrackingSegmentM(Segment):
    source = HprimString((3, 1))
    action = HprimString((3, 2))
    sample_id = HprimString(4)
    date = HprimDate(5)
    instrument_code = HprimString(6)
    code = HprimList((7, 1))
    # Description d'une erreur
    # Information sur le tube (Cap color)
    information = HprimString(8)
    user = HprimString(9)
    # Test code for host used by host when has been ordered
    test_code = HprimString(10)
    instrument_aliquoter = HprimString(11)


class DmsTrackingSegmentL(Segment):
    terminaison_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "dms_tracking",
        "label": "Dms Tracking Flexlab"
    }
    SEGMENT_CLASS_MAP = {
        "H": DmsTrackingSegmentH,
        "M": DmsTrackingSegmentM,
        "L": DmsTrackingSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "nom_chaine_dms",
            label="Nom de la chaîne DMS dans le cas de plusieurs chaines connectées",
            default="DMS"
        ),
    ]

    def __init__(self, nom_chaine_dms="DMS", **kwargs):
        self.nom_chaine_dms = nom_chaine_dms
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = DmsTrackingSegmentH(type_segment="H", **kwargs)
        message.end = DmsTrackingSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])
        for sample in worklist.samples:
            for order in sample.orders:
                if order.action == "del":
                    source = "DISABLE"
                else:
                    source = "ENABLE"
                seg_m = message.infos.add(
                    source=source,
                    sample_id=sample.sample_id,
                )
                seg_m.set_raw("date", order.code)
        if message.infos:
            return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_m in message.infos:
            sample = worklist.get_or_create(
                sample_id=seg_m.sample_id,
                type="patient",
            )
            commentaires = []
            flags = []
            if seg_m.source == "STOR":
                rack = "-".join(seg_m.code)
                sample.rack = "%s-%s" % (seg_m.instrument_code, rack)
                commentaires.append(rack)
            elif seg_m.source == "QRY":
                # Liste des tests demandés au DMS
                commentaires = seg_m.code
            elif seg_m.source == "INSTRUMENTEXCEPTION" and seg_m.information:
                # Description error exception ou de warning
                comments = "%s %s" % (seg_m.information, seg_m.test_code,)
                commentaires.append(comments)
            elif seg_m.information:
                commentaires.append(seg_m.information)

            if seg_m.source not in ("QRY", "STOR"):
                flags.extend(seg_m.code)
            if seg_m.source == "ALIQINFO":
                if self.nom_chaine_dms:
                    instrument = "%s_%s" % (
                        self.nom_chaine_dms,
                        seg_m.instrument_aliquoter,
                    )
                else:
                    instrument = seg_m.instrument_aliquoter
            elif self.nom_chaine_dms and seg_m.source != "ALIQINFO" and seg_m.instrument_code:
                instrument = "%s_%s" % (
                    self.nom_chaine_dms,
                    seg_m.instrument_code,
                )
            else:
                instrument = seg_m.instrument_code
            if seg_m.source == "CKIN":
                date_plateau = seg_m.date
            else:
                date_plateau = None
            sample.results.add(
                code="TRACKING",
                value=seg_m.source,
                flags=flags,
                comments=commentaires,
                date=seg_m.date,
                instrument=instrument,
                date_plateau=date_plateau,
            )
        return worklist
