# -*- coding: utf8 -*-
from __future__ import unicode_literals

from unidecode import unidecode

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimString,
    HprimDate,
    HprimForeign,
)


class DxiSegmentH(HeaderSegment):
    sender_name = HprimString((5, 1))
    # Identifiant paramétrable
    sender_id = HprimString((5, 2))
    receiver_id = HprimString((10, 1), default="LIS")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class DxiSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    attending_physician = HprimString(14)

    orders = HprimForeign("O", "patient")


class DxiSegmentO(Segment):
    specimen_id = HprimString(3)
    rack = HprimString((4, 2))
    position = HprimString((4, 3))
    test_code = HprimString((5, 4))
    replicate_test = HprimString((5, 5))
    # S: STAT
    # R: Routine
    # A: As soon as possible
    priority = HprimString(6)
    # N: New test
    # A: Add
    # C: Cancel
    action_code = HprimString(12)
    specimen_type = HprimString((16, 1))
    # F: Final results
    # X: Results cannot be generated, request cancelled
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class DxiSegmentR(Segment):
    test = HprimString((3, 4))
    replicate_test = HprimString((3, 5))
    value = HprimString((4, 1))
    interpretation = HprimString((4, 2))
    units = HprimString(5)
    reference_ranges = HprimString((6, 1))
    reference_type = HprimString((6, 2))
    # L: Below normal reference low limit
    # H: Above normal reference high limit
    # LL: Below panic low
    # HH: Above panic high
    # >: Out of range high
    # N: Normal result
    # <: Under for positive slope assays
    abnormal_flags = HprimList(7)
    nature_abnormality = HprimString(8)
    # F: Final results
    # X: Results cannot be generated, request is not honored
    result_status = HprimString(9)
    date_completed = HprimDate(13)
    reagent_lot = HprimString((15, 3))
    reagent_serial_number = HprimString((15, 4))


class DxiSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimList(4)
    comment_type = HprimString(5)


class DxiSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    universal_test_id = HprimString(5)
    request_status_code = HprimString(13)


class DxiSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "dxi",
        "label": "UniCel DxI 800"
    }
    SEGMENT_CLASS_MAP = {
        "H": DxiSegmentH,
        "P": DxiSegmentP,
        "O": DxiSegmentO,
        "R": DxiSegmentR,
        "C": DxiSegmentC,
        "Q": DxiSegmentQ,
        "L": DxiSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = DxiSegmentH(type_segment="H", **kwargs)
        message.end = DxiSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=unidecode(patient.last_name),
                first_name=unidecode(patient.first_name),
                maiden_name=unidecode(patient.maiden_name),
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_code=order.code,
                        priority=priority,
                        action_code=action,
                        specimen_type=sample.specimen,
                    )
                    for physician in sample.physicians:
                        seg_p.attending_physician = physician.code
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.specimen_id.startswith("CQ"):
                        sample = worklist.get_or_create(
                            control_lot=seg_o.specimen_id,
                            sample_id=seg_o.specimen_id,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            type="patient"
                        )
                    if seg_o.rack and seg_o.position:
                        sample.rack = "%s-%s" % (
                            seg_o.rack,
                            seg_o.position,
                        )
                    for seg_r in seg_o.results:
                        alarmes = seg_r.abnormal_flags[:]
                        commentaires = []
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_type == "I":
                                alarmes.extend(seg_c.comment_text)
                            elif seg_c.comment_type == "G":
                                commentaires.extend(seg_c.comment_text)
                        # lots réactifs
                        if seg_r.reagent_lot and seg_r.reagent_serial_number:
                            lot = "%s-%s" % (
                                seg_r.reagent_lot,
                                seg_r.reagent_serial_number,
                            )
                        else:
                            lot = None
                        # Résultat quantitatif
                        if seg_r.value:
                            result = sample.results.add(
                                code=seg_r.test,
                                value=seg_r.value,
                                flags=alarmes,
                                comments=commentaires,
                                date=seg_r.date_completed,
                            )
                            if lot:
                                result.lots.add(
                                    type="reagent",
                                    lot=lot,
                                )
                        # Résultat interprété
                        if seg_r.interpretation:
                            code_test = "%sI" % (seg_r.test,)
                            result = sample.results.add(
                                code=code_test,
                                value=seg_r.interpretation,
                                flags=alarmes,
                                comments=commentaires,
                                date=seg_r.date_completed,
                            )
                            if lot:
                                result.lots.add(
                                    type="reagent",
                                    lot=lot,
                                )
        return worklist
