# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)

from driverng.exceptions import ParametrageError


class SegmentY(Segment):
    END_LINE_WITH_SEPARATOR = True


class EasylinkSegmentH(HeaderSegment):

    END_LINE_WITH_SEPARATOR = True
    sender = HprimString(5, max_length=40)
    receiver = HprimString(10, max_length=40)

    # P - Production
    # Q - Quality Control
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="DB 100102")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class EasylinkSegmentP(SegmentY):
    END_LINE_WITH_SEPARATOR = True
    practice_assigned_id = HprimString(3, max_length=20)
    lab_assigned_id = HprimString(4, max_length=20)
    last_name = HprimString((6, 1), max_length=20)
    first_name = HprimString((6, 2), max_length=20)
    initial = HprimString((6, 3), max_length=1)
    suffix = HprimString((6, 4), max_length=5)

    birthdate = HprimDate(8, fmt="%Y%m%d")
    # M: Male
    # F: Female
    # U: Unknown
    sex = HprimString(9)

    height = HprimString(17)
    weight = HprimString(18)

    # Admission date et discharge date
    date = HprimList(24, basefield=HprimDate(0, fmt="%Y%m%d"))

    location_code = HprimString((26, 1), max_length=10)
    location_text = HprimString((26, 2), max_length=40)

    orders = HprimForeign("O", "patient")


class EasylinkTestField(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "code": HprimString(4, max_length=10, required=True),
            # 0401: Dimension RxL, ExL
            # 2202: Dimension Vista
            "instrument_type": HprimString(5, max_length=10),
            # Ignored from LIS
            "description": HprimString(6, max_length=10),
        })
        super(EasylinkTestField, self).__init__(*position, **kwargs)


class EasylinkSegmentO(SegmentY):
    SAMPLE_TYPES = ["S", "C", "P", "U", "W", "WB", "O"]

    specimen_id = HprimString(3, max_length=30, required=True)
    instrument_specimen_id = HprimString(4)

    # Liste des codes tests
    # ^^^Test^^Description
    tests = HprimList(5, basefield=EasylinkTestField(0))

    # Priority
    # S: stat
    # A: ASAP
    # R: Routine
    priority = HprimString(6, max_length=1)

    # Requested/Ordered Date and time
    date_requested = HprimDate(7)

    collection_date = HprimDate(8)
    collection_end_time = HprimDate(9)
    collection_volume = HprimString(10)

    # N: Rerun
    # A: Add
    action_code = HprimString(12, required=True)

    date_specimen_recevied = HprimDate(15)

    # S: Serum
    # C: CSF
    # P: Plasma
    # U: Urine
    # W: Whole blood
    # WB: Whole blood
    # O: Other
    # Note: O is not supported on downloads but will be returned
    # on uploads when not otherwise specified by the instrument
    sample_type = HprimString((16, 1), required=True)

    # Empty/Nil: Patient Result
    # QCn: Where n represents QC level
    # CALn: Where n represent Calibrator level
    sample_source = HprimString((16, 2))

    # Defaults to 1 (No dilution)
    sample_dilution = HprimString(19, max_length=6)

    total_number_of_tests = HprimString(20, max_length=3)

    results_reported_date = HprimDate(23)

    # O: Order (supported from host)
    # F: Final
    # P: Preliminaire
    # Q: Reponse to query (supported from host)
    report_type = HprimString(26, required=True)

    location_or_ward = HprimString(28)

    results = HprimForeign("R", "order")

    def to_string(self, separators=None):
        self.total_number_of_tests = len(self.tests)
        return super(EasylinkSegmentO, self).to_string(separators=separators)


class EasylinkSegmentR(SegmentY):

    END_LINE_WITH_SEPARATOR = True
    test = EasylinkTestField(3)
    value = HprimString(4, max_length=20)
    units = HprimString(5, max_length=20)
    reference_ranges = HprimString(6, max_length=20)

    # Pour le vista c'est envoyé dans les segments C
    # abnormal_flags = HprimString(7, max_length=20)

    status = HprimString(9)
    operator = HprimString(11, max_length=20)
    date_started = HprimDate(12)
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


class EasylinkSegmentC(SegmentY):
    END_LINE_WITH_SEPARATOR = True
    # I: Instrument
    source = HprimString(3, default="I")
    code = HprimString((4, 1), max_length=6)
    text = HprimString((4, 2), max_length=35)
    comment_type = HprimString(5)


class EasylinkSegmentL(SegmentY):
    # Termination code
    # N: Normal
    # R: Receiver requested abort
    # E: Unknown Error
    # Q: Error in request information
    # I: No information available
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "easylink",
        "label": "Easylink ASTM",
    }
    SEGMENT_CLASS_MAP = {
        "H": EasylinkSegmentH,
        "P": EasylinkSegmentP,
        "O": EasylinkSegmentO,
        "C": EasylinkSegmentC,
        "R": EasylinkSegmentR,
        "L": EasylinkSegmentL,
    }
    DEFAULT_SEPARATORS = r"|\^&"

    OPTIONS = [
        AstmMessageV2.Option("easylink_sender", label="Code emetteur", default="EL"),
        AstmMessageV2.Option("easylink_receiver", label="Code recepteur", default="SIL"),
    ]

    def __init__(self, easylink_sender="EL", easylink_receiver="SIL", **kwargs):
        self.easylink_sender = easylink_sender
        self.easylink_receiver = easylink_receiver
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = EasylinkSegmentH(type_segment="H", **kwargs)
        message.end = EasylinkSegmentL(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(
            context="order",
            sender_id=message.sender,
            receiver_id=message.receiver,
        )
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_o.specimen_id,
                    request_id=seg_o.specimen_id,
                    patient_id=seg_p.lab_assigned_id or "IPP" + seg_o.specimen_id,
                    last_name=seg_p.last_name,
                    first_name=seg_p.first_name,
                    birthdate=seg_p.birthdate,
                    sex=seg_p.sex,
                    height=seg_p.height,
                    weight=seg_p.weight,
                    admission_date=seg_p.date[0] if seg_p.date else None,
                    location=seg_p.location_code,
                    location_name=seg_p.location_text,
                    collection_date=seg_o.collection_date,
                    specimen=seg_o.sample_type,
                )
                for test in seg_o.tests:
                    sample.orders.get_or_create(
                        code=test.code + seg_o.sample_type,
                        priority=seg_o.priority,
                        action="add",
                    )

        return worklist

    def encode_worklist(self, worklist, context):
        if worklist.context != "result":
            raise NotImplementedError("context %s" % (worklist.context,))

        message = self.create(
            sender=self.easylink_sender,
            receiver=self.easylink_receiver,
            date=context["now"],
        )

        worklist_sample_types = list(set([s.type for s in worklist.samples]))
        if worklist_sample_types == ["patient"]:
            message.processing_id = "P"
        elif worklist_sample_types == ["control"]:
            message.processing_id = "Q"
        else:
            raise NotImplementedError("Multiple message types {}".format(worklist_sample_types))

        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            priority={"S": "S", "A": "A", worklist.ELSE: "R"},
        ):
            seg_p = message.patients.add(
                last_name=patient.last_name,
                first_name=patient.first_name,
                lab_assigned_id=patient.lab_patient_id or patient.patient_id,
                birthdate=patient.birthdate,
                sex=patient.sex if patient.sex in ("M", "F") else "U",
                height=patient.height,
                weight=patient.weight,
                location_code=patient.location,
                location_text=patient.location_name,
            )
            for sample, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority or "R",
                    action_code="N",
                    collection_date=sample.collection_date,
                    collection_end_time=sample.collection_end_date,
                    report_type="F",
                )

                if sample.specimen in seg_o.SAMPLE_TYPES:
                    seg_o.sample_type = sample.specimen

                if sample.type == "control" and sample.control_level:
                    seg_o.sample_source = "QC" + sample.control_level

                for result in sample.results:
                    if (
                        result.code[-1] in seg_o.SAMPLE_TYPES and
                        result.code[:-1].isdigit()
                    ):
                        # Code sous la forme ID + Specimen type
                        # On envoie que l'ID
                        code = result.code[:-1]
                        if (
                            seg_o.sample_type and
                            seg_o.sample_type != result.code[-1]
                        ):
                            raise ParametrageError(
                                "Le résultat (code {}, type {}) et l'échantillon "
                                "({}, type {}) n'ont pas le même type de "
                                "prélèvement. Impossible de transmettre ce "
                                "résultat.".format(
                                    result.code, result.code[-1],
                                    seg_o.specimen_id, seg_o.sample_type,
                                )
                            )
                        seg_o.sample_type = result.code[-1]
                    else:
                        code = result.code

                    if isinstance(result.value, basestring):
                        values = [{"value": result.value}]
                    elif isinstance(result.value, dict):
                        values = [result.value]
                    elif isinstance(result.value, list):
                        values = result.value
                    else:
                        raise NotImplementedError

                    if len(values) != 1:
                        raise NotImplementedError

                    value = values[0]

                    test = seg_o.tests.add()
                    test.code = code
                    test.instrument_type = "2202"
                    if result.operator and (not result.validator or result.validator == 'auto'):
                        operator = result.operator
                    elif result.validator:
                        operator = result.validator
                    else:
                        operator = 'auto'
                    seg_r = seg_o.results.add(
                        value=value["value"],
                        date_completed=result.date,
                        instrument_id=result.instrument,
                        units=result.unit,
                        operator=operator,
                        status="F",
                        reference_ranges=result.reference_ranges,
                    )
                    seg_r.test.code = code
                    seg_r.test.instrument_type = "2202"

                    for flag in result.flags:
                        seg_r.commentaires.add(
                            code=flag,
                            comment_type="E",
                        )

                    for lot in result.lots:
                        lot_code, name_code, expiration_code = {
                            "reagent": ("R001", None, "R002"),
                            "control": ("Q001", "Q004", "Q002"),
                            "calibration": ("C002", "C001", "C003"),
                        }[lot.type]
                        seg_r.commentaires.add(
                            code=lot_code,
                            text=lot.lot,
                            comment_type=lot_code[0],
                        )
                        if lot.name and name_code is not None:
                            seg_r.commentaires.add(
                                code=name_code,
                                text=lot.name,
                                comment_type=name_code[0],
                            )
                        if lot.expiration and expiration_code is not None:
                            seg_r.commentaires.add(
                                code=expiration_code,
                                text=lot.expiration.strftime("%Y%m%d%H%M"),
                                comment_type=expiration_code[0],
                            )
        return message
