# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class EliteSegmentH(HeaderSegment):

    sender_id = HprimString((5, 1), default='MCA')
    sender_instrument = HprimString((5, 2))
    sender_name = HprimString((5, 3))
    processing_id = HprimString(12, default='P')
    version = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class EliteSegmentQ(Segment):
    starting_range_id = HprimString((3, 2))


class EliteSegmentP(Segment):

    patient_id = HprimString(3)
    sample_id = HprimString(4)
    patient_ext_id = HprimString(5)
    name = HprimString(6)
    nomjf = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    physician = HprimString(14)
    orders = HprimForeign("O", "patient")


class EliteSegmentO(Segment):

    specimen_id = HprimString((3, 1))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))

    # S: Stat
    # R: Routine
    priority = HprimString(6, default='R')
    # O order, F final result
    report_type = HprimString(26, default='O')

    results = HprimForeign("R", "order")


class EliteSegmentR(Segment):

    test = HprimString((3, 4))
    test_name = HprimString((3, 5))
    value = HprimString(4)
    units = HprimString(5)
    # F: Final
    status = HprimString(9)
    date = HprimDate(13)
    instrument = HprimString(14)


class EliteSegmentC(Segment):
    # C|1|I|XXXXXX non precise dans le protocole

    source = HprimString(3)
    commentaire = HprimString(4)


class EliteSegmentL(Segment):

    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "elite_astm",
        "label": "Elite ingenius ASTM",
    }
    SEGMENT_CLASS_MAP = {
        "H": EliteSegmentH,
        "Q": EliteSegmentQ,
        "P": EliteSegmentP,
        "O": EliteSegmentO,
        "R": EliteSegmentR,
        "C": EliteSegmentC,
        "L": EliteSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.Option("qc_prefix", label="Préfixe QC"),
    ]

    def __init__(self, qc_prefix=None, **kwargs):
        self.qc_prefix = qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = EliteSegmentH(type_segment="H", **kwargs)
        message.end = EliteSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE},
        ):
            for sample, priority in patient.samples:
                seg_p = msg.patients.add(
                    patient_id=sample.sample_id,
                    sample_id=sample.sample_id,
                    name="%s %s" % (patient.last_name, patient.first_name,),
                    birthdate=patient.birthdate,
                    sex=patient.sex,

                )
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        priority=priority,
                    )
                    test = seg_o.tests.add()
                    test.code = order.code

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id.strip(),
                )
            return worklist
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample_id = seg_o.specimen_id or seg_p.sample_id
                control = self.qc_prefix and sample_id.startswith(self.qc_prefix)
                sample = worklist.get_or_create(
                    sample_id=sample_id,
                    type="control" if control else "patient",
                )
                if sample.type == "control":
                    sample.control_lot = sample_id
                for seg_r in seg_o.results:
                    # uniquement les resultats avec le statut F
                    if seg_r.status != "F":
                        continue
                    tests = seg_r.value.split('/')
                    for t in tests:
                        try:
                            code, val = t.split(':')
                        except Exception:
                            continue
                        sample.results.add(
                            code=code,
                            date=seg_r.date,
                            value=val,
                            instrument=seg_r.instrument,
                        )
        return worklist
