# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class EngenSegmentH(HeaderSegment):
    sending_application = HprimString(5)
    version_driver = HprimString(11)
    processing_id = HprimString(12, default="P")
    version_number = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    infos = HprimForeign("M", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class EngenSegmentP(Segment):
    patient_id = HprimString(3)
    ssn = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    ordering_physician = HprimMultiple(
        14,
        code=HprimString(1),
        last_name=HprimString(2),
        first_name=HprimString(3),
        middle_initial=HprimString(4),
    )
    practice_field = HprimString(23)
    patient_status = HprimString(25)
    patient_location = HprimMultiple(
        26,
        facility=HprimString(1),
        ward=HprimString(2),
        room=HprimString(3),
        bed=HprimString(4),
        nurses_station=HprimString(5),
    )

    orders = HprimForeign("O", "patient")


class EngenSegmentO(Segment):
    specimen_id = HprimString((3, 1))
    sequence_number = HprimString((4, 1))
    rack = HprimString((4, 2))
    cup = HprimString((4, 3))
    tests = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            code=HprimString(4),
            dilution=HprimString(5),
        ))

    # R: Routine
    # S: Stat
    priority = HprimString(6)
    collection_date_field1 = HprimDate(8)
    # A: Add ordered tests
    # N or O: overwrite all existing tests with new tests
    # C or D: cancel tests on existing order
    # Q: QC result
    # Blank: Patient result
    action_code = HprimString(12)
    collection_date_field2 = HprimDate(15)
    specimen_type = HprimString((16, 1))
    user_field = HprimString((19, 1))
    destination_connection = HprimString(25)

    results = HprimForeign("R", "order")


class EngenSegmentR(Segment):
    test_code = HprimString((3, 4))
    result = HprimString(4)
    units = HprimString(5)
    reference_range = HprimString(6)
    error_flags = HprimList(7)
    result_status = HprimString(9)
    operator_id = HprimMultiple(
        11,
        test_operator_id=HprimString(1),
        test_results=HprimString(2),
        specimen_results=HprimString(3),
    )
    result_date = HprimDate(13)
    instrument_id = HprimString(14)


class EngenSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class EngenSegmentM(Segment):
    # Segment non décrit dans le protocole de communication mais pouvant être
    # transmis selon le paramétrage de l'appareil
    code = HprimString(3)
    tests = HprimList(4)
    sample_id = HprimString(6)


class EngenSegmentL(Segment):
    flag = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "engen",
        "label": "Engen Ortho"
    }
    SEGMENT_CLASS_MAP = {
        "H": EngenSegmentH,
        "P": EngenSegmentP,
        "O": EngenSegmentO,
        "R": EngenSegmentR,
        "C": EngenSegmentC,
        "M": EngenSegmentM,
        "L": EngenSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = EngenSegmentH(type_segment="H", **kwargs)
        message.end = EngenSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "N"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    collection_date_field1=sample.collection_date,
                    collection_date_field2=sample.collection_date,
                    specimen_type=sample.specimen,
                )
                for physician in sample.physicians:
                    seg_p.ordering_physician.code = physician.code
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order["code"]
                    t.dilution = order["dilution"]

        if len(msg.patients):
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # QC
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        control_lot=seg_o.specimen_id,
                        sample_id=seg_o.specimen_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="patient",
                    )
                if seg_o.rack and seg_o.cup:
                    sample.rack = "%s-%s" % (
                        seg_o.rack,
                        seg_o.cup,
                    )
                for seg_r in seg_o.results:
                    comments = []
                    if seg_r.result_status[0] not in ('V', 'F'):
                        continue
                    flags = seg_r.error_flags[:]
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type == "G":
                            comments.append(seg_c.comment_text)
                        elif seg_c.comment_type == "I":
                            flags.append(seg_c.comment_text)
                    r = sample.results.add(
                        code=seg_r.test_code,
                        value=seg_r.result,
                        flags=[e.strip() for e in flags if e.strip()],
                        comments=comments,
                        date=seg_r.result_date,
                        instrument=seg_r.instrument_id,
                    )
                    if sample.type == 'patient' and seg_r.operator_id.test_operator_id:
                        r.operator = seg_r.operator_id.test_operator_id.strip()
        return worklist
