# -*- coding: utf8 -*-
from __future__ import unicode_literals

import re

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Multiple, Date


class EurolinescanBase(Splitted):
    SEPARATOR = ";"


class EurolinescanOrder(EurolinescanBase):
    FIELDS = (
        String("code_test"),
        String("id_patient"),
        String("nom_patient"),
        Date("date_naissance", fmt_date="%d/%m/%Y"),
        Date("date_echantillon", fmt_date="%d/%m/%Y"),
    )


class EurolinescanResult(EurolinescanBase):
    FIELDS = (
        String("type_resultat"),
        String("id_patient"),
        String("nom_patient"),
        Date("date_naissance", fmt_date="%d/%m/%Y"),
        String("no_labo"),
        String("no_bande"),
        String("no_lot_kit"),
        String("no_batch"),
        String("expediteur"),
        String("resultat_qualitatif"),
        Multiple("results", (
            String("code_antigene"),
            String("resu"),
        ), separator=" "),
    )


class EurolinescanMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: EurolinescanOrder,
        MessageV2.RESULT: EurolinescanResult,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not len(line):
                continue
            if len(line.split(";")) == 5:
                type_message = MessageV2.ORDER
            else:
                type_message = MessageV2.RESULT
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "eurolinescan",
        "label": "EuroLineScan",
    }

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: EurolinescanOrder,
        MessageV2.RESULT: EurolinescanResult,
    }

    def create(self):
        return EurolinescanMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = EurolinescanMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) == 0:
            return None
        elif len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = samples[0][0]
            nom_patient = "%s %s" % (sample.last_name, sample.first_name,)
            for order in sample.orders:
                msg.add(
                    type_message=MessageV2.ORDER,
                    code_test=order.code,
                    id_patient=sample.sample_id,
                    nom_patient=nom_patient,
                    date_naissance=sample.birthdate,
                    date_echantillon=sample.request_date,
                )
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample = worklist.get_or_create(
                sample_id=m.id_patient,
                type="patient",
            )
            for result in m.results:
                # un resultat est composé ainsi:
                # <resultat_qualitatif>[<resultat_quantitatif>]
                r = re.compile(r"^([^\[]+)(\[([^\]]+)\])?$")
                res = r.match(result.resu)
                resu = res.groups()
                if len(resu) == 3:
                    # resultat qualitatif
                    sample.results.add(
                        code="%s%sQUAL" % (m.type_resultat, result.code_antigene,),
                        value=resu[0],
                    )
                    # resultat quantitatif
                    sample.results.add(
                        code="%s%sQUANT" % (m.type_resultat, result.code_antigene,),
                        value=resu[2],
                    )
        return worklist
