# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class Gem4000SegmentH(HeaderSegment):
    sender = HprimMultiple(
        5,
        analyser_label=HprimString(1),
        analyser_version=HprimString(2),
        analyser_area_name=HprimString(3),
        analyser_name=HprimString(4),
        analyser_model=HprimString(5),
        analyser_serie=HprimString(6),
        analyser_serie_card=HprimString(7),
        analyser_sw_version=HprimString(6)
    )
    proccessing = HprimString(12, default='P')
    version = HprimString(13, default='LIS2-A')
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    infos = HprimForeign("M", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Gem4000SegmentP(Segment):
    patient_id = HprimString(3)
    patient_lab_id = HprimString(4)
    alternate_patient_identifier = HprimString(5)
    last_name = HprimString((6, 1), max_length=24, trunc=True)
    first_name = HprimString((6, 2), max_length=24, trunc=True)
    midle_name = HprimString((6, 3), max_length=1, trunc=True)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class Gem4000SegmentO(Segment):
    specimen_id = HprimString(3)
    instrument_specimen_id = HprimString(4)
    collection_date = HprimDate(8)
    sample = HprimMultiple(
        16,
        sample_type=HprimString(1),
        nlot=HprimString(2),
        nlot_descriptor=HprimString(3),
        lot_level=HprimString(4)
    )

    results = HprimForeign("R", "order")


class Gem4000SegmentR(Segment):
    universal_test_id = HprimString((3, 4))
    result = HprimString(4)
    units = HprimString(5)
    reference_range = HprimList(6)
    abnormal_flags = HprimList(7)
    operator_identification = HprimString((11, 1))
    date_test_completed = HprimDate(13)  # Attention seulement le 1er resultat
    analyser = HprimString(14)


class Gem4000SegmentC(Segment):
    comment_source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))
    comment_value = HprimString((4, 3))
    comment_unit = HprimString((4, 4))
    comment_type = HprimString(5)


class Gem4000SegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    specimen_type = HprimString((3, 4))
    request_info = HprimString(13)


class Gem4000SegmentM(Segment):
    report_type = HprimString(3)
    report_date = HprimString(4)
    report_lang = HprimString(5)


class Gem4000SegmentL(Segment):
    pass


class Driver(AstmMessageV2):
    SPEC = {
        "code": "gem4000",
        "label": "Gem4000/Gem5000 ASTM (native mode)"
    }
    SEGMENT_CLASS_MAP = {
        "H": Gem4000SegmentH,
        "P": Gem4000SegmentP,
        "O": Gem4000SegmentO,
        "R": Gem4000SegmentR,
        "C": Gem4000SegmentC,
        "Q": Gem4000SegmentQ,
        "M": Gem4000SegmentM,
        "L": Gem4000SegmentL,
    }
    DEFAULT_SEPARATORS = "|@^&"

    def create(self, **kwargs):
        message = Gem4000SegmentH(type_segment="H", **kwargs)
        message.end = Gem4000SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        if not context["query"] or not context["query"].queries:
            raise NotImplementedError("Pas de chargement sans requette ,context %s" % (worklist.context,))
        msg = self.create(
            message_id=context["now"],
            date=context["now"]
        )
        msg.queries = context["query"].queries
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={'del': worklist.IGNORE, worklist.ELSE: ""},
        ):
            msg.patients.add(
                patient_id=patient.patient_id,
                patient_lab_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )

        if not msg.patients:
            msg.queries[0].request_info = 'X'
        else:
            msg.queries[0].request_info = 'F'
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                if seg_q.specimen_type == 'SPECIMENID':
                    worklist.get_or_create(
                        type="patient",
                        sample_id=seg_q.specimen_id
                    )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:

                for seg_o in seg_p.orders:
                    if seg_o.sample.nlot_descriptor[:7] == "GEM CVP":
                        sample = worklist.get_or_create(
                            type="control",
                            control_lot=seg_o.sample.nlot,
                            sample_id=seg_o.sample.nlot,
                            control_level=seg_o.sample.lot_level,
                        )
                    else:
                        sample = worklist.get_or_create(
                            type="patient",
                            sample_id=seg_o.instrument_specimen_id,
                        )

                    date = None
                    operator = None

                    for seg_r in seg_o.results:
                        test = seg_r.universal_test_id

                        if seg_r.operator_identification:
                            operator = seg_r.operator_identification

                        if seg_r.date_test_completed:
                            date = seg_r.date_test_completed

                        flags = set(seg_r.abnormal_flags[:])
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_code:
                                flags.add(seg_c.comment_code)

                        r = sample.results.add(
                            code=test,
                            value=seg_r.result,
                            flags=sorted(flags),
                            date=date,
                            instrument=seg_r.analyser or message.sender.analyser_name,
                        )
                        if sample.type == 'patient' and operator:
                            r.operator = operator

                    if sample.type == "patient" and sample.results:

                        comments = []
                        for seg_c in seg_o.commentaires:
                            if seg_c.comment_code == 'COMMENT' and seg_c.comment_text:
                                comments.append(seg_c.comment_text)
                        if comments:
                            sample.results.add(
                                code='COMMENT',
                                value=";".join(comments),
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
                        if operator:
                            sample.results.add(
                                code='OPE',
                                value=operator,
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
                        if seg_o.sample.sample_type:
                            sample.results.add(
                                code='PRELEV',
                                value=seg_o.sample.sample_type,
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
        return worklist
