# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.worklist import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class Ha8180SegmentH(HeaderSegment):
    sender_name = HprimMultiple(
        5,
        instrument_name=HprimString(1),
        instrument_no=HprimString(2),
        version=HprimString(3),
        surplus=HprimString(4, default="    "),
    )
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class Ha8180SegmentP(Segment):
    orders = HprimForeign("O", "patient")


class Ha8180SegmentO(Segment):
    patient_id = HprimMultiple(
        3,
        barcode_id=HprimString(1),
        # !! C'est a l'envers dans la doc !!
        rack_id=HprimString(2),
        port_no=HprimString(3),
    )
    measurement_number = HprimString(4)
    measurement = HprimList(5, basefield=HprimMultiple(
        0,
        # code demande par défaut HbA1c
        measure=HprimString(4),
    ))
    # R: Normal sample
    # S: STAT
    # C: Control
    priority = HprimString(6)
    sample_type = HprimString(16)
    additional_comment = HprimMultiple(
        20,
        measurement_error=HprimString(1, default="00000000"),
        reserve=HprimString(2, default="00000000"),
        degradation_info=HprimString(3),
        mode=HprimString(4),
        eluant_a_lot=HprimString(5),
        eluant_a_date=HprimString(6),
        eluant_b_lot=HprimString(7),
        eluant_b_date=HprimString(8),
        eluant_cv_lot=HprimString(9),
        eluant_cv_date=HprimString(10),
        hemorisys_lot=HprimString(11),
        hemorisys_date=HprimString(12),
        calibrator_lot=HprimString(13),
        calibrator_date=HprimString(14),
    )
    results = HprimForeign("R", "order")

    def get_sample_id(self):
        return self.patient_id.barcode_id.replace("-", "")


class Ha8180SegmentR(Segment):
    test = HprimString((3, 4))
    value = HprimMultiple(
        4,
        **dict(("value%d" % i, HprimString(i)) for i in range(1, 31))
    )
    unit = HprimMultiple(
        5,
        unit1=HprimString(1),
        unit2=HprimString(2),
        unit3=HprimString(3),
    )
    abnormal_flag = HprimString(7)
    result_status = HprimString(9)
    date = HprimDate(12)


class Ha8180SegmentQ(Segment):
    patient_id = HprimMultiple(
        3,
        barcode_id=HprimString(1),
        rack_id=HprimString(2),
        port_no=HprimString(3),
    )
    query_code = HprimString(13)

    def get_sample_id(self):
        return self.patient_id.barcode_id.replace("-", "")


class Ha8180SegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "ha8180",
        "label": "Ha8180 en direct"
    }
    SEGMENT_CLASS_MAP = {
        "H": Ha8180SegmentH,
        "P": Ha8180SegmentP,
        "O": Ha8180SegmentO,
        "Q": Ha8180SegmentQ,
        "R": Ha8180SegmentR,
        "L": Ha8180SegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = Ha8180SegmentH(type_segment="H", **kwargs)
        message.end = Ha8180SegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        message = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            seg_p = message.patients.add()
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(priority=priority)
                seg_o.patient_id.barcode_id = sample.sample_id
                if context["query"] is not None:
                    for seg_q in context["query"].queries:
                        if seg_q.get_sample_id() == sample.sample_id:
                            seg_o.patient_id.barcode_id = seg_q.patient_id.barcode_id
                            seg_o.patient_id.port_no = seg_q.patient_id.port_no
                            seg_o.patient_id.rack_id = seg_q.patient_id.rack_id
                for order in sample.orders:
                    test = seg_o.measurement.add()
                    test.measure = order.code

        if message.patients:
            return message
        elif context["query"]:
            for seg_q in context["query"].queries:
                seg_p = message.patients.add()
                seg_o = seg_p.orders.add(
                    priority="R",
                    sample_type="NULL",
                )
                seg_o.patient_id.barcode_id = seg_q.patient_id.barcode_id
                seg_o.patient_id.port_no = seg_q.patient_id.port_no
                seg_o.patient_id.rack_id = seg_q.patient_id.rack_id
                seg_o.measurement.measure = "NULL"
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.get_sample_id())
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample_id = seg_o.get_sample_id()
                    if seg_o.priority == "C":
                        sample = worklist.get_or_create(
                            type="control",
                            sample_id=sample_id,
                            control_lot=sample_id,
                        )
                    else:
                        sample = worklist.get_or_create(type="patient", sample_id=sample_id)

                    if seg_o.patient_id.port_no and seg_o.patient_id.rack_id:
                        sample.rack = "%s-%s" % (
                            seg_o.patient_id.rack_id,
                            seg_o.patient_id.port_no,
                        )

                    lots = []
                    for label, attr in (
                        ("ELUANT 80A", "eluant_a"),
                        ("ELUANT 80B", "eluant_b"),
                        ("ELUANT 80CV", "eluant_cv"),
                        ("SOLUTION HEMOLYSANTE 80H", "hemorisys"),
                        ("CALIBRATOR KIT 80", "calibrator"),
                    ):
                        lot = getattr(seg_o.additional_comment, attr + "_lot")
                        peremption = getattr(seg_o.additional_comment, attr + "_date") or ""
                        if lot:
                            lots.append({
                                "type": "reagent",
                                "lot": "%s: %s %s" % (label, lot, peremption),
                            })

                    date = None

                    # pic -> occurrence
                    # None correspond au pics non nommés (il commencent à 1)
                    # Pour les pics nommés la première occurence n'est pas
                    # suffixés et le suffixe des autres commencent à 2.
                    multiples = {None: 1}
                    for seg_r in seg_o.results:
                        if seg_r.date is not None:
                            date = seg_r.date

                        if (
                            seg_r.result_status != "F" or
                            seg_r.test[:4] == "Time" or
                            seg_r.test[:7] == "ADCount" or
                            seg_r.test in ("ValueGain",)
                        ):
                            continue

                        if seg_r.test[:4] == "Peak":
                            if seg_r.test == "Peak":
                                code = "Peak%d" % (multiples[None],)
                                multiples[None] += 1
                            else:
                                multiples.setdefault(seg_r.test, 1)
                                if multiples[seg_r.test] > 1:
                                    code = "%s%d" % (seg_r.test, multiples[seg_r.test])
                                else:
                                    code = seg_r.test
                                multiples[seg_r.test] += 1
                            for suffix, attr in (
                                ("sec", "value1"),
                                ("count", "value2"),
                                ("%", "value3"),
                            ):
                                sample.results.add(
                                    code="%s_%s" % (code, suffix),
                                    value=getattr(seg_r.value, attr),
                                    date=seg_r.date or date,
                                )
                        else:
                            result = sample.results.add(
                                code=seg_r.test,
                                value=seg_r.value.value1,
                                date=seg_r.date or date,
                            )
                            if seg_r.abnormal_flag:
                                result.flags = [seg_r.abnormal_flag]
                            if sample.type == "patient":
                                result.lots = lots
                    if sample.type == "patient":
                        self.set_graphes(sample, seg_o)
                        sample.results.add(
                            code="SAMPLE_TYPE",
                            value=seg_o.sample_type or "WB",
                            date=date,
                        )
                        sample.results.add(
                            code="MEASUREMENT_NUMBER",
                            value=seg_o.measurement_number,
                            date=date,
                        )
                        sample.results.add(
                            code="DEGRADATION_INFO",
                            value=seg_o.additional_comment.degradation_info,
                            date=date,
                        )
                        errors = [int(e, 16) for e in seg_o.additional_comment.measurement_error]
                        values = []
                        for index, flags in {
                            0: ("A1c_R.T fast", "L-A1c tail", None, None),
                            1: ("A1c tail", "A0_R.T", "A1c_R.T move", "A1c_R.T late"),
                            2: ("Noise", "Large A0", "Little Area", "Drift"),
                            3: ("Low Level", "Peak Count", "No A1c Peak", "Duplex Peak"),
                            4: ("Temp", "Lamp", "High Press", "High Conc"),
                            5: (None, None, "Press move", "Low Flow"),
                            6: ("R.T", "A0 bottom", "Peak D", "Peak E"),
                        }.items():
                            for i, flag in enumerate(flags):
                                if errors[index] & (1 << i):
                                    values.append(flags[i])
                        if values:
                            sample.results.add(
                                code="HB_ERR",
                                value=";".join(sorted(values)),
                                flags=sorted(values),
                                date=date,
                            )

        return worklist

    def set_graphes(self, sample, seg_o):
        points = []
        peaks = []
        date = None
        for seg_r in seg_o.results:
            if seg_r.date is not None:
                date = seg_r.date
            if seg_r.result_status != "F":
                continue
            if seg_r.test[:4] == "Peak":
                peaks.append({
                    "name": seg_r.test[4:],
                    # abscisse du sommet du pic (arrondi à la seconde).
                    "top": int(seg_r.value.value1) * 10,
                    "area": float(seg_r.value.value2),
                    "area_percent": float(seg_r.value.value3),
                })
            elif seg_r.test[:7] == "ADCount":
                for i in range(1, 31):
                    point = getattr(seg_r.value, "value%d" % i)
                    points.append(int((point), 16))

        # On supprime les points à 0 à la fin du graphe
        while points[-20:] == 20 * [0]:
            points.pop()

        if points:
            for peak in peaks:
                # Le top est un arrondi à la seconde et le pas du graphe est
                # d'un dixième de seconde. Du coup on obtient une précision
                # plus grande en regardant le max sur l'intervalle de 10 points
                # autour de l'arrondi du top
                portion = points[peak["top"] - 5:peak["top"] + 5]
                if portion:
                    peak["top"] -= (5 - portion.index(max(portion)))

            peaks = sorted(peaks, cmp=lambda a, b: a["top"] > b["top"])
            for i, peak in enumerate(peaks):
                # Ici on cherche à determiner le début et la fin du pic On
                # regarde le minimum entre le sommet du pic et les pics
                # précédent et suivant.
                if i == 0:
                    peak["start"] = 0

                if i == len(peaks) - 1:
                    peak["end"] = len(points)
                else:
                    portion = points[peak["top"]:peaks[i + 1]["top"]]
                    peaks[i + 1]["start"] = peak["end"] = portion.index(min(portion)) + peak["top"]

            sample.results.add(
                code="ADCount",
                value="GRAPHE",
                graph={"graph_type": "area", "points": points},
                date=date,
            )
            sample.results.add(
                code="GRAPHE_A1C",
                value="GRAPHE",
                graph={"graph_type": "a1c", "peaks": peaks, "points": points},
                date=date,
            )
