# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class HeliosSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class HeliosSegmentP(Segment):
    patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))

    orders = HprimForeign("O", "patient")


class HeliosSegmentO(Segment):
    specimen_id = HprimString(3)
    # un segment O par test
    test_id = HprimString((5, 4))
    dilution = HprimString((5, 5))
    priority = HprimString(6, default="R")

    results = HprimForeign("R", "order")


class HeliosSegmentR(Segment):
    test = HprimString((3, 4))
    dilution = HprimString((3, 5))
    value = HprimString((4, 1))
    aspect1 = HprimString((4, 2))
    aspect2 = HprimString((4, 3))
    aspect3 = HprimString((4, 4))
    status = HprimString(9)
    operator = HprimString(11)
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


class HeliosSegmentC(Segment):
    comment_source = HprimString(3)
    comment_code = HprimString((4, 1))
    comment_dilution = HprimString((4, 2))
    comment_text = HprimString((4, 3))
    comment_type = HprimString(5)


class HeliosSegmentQ(Segment):
    starting_range_id = HprimString((3, 2))
    universal_test_id = HprimString(5)


class HeliosSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "helios",
        "label": "HELIOS AESKU INGEN"
    }
    SEGMENT_CLASS_MAP = {
        "H": HeliosSegmentH,
        "P": HeliosSegmentP,
        "O": HeliosSegmentO,
        "R": HeliosSegmentR,
        "C": HeliosSegmentC,
        "Q": HeliosSegmentQ,
        "L": HeliosSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = HeliosSegmentH(type_segment="H", **kwargs)
        message.end = HeliosSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            for sample, action in patient.samples:
                seg_p = msg.patients.add(
                    patient_id=sample.sample_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                )
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_id=order["code"],
                        dilution=order["dilution"] if order["dilution"] else ""
                    )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.starting_range_id
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    for seg_r in seg_o.results:
                        comments = []
                        for seg_c in seg_r.commentaires:
                            # Ne traiter que les commentaires de type "T"
                            if seg_c.comment_type in ('C', 'I'):
                                continue
                            if seg_c.comment_code or seg_c.comment_dilution or seg_c.comment_text:
                                commentaires = "{}^{}^{}".format(
                                    seg_c.comment_code,
                                    seg_c.comment_dilution,
                                    seg_c.comment_text
                                )
                                comments.append(commentaires)
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id,
                            type="patient",
                        )
                        # Resultat par dilution
                        if seg_r.value.strip():
                            value = seg_r.value
                            if seg_r.aspect1:
                                value = "{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip()
                                )
                            if seg_r.aspect2:
                                value = "{}^{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip(),
                                    seg_r.aspect2.strip()
                                )
                            if seg_r.aspect3:
                                value = "{}^{}^{}^{}".format(
                                    seg_r.value,
                                    seg_r.aspect1.strip(),
                                    seg_r.aspect2.strip(),
                                    seg_r.aspect3.strip()
                                )
                            sample.results.add(
                                code="{}_RESULT_{}".format(seg_r.test, seg_r.dilution),
                                value=value,
                                comments=comments,
                                date=seg_r.date_completed,
                                dilution=seg_r.dilution,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 1
                        if seg_r.aspect1:
                            sample.results.add(
                                code="{}_ASPECT1".format(seg_r.test),
                                value=seg_r.aspect1,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 2
                        if seg_r.aspect2:
                            sample.results.add(
                                code="{}_ASPECT2".format(seg_r.test),
                                value=seg_r.aspect2,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Aspect no 3
                        if seg_r.aspect3:
                            sample.results.add(
                                code="{}_ASPECT3".format(seg_r.test),
                                value=seg_r.aspect3,
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
                        # Les deux aspects
                        if seg_r.aspect1 or seg_r.aspect2:
                            sample.results.add(
                                code="{}_ASPECT".format(seg_r.test),
                                value="{}^{}".format(seg_r.aspect1, seg_r.aspect2),
                                date=seg_r.date_completed,
                                operator=seg_r.operator,
                                instrument=seg_r.instrument_id,
                            )
        return worklist
