# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class HemohubSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1.0")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class HemohubSegmentP(Segment):
    patient_id = HprimString((3, 1))
    laboratory_patient_id = HprimString(4)
    patient_id_no_3 = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    description = HprimString((6, 3))
    # Flag QC dans ce champ lorsqu'il s'agit d'un QC
    flag_qc = HprimString((6, 4))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    attending_physicians = HprimString(14)
    special_field1 = HprimString(15)
    special_field2 = HprimString(16)
    patient_diagnosis = HprimString(19)
    # Order comments
    practice_field_no2 = HprimString(23)
    location = HprimString(26)
    service_id_code = HprimString((33, 1))
    description = HprimString((33, 2))
    destination_code = HprimString(34)

    orders = HprimForeign("O", "patient")


class HemohubSegmentO(Segment):
    specimen_id = HprimString((3, 1))
    reagent_lot_number = HprimString((3, 2))
    instrument_specimen_id = HprimString(4)
    test_code = HprimString((5, 4))
    test_flag = HprimString((5, 5))
    test_instrument = HprimString((5, 7))
    priority = HprimString(6)
    request_date = HprimDate(7)
    action_code = HprimString(12)
    ordering_physician = HprimString(17)
    collection_center = HprimString(19)
    # Re-run flag R: Rerun
    flag_rerun = HprimString(22)
    report_type = HprimString(26)
    service = HprimString(30)

    results = HprimForeign("R", "order")


class HemohubSegmentR(Segment):
    test = HprimString((3, 4))
    result = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimString(7)
    nature_abnormal_testing = HprimString(8)
    result_status = HprimString(9)
    date_change = HprimDate(10)
    operator = HprimString((11, 1))
    validator = HprimString((11, 2))
    instrument_id = HprimString((14, 1))
    instrument_name = HprimString((14, 2))


class HemohubSegmentC(Segment):
    comment_text = HprimString(4)


class HemohubSegmentL(Segment):
    terminator_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "hemohub",
        "label": "Hemohub Werfen"
    }

    SEGMENT_CLASS_MAP = {
        "H": HemohubSegmentH,
        "P": HemohubSegmentP,
        "O": HemohubSegmentO,
        "R": HemohubSegmentR,
        "C": HemohubSegmentC,
        "L": HemohubSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = HemohubSegmentH(type_segment="H", **kwargs)
        message.end = HemohubSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority", "rerun"],
            action={"del": "C", "add": "A", "new": "A"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                laboratory_patient_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority, rerun in patient.samples:
                for physician in sample.physicians:
                    seg_p.attending_physicians = physician.code
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_code=order.code,
                        request_date=sample.request_date,
                        action_code=action,
                        priority=priority,
                    )
                    if order.action == "add" and rerun:
                        seg_o.flag_rerun = "R"
                seg_p.location = sample.location
        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            lot_number = ""
            if seg_p.flag_qc == "QC":
                lot_number = seg_p.patient_id if seg_p.patient_id else ""
            for seg_o in seg_p.orders:
                if seg_o.action_code == "Q":
                    # CQ
                    sample = worklist.get_or_create(
                        sample_id=lot_number,
                        control_lot=lot_number,
                        control_name=seg_p.laboratory_patient_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="patient",
                    )
                for seg_r in seg_o.results:
                    # Ignorer les résultats de type X "Cancel"
                    # F: Final results
                    # X: For cancel
                    if seg_r.result_status == "X":
                        continue
                    comments = []
                    for seg_c in seg_r.commentaires:
                        comments.append(seg_c.comment_text)
                    alarms = []
                    if seg_r.abnormal_flags:
                        alarms.append(seg_r.abnormal_flags)
                    if seg_r.nature_abnormal_testing:
                        alarms.append(seg_r.nature_abnormal_testing)
                    sample.results.add(
                        code=seg_r.test,
                        value=seg_r.result,
                        flags=alarms,
                        comments=comments,
                        unit=seg_r.units,
                        date=seg_r.date_change,
                        operator=seg_r.operator,
                        validator=seg_r.validator,
                        instrument=seg_r.instrument_id,
                    )
        return worklist
