# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimPN,
    HprimList,
    HprimAD,
    HprimCNA,
    HprimFloat,
    HprimCE,
    HprimForeign,
)
from driverng.drivers.hprim_spec import HprimSegmentOBX, HprimSegmentOBR, HprimSegmentC

logger = logging.getLogger("driverng.hprim_facturation")


class HprimSegmentH(HeaderSegment):

    nom_fichier = HprimString(3, max_length=12)
    emetteur = HprimMultiple(5,
                             code=HprimString(1, obligatoire=True),
                             nom=HprimString(2, obligatoire=True),
                             )

    adresse = HprimAD(6)

    # ORM: Demande
    # ORU: Resultat
    # ADM: Admission
    # FAC: Facturation
    # REG: Reglement
    # ERR: Erreur
    type_message = HprimString(7, obligatoire=True, max_length=7)
    telephone = HprimList(8)

    recepteur = HprimMultiple(10,
                              code=HprimString(1, obligatoire=True),
                              nom=HprimString(2, obligatoire=True),
                              )

    # P: Production
    # T: Test
    # D: Debug
    mode_traitement = HprimString(12, max_length=1, default="P")
    version = HprimString((13, 1), default="H2.1")
    type_liaison = HprimString((13, 2), default="L")
    date = HprimDate(14, obligatoire=True)

    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class HprimSegmentP(Segment):

    patient_demandeur = HprimString((3, 1), max_length=16)
    patient_fusion = HprimString((3, 2))
    patient_executant = HprimString((4, 1), max_length=16)
    no_adm_cli = HprimString((5, 1))
    rang_naissance = HprimString((5, 2), max_length=2)
    nom = HprimPN(6, max_length=48)
    nomjf = HprimString(7, max_length=24)
    date_nai = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sexe = HprimString(9, max_length=1)
    adresse = HprimAD(11)
    telephone = HprimList(13)
    medecins = HprimList(14, basefield=HprimCNA(0))
    traite_lc1 = HprimString(15)
    traite_lc2 = HprimString(16)
    taille = HprimString(17)
    poids = HprimFloat(18)
    diag_pat = HprimString(19)
    diagnostic_patient = HprimList(19, basefield=HprimCE(0))
    trait_medical = HprimString(20)
    traitement_medical = HprimList(20)
    regime = HprimString(21)
    comment_dem1 = HprimString((22, 1))
    comment_dem1_libelle = HprimString((22, 2))
    comment_dem2 = HprimString(23)
    date_entree = HprimDate((24, 1), fmt_date="%Y%m%d")
    date_sortie = HprimDate((24, 2), fmt_date="%Y%m%d")
    stat_adm = HprimString(25, max_length=2)
    localisation = HprimMultiple(26, max_length=100,
                                 lit=HprimString(1),
                                 chambre=HprimString(2),
                                 corr=HprimString(3),
                                 )
    class_diag = HprimString(27)
    situ_maritale = HprimString(29)
    precaution = HprimString(30)
    langue = HprimString(31)
    stat_conf = HprimString(32)
    date_modif = HprimDate(33, fmt_date="%Y%m%d")
    date_deces = HprimDate(34, fmt_date="%Y%m%d")

    orders = HprimForeign("FAC", "billing")


class HprimSegmentFAC(Segment):
    no_adm_cli = HprimString((3, 1))
    rang_naissance = HprimString((3, 2), max_length=2)
    no_executant = HprimString(4)
    date_acte = HprimDate(5)
    prescripteur = HprimList(6, basefield=HprimMultiple(
        0,
        code_medecin=HprimString(2),
        medecin=HprimPN(3),
    ))
    preleveur = HprimList(7, basefield=HprimMultiple(
        0,
        code_preleveur=HprimString(2, max_length=9),
        preleveur=HprimString(3, max_length=40),
    ))
    # O
    # N
    # demande_non_facture = HprimString(8, max_length=1, default='N')

    results = HprimForeign('ACT', 'billing')


class HprimSegmentACT(Segment):
    lettre_cle = HprimString(3, max_length=10)
    coefficient = HprimFloat(4)
    denombrement = HprimFloat(5)
    quantite = HprimFloat(6)
    libelle_acte = HprimString(7)
    type_majoration = HprimString(8, max_length=3)
    no_executant = HprimString((9, 1), max_length=9)
    code_executant = HprimString((9, 2), max_length=9)
    nom_executant = HprimString((9, 3), max_length=40)
    code_nomenclature = HprimString((10, 1))
    catalogue = HprimString((10, 2))
    # HR
    # RX
    # R
    code_rmo = HprimString(11, max_length=5)


class HprimSegmentL(Segment):

    nombre_segment_patient = HprimString(4, max_length=4)
    nombre_segment = HprimString(5, max_length=10)
    numero_lot = HprimString(6, max_length=12)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "hprim_facturation",
        "label": "Hprim 2.1 (Facturation)",
    }
    DEFAULT_SEPARATORS = "|~^\&"

    SEGMENT_CLASS_MAP = {
        "H": HprimSegmentH,
        "P": HprimSegmentP,
        "FAC": HprimSegmentFAC,
        "ACT": HprimSegmentACT,
        "L": HprimSegmentL,
        "OBR": HprimSegmentOBR,
        "OBX": HprimSegmentOBX,
        "C": HprimSegmentC,
    }
    ADDENDUM_SEGMENT_SIZE = 220

    OPTIONS = [
        AstmMessageV2.Option("hprim_code_emetteur", label="Code emetteur", default="MCA"),
        AstmMessageV2.Option("hprim_nom_emetteur", label="Nom emetteur"),
        AstmMessageV2.Option("hprim_code_recepteur", label="Code recepteur"),
        AstmMessageV2.Option("hprim_nom_recepteur", label="Nom recepteur"),
        AstmMessageV2.BooleanOption(
            "option_comments_in_flags",
            label="Intégrer les commentaires OBX en alarme",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoi_technicien",
            label="Envoyer le code technicien transmis par l'automate au lieu du code du valideur en cas de validation automatique",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoi_sample_id",
            label="Multi SIL (OBR.9.3.2=No tube)",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_ipp_executant",
            label="Renvoyer l'IPP du patient dans le champs 8.4",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoyer_fic",
            label="Envoyer les pièces jointes de la demande dans des segments OBX",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_identification_demandeur",
            label="Identification demandeur OBR.9.19",
            default=False),
        AstmMessageV2.IntegerOption(
            "option_annulation_sans_ipp_age_max",
            label="Accepter les demandes d'annulation sans IPP ni OBR 9.3.2. Préciser l'age maximum de la demande (en jour).",
            default=0),
    ]

    def __init__(
        self, hprim_code_emetteur="MCA", hprim_code_recepteur="",
        hprim_nom_emetteur="", hprim_nom_recepteur="",
        option_comments_in_flags=False, option_envoi_technicien=False,
        option_envoi_sample_id=False, option_ipp_executant=False,
        option_envoyer_fic=False, option_identification_demandeur=False,
        option_annulation_sans_ipp_age_max=0, **kwargs
    ):
        self.code_emetteur = hprim_code_emetteur
        self.nom_emetteur = hprim_nom_emetteur or ""
        self.code_recepteur = hprim_code_recepteur or ""
        self.nom_recepteur = hprim_nom_recepteur or ""
        self.option_comments_in_flags = option_comments_in_flags
        self.option_envoi_technicien = option_envoi_technicien
        self.option_envoi_sample_id = option_envoi_sample_id
        self.option_ipp_executant = option_ipp_executant
        self.option_envoyer_fic = option_envoyer_fic
        self.option_identification_demandeur = option_identification_demandeur
        self.option_annulation_sans_ipp_age_max = option_annulation_sans_ipp_age_max
        super(Driver, self).__init__(**kwargs)

    def to_string(self, message, *args, **kwargs):
        message.end.nombre_segment_patient = len(message.patients)
        message.end.nombre_segment = len(message.get_segments())
        return super(Driver, self).to_string(message, *args, **kwargs)

    def create(self, **kwargs):
        message = HprimSegmentH(type_segment="H", **kwargs)
        message.end = HprimSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context not in ('result', 'billing', 'order'):
            raise NotImplementedError("context %s" % (worklist.context,))

        if worklist.context in ('result', 'order'):
            # Utiliser l'hprim normal
            raise NotImplementedError("context %s" % (worklist.context,))

        msg = self.create(
            date=context["now"],
            type_message="FAC",
            nom_fichier=context.get("filename"),
        )
        msg.emetteur.code = worklist.sender_id or self.code_emetteur
        msg.emetteur.nom = worklist.sender_name or self.nom_emetteur
        msg.recepteur.code = worklist.receiver_id or self.code_recepteur
        msg.recepteur.nom = worklist.receiver_name or self.nom_recepteur

        for patient in worklist.samples:
            patient_demandeur = patient.patient_id

            seg_p = msg.patients.add(
                no_adm_cli=patient.alternate_patient_id,
                date_nai=patient.birthdate,
                sexe=patient.sex,
                nomjf=patient.maiden_name,
                date_entree=patient.admission_date,
                date_sortie=patient.admission_end_date,
                poids=patient.weight,
                taille=patient.height,
                rang_naissance=patient.birth_row,
            )
            seg_p.date_deces = patient.deathdate
            if self.option_ipp_executant:
                seg_p.patient_executant = patient_demandeur
            else:
                seg_p.patient_demandeur = patient_demandeur

            if patient.phone:
                if patient.phone == "T":
                    # Spécifique Clarilab
                    seg_p.set_raw("telephone", "~T")
                else:
                    seg_p.telephone = [p for p in patient.phone.split(";") if p]
            if patient.medications:
                seg_p.traitement_medical = patient.medications
            seg_p.nom.nom = patient.last_name
            seg_p.nom.prenom = patient.first_name
            seg_p.nom.civilite = patient.title
            seg_p.stat_adm = patient.admission_status
            seg_p.localisation.lit = patient.bed
            seg_p.localisation.chambre = patient.room
            seg_p.localisation.corr = patient.location
            seg_p.adresse.ville = patient.city
            seg_p.adresse.pays = patient.country
            seg_p.adresse.code_postal = patient.postal_code
            if patient.address:
                if "~" in patient.address:
                    seg_p.adresse.ligne1, seg_p.adresse.ligne2 = patient.address.split("~", 1)
                else:
                    seg_p.adresse.ligne1 = patient.address
            for physician in patient.physicians:
                if physician.type == "attending":
                    med = seg_p.medecins.add()
                    med.code = physician.code
                    med.nom.nom = physician.last_name
                    med.nom.prenom = physician.first_name
                    med.type = physician.codification
                    med.nom.civilite = physician.title
                    med.nom.sobriquet = physician.alias

            seg_fac = seg_p.orders.add(
                no_adm_cli=patient.sample_id,
                no_executant=patient.exe_request_id,
                date_acte=patient.request_date,
            )
            if patient.physicians:
                prescripteur = seg_fac.prescripteur.add()
                preleveur = seg_fac.preleveur.add()
                for physician in patient.physicians:
                    if physician.type == "prescriber":
                        prescripteur.code_medecin = physician.code
                        prescripteur.medecin.nom = physician.last_name
                    elif physician.type == "sampler":
                        preleveur.code_preleveur = physician.code
                        preleveur.preleveur = physician.last_name

            for billing in patient.billings:
                # if not msg.emetteur.code:
                #    msg.emetteur.code = billing.lab
                seg_fac.results.add(
                    lettre_cle=billing.lettre_cle,
                    coefficient=billing.coefficient,
                    denombrement=billing.denombrement,
                    quantite=billing.quantite,
                    libelle_acte=billing.libelle_acte,
                    type_majoration=billing.type_majoration,
                    no_executant=billing.no_executant,
                    code_executant=billing.code_executant,
                    nom_executant=billing.nom_executant,
                    code_nomenclature=billing.code_nomenclature,
                    catalogue=billing.catalogue,
                    code_rmo=billing.code_rmo,
                )
        if msg.patients:
            return msg

    def add_billing(self, worklist, sample, seg_act):
        sample.billings.get_or_create(
            lettre_cle=seg_act.lettre_cle,
            coefficient=seg_act.coefficient,
            denombrement=seg_act.denombrement,
            quantite=seg_act.quantite,
            libelle_acte=seg_act.libelle_acte,
            type_majoration=seg_act.type_majoration,
            no_executant=seg_act.no_executant,
            code_executant=seg_act.code_executant,
            nom_executant=seg_act.nom_executant,
            code_nomenclature=seg_act.code_nomenclature,
            catalogue=seg_act.catalogue,
            code_rmo=seg_act.code_rmo,
        )

    def decode_message(self, message):
        if message.type_message not in ("FAC", "ORM", "ORU"):
            raise NotImplementedError("type message %s" % (message.type_message,))

        if message.type_message in ("ORM", "ORU"):
            # Utiliser Hprim normal
            raise NotImplementedError("type message %s" % (message.type_message,))

        worklist = Worklist(
            context="billing",
            sender_id=message.emetteur.code,
            receiver_id=message.recepteur.code,
            filename=message.nom_fichier,
        )
        for seg_p in message.patients:
            if message.type_message == "FAC":
                for seg_fac in seg_p.orders:
                    sample = worklist.get_or_create(
                        sample_id=seg_fac.no_adm_cli,
                        exe_request_id=seg_fac.no_executant,
                        patient_id=seg_p.patient_demandeur or seg_p.patient_executant or seg_fac.no_demandeur,
                    )
                    if seg_p.telephone == [["", "T"]]:
                        # À telephoner clarilab...
                        phone = ""
                    else:
                        phone = ";".join([p for p in seg_p.telephone if p])
                    sample.update(
                        alternate_patient_id=seg_p.no_adm_cli,
                        last_name=seg_p.nom.nom,
                        first_name=seg_p.nom.prenom,
                        title=seg_p.nom.civilite,
                        maiden_name=seg_p.nomjf,
                        birthdate=seg_p.date_nai,
                        birth_row=seg_p.rang_naissance,
                        sex=seg_p.sexe if seg_p.sexe in ("M", "F") else "U",
                        request_date=(
                            seg_fac.date_acte or
                            message.date),
                        type="patient",
                        diagnostic=";".join([e.code1 for e in seg_p.diagnostic_patient]),
                        location=seg_p.localisation.corr,
                        room=seg_p.localisation.chambre,
                        bed=seg_p.localisation.lit,
                        collection_date=seg_fac.date_acte,
                        collection_end_date=seg_fac.date_acte,
                        admission_date=seg_p.date_entree,
                        admission_end_date=seg_p.date_sortie,
                        height=seg_p.taille,
                        phone=phone,
                        city=seg_p.adresse.ville,
                        postal_code=seg_p.adresse.code_postal,
                        country=seg_p.adresse.pays,
                        address="~".join([
                            l for l in [seg_p.adresse.ligne1, seg_p.adresse.ligne2]
                            if l
                        ]),
                        admission_status=seg_p.stat_adm,
                        deathdate=seg_p.date_deces,
                        physiologie=self.get_physiologie(seg_p),
                    )
                    if seg_p.poids:
                        sample.weight = unicode(seg_p.poids)
                    for item in [
                        seg_p.traite_lc1, seg_p.traite_lc2,
                        seg_p.comment_dem1, seg_p.comment_dem2,
                    ]:
                        if item:
                            sample.comments.get_or_create(type="patient", text=item)
                    for med in seg_p.medecins:
                        if med.code not in [p.code for p in sample.physicians]:
                            sample.physicians.get_or_create(
                                code=med.code,
                                last_name=med.nom.nom,
                                first_name=med.nom.prenom,
                            )
                    self.get_medecin_prescripteur(sample, seg_fac)

                    if seg_fac.preleveur[0].code_preleveur:
                        sample.physicians.get_or_create(
                            type="sampler",
                            code=seg_fac.preleveur[0].code_preleveur,
                            last_name=seg_fac.preleveur[0].preleveur.nom,
                        )
                self.decode_fac(seg_fac, worklist, sample)
            else:
                pass
        return worklist

    def decode_fac(self, seg_fac, worklist, sample):
        for seg_act in seg_fac.results:
            self.add_billing(worklist, sample, seg_act)

    def get_physiologie(self, seg_p):
        return ""

    def get_medecin_prescripteur(self, sample, seg_fac):
        for prescripteur in seg_fac.prescripteur:
            if (
                prescripteur.code_medecin and
                prescripteur.code_medecin not in [p.code for p in sample.physicians]
            ):
                sample.physicians.get_or_create(
                    code=prescripteur.code_medecin,
                    last_name=prescripteur.medecin.nom,
                    type="prescriber",
                )
