# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class ImmucorSegmentH(HeaderSegment):
    sender = HprimString(5, default="MCA")
    reciever = HprimString(10, default="RVT")
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", required=True)


class ImmucorSegmentP(Segment):
    patient_id = HprimString(3)
    patient_lab_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    middle_name = HprimString((6, 3))
    mother_maiden = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class ImmucorSegmentO(Segment):
    specimen_id = HprimString(3)
    test = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))
    # R
    priority = HprimString(6, default="R")
    # C: Cancel future use
    # N: New
    action_code = HprimString(12)
    sample_type = HprimString((16, 1), default="Blood")
    # Patient or Product
    # Note:Patient is used when sample is from a patient
    # Product is used when sample is from a donor unit or
    # product.
    sample_source = HprimString((16, 2), default="Patient")

    results = HprimForeign("R", "order")
    comment = HprimForeign("C", "order")
    # O Order record
    # Y No order on record for this test (in response to query)
    # Z No record of this patient (in response to query)
    report_type = HprimString(26)


class ImmucorSegmentC(Segment):
    source = HprimString(3, default="L")
    code = HprimString((4, 1))
    texte = HprimString((4, 2))


class ImmucorSegmentQ(Segment):
    sample_id = HprimString((3))
    test = HprimString(5, default="ALL")


class ImmucorSegmentR(Segment):
    analyte_code = HprimString((3, 4))
    result_original = HprimString((4, 1))
    result_edited = HprimString((4, 2))
    result_measured = HprimString((4, 3))
    flag = HprimString(7)
    # F: Final result
    status = HprimString(9)
    operator = HprimString(11)
    date = HprimDate(13)
    instrument_id = HprimString(14)


class ImmutorSegmentC(Segment):
    comment_source = HprimString(3, default="L")
    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))


class ImmucorSegmentL(Segment):
    terminaison = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "immucor",
        "label": "Immucor",
    }
    SEGMENT_CLASS_MAP = {
        "H": ImmucorSegmentH,
        "P": ImmucorSegmentP,
        "O": ImmucorSegmentO,
        "C": ImmucorSegmentC,
        "Q": ImmucorSegmentQ,
        "R": ImmucorSegmentR,
        "C": ImmucorSegmentC,
        "L": ImmucorSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = ImmucorSegmentH(type_segment="H", **kwargs)
        message.end = ImmucorSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.sample_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient["patient_id"],
                patient_lab_id=patient["lab_patient_id"],
                last_name=patient["last_name"],
                first_name=patient["first_name"],
                mother_maiden=patient["maiden_name"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
            )
            tests = set()
            for sample, _, _ in patient["samples"]:
                tests = tests.union([o["code"] for o in sample["orders"]])
            for sample, action, priority in patient["samples"]:
                for order in sample["orders"]:
                    # Si ABO0 et GrBB on ne demande pas ABOD
                    if order["code"] == "ABOD" and "GrBB" in tests:
                        continue
                    seg_o = seg_p.orders.add(
                        specimen_id=sample["sample_id"],
                        action_code=action,
                        priority=priority,
                    )
                    test = seg_o.test.add()
                    test.code = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            seg_p = msg.patients.add()
            for sample_id in self.get_queries(context["query"]):
                seg_p.orders.add(specimen_id=sample_id, report_type="Z")
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                resultats = results.setdefault(
                    seg_o.specimen_id,
                    {"resultats": [
                        {"test": "NOM", "resu": seg_p.last_name},
                        {"test": "PRENOM", "resu": seg_p.first_name},
                        {"test": "DNAIS", "resu": seg_p.birthdate.strftime("%d/%m/%Y")},
                    ]},
                )["resultats"]
                for seg_r in seg_o.results:
                    if seg_r.result_edited:
                        value = seg_r.result_edited
                    else:
                        value = seg_r.result_original
                    resultats.append({
                        "test": seg_r.analyte_code,
                        "resu": value,
                        "date": seg_r.date,
                        "operator": seg_r.operator,
                    })
        return results
