# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class ImmuliteSegmentH(HeaderSegment):

    password = HprimString(4)

    # Sender Name or ID
    emetteur = HprimString(5, default="WST")

    # Sender Telephone Number
    emetteur_tel = HprimString(8)

    # Characteristics of Sender (No Parity 8 bits 1 Stop Bit)
    emetteur_characteristics = HprimString(9)
    recepteur = HprimString(10, default="WST")
    type_message = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class ImmuliteSegmentP(Segment):
    # Patient ID
    no_patient = HprimString(3)

    # Nom du Patient
    nom = HprimString((6, 1), max_length=20)

    # Prenom du Patient
    prenom = HprimString((6, 2), max_length=15)

    # date de naissance
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")

    # sexe  M/F
    sexe = HprimString(9)

    # code medecin prescripteur
    code_prescripteur = HprimString(14)

    orders = HprimForeign("O", "patient")


class ImmuliteSegmentO(Segment):

    # L'identification des controles de qualite se fait
    # par les deux premiers caracteres du tube
    #   - First 2 characters : ~C (QC)
    #   - Next 6 characters : Control Name, if fewer than 6 characters
    #     right-padded with spaces (should not be empty)
    #   - Next 3 characters : Control Lot (use 3 numeric digits;
    #     should not be empty)
    #   - Next 6 characters : Expiration Date (format YYYYMM;
    #     should not be empty)
    #   - Last 1 character : Control Level (should not be empty)
    tube = HprimString((3, 1), max_length=30)

    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution=HprimString(5),
    ))

    # urgence - R : Routine - S : Stat (urgence)
    urgence = HprimString(6, max_length=1)

    # Requested/Ordered Date and time
    date_requested = HprimDate(7)

    # Date et heure de prelevement
    date_prelevement = HprimDate(8)

    results = HprimForeign("R", "order")


class ImmuliteSegmentQ(Segment):

    # Starting Range ID Number
    # Sample ID - Tube primaire
    tube = HprimString((3, 2))

    # Universal Test ID Valeur : ALL pour tous les tests
    test = HprimString(5, default="ALL")

    # Request information status codes
    # valeurs permises :
    #  A : Abort / Cancel last request
    #  O : Requesting orders and demographics
    request = HprimString(13)


class ImmuliteSegmentR(Segment):

    # code test
    # ^^^TEST_CODE
    test = HprimString((3, 4), max_length=15)

    # valeur du resultat
    valeur = HprimString(4)

    # unite courante de l'analyseur DPC
    unite = HprimString(5)

    # Reference Ranges
    # Low / Panic Low / High / Panic High
    ref_ranges = HprimList(
        6,
        basefield=HprimMultiple(
            0,
            low=HprimString(1),
            panic_low=HprimString(2),
        ),
    )

    # Result Abnormal Flags
    # L : Below Normal
    # H : Above Normal
    # < : Below readable limit
    # > : Above readable limit
    # N : Normal
    flag_alarm = HprimList(7)

    # Nature of Abnormality Testing
    # valeur possible  N : Normal generic range
    flag_nature = HprimString(8)

    # F: Final Results
    # R: Previously transmitted
    status = HprimString(9)

    # Operator Name ID
    operator = HprimString(11)

    # Date + Time Test Started
    date_debut_test = HprimDate(12)

    # Date + Time Test Completed
    date_result = HprimDate(13)

    # Instrument ID
    instrumentID = HprimString(14)


class ImmuliteSegmentL(Segment):
    # code de terminaison
    # N : Normal termination
    # Q : Error in last request for information (required with query)
    # I : No information available from last query (required with query)
    # F : Last request for information Processed (required with query)
    code = HprimString(3, default="F")


class Driver(AstmMessage):
    SPEC = {
        "code": "immulite",
        "label": "Immulite",
    }
    SEGMENT_CLASS_MAP = {
        "H": ImmuliteSegmentH,
        "P": ImmuliteSegmentP,
        "O": ImmuliteSegmentO,
        "Q": ImmuliteSegmentQ,
        "R": ImmuliteSegmentR,
        "L": ImmuliteSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = ImmuliteSegmentH(type_segment="H", **kwargs)
        message.end = ImmuliteSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.tube)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", "A": "A", "C": "C", "P": "P",
                      worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                no_patient=patient["patient_id"],
                nom=patient["last_name"],
                prenom=patient["first_name"],
                date_naissance=patient["birthdate"],
                sexe=patient["sex"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    tube=sample["sample_id"],
                    urgence=priority,
                    date_prelevement=sample["collection_date"],
                )
                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.code = order["code"]
                    t.dilution = order["dilution"]

        if msg.patients or context["query"]:
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # exclusion des CQ
                if "~C" in seg_o.tube[0:2]:
                    continue
                results.setdefault(seg_o.tube, {"resultats": []})

                for seg_r in seg_o.results:
                    r = {
                        "test": seg_r.test,
                        "resu": seg_r.valeur,
                        "date": seg_r.date_result,
                        "alarmes": seg_r.flag_alarm,
                    }
                    if seg_r.operator:
                        r['operator'] = seg_r.operator.strip()
                    results[seg_o.tube]["resultats"].append(r)

        return results

    def get_qc_result_list(self, message):
        results = []
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # uniquement les CQ
                if seg_o.tube[0:2] != "~C":
                    continue
                # reference du lot
                lot = seg_o.tube[2:]
                for seg_r in seg_o.results:
                    results.append({
                        "lot": lot,
                        "test": seg_r.test,
                        "resu": seg_r.valeur,
                        "date": seg_r.date_result,
                    })

        return results
