# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class ImmunocapSegmentH(HeaderSegment):

    sender_id = HprimMultiple(
        5,
        name=HprimString(1, default="MCA"),
        software_version=HprimString(2),
        uniface_version=HprimString(3),
    )

    receiver_id_hostname = HprimString((10, 1))
    processing_id = HprimString(12, default="P")
    version_number = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class ImmunocapSegmentP(Segment):
    practive_pid = HprimString(3)
    laboratory_pid = HprimString(4)
    patient_name = HprimString(6)
    mother_maiden_name = HprimString(7)
    date_birth = HprimDate(8, fmt_date="%Y%m%d")

    # M/F/U
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class ImmunocapSegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        specimen_id=HprimString(1),
        tube=HprimString(2),
        rack_id_no=HprimString(3),
        rack_position=HprimString(4),
    )
    instrument_specimen_id = HprimString(4)
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        test=HprimString(4),
        method=HprimString(5),
        dilution=HprimString(6),
        supress_reflex=HprimString(7),
    ))
    request_date = HprimDate(7)
    collection_date = HprimDate(8)

    # code action:
    # N: New test
    # Q: QC
    # A: Add
    # R: Retest
    action_code = HprimString(12)

    # nom ou ID de l'analyseur
    instrument_id = HprimString(25)
    # Report types
    #  O: Order
    #  F: Final results
    #  X: Request cannot be done
    #  I: Instrument pending
    report_type = HprimString(26, default="O")

    # A value of 1 means that the specimen is not diluted
    # if left blank, the method default dilution is assumed
    information_dilution = HprimString(14, default="1")
    results = HprimForeign("R", "order")


class ImmunocapSegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        test=HprimString(4),
        method=HprimString(5),
        dilution=HprimString(6),
    )
    data = HprimMultiple(
        4,
        concentration=HprimString(1),
        classe=HprimString(2),
        cut_off=HprimString(3),
        cut_off2=HprimString(4),
        quotient=HprimString(5),
        sensitization=HprimString(6),
        clin_relation=HprimString(7),
        comment_id=HprimString(8),
        lot_numbers=HprimString(9),
    )
    units = HprimString(5)
    # Status du resultat
    #  F: Final results
    #  P: Preliminary results
    #  I: Result pending
    #  X: Test cannot be completed
    #  M: Final result transmitted with manuel defined tests
    result_status = HprimString(9)
    operator_id = HprimString(11)
    date = HprimDate(13)
    # nom ou ID de l'analyseur
    instrument_id = HprimString(14)


class ImmunocapSegmentC(Segment):
    # source possible:
    #  I: Inventory information from specific instrument
    #  O: ImmunoCAP data manager operator software
    #  A: AllerQuest
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    # type possible:
    #  G: Generic/free text comment
    #  I: Automatic comment
    comment_type = HprimString(5)


class ImmunocapSegmentL(Segment):
    # N: Normal termination
    # F: Finished processing last request
    # I: Information not available on last request
    termination_code = HprimString(3, default="F")


class ImmunocapSegmentQ(Segment):
    starting_range_id = HprimMultiple(
        3,
        patient_id=HprimString(1),
        specimen_id=HprimString(2),
    )
    request_status_codes = HprimString(13)


class Driver(AstmMessage):
    SPEC = {
        "code": "immunocap",
        "label": "ImmunoCAP PHADIA",
    }
    SEGMENT_CLASS_MAP = {
        "H": ImmunocapSegmentH,
        "P": ImmunocapSegmentP,
        "O": ImmunocapSegmentO,
        "Q": ImmunocapSegmentQ,
        "R": ImmunocapSegmentR,
        "C": ImmunocapSegmentC,
        "L": ImmunocapSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessage.BooleanOption(
            "idm_version",
            label="Version IDM",
            default=False,
        ),
    ]

    def __init__(self, idm_version=False, **kwargs):
        self.idm_version = idm_version
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = ImmunocapSegmentH(type_segment="H", **kwargs)
        message.end = ImmunocapSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER
        return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.starting_range_id.specimen_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add(
                practive_pid=patient["patient_id"],
                patient_name="%s %s" % (
                    patient["last_name"],
                    patient["first_name"],
                ),
                mother_maiden_name=patient["maiden_name"],
                date_birth=patient["birthdate"],
                sex=patient["sex"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    report_type="O",
                    action_code=action,
                )
                seg_o.specimen_id.specimen_id = sample["sample_id"]
                for order in sample["orders"]:
                    t = seg_o.universal_test_id.add()
                    if "^" in order["code"]:
                        t.test, t.method = order["code"].split("^", 1)
                    else:
                        t.test = order["code"]
                        t.method = sample["specimen"]

        if msg.patients or context["query"]:
            return msg

    @staticmethod
    def get_resultats(obj):
        results = []
        for attr, code in (
            ("concentration", ""),
            ("classe", "CLA"),
            ("cut_off", "CUT1"),
            ("cut_off2", "CUT2"),
            ("quotient", "QUO"),
            ("sensitization", "SENSI"),
            ("clin_relation", "CLIN"),
        ):
            value = getattr(obj.data, attr)
            if value:
                results.append((
                    "%s%s" % (
                        obj.universal_test_id.test,
                        code,
                    ),
                    value,
                    obj.date,
                ))
        return results

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                # exclusion des CQ
                if seg_o.action_code == "Q":
                    continue
                if seg_o.specimen_id.rack_id_no and seg_o.specimen_id.rack_position:
                    rack = "%s-%s" % (
                        seg_o.specimen_id.rack_id_no,
                        seg_o.specimen_id.rack_position,
                    )
                else:
                    rack = None
                tube = seg_o.instrument_specimen_id
                results.setdefault(tube, {
                    "resultats": [],
                    "rack": rack,
                })
                lots_reactifs = []
                for seg_r in seg_o.results:
                    commentaires = []
                    operator = ""
                    analyseur = ""
                    if self.idm_version:
                        if seg_r.data.lot_numbers:
                            lots_reactifs.append({
                                "ref": seg_r.data.lot_numbers,
                            })
                        if seg_r.operator_id:
                            operator = seg_r.operator_id
                        if seg_r.instrument_id:
                            analyseur = seg_r.instrument_id
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type:
                            commentaires.append(seg_c.comment_text.strip())

                    for test, resu, date in self.get_resultats(
                            seg_r):
                        results[tube]["resultats"].append({
                            "test": test,
                            "resu": resu,
                            "commentaires": commentaires,
                            "date": date,
                            "analyseur": analyseur,
                            "operator": operator,
                            "lots_reactifs": lots_reactifs,
                        })
        return results

    def get_qc_result_list(self, message):
        results = []
        for seg_p in message.patients:

            for seg_o in seg_p.orders:
                # exclusion des resus non CQ
                if seg_o.action_code != "Q":
                    continue
                for seg_r in seg_o.results:
                    lot = seg_o.instrument_specimen_id.strip()
                    for seg_c in seg_r.commentaires:
                        # selon les versions le lot suit la mention Flacon QC
                        # dans le commentaire
                        if "Flacon QC" in seg_c.comment_text:
                            lot = seg_c.comment_text[9:].strip()
                    for test, resu, date in self.get_resultats(
                            seg_r):
                        results.append({
                            "lot": lot,
                            "test": test,
                            "resu": resu,
                            "date": date,
                            "analyseur": seg_r.instrument_id if seg_r.instrument_id else ""
                        })
        return results

    def enregistrer(self, data, message=None):
        # En full download on n'enregistre pas les
        # query ALL
        if data:
            lines = data.splitlines()
            if len(lines) == 3 and lines[1] == "Q|1|^ALL||||||||||O":
                return False
        return True
