# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import xml.etree.cElementTree as ET

from driverng import Message
from driverng.utils import format_date


class Driver(Message):
    SPEC = {
        "code": "iq200xml",
        "label": "Iq200 XML",
    }

    def parse(self, data):
        return ET.fromstring(data.encode("utf16"))

    def get_message_type(self, message):
        if message.tag == "SIQ":
            return self.QUERY
        else:
            return self.ORDER

    def to_string(self, message):
        header = '<?xml version="1.0" encoding="UTF-16"?>'
        return header + ET.tostring(message).decode()

    def get_order_message(self, worklist, context):
        root = ET.Element("SI")
        root.set("BF", "URN")
        root.set("WO", "run")
        has_order = False
        for sample in worklist.samples:
            root.text = sample["sample_id"]
            demog = ET.SubElement(root, "DEMOG")
            for order in sample["orders"]:
                # Les suppressions ne sont pas gérées
                if order["action"] == "del":
                    continue
                has_order = True
                demog.set("LName", sample["last_name"])
                demog.set("FName", sample["first_name"])
                demog.set("RecNum", sample["patient_id"])
                demog.set("Gender", sample["sex"])
                demog.set("DOB", format_date(sample["birthdate"], fmt="%Y-%m-%d"))
        if has_order:
            return root

    def get_queries(self, message):
        queries = []
        if message.tag == "SIQ":
            queries.append(message.text)
        return queries

    def get_result_dict(self, message):
        results = {}
        sa = message.tag
        if sa == "SA":
            sample_id = message.get('ID')
            if sample_id.startswith("UR"):
                return results
            result = results.setdefault(
                sample_id, {"resultats": []})["resultats"]
            cristal = set()
            cylindre = set()
            autres_cellules = set()
            # dilution effectuée sur l'automate
            dilution = message.get("DILD")

            for ac in message.findall('AC'):
                if ac.get('AT') == 'Sediment' or ac.get('AT') == 'Chemistry':
                    for ar in ac.findall('AR'):
                        test = ar.get('Key')
                        resu = ar.text.split(" ")
                        if test in ("HYAL", "EPIC", "WBCT", "RBCT", "GRAN"):
                            if resu[0] == "[none]":
                                cylindre.add(resu[0])
                            else:
                                cylindre.add("%s%s" % (test, resu[0]))
                        elif test in (
                            "CACB", "CAPH", "CAOX", "TPO4", "URIC", "LEUC",
                            "CYST", "TYRO", "AMOR"
                        ):
                            if resu[0] == "[none]":
                                cristal.add(resu[0])
                            else:
                                cristal.add("%s%s" % (test, resu[0]))
                        elif test in (
                            "NSE", "WBCC", "BYST", "TRCH", "SPRM"
                        ):
                            if resu[0] == "[none]":
                                autres_cellules.add(resu[0])
                            else:
                                autres_cellules.add("%s%s" % (test, resu[0]))
                        if test in ("COL", "CLA"):
                            resu = ar.text
                        else:
                            resu = resu[0]
                        result.append({
                            "test": test,
                            "resu": resu,
                            "dilution": dilution,
                        })

            for test, resu in (
                ("CYLINDRE", cylindre),
                ("CRISTAL", cristal),
                ("AUTRES", autres_cellules),
            ):
                if not len(resu):
                    continue
                elif len(resu) > 1 and "[none]" in resu:
                    # Plus d'un resultat != [none]
                    resu.remove("[none]")
                result.append({
                    "test": test,
                    "resu": ";".join(sorted(resu)),
                    "dilution": dilution,
                })
        return results

    def get_qc_result_list(self, message):
        results = []
        cq = message.tag
        sa = message.tag
        # CQ
        if cq == "CQC":
            sample_id = message.get('LOT')
            code = message.get('PRODID')
            resu = message.find("COUNT").text
            date = datetime.datetime.strptime(
                message.get('RDTS'),
                "%Y-%m-%d %H:%M:%S"
            )
            results.append({
                "lot": sample_id,
                "test": code,
                "resu": resu,
                "date": date,
            })
        elif sa == "SA":
            sample_id = message.get('ID')
            if sample_id.startswith("UR") is False:
                return results
            cristal = set()
            cylindre = set()
            date = datetime.datetime.strptime(
                message.get('RDTS'),
                "%Y-%m-%d %H:%M:%S"
            )
            for ac in message.findall('AC'):
                if ac.get('AT') == 'Sediment':
                    for ar in ac.findall('AR'):
                        test = ar.get('Key')
                        resu = ar.text.split(" ")
                        if test in ("HYAL", "EPIC", "WBCT", "RBCT", "GRAN"):
                            if resu[0] == "[none]":
                                cylindre.add(resu[0])
                            else:
                                cylindre.add("%s%s" % (test, resu[0]))
                        elif test in (
                            "CACB", "CAPH", "CAOX", "TPO4", "URIC", "LEUC",
                            "CYST", "TYRO", "AMOR"
                        ):
                            if resu[0] == "[none]":
                                cristal.add(resu[0])
                            else:
                                cristal.add("%s%s" % (test, resu[0]))
                        if resu[0] == "[none]":
                            continue
                        results.append({
                            "lot": sample_id,
                            "test": test,
                            "resu": resu[0],
                            "date": date,
                        })
            for test, resu in (
                ("CYLINDRE", cylindre),
                ("CRISTAL", cristal),
            ):
                if not len(resu):
                    continue
                elif len(resu) > 1 and "[none]" in resu:
                    resu.remove("[none]")
                results.append({
                    "lot": sample_id,
                    "test": test,
                    "resu": ";".join(sorted(resu)),
                    "date": date,
                })

        return results
