# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
)


class KpcrAstmSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    processing_id = HprimString(12)
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KpcrAstmSegmentP(Segment):
    # Uniquement transmis U (Unknown)
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class KpcrAstmSegmentO(Segment):
    specimen_id = HprimString(3)
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4)
    ))
    priority = HprimString(6)
    requested_date = HprimDate(7)
    specimen_descriptor = HprimString(16)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class KpcrAstmSegmentR(Segment):
    test = HprimString((3, 4))
    data = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)
    result_abnormal_flags = HprimList(7)
    result_status = HprimString(9)
    operator_identification = HprimString(11)
    date_result = HprimDate(13)
    instrument = HprimString(14)


class KpcrAstmSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    comment_type = HprimString(5)


class KpcrAstmSegmentL(Segment):
    terminaison_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "kpcr_astm",
        "label": "kPCR Astm"
    }
    SEGMENT_CLASS_MAP = {
        "H": KpcrAstmSegmentH,
        "P": KpcrAstmSegmentP,
        "O": KpcrAstmSegmentO,
        "R": KpcrAstmSegmentR,
        "C": KpcrAstmSegmentC,
        "L": KpcrAstmSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = KpcrAstmSegmentH(type_segment="H", **kwargs)
        message.end = KpcrAstmSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = msg.patients.add()
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                )
                for order in sample.orders:
                    t = seg_o.universal_test_id.add()
                    t.code = order.code

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_o.specimen_id,
                )
                for seg_r in seg_o.results:
                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        commentaires.append(seg_c.comment_text)

                    sample.results.add(
                        code=seg_r.test,
                        value=seg_r.data,
                        flags=seg_r.result_abnormal_flags,
                        comments=commentaires,
                        date=seg_r.date_result,
                        instrument=seg_r.instrument,
                    )
        return worklist
