# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimForeign,
    HprimList,
    HprimDate,
)


class KryptorSegmentH(HeaderSegment):
    host = HprimString(5)
    receiver = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KryptorSegmentP(Segment):
    ref_patient = HprimString(3)
    nom = HprimString((6, 1))
    prenom = HprimString((6, 2))
    sexe = HprimString(9)
    orders = HprimForeign("O", "patient")


class KryptorSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    test_order = HprimString(5)
    report_type = HprimString(13)


class KryptorSegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        sample_id=HprimString(1),
        cassette=HprimString(2),
        position=HprimString(3),
    )
    test = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution=HprimString(6, default="1"),
        previous_result=HprimString(7),
    ))
    priority = HprimString(6)
    action_code = HprimString(12)
    # O: en mode download
    # Q: en reponse d'un query
    # Z: si tube inconne lors d'un query
    report_type = HprimString(26)
    results = HprimForeign("R", "order")


class KryptorSegmentR(Segment):
    test = HprimMultiple(
        3,
        code=HprimString(4),
        dilution=HprimString(6),
        previous_status=HprimString(8),
    )
    result = HprimString(4)
    # L: pathological low
    # H: pathological high
    # <: below range
    # >: above range
    flags = HprimList(7)
    # F: Final result
    # X: if the test has not been performed(rejected, cancelled, failed)
    status = HprimString(9)
    date_started = HprimDate(12)
    date_result = HprimDate(13)
    kit_id = HprimString(14)


class KryptorSegmentC(Segment):
    comment_text = HprimString(3)
    # 0 System Warning: Low Laser Power
    # 1 System Error: Reaction Plate Position
    # 2 System Warning :Reagent Cooler Out Of Range
    # 3 System Warning: Incubator Out Of Range
    # 4 System Error: Missed Reading
    # 5 System Error: Unknown
    # 6 System Error: Insufficient Sample Volume
    # 7 System Error: Insufficient Reagent Volume
    # 8 System Error: Insufficient Diluant Volume
    # 9 System Warning: Late T0
    # 10 System Warning: Late T1
    # 11 System Warning: Late TE
    # 12 System Error: Pipetting
    # 13 System Error: Clot Detected
    # 14 System Error: Pipetting in Sample Area
    # 15 System Error: Pipetting in Dilution Area
    # 16 System Error: Pipetting in Reagent Area
    # 17 System Error: Pipetting in Wash Area
    # 18 System Error: Pipetting in Reaction Area
    # 19 System Warning: Heated Tip Out Of Range
    # 20 System Error: Interrupted
    # 21 System Warning: Not Performed
    # 22 Calibration: Removed by User
    # 24 Check Results
    # 25 Math Error
    # 26 Reflex Test Launched
    # 27 Rerun by Reflex testing
    # 28 Generated by Reflex testing
    # 30 Data Error: Unknown Error
    # 31 Data Error: Ratio
    # 32 Data Error: Response
    # 33 Data Warning: Abnormal
    # 34 Data Warning: Out Of Range
    # 35 Data Warning: Detection Limit
    # 36 Data Warning: Above Max Range
    # 37 Data Warning: Standard Warning
    # 38 Data Warning: Calibrator Warning
    # 39 Data Warning: Below Normal
    # 40 Data Warning: Above Normal
    # 41 Data Warning: Imposed
    # 42 Concentration not consistent with dilution used
    # 43 Data Warning: Control Warning
    # 44 Data Warning: Abnormal Kinetics
    # 45 Data Warning: Internal flag
    # 46 Data Warning: Internal flag
    # 47 Data Warning:  25 mUI hCG
    comment_flag = HprimList(4)


class KryptorSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessage):
    SPEC = {
        "code": "kryptor",
        "label": "Kryptor",
    }
    SEGMENT_CLASS_MAP = {
        "H": KryptorSegmentH,
        "P": KryptorSegmentP,
        "Q": KryptorSegmentQ,
        "O": KryptorSegmentO,
        "R": KryptorSegmentR,
        "C": KryptorSegmentC,
        "L": KryptorSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = KryptorSegmentH(type_segment="H", **kwargs)
        message.end = KryptorSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.sample_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "A"},
            priority={
                "S": "S",
                "A": "A",
                worklist.ELSE: "R"
            }
        ):
            seg_p = msg.patients.add(
                ref_patient=patient["patient_id"],
                nom=patient["last_name"],
                prenom=patient["first_name"],
                sexe=patient["sex"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    priority=priority,
                    action_code=action,
                    report_type="Q" if context["query"] else "O",
                )
                seg_o.specimen_id.sample_id = sample["sample_id"]
                for order in sample["orders"]:
                    test = seg_o.test.add()
                    test.code = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            for sample_id in self.get_queries(context["query"]):
                seg_p = msg.patients.add()
                seg_o = seg_p.orders.add()
                seg_o.specimen_id.sample_id = sample_id
                seg_o.report_type = "Z"
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                tube = seg_o.specimen_id.sample_id
                # exclusion des CQ
                if len(tube) == 8 and tube[3:5] in ['C1', 'C2', 'C3']:
                    continue
                if seg_o.specimen_id.cassette and seg_o.specimen_id.position:
                    rack = "%s-%s" % (
                        seg_o.specimen_id.cassette,
                        seg_o.specimen_id.position,
                    )
                else:
                    rack = None
                results.setdefault(tube, {
                    "resultats": [],
                    "rack": rack,
                })
                lots_reactifs = []
                for seg_r in seg_o.results:
                    lots_reactifs.append({
                        "ref": seg_r.kit_id,
                        "nom": seg_r.kit_id,
                    })
                    alarmes = seg_r.flags[:]
                    for seg_c in seg_r.commentaires:
                        # Contournement, le kryptor peut envoyer ~
                        # à la place du repetiteur.
                        for flag in seg_c.comment_flag:
                            alarmes.extend(flag.split("~"))
                    results[tube]["resultats"].append({
                        "test": seg_r.test.code,
                        "resu": seg_r.result,
                        "alarmes": alarmes,
                        "date": seg_r.date_result,
                        "lots_reactifs": lots_reactifs,
                    })
        return results

    def get_qc_result_list(self, message):
        results = []
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                lot = seg_o.specimen_id.sample_id
                # exclusion des resultats non CQ
                if len(lot) != 8 or lot[3:5] not in ['C1', 'C2', 'C3']:
                    continue
                lots_reactifs = []
                for seg_r in seg_o.results:
                    lots_reactifs.append({
                        "ref": seg_r.kit_id,
                        "nom": seg_r.kit_id,
                    })
                    results.append({
                        "lot": lot,
                        "test": seg_r.test.code,
                        "resu": seg_r.result,
                        "date": seg_r.date_result,
                        "lots_reactifs": lots_reactifs,
                    })
        return results
