# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.fixed import (
    FixedLengthFormat,
    Multiple,
    String,
    Date,
)


class KX21Base(FixedLengthFormat):
    pass


class KX21Result(KX21Base):
    TEST_CODE = [
        'WBC', 'RBC', 'HGB', 'HCT', 'MCV', 'MCH', 'MCHC', 'PLT',
        'LYM%', 'MXD%', 'NEUT%', 'LYM#', 'MXD#', 'NEUT#', 'RDW', 'PDW', 'MPV', 'P-LCR'
    ]

    FMT_DESCRIPTION = [
        "D",
        "1",
        "U",
        Date("date", 6, fmt_date="%y%m%d"),
        String("analys_information", 1),
        String("sample_id", 12),
        String("pda_information", 6),
        String("rdw_select", 1),  # S or C
    ] + [Multiple("results", [
        String("value", 4),
        # flag
        # 0: Normal
        # 1: +
        # 2: -
        # 3: Out of linearity limit
        # 4: Reliability is low
        String("flag", 1),
    ])]


class KX21Control(KX21Base):
    TEST_CODE = [
        'WBC', 'LYM%', 'MXD%', 'NEUT%', 'LYM#', 'MXD#', 'NEUT#', 'RBC', 'HGB', 'HCT',
        'MCV', 'MCH', 'MCHC', 'RDW', 'PLT', 'PDW', 'MPV', 'P-LCR', 'W-SMV', 'W-LMV'
    ]

    FMT_DESCRIPTION = [
        "D",
        "1",
        "C",
        String("data_distinction", 1),  # X or L
        Date("date", 10, fmt_date="%y%m%d%H%M"),
        String("control", 1),
        String("futur", 1),
        String("rdw_select", 1),  # S or C
    ] + [Multiple("results", [
        String("value", 4),
    ])]


class Driver(MessageV2):
    SPEC = {
        "code": "kx21",
        "label": "Sysmex KX21",
    }

    def create(self, message_type, **kwargs):
        if message_type == self.RESULT:
            return KX21Result(**kwargs)
        elif message_type == self.QC:
            return KX21Control(**kwargs)
        else:
            raise NotImplementedError

    def parse(self, data):
        if data[2] == "C" and len(data) >= 97:
            klass = KX21Control
        elif data[2] == "U" and len(data) >= 119:
            klass = KX21Result
        else:
            raise NotImplementedError

        obj = klass()
        obj.parse(data)
        return obj

    def to_string(self, message):
        return message.to_string()

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if isinstance(message, KX21Result):
            sample_id = message.sample_id.replace('-', '')
            sample = worklist.get_or_create(
                type="patient",
                sample_id=sample_id,
            )
        elif isinstance(message, KX21Control):
            sample = worklist.get_or_create(
                type="control",
                control_lot=message.control,
                sample_id=message.control,
            )
        else:
            raise NotImplementedError

        for idx, result in enumerate(message.results):
            code = message.TEST_CODE[idx]
            if code == "PLT":
                value = result.value
            elif code == "RBC":
                value = result.value[:2] + "." + result.value[2:]
            else:
                value = result.value[:3] + "." + result.value[-1]
            r = sample.results.add(
                code=message.TEST_CODE[idx],
                value=value,
                date=message.date,
            )
            if sample.type == "patient" and result.flag != "0":
                r.flags = [result.flag]

        if sample.type == 'patient':
            alarme = []
            idx = 0
            for code in ("WBC", "RBC", "PLT"):
                info, flag = message.pda_information[idx], message.pda_information[idx + 1]
                if info != "0" or flag != "0":
                    alarme.append(code + flag)
                idx += 2

            sample.results.add(
                code="ALARME",
                value=";".join(alarme),
                date=message.date,
            )
        return worklist
