# -*- coding: utf8 -*-

from __future__ import unicode_literals
from driverng.hl7 import (
    AstmMessage,
    Segment,
    HL7Foreign,
    HL7String,
    HL7CE,
    HL7List,
    HL7HD,
    HL7DTM,
    HL7NM,
    HL7RP,
    HL7CompositePrice,
    HL7CompositeQuantity,
    HL7SegmentMSH,
)
from driverng import Worklist


class HL7SegmentMFI(Segment):
    """Master file Identifier
    Identifie le catalogue.
    """
    type_catalogue = HL7String(2,
                               obligatoire=True,
                               max_length=250,
                               default="OMC")
    # champ modifié par le IHE
    # APPLICATION_IDENTIFIER doit être utilisé
    # Exemple: CLARISYS_OMC_FRA dans le sous champ 1
    version_catalogue = HL7HD(3, obligatoire=True, max_length=180)
    code_traitement = HL7String(4,
                                obligatoire=True,
                                max_length=3,
                                default='REP')
    date_effet = HL7DTM(6, obligatoire=True)
    # Available: AL => Always, NE => Never
    response_level = HL7String(7, obligatoire=True, default='NE')
    mfe = HL7Foreign("MFE", "mfi")


class HL7SegmentMFE(Segment):
    """Master File Entry
    Identifie une entrée dans le catalogue.
    """
    code_traitement = HL7String(2, obligatoire=True, default='MAD')
    control_id = HL7String(3)
    date_effet = HL7DTM(4)
    pkey = HL7CE(5, obligatoire=True, max_length=200)
    pkey_type = HL7String(6, obligatoire=True, default="CE")

    om1 = HL7Foreign("OM1", "mfe")
    om4 = HL7Foreign("OM4", "mfe")
    om5 = HL7Foreign("OM5", "mfe")
    zca = HL7Foreign("ZCA", "mfe")


class HL7SegmentOM1(Segment):
    """Identifie une analyse."""
    seq = HL7NM(2, obligatoire=True, default=1)
    analyse = HL7CE(3, obligatoire=True)
    codification = HL7String(4, default="L")
    echantillon_requis = HL7String(5, default="Y")
    service_production = HL7CE(6, obligatoire=True)
    synonymes = HL7List(9, basefield=HL7String(0))
    technique = HL7List(15, basefield=HL7CE(0))
    specialite = HL7List(17, basefield=HL7CE(0))
    nature = HL7List(19,
                     basefield=HL7String(0, max_length=20, default="A"))
    # En minutes
    delai_realisation = HL7NM(24)
    frequence_realisation = HL7String(41)
    commentaire = HL7String(42, max_length=65536)


class HL7SegmentOM4(Segment):
    """Observations That Require Specimens Segment
    Une occurrence de ce segment contient des informations liées
    à un type d’échantillon nécessaire à la réalisation de l’examen.
    """
    seq = HL7NM(2, obligatoire=True)
    container_description = HL7String(4, max_length=60)
    container_volume = HL7NM(5)
    container_unit = HL7CE(6)
    nature = HL7CE(7)
    additif = HL7CE(8)
    conditions_conservation = HL7CE(10)
    volume_requis = HL7CompositeQuantity(11)


class HL7SegmentOM5(Segment):
    """Défini un "groupe d'anlyses"
    Pour un analyse unique il faut mettre la meme chose que OM1-2 dans OM5-2.
    """
    seq = HL7NM(2)
    analyses = HL7List(3, obligatoire=True, basefield=HL7CE(0))


class HL7SegmentZCA(Segment):
    """Informations tarifaires. Spécificité IHE france."""
    prix_hn = HL7CompositePrice(2)
    prix_fixe = HL7String(3, default="Y")
    entente_prealable = HL7String(4, default="N")
    consentement = HL7String(5, default="N")
    examen_supplemtaire = HL7String(6)
    code_nabm = HL7List(7, basefield=HL7CE(0))
    url_documentaire = HL7RP(8)
    conditions_tarifaires = HL7String(9, max_length=250)


class Driver(AstmMessage):
    SPEC = {
        "code": "HL7_LCSD",
        "label": "LCSD National Extension 1.2",
    }
    FIRST_SEP_INDEX = 3
    DEFAULT_SEPARATORS = r"|^~\&"

    ENCODING_HL7_MAP = {
        "ISO-8859-15": "8859/15",
        "UTF-8": "UNICODE UTF-8",
    }

    SEGMENT_CLASS_MAP = {
        "MSH": HL7SegmentMSH,
        "MFI": HL7SegmentMFI,
        "MFE": HL7SegmentMFE,
        "OM1": HL7SegmentOM1,
        "OM4": HL7SegmentOM4,
        "OM5": HL7SegmentOM5,
        "ZCA": HL7SegmentZCA,
    }
    ADDENDUM_SEGMENT_SIZE = None

    def create(self, **kwargs):
        msg = self.SEGMENT_CLASS_MAP["MSH"](type_segment="MSH", **kwargs)
        msg.encoding = self.ENCODING_HL7_MAP[
            kwargs.get("encoding", "ISO-8859-15")]
        msg.type_message.message_type = "MFM"
        msg.type_message.event_type = "M10"
        msg.type_message.structure = "MFM_M10"
        msg.mode_traitement = "D"
        return msg

    def decode_message(self, message):
        msh = message
        if (
            msh.type_message.message_type != "MFN" or
            msh.type_message.event_type != "M10" or
            msh.type_message.structure != "MFN_M10"
        ):
            raise NotImplementedError("message_type {}".format(msh.type_message))
        mfi = msh.mfi[0]
        worklist = Worklist(context="dictionary")
        dictionary = worklist.dictionary
        dictionary.emetteur = msh.emetteur.nsid.label
        dictionary.production = msh.mode_traitement == "P"
        dictionary.unique_identifier = msh.message_control_id
        dictionary.version = mfi.version_catalogue.nsid.label
        dictionary.effective_date = mfi.date_effet
        for mfe in mfi.mfe:
            if not mfe.om1:
                continue
            om1 = mfe.om1[0]
            analyse = dictionary.entries.add()
            analyse.pkey = mfe.pkey.code1
            analyse.code = om1.analyse.code1
            analyse.libelle = om1.analyse.label1
            if om1.codification == "LN":
                analyse.codification = "LN"
            else:
                analyse.codification = "L"
            analyse.sample_required = om1.echantillon_requis == "Y"
            if om1.technique:
                analyse.method = om1.technique[0].label1
            analyse.battery = om1.nature != "A"
            if om1.delai_realisation is not None:
                try:
                    analyse.turnaroundtime = int(om1.delai_realisation)
                except ValueError:
                    pass
            # Need to see "Repeat Pattern HL7 (like Q6H)"
            # try:
            #     analyse.frequency = int(om1.frequence_realisation)
            # except ValueError:
            #     pass
            if om1.commentaire:
                analyse.comments = om1.commentaire
            for om4 in mfe.om4:
                echantillon = analyse.samples.add()
                echantillon.container_description = om4.container_description
                if om4.container_volume is not None:
                    try:
                        echantillon.container_volume = int(om4.container_volume)
                    except ValueError:
                        pass
                if om4.container_unit and om4.container_unit.code1:
                    echantillon.container_unit = om4.container_unit.code1
                if om4.nature:
                    if om4.nature.code1:
                        echantillon.nature_code = om4.nature.code1
                    if om4.nature.label1:
                        echantillon.nature_libelle = om4.nature.label1
                    if om4.nature.code2:
                        echantillon.alt_nature_code = om4.nature.code2
                    if om4.nature.label2:
                        echantillon.alt_nature_libelle = om4.nature.label2
                if om4.additif and om4.additif.label1:
                    echantillon.additive = om4.additif.label1
                if om4.conditions_conservation:
                    if om4.conditions_conservation.code1:
                        echantillon.transport_conditions_code = om4.conditions_conservation.code1
                    if om4.conditions_conservation.label1:
                        echantillon.transport_conditions_libelle = om4.conditions_conservation.label1
                if om4.volume_requis and om4.volume_requis.quantity:
                    echantillon.minimum_volume = "{}{}".format(
                        om4.volume_requis.quantity,
                        " {}" if om4.volume_requis.unit.code1 else '')
            zca = mfe.zca[0] if mfe.zca else None
            if zca:
                hn_price = zca.prix_hn.price
                if hn_price.currency == "EUR" and hn_price.quantity:
                    analyse.price_hn_eur_cents = int(
                        100 * zca.prix_hn.price.quantity)
                analyse.price_fixed = zca.prix_fixe == "Y"
                analyse.entente_prealable = zca.entente_prealable == "Y"
                analyse.consentement = zca.consentement == "Y"
                if zca.examen_supplemtaire:
                    analyse.analyse_declenchee = zca.examen_supplemtaire
                for nabm in zca.code_nabm:
                    analyse.nabms.add(nabm.code1)
                if zca.url_documentaire.pointer:
                    analyse.url_doc = zca.url_documentaire.pointer
                if zca.conditions_tarifaires:
                    analyse.conditions_tarifaires = zca.conditions_tarifaires
        return worklist
