# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class LxlSegmentH(HeaderSegment):

    sender_id = HprimString(5)
    receiver_id = HprimString(10, max_length=10)
    processing_id = HprimString(12)
    version = HprimString(13, default="1")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    infos = HprimForeign("M", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class LxlSegmentP(Segment):

    patient_id = HprimString(3, max_length=11)
    last_name = HprimString((6, 1), max_length=26)
    first_name = HprimString((6, 2), max_length=26)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    physician_id = HprimString(14, max_length=6)
    orders = HprimForeign("O", "patient")


class LxlSegmentQ(Segment):
    range_id = HprimList(3)
    test_id = HprimString(5, default="ALL")
    status_code = HprimString(13, default="O")


class LxlSegmentO(Segment):

    specimen_id = HprimString(3, max_length=17)
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution_ratio=HprimString(5),
    ))

    # S: Stat
    # N: Normal
    priority = HprimString(6)

    request_date = HprimDate(7)

    # Q pour QC
    action_code = HprimString(12)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class LxlSegmentR(Segment):

    test = HprimString((3, 4))
    # valeur possible:
    # INTR: valeur interprétée
    # DOSE: valeur quantitative
    aspect = HprimString((3, 6))
    value = HprimString(4)
    units = HprimString(5)

    # L: Database flag LOW, under a defined normal range
    # H: Database flag HIGH, over a defined normal range
    # <: under range
    # >: over range
    # N: Within normal and assay range
    abnormal_flag = HprimList(7)

    # F: Final
    # I: Instrument pending
    # S: Partial result
    # Q: response to a Query
    # C: Correction
    # X: Rejected
    status = HprimString(9)
    date = HprimDate(13)
    infos = HprimForeign("M", "result")


class LxlSegmentC(Segment):

    # I: Clinical instrument system
    source = HprimString(3)
    text = HprimString(4)
    # I: Instrument flag comments
    comment_type = HprimString(5)


class LxlSegmentM(Segment):

    kit_number = HprimString(3)
    expiration_date = HprimDate(4, fmt_date="%Y%m%d")
    lot_number = HprimString(5)
    calibration_id = HprimString(6)
    sample_lane_number = HprimString((7, 1))
    sample_tube_position = HprimString((7, 2))


class LxlSegmentL(Segment):

    # N: Normal termination
    # I: No info available
    code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "lxl",
        "label": "LxL Diasorin",
    }
    SEGMENT_CLASS_MAP = {
        "H": LxlSegmentH,
        "P": LxlSegmentP,
        "O": LxlSegmentO,
        "Q": LxlSegmentQ,
        "C": LxlSegmentC,
        "R": LxlSegmentR,
        "M": LxlSegmentM,
        "L": LxlSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessage.Option("receiver_id", label="Code emetteur"),
        AstmMessage.BooleanOption(
            "ref_qc_tronquee",
            label="Ignorer le premier caractère '#' des lots de QC",
            default=True),
    ]

    def __init__(self, receiver_id="", ref_qc_tronquee=True, **kwargs):
        self.receiver_id = receiver_id or ""
        self.ref_qc_tronquee = ref_qc_tronquee
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = LxlSegmentH(type_segment="H", **kwargs)
        message.end = LxlSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.extend(seg_q.range_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(
            receiver_id=self.receiver_id,
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                action={"del": "C", worklist.ELSE: "N"},
                priority={"S": "S", worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient["patient_id"],
                last_name=patient["last_name"],
                first_name=patient["first_name"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
                physician_id=patient["physician"],
            )
            for sample, action, priority in patient["samples"]:
                seg_o = seg_p.orders.add(
                    specimen_id=sample["sample_id"],
                    priority=priority,
                    action_code=action,
                    request_date=sample["request_date"],
                    report_type="O",
                )
                for order in sample["orders"]:
                    test = seg_o.tests.add()
                    test.code = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            msg.end.code = "I"
            return msg

    def get_result_dict(self, message):
        results = {}
        racks = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # exclusion des QC
                if seg_o.action_code == 'Q':
                    continue
                for seg_r in seg_o.results:
                    # uniquement les resultats avec le statut F
                    if seg_r.status != "F":
                        continue
                    test = "%s%s" % (seg_r.test, seg_r.aspect)
                    alarmes = []
                    if seg_r.abnormal_flag:
                        alarmes = seg_r.abnormal_flag
                    for seg_c in seg_r.commentaires:
                        alarmes.append(seg_c.text)
                    lots_reactifs = {}
                    for seg_m in seg_r.infos:
                        if seg_m.lot_number:
                            lots_reactifs[seg_m.lot_number] = {
                                "ref": seg_m.lot_number,
                                "peremption": seg_m.expiration_date,
                            }
                        if seg_m.sample_lane_number and seg_m.sample_tube_position:
                            racks[seg_o.specimen_id] = "%s-%s" % (
                                seg_m.sample_lane_number,
                                seg_m.sample_tube_position,
                            )
                    results.setdefault(
                        seg_o.specimen_id,
                        {"resultats": []})["resultats"].append(
                        {
                            "test": test,
                            "date": seg_r.date,
                            "alarmes": alarmes,
                            "resu": seg_r.value,
                            "lots_reactifs": lots_reactifs.values(),
                        })
        for sample_id, rack in racks.items():
            results.setdefault(sample_id, {"resultats": []})["rack"] = rack
        return results

    def get_qc_result_list(self, message):
        result = []
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # uniquement les QC
                if seg_o.action_code != "Q":
                    continue
                if seg_o.specimen_id[0] == '#' and self.ref_qc_tronquee:
                    lot = seg_o.specimen_id[1:]
                else:
                    lot = seg_o.specimen_id
                for seg_r in seg_o.results:
                    if seg_r.status != "F" or not seg_r.value:
                        continue
                    test = "%s%s" % (seg_r.test, seg_r.aspect)
                    result.append({
                        "lot": lot,
                        "test": test,
                        "resu": seg_r.value,
                        "date": seg_r.date,
                    })
        return result
