# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)
from driverng.utils import num_years


class MeqnetSegmentH(HeaderSegment):
    access_password = HprimString(4)
    sender_name = HprimString(5)
    receiver_id = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="E1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class MeqnetSegmentP(Segment):
    practice_patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    mother_maiden_name = HprimString(7)
    birthdate = HprimMultiple(
        8,
        birthdate=HprimDate(1, fmt_date="%Y%m%d"),
        age=HprimString(2),
        # en année uniquement: Y
        age_unit=HprimString(3)
    )
    sex = HprimString(9)
    attending_physician = HprimString(14)
    patient_height = HprimString((17, 1))
    height_unit = HprimString((17, 2))
    patient_weight = HprimString((18, 1))
    weight_unit = HprimString((18, 2))
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class MeqnetSegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        sample_id=HprimString(1),
        rack=HprimString(2),
        position=HprimString(3),
    )
    universal_test_id = HprimList(5, basefield=HprimMultiple(
        0,
        # code demande par défaut HbA1c%
        code=HprimString(4),
        # relicat 0 par defaut
        # 1 si demande de repasse
        replicate=HprimString(5, default="0"),
    ))
    # S: Stat
    # R: Routine
    priority = HprimString(6)
    collection_date = HprimDate(8)
    # A: Update action to the order for a patient
    # N: New order for a patient
    # C: Cancel the order for a patient
    # R: Normal sample
    # Q: Control
    action_code = HprimString(12)
    specimen_descriptor = HprimString(16, default="S")
    # Q or O: New order asked to be performed
    # Y or Z No order (as response fro HQ)
    report_type = HprimString(26)

    infos = HprimForeign("M", "order")
    results = HprimForeign("R", "order")


class MeqnetSegmentR(Segment):
    test = HprimString((3, 4))
    data_type = HprimString((3, 10))
    value = HprimString(4)
    unit = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flag = HprimList(7)
    result_status = HprimString(9)
    operator_identification = HprimString(11)
    date = HprimDate(13)
    instrument = HprimString(14)


class MeqnetSegmentM(Segment):
    manufacturer_record = HprimString(3)
    name = HprimString(4)
    lot_number = HprimString(5)
    serial_number = HprimString(6)
    expiration_date = HprimDate(7)
    setup_date = HprimDate(8)
    type_calibration = HprimString(9)


class MeqnetSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)
    # G: Generic/free text comment
    # I: Instrument flag comments
    comment_type = HprimString(5)


class MeqnetSegmentQ(Segment):
    specimen_id = HprimString((3, 2))
    request_status = HprimString(13)


class MeqnetSegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "meqnet",
        "label": "Meqnet"
    }
    SEGMENT_CLASS_MAP = {
        "H": MeqnetSegmentH,
        "P": MeqnetSegmentP,
        "O": MeqnetSegmentO,
        "Q": MeqnetSegmentQ,
        "M": MeqnetSegmentM,
        "C": MeqnetSegmentC,
        "R": MeqnetSegmentR,
        "L": MeqnetSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = MeqnetSegmentH(type_segment="H", **kwargs)
        message.end = MeqnetSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "N"},
        ):
            sample_ids = list(set([s.sample_id for s, _, _ in patient.samples]))
            if len(sample_ids) > 1:
                raise NotImplementedError("Multi sample on same patient is not implemented")
            seg_p = msg.patients.add(
                practice_patient_id=patient.patient_id,
                laboratory_patient_id=sample_ids[0],
                mother_maiden_name=patient.maiden_name,
                last_name=patient.last_name,
                first_name=patient.first_name,
            )

            if patient.birthdate:
                seg_p.birthdate.birthdate = patient.birthdate
                seg_p.birthdate.age = num_years(
                    patient.birthdate, patient.collection_date
                )
                seg_p.birthdate.age_unit = "Y"

            if patient.height:
                seg_p.patient_height = patient.height
                seg_p.height_unit = "cm"

            if patient.weight:
                seg_p.patient_weight = patient.weight
                seg_p.weight_unit = "kg"

            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    priority=priority,
                    action_code=action,
                    collection_date=sample.collection_date,
                    report_type="Q" if context["query"] else "O",
                )
                seg_o.specimen_id.sample_id = sample.sample_id
                for order in sample.orders:
                    test = seg_o.universal_test_id.add()
                    test.code = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            for seg_q in context["query"].queries:
                seg_p = msg.patients.add(laboratory_patient_id=seg_q.specimen_id)
                seg_o = seg_p.orders.add(report_type="Y")
                seg_o.specimen_id.sample_id = seg_q.specimen_id
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.specimen_id)
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    # QC
                    if seg_o.action_code == "Q":
                        sample = worklist.get_or_create(
                            control_lot=seg_o.specimen_id.sample_id,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.specimen_id.sample_id,
                            type="patient",
                        )
                    if seg_o.specimen_id.rack and seg_o.specimen_id.position:
                        sample.rack = "%s-%s" % (
                            seg_o.specimen_id.rack,
                            seg_o.specimen_id.position,
                        )
                    for seg_r in seg_o.results:
                        comments = []
                        if seg_r.abnormal_flag:
                            flags = [seg_r.abnormal_flag]
                        else:
                            flags = []
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_type == "G":
                                comments.append(seg_c.comment_text)
                            elif seg_c.comment_type == "I":
                                flags.append(seg_c.comment_text)
                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.value,
                            flags=flags,
                            comments=comments,
                            date=seg_r.date,
                            instrument=seg_r.instrument,
                            operator=seg_r.operator_identification.strip(),
                        )
        return worklist
