# -*- coding: utf8 -*-
from __future__ import unicode_literals
import csv
from StringIO import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String, Multiple


class NimbusBase(Splitted):
    SEPARATOR = ""


class NimbusOrder(NimbusBase):
    FIELDS = (
        String("no_liste"),
        String("no_dossier"),
        Multiple("tests", (
            String("test"),
        ), separator=""),
    )


class NimbusResult(object):
    FIELDS = [
        String("sample_no"),
        String("patient_id"),
        String("name"),
        String("well"),
        String("target"),
        String("ct"),
        String("niveau"),
        String("auto_interpretation"),
        String("test_kit"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for idx, elt in enumerate(line):
            field = fields[idx]
            setattr(self, field.name, elt)


class NimbusMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: NimbusOrder,
        MessageV2.RESULT: NimbusResult,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if data.startswith('Sample No') or data.startswith(u'N° échantillon'):
            result = data.replace('""', '"').replace(u'\x99', '').replace(u'\xb0', '').replace(u'\xe9', '')
            buf = StringIO(result)
            reader = csv.reader(buf, skipinitialspace=True)
            type_message = MessageV2.RESULT
            for index, line in enumerate(list(reader)):
                if index == 0:
                    continue
                obj = self.add(type_message=type_message)
                obj.parse(line)
        else:
            type_message = MessageV2.ORDER
            for line in data.splitlines():
                if not len(line):
                    continue
                obj = self.add(type_message=type_message)
                obj.parse(line)

    def to_string(self):
        return "".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "nimbus",
        "label": "Eurobio Nimbus",
    }

    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: NimbusOrder,
        MessageV2.RESULT: NimbusResult,
    }

    def create(self):
        return NimbusMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = NimbusMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create()
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        seq = 1
        if len(samples) == 0:
            return None
        elif len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = samples[0][0]
            message_order = msg.add(
                type_message=MessageV2.ORDER,
                no_liste=str(seq).ljust(5),
                no_dossier=sample.sample_id.ljust(20),
            )
            for order in sample.orders:
                message_order.tests.add(
                    test=order.code.ljust(15)
                )
            idx = 1
            while idx <= 10 - len(sample.orders):
                message_order.tests.add(
                    test=" ".ljust(15)
                )
                idx += 1
        seq += 1
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        sample_id = rack = ""
        for m in message:
            if m.name and m.name != 'Nom':
                sample_id = m.name
                rack = m.well
            if sample_id:
                sample = worklist.get_or_create(
                    sample_id=sample_id,
                    rack=rack,
                    type="patient",
                )
                sample.results.add(
                    code=m.target,
                    value=m.ct,
                )
                # niveau de positivité
                # lorsque le résulat est négatif, le niveau "N/A" est transmis
                if m.niveau and m.niveau != "N/A":
                    sample.results.add(
                        code="LEVEL_%s" % (m.target,),
                        value=m.niveau,
                    )
        return worklist
