# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import re

from StringIO import StringIO

from driverng import Message, GraphContainer
from driverng.utils import get_date, format_date
from driverng.lazy import lazy


logger = logging.getLogger("driverng.drivers.pentra120")


class Pentra120GraphContainer(GraphContainer):
    THRESHOLD_MAP = {
        # Histogrames
        0x57: 0x5D,
        0x58: 0x5E,
        0x59: 0x5F,
        0x5A: 0x60,
        # Lmne
        0x5B: 0x61,
        # Double Lmne
        0x63: 0xC8,
        # Retic
        0x5C: 0x62,
    }
    GRAPH_ID = THRESHOLD_MAP.keys() + THRESHOLD_MAP.values()

    def __init__(self, code, values, type_fichier="png"):
        self.code = code
        self.values = values
        super(Pentra120GraphContainer, self).__init__(type_fichier)

    @staticmethod
    def generate_histogram_graph(code, values):
        pylab = lazy.pylab
        taby = [ord(c) - 32 for c in values[code]]
        if taby[0] != 0 or taby[-1] != 0:
            taby = [0] + taby + [0]
        taby = map(float, taby)
        lig = values[Pentra120GraphContainer.THRESHOLD_MAP[code]]
        thi = [int(i) for i in lig.split() if int(i)]
        if code == 0x5A:
            taby = [0] * 9 + taby
        else:
            taby = [0] * 6 + taby
        pylab.figure(figsize=(5, 2), edgecolor="r")
        pylab.fill(list(range(len(taby))), taby, 'b')
        pylab.axis('on')
        ax = pylab.gca()
        ax.xaxis.set_visible(False)
        pylab.vlines(thi, min(taby), max(taby))
        graph = StringIO()
        pylab.savefig(graph, dpi=100)
        pylab.close()
        return graph.getvalue()

    @staticmethod
    def hex_data_to_img_string(data):
        assert len(data) % 2 == 0
        return b''.join(
            [chr(int(data[i:i + 2], 16)) for i in range(0, len(data), 2)])

    @staticmethod
    def get_retic_polygons(threshold):
        # Attention, YM est "en bas" du graphe et YZ "en haut"
        XZ, YZ, XM, YM = (-1, -1, 128, 128)
        X1, X2, X3, X4 = threshold[0:4]
        Y1, Y2, Y3, Y4 = [a / 2 for a in threshold[4:8]]
        return [
            ['#bababa', (XZ, YM), (XZ, YZ), (XM, YZ), (XM, YM),
                (X4, YM), (X4, Y3), (X2, Y3), (X2, YM)],
            ['#ff0000', (X2, YM), (X2, Y3), (X3, Y3), (X3, YM)],
            ['#00ffff', (X3, YM), (X3, Y3), (X4, Y3), (X4, YM)],
        ], [
            ['#0000ff', (X1, YZ), (X1, YM)],
            ['#0000ff', (X2, YZ), (X2, YM)],
            ['#0000ff', (X3, YZ), (X3, YM)],
            ['#0000ff', (X4, YZ), (X4, YM)],
            ['#00ffff', (X1, Y1), (XM, Y1)],
            ['#00ffff', (X1, Y2), (XM, Y2)],
            ['#00ffff', (X1, Y3), (XM, Y3)],
            ['#00ffff', (X1, Y4), (XM, Y4)],
        ]

    @staticmethod
    def get_lmne_polygons(threshold):
        # Attention, YM est "en bas" du graphe et YZ "en haut"
        XZ, YZ, XM, YM = (-1, -1, 128, 128)
        NOL, NON, NOE, LN, RN, LL, AL, LMU, LMD, LMN, MN, RM = threshold[0:12]
        NL, NE, RMN = [127 - t for t in threshold[12:12 + 3]]
        return [
            ['#bababa', (XZ, YM), (NOL, YM), (NOL, NL), (NON, NL), (NON, NE),
                (NOE, NE), (NOE, YZ), (XZ, YZ)],  # NOISE
            ['#F3E037', (NOE, NE), (NOE, YZ), (XM, YZ), (XM, NE)],  # EOS
            ['#50AAFF', (NOL, YM), (NOL, NL), (LMU, NL), (LMD, YM)],  # LYMPHO
            ['#C98AE5', (NON, NL), (NON, NE), (XM, NE), (XM, RMN), (MN, RMN),
                (LMN, NL)],  # NEUTRO
            ['#6FE035', (LMN, NL), (MN, RMN), (XM, RMN), (XM, YM), (LMD, YM),
                (LMU, NL)],  # MONO
        ], [
            ['#C98AE5', (LN, NE), (LN, NL)],  # séparation LEFT NEU / NEUTRO
            ['#C98AE5', (RN, RMN), (RN, NE)],  # séparation RIGHT NEU / NEUTRO
            ['#50AAFF', (LL, YM), (LL, NL)],  # LEFT LYM / LYMPHO
            ['#50AAFF', (AL, YM), (AL, NL)],  # ATT LYM / LYMPHO
            ['#6FE035', (RM, YM), (RM, RMN)],  # RIGHT MON / MONO
        ]

    @staticmethod
    def get_double_lmne_polygons(threshold):
        # Attention, YM est "en bas" du graphe et YZ "en haut"
        XZ, YZ, XM, YM = (-1, -1, 256, 128)
        BFL, BFN, BFE, NG, ND, LG, LYA, LMN, LI, MI, MDI, IGI = threshold[0:12]
        NL, NE, MND, LMI, MMD, IB, IH, IGIB, IGIH, GIB, GIH = [
            127 - t for t in threshold[12:12 + 11]]
        return [
            ['#bababa', (XZ, YM), (BFL, YM), (BFL, NL), (BFN, NL), (BFN, NE),
                (BFE, NE), (BFE, YZ), (XZ, YZ)],  # (1)
            ['#50AAFF', (BFL, YM), (BFL, NL), (LMN, NL), (LI, LMI),
                (LI, YM)],  # (2,3,4)
            ['#6FE035', (LMN, NL), (ND, MND), (MI, MND), (MI, MMD),
                (LI, LMI)],  # (5)
            ['#C98AE5', (BFN, NL), (BFN, NE), (MI, NE), (MI, MND), (ND, MND),
                (LMN, NL)],  # (6,7,14)
            ['#f3e037', (BFE, NE), (BFE, YZ), (MI, YZ), (MI, NE)],  # (8)
            ['#ff00ff', (LI, YM), (LI, LMI), (MI, MMD), (MDI, IB), (IGI, IGIB),
                (XM, GIB), (XM, YM)],  # (9,10)
            ['#ffa500', (MI, MMD), (MI, MND), (MDI, IH), (IGI, IGIH),
                (XM, GIH), (XM, GIB), (IGI, IGIB), (MDI, IB)],  # (11,12,13)
            ['#a5ff00', (MI, MND), (MI, YZ), (XM, YZ), (XM, GIH), (IGI, IGIH),
                (MDI, IH)],  # (15,16)
        ], [
            ['#50AAFF', (LG, YM), (LG, NL)],  # 2/3
            ['#50AAFF', (LYA, YM), (LYA, NL)],  # 3/4
            ['#C98AE5', (NG, NL), (NG, NE)],  # 6/7
            ['#C98AE5', (ND, MND), (ND, NE)],  # 7/14
            ['#ff00ff', (IGI, YM), (IGI, IGIB)],  # 9/10
            ['#ffa500', (MDI, IB), (MDI, IH)],  # 11/12
            ['#ffa500', (IGI, IGIB), (IGI, IGIH)],  # 12/13
            ['#a5ff00', (IGI, IGIH), (IGI, YZ)],  # 15/16
        ]

    @staticmethod
    def generate_lmne_graph(code, values, width=128, height=128,
                            polygons_func=None, draw_poly_bounds=True):
        image = lazy.Image
        imagedraw = lazy.ImageDraw
        scale = 3
        if polygons_func is None:
            polygons_func = Pentra120GraphContainer.get_lmne_polygons
        data = Pentra120GraphContainer.hex_data_to_img_string(values[code])
        assert len(data) == width * height / 8
        # On prépare le masque (pixels allumés ou non)
        img = image.fromstring('1', (width, height), data)
        width = width * scale
        height = height * scale
        img = img.resize((width, height))
        mask = image.eval(img, lambda x: bool(x))

        lmne_tab = values[Pentra120GraphContainer.THRESHOLD_MAP[code]]
        tl = map(int, lmne_tab.split())
        polygons, dashed_lines = polygons_func(tl)
        # On va dessiner sur deux "calques"
        # Sur l'un des calques on dessine juste les contours des polygones
        # et les lignes "pointillées"
        img = image.new('RGB', (width, height))
        # sur l'autre on les "rempli"
        imgpolygons = image.new('RGB', (width, height))
        # On commence par tracer les lignes qu'on veut "pointiller"
        draw = imagedraw.Draw(img)
        for line in dashed_lines:
            color = line.pop(0)
            line = [tuple([c * scale for c in p]) for p in line]
            draw.line(line, fill=color)
        # On prend l'image qui contient les lignes qu'on veut pointiller et
        # on met des pixels noirs
        # par-ci par-là
        for x in range(width):
            for y in range(height):
                if (x + y) % 2 == 0:
                    img.putpixel((x, y), 1)
        # Maintenant, on trace les contours des polygones sur ce même calque
        # et les polygones "remplis" sur l'autre calque
        drawpoly = imagedraw.Draw(imgpolygons)
        for poly in polygons:
            color = poly.pop(0)
            poly.append(poly[0])  # Pour que le polygone "boucle"
            poly = [tuple([c * scale for c in p]) for p in poly]
            drawpoly.polygon(poly, outline=color, fill=color)
            if draw_poly_bounds:
                draw.polygon(poly, outline=color)
        # Ensuite, grace au masque des pixels contenus dans la trame,
        # on applique le calque qu'on a rempli sur celui où il y a juste
        # les contours
        img.paste(imgpolygons, None, mask)
        graph = StringIO()
        img.save(graph, format='png')
        # Pour debug :
        # img.save('/tmp/testpentra.png', format='png')
        return graph.getvalue()

    def get_bytes(self):
        if self.code in (0x57, 0x58, 0x59, 0x5A):  # Histograme
            return self.generate_histogram_graph(self.code, self.values)
        elif self.code == 0x5B:
            return self.generate_lmne_graph(self.code, self.values)
        elif self.code == 0x63:
            return self.generate_lmne_graph(
                self.code, self.values, width=256, height=128,
                polygons_func=Pentra120GraphContainer.get_double_lmne_polygons)
        elif self.code == 0x5C:
            return self.generate_lmne_graph(
                self.code, self.values,
                polygons_func=Pentra120GraphContainer.get_retic_polygons,
                draw_poly_bounds=False)
        else:
            raise NotImplementedError


class AbxString(object):
    def __init__(self, name, length):
        self.name = name
        self.length = length
        super(AbxString, self).__init__()

    def to_python(self, value):
        return value.strip()

    def to_string(self, value):
        fmt = "%%-%d.%ds" % (self.length, self.length)
        return fmt % (value,)


class AbxDate(AbxString):
    def __init__(self, name, length, fmt="%Y%m%d %H:%M"):
        self.fmt = fmt
        super(AbxDate, self).__init__(name, length)

    def to_python(self, value, fmt=None):
        fmt = fmt or self.fmt
        return get_date(super(AbxDate, self).to_string(value),
                        fmt=fmt)

    def to_string(self, value, fmt=None):
        fmt = fmt or self.fmt
        return format_date(value, fmt)


class Pentra120Test(object):
    def __init__(self, code, value):
        self.code = code
        self.value = value
        super(Pentra120Test, self).__init__()


class Pentra120TestContainer(list):
    def add(self, *args):
        obj = Pentra120Test(*args)
        self.append(obj)
        return obj


class _Pentra120Message(object):
    FIELDS = (
        (0xFF, AbxString("type", 8)),
        (0x6B, AbxString("reagents", 168)),
        (0x6C, AbxString("instrument_serial", 20)),
        (0x70, AbxString("analyzer_number", 2)),
        (0x71, AbxDate("analysis_date", 19, fmt=None)),
        (0x72, AbxString("analyzer_run_number", 16)),
        (0x73, AbxString("analyzer_sequence_number", 9)),

        # M: Manual (open tube)
        # R: Rack (close tube)
        (0x74, AbxString("sampling_mode", 1)),
        (0x75, AbxString("sample_id", 16)),
        (0x76, AbxString("patient_name", 30)),
        # Date de naissance
        (0x77, AbxDate("birth_date", 6, fmt="%y/%m/%d")),
        (0x78, AbxString("age", 3)),
        (0x79, AbxString("sex", 1)),

        # 0 or blank: Unspecified
        # 1: Male
        # 2: Female
        (0x7A, AbxString("origin", 1)),
        (0x7B, AbxString("doctor", 15)),
        (0x7C, AbxString("department", 10)),
        (0x7D, AbxDate("collection_date", 14, fmt="%d/%m/%y %Hh%M")),
        (0x7F, AbxString("blood_type", 16)),

        # A: CBC
        # B: DIF
        # C: RET
        # E: CBR
        # F: DIR
        # G: No analysis
        # H: ERB
        # I: CBE
        # J: CBF
        (0x80, AbxString("analysis_type", 1)),

        # 0: 10 position
        # 1: 15 position
        # 2: 32 position
        (0x81, AbxString("sample_rack_type", 1)),
        (0x82, AbxString("number_of_runs", 1)),
        (0x83, AbxString("operator_code", 3)),
        (0x89, AbxString("sps_smearing_mode", 3)),
        (0x8A, AbxString("sps_smearing_profile", 4)),
        (0x8B, AbxString("patient_id", 30)),
        (0xFB, AbxString("analyzer_name", 8)),
        (0xFE, AbxString("format_version", 5)),
    )

    def __init__(self, format_analysis_date, **kwargs):
        self.format_analysis_date = format_analysis_date
        self.tests = {}
        for abx_id, field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        super(_Pentra120Message, self).__init__()

    def parse(self, data):
        fields = dict(self.FIELDS)
        for line in data.split('\r'):
            if not line:
                continue
            abx_id = ord(line[0])
            try:
                field = fields[abx_id]
            except KeyError:
                value = line[1:].strip()
                self.tests[abx_id] = value
            else:
                if abx_id == 0x71:
                    setattr(self, field.name, field.to_python(line[2:],
                            fmt=self.format_analysis_date))
                else:
                    setattr(self, field.name, field.to_python(line[2:]))

    def to_string(self):
        data = ""
        for abx_id, field in self.FIELDS:
            value = getattr(self, field.name)
            if value is not None:
                if abx_id == 0x71:
                    v = field.to_string(
                        value,
                        fmt=self.format_analysis_date)
                else:
                    v = field.to_string(value)
                data += "%c %s\r" % (
                    chr(abx_id).decode("ISO-8859-15"),
                    v,
                )
        for abx_id, value in self.tests.iteritems():
            data += "%c %s\r" % (
                chr(abx_id).decode("ISO-8859-15"),
                value,
            )
        return data


class Driver(Message):
    SPEC = {
        "code": "pentra120",
        "label": "Pentra Nexus/120DX/120DF/120R/80/60",
    }
    ENCODING = "latin1"
    PATHOLOGY_FIELDS = [0x54, 0x55, 0x56, 0x69]
    FLAG_FIELDS = [0x50, 0x51, 0x52, 0x53, 0x66, 0x67, 0x68, 0xA2]
    ALARME_FIELDS = PATHOLOGY_FIELDS + FLAG_FIELDS
    TYPE_RESULT, TYPE_QC, TYPE_OTHER = range(3)

    OPTIONS = [
        Message.Option(
            "format_analysis_date",
            label="Format Analysis Date (0x71)",
            choices=(
                ("%d/%m/%y %Hh%Mmn%Ss", "%d/%m/%y %Hh%Mmn%Ss"),
                ("%y/%m/%d %Hh%Mmn%Ss", "%y/%m/%d %Hh%Mmn%Ss"),
            ),
            default="%d/%m/%y %Hh%Mmn%Ss"),
        Message.Option("qc_prefix", label="Préfixe QC"),
    ]

    def __init__(self, format_analysis_date="%d/%m/%y %Hh%Mmn%Ss", qc_prefix=None, **kwargs):
        self.format_analysis_date = format_analysis_date
        self.qc_prefix = qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return _Pentra120Message(self.format_analysis_date, **kwargs)

    def get_message_type(self, message):
        if message.type == "FILE":
            return self.QUERY
        else:
            return self.RESULT

    def to_string(self, message, crlf="\r\n"):
        return message.to_string()

    def parse(self, data):
        obj = _Pentra120Message(self.format_analysis_date)
        obj.parse(data)
        return obj

    def get_queries(self, message):
        return [message.sample_id]

    def get_order_message(self, worklist, context):
        if len(worklist.samples) == 0:
            return
        elif len(worklist.samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        else:
            sample = worklist.samples[0]
            msg = self.create(
                type="FILE",
                sample_id=sample["sample_id"],
                patient_name="%s %s" % (
                    sample["last_name"], sample["first_name"]),
                birth_date=sample["birthdate"],
                sexe={"M": "1", "F": "2"}.get(sample["sex"], "0"),
                collection_date=sample["collection_date"],
                blood_type=sample["specimen"],
            )
            age = None
            if sample["birthdate"] and sample["collection_date"]:
                age = (
                    sample["collection_date"] - sample["birthdate"]).days / 365
            if age and age < 100:
                msg.age = "%sy" % (age,)
            elif age and age >= 100:
                msg.age = "%s" % (age,)
            m = {
                "CBC": 1 << 0,
                "DIF": 1 << 1,
                "RET": 1 << 2,
                "G": 1 << 3,
                "ERB": 1 << 4,
                "CBF": 1 << 5,
                "DIR": 1 << 6,
            }
            code = 0
            lame = 0
            for order in sample["orders"]:
                if order["action"] != "del":
                    if order["code"] in m:
                        if order["code"] == "G":
                            # detection du code G (LAME)
                            lame = 1
                        code |= m[order["code"]]
                    else:
                        logger.warning("Code demande non valide: %s", code)

            if code == 0:
                # Pas de demande
                return

            try:
                msg.analysis_type = {
                    m["CBC"]: "A",
                    m["CBC"] | m["G"]: "A",
                    m["DIF"]: "B",
                    m["DIF"] | m["G"]: "B",
                    m["CBC"] | m["DIF"]: "B",
                    m["CBC"] | m["DIF"] | m["G"]: "B",
                    m["RET"]: "C",
                    m["RET"] | m["G"]: "C",
                    m["CBC"] | m["RET"]: "E",
                    m["CBC"] | m["RET"] | m["G"]: "E",
                    m["DIF"] | m["RET"]: "F",
                    m["DIF"] | m["RET"] | m["G"]: "F",
                    m["CBC"] | m["DIF"] | m["RET"]: "F",
                    m["CBC"] | m["DIF"] | m["RET"] | m["G"]: "F",
                    m["DIR"]: "F",
                    m["CBC"] | m["DIF"] | m["DIR"]: "F",
                    m["CBC"] | m["DIF"] | m["DIR"] | m["G"]: "F",
                    m["CBC"] | m["DIF"] | m["DIR"] | m["RET"]: "F",
                    m["CBC"] | m["DIF"] | m["DIR"] | m["RET"] | m["G"]: "F",
                    m["G"] | m["DIR"]: "F",
                    m["DIF"] | m["G"]: "B",
                    m["DIF"] | m["DIR"]: "F",
                    m["DIF"] | m["DIR"] | m["G"]: "F",
                    m["G"]: "G",
                    m["ERB"]: "H",
                    m["ERB"] | m["G"]: "H",
                    m["CBC"] | m["ERB"]: "I",
                    m["CBC"] | m["ERB"] | m["G"]: "I",
                    m["CBF"]: "J",
                    m["CBF"] | m["G"]: "J",
                }[code]
            except KeyError:
                combinaison = ",".join([k for k in m.keys() if code & m[k]])
                raise RuntimeError((
                    "Combinaison de code demande non valide sur le tube %s: %s. "
                    "Voir la documentation du driver"
                ) % (sample["sample_id"], combinaison))

            if msg.analysis_type == "G" or lame == 1:
                # A slide for Smearing and Staining is done before test.
                # Reflex testing conditions are inhibited
                msg.sps_smearing_mode = "EC"

            return msg

    def get_value_and_flag(self, value):
        number, flags = "", []
        if value and value[0].isdigit():
            for c in value:
                if c.isdigit() or c == ".":
                    number += c
                elif c != " ":
                    flags.append(c)
            return (number, flags)
        else:
            return (value, [])

    @staticmethod
    def get_analyseur(message):
        if message.analyzer_name and message.analyzer_number:
            return "%s%s" % (
                message.analyzer_name,
                message.analyzer_number,
            )
        else:
            return None

    def _get_message_type(self, message):
        if message.type in ("RESULT", "RES-RR", "RES-BLK"):
            if self.qc_prefix and message.sample_id.startswith(self.qc_prefix):
                return self.TYPE_QC
            else:
                return self.TYPE_RESULT
        elif message.type in ("QC-RES-H", "QC-RES-M", "QC-RES-L"):
            return self.TYPE_QC
        else:
            return self.TYPE_OTHER

    def get_result_dict(self, message):
        graphics_available = lazy.import_all()

        if self._get_message_type(message) != self.TYPE_RESULT:
            return {}

        results = {
            message.sample_id: {
                "rack": message.analyzer_run_number,
                "resultats": [],
            },
        }

        analyseur = self.get_analyseur(message)
        if message.operator_code:
            operator = message.operator_code.strip()
        else:
            operator = ""
        alarmes_hemato = []

        for abx_id, value in message.tests.iteritems():

            if not value:
                continue

            if (
                graphics_available and
                abx_id in Pentra120GraphContainer.THRESHOLD_MAP.keys()
            ):
                results[message.sample_id]["resultats"].append({
                    "test": "%02x" % (abx_id, ),
                    "resu": "GRAPHE",
                    "graph": Pentra120GraphContainer(abx_id, message.tests),
                    "analyseur": analyseur,
                })
            elif abx_id in self.ALARME_FIELDS:
                if abx_id == 0x51:
                    if 'LI1' in value:
                        alarmes_hemato.append('LI1')
                    # uniquement des flags sur deux caractères
                    value_flags = value.replace('LI1', '')
                    alarmes_hemato.extend(re.findall('..', value_flags))
                else:
                    alarmes_hemato.extend([e for e in value.split() if e])
            elif abx_id not in Pentra120GraphContainer.GRAPH_ID:
                resu, alarmes = self.get_value_and_flag(value)
                results[message.sample_id]["resultats"].append({
                    "test": "%02x" % (abx_id, ),
                    "resu": resu,
                    "date": message.analysis_date,
                    "analyseur": analyseur,
                    "alarmes": alarmes,
                    "operator": operator,
                })

        if alarmes_hemato:
            # DEPRECATED !!

            results[message.sample_id]["resultats"].append({
                "test": "ALARM",
                "resu": ";".join(sorted(list(set(alarmes_hemato)))),
                "date": message.analysis_date,
                "analyseur": analyseur,
                "operator": operator,
            })
            results[message.sample_id]["resultats"].append({
                "test": "ALARME",
                "resu": "",
                "commentaires": sorted(list(set(alarmes_hemato))),
                "date": message.analysis_date,
                "analyseur": analyseur,
                "operator": operator,
            })
        return results

    def get_qc_result_list(self, message):
        if self._get_message_type(message) != self.TYPE_QC:
            return []

        results = []

        analyseur = self.get_analyseur(message)

        for abx_id, value in message.tests.iteritems():

            if abx_id in self.ALARME_FIELDS + Pentra120GraphContainer.GRAPH_ID:
                continue

            resu, _ = self.get_value_and_flag(value)
            results.append({
                "lot": message.sample_id,
                "test": "%02x" % (abx_id,),
                "resu": resu,
                "date": message.analysis_date,
                "analyseur": analyseur,
            })
        return results
