# -*- coding: utf8 -*-

from __future__ import unicode_literals

import json
import datetime
from collections import OrderedDict

from driverng import (
    Message,
    GraphContainer,
)
from driverng.utils import splitlines, RS
from driverng.fixed import (
    FixedLengthFormat,
    String,
    Date,
)


class PhoresysGraphContainer(GraphContainer):
    def __init__(self, points):
        self.points = points
        super(PhoresysGraphContainer, self).__init__(type_fichier="json")

    def get_bytes(self):
        return json.dumps({"points": self.points})


class PhoresysBase(FixedLengthFormat):
    pass


class PhoresysQuery(PhoresysBase):
    FMT_DESCRIPTION = [
        String("code", 1),
        String("patient_id", 15),
    ]


class PhoresysDemande(PhoresysBase):
    FMT_DESCRIPTION_HEADER = [
        String("code", 1),
        String("sample", 4),
        String("patient_id", 15),
        String("patient_name", 30),
        Date("birthdate", 8, fmt_date="%d%m%Y"),
        String("sex", 1),  # M/F
        String("age", 3, right_justified=True, empty="0"),
        String("department", 20, truncable=True),
        Date("sample_date", 8, fmt_date="%d%m%Y"),
        String("concentration", 5),
    ]
    FMT_DESCRIPTION = FMT_DESCRIPTION_HEADER + [
        String("free{}".format(i), 30) for i in range(1, 6)
    ]


class PhoresysExtraData(PhoresysBase):
    FMT_DESCRIPTION = [
        "**",
    ] + [String("programmable_field{}".format(i), 15) for i in range(1, 41)] + [
        String("card_comment", 256),
    ]


class PhoresysResultData(PhoresysBase):
    FMT_DESCRIPTION = [
        String("ndots", 4, right_justified=True, empty="0"),
        # A: Automatic scale
        # M: Manual scale
        String("curve_scale_flag", 1),
        String("curve_scale_factor", 4, right_justified=True, empty="0"),
    ]
    for i in range(1, 5):
        FMT_DESCRIPTION += [
            String("peak{}_start".format(i), 4, right_justified=True, empty="0"),
            String("peak{}_end".format(i), 4, right_justified=True, empty="0"),
        ]

    def __init__(self, *args, **kwargs):
        self._locked = False
        self.points = kwargs.pop("points", "")
        self.extra_data = PhoresysExtraData()
        self.extended_comment = ""
        super(PhoresysResultData, self).__init__(*args, **kwargs)

    def parse(self, data):
        remain = super(PhoresysResultData, self).parse(data)
        if "|" in remain:
            self.points, remain = remain.split("|", 1)
            if remain:
                self.extended_comment = self.extra_data.parse(remain)
            else:
                self.extended_comment = ""
        else:
            self.points = ""
            self.extended_comment = ""
        return remain

    def to_string(self):
        return "{}{}|{}".format(
            super(PhoresysResultData, self).to_string(),
            self.points,
            self.extra_data.to_string(),
            self.extended_comment,
        )


class PhoresysResult(PhoresysBase):
    FMT_DESCRIPTION = PhoresysDemande.FMT_DESCRIPTION_HEADER + [
        String("measurement_unit", 8),
    ] + [String("free{}".format(i), 30) for i in range(1, 6)] + [
        String("operator_id", 3),
        Date("date", 8, fmt_date="%d%m%Y"),
        String("fractions", 2, right_justified=True, empty="0"),
    ] + [
        String("fraction{}_name".format(i), 10) for i in range(1, 11)
    ] + [
        String("fraction{}_value".format(i), 5) for i in range(1, 11)
    ] + [
        String("fraction{}_conc".format(i), 5) for i in range(1, 11)
    ] + [
        String("area{}_name".format(i), 10) for i in range(1, 5)
    ] + [
        String("area{}_value".format(i), 5) for i in range(1, 5)
    ] + [
        String("area{}_conc".format(i), 5) for i in range(1, 5)
    ] + [
        # 0: Normal
        # 1: Pathological
        String("pathological_flag", 1),
        String("ratio1", 5),
        String("ratio2", 5),
        String("comment", 230, right_justified=True),
        # 0: Normal pattern
        # 1: Reference pattern
        String("reference_pattern_flag", 1),
        # 0: Normal sample
        # 1: QC sample
        String("qc_sample_flag", 1),
    ]

    def __init__(self, *args, **kwargs):
        # TODO
        self._locked = False
        self.data = None
        super(PhoresysResult, self).__init__(*args, **kwargs)

    def parse(self, data):
        remain = super(PhoresysResult, self).parse(data)
        if remain:
            self.data = PhoresysResultData()
            return self.data.parse(remain)
        return remain

    def to_string(self):
        return "{}{}".format(
            super(PhoresysResult, self).to_string(),
            self.data.to_string(),
        )


class PhoresysMultiple(list):
    QUERY, ORDER, RESULT = list(range(3))

    def add(self, type_message, **kwargs):
        m = {
            self.QUERY: PhoresysQuery,
            self.ORDER: PhoresysDemande,
            self.RESULT: PhoresysResult,
        }[type_message](**kwargs)
        self.append(m)
        return m


class Driver(Message):
    SPEC = {
        "code": "phoresys",
        "label": "Phoresys (Capillarys)",
    }

    ENCODING = "cp1252"
    OPTIONS = [
        Message.BooleanOption("qc_legacy", label="QC legacy", default=False),
        Message.BooleanOption(
            "qc_capillaire",
            label="Intégration du capillaire dans la référence de QC",
            default=False
        ),
        Message.BooleanOption(
            "phoresys_comment_textecode",
            label="Traiter les commentaires en textes codés",
            default=False,
        ),
        Message.Option("qc_prefix", "Préfixe QC"),
        Message.BooleanOption(
            "immunotyping",
            label="Gestion du profil immunotypage (immunotyping)",
            default=False,
        ),
        Message.BooleanOption(
            "envoi_ipp",
            label="Envoyer l'IPP dans la prescription",
            default=True,
        ),
        Message.Option(
            "position_analyseur",
            label="Positionnement de l'analyseur dans le champ additionnel dans le logiciel Phoresys",
            choices=[("1", u"1 - Analyseur"), ("2", u"2 - Analyseur")],
            default="1"),
        Message.Option(
            "position_no_capillaire",
            label="Positionnement du capillaire dans le champ additionnel dans le logiciel Phoresys",
            choices=[("0", u"Pas de gestion du capillaire"), ("1", u"1 - No Capillaire"), ("2", u"2 - No Capillaire")],
            default="1"),
    ]

    def __init__(
            self, qc_legacy=False, qc_capillaire=False,
            phoresys_comment_textecode=False, qc_prefix=None,
            immunotyping=False, envoi_ipp=True, position_analyseur="1",
            position_no_capillaire="1", **kwargs
    ):
        self.qc_legacy = qc_legacy
        self.qc_capillaire = qc_capillaire
        self.phoresys_comment_textecode = phoresys_comment_textecode
        self.qc_prefix = qc_prefix
        self.immunotyping = immunotyping
        self.envoi_ipp = envoi_ipp
        self.position_analyseur = position_analyseur
        self.position_no_capillaire = position_no_capillaire
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return PhoresysMultiple()

    def parse(self, data):
        messages = PhoresysMultiple()
        for line in splitlines(data):
            m = {
                16: PhoresysQuery,
                245: PhoresysDemande,
            }.get(len(data), PhoresysResult)()
            m.parse(data)
            messages.append(m)
        return messages

    def to_string(self, message):
        return "\n".join([m.to_string() for m in message])

    def get_message_type(self, message):
        for m in message:
            if isinstance(m, PhoresysQuery):
                return self.QUERY
            else:
                return self.ORDER

    def get_queries(self, message):
        queries = []
        for m in message:
            if isinstance(m, PhoresysQuery):
                queries.append(m.patient_id)
        return queries

    def get_order_message(self, worklist, context):

        if not context["query"]:
            return None

        msg = self.create()
        known_samples = []
        # sample_id -> profil
        profils = OrderedDict()

        for m in context["query"]:
            if isinstance(m, PhoresysQuery):
                profils[m.patient_id] = m.code

        for sample, action, priority in worklist.get_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "Urgent", "S": "Urgent", worklist.ELSE: ""},
        ):
            concentration = None
            for r in sample["results"]:
                # ne pas transmettre de résultat supérieur à 5 digits non
                # permis dans le protocole de communication
                if r["code"] == "PROTEINE" and len(r["value"]) <= 5:
                    concentration = r["value"]
            sex = {
                "M": "M",
                "F": "F",
            }.get(sample["sex"], "")
            patient_name = "{} {}".format(
                sample["last_name"],
                sample.get("first_name", ""))
            service = ""
            if sample["service"]:
                service = "{} {}".format(
                    sample["service"],
                    sample["service_name"] if sample["service_name"] else '',
                )
            elif sample["services"]:
                service = " ".join([
                    sample["services"][0]["code"],
                    sample["services"][0]["name"] if sample["services"][0]["name"] else '',
                ])
            elif sample.get("location_name"):
                service = "{} {}".format(
                    sample["location"],
                    sample.get("location_name"),
                )
            else:
                service = sample["location"]
            free2 = "{} {}".format(
                    sample["physician"] if sample["physician"] else '',
                    sample["physician_name"] if sample["physician_name"] else '')
            msg.add(
                msg.ORDER,
                code=profils[sample["sample_id"]],
                patient_id=sample["sample_id"],
                patient_name=patient_name[:30],
                birthdate=sample["birthdate"],
                sex=sex,
                sample_date=sample["collection_date"],
                department=service,
                concentration=concentration,
                free1=sample["lab_patient_id"] if self.envoi_ipp else "",
                free2=free2[:30],  # tronquer le nom du service prescripteur si trop long
                free5=priority,
            )
            known_samples.append(sample["sample_id"])

        for sample_id, profil in profils.items():
            if sample_id not in known_samples:
                msg.add(
                    msg.ORDER,
                    code=profil,
                    patient_id=sample_id,
                    patient_name="Inconnu",
                )

        if len(msg):
            return msg

    def get_resultats(self, message):
        r = []
        for i in range(1, 11):
            name, value, conc = [
                getattr(message, "fraction{}_{}".format(i, k)) for k in (
                    "name", "value", "conc")]
            if name and value:
                r.append({
                    "test": "{}%".format(name,),
                    "resu": value,
                })
            if name and conc:
                r.append({
                    "test": "{}#".format(name,),
                    "resu": conc,
                })
        return r

    @staticmethod
    def get_analyseur(position_analyseur, message):
        if (
            position_analyseur == "1" and
            message.data and
            message.data.extra_data and
            message.data.extra_data.programmable_field1
        ):
            return message.data.extra_data.programmable_field1
        elif(
            position_analyseur == "2" and
            message.data and
            message.data.extra_data and
            message.data.extra_data.programmable_field2
        ):
            return message.data.extra_data.programmable_field2
        else:
            return None

    @staticmethod
    def get_no_capillaire(position_no_capillaire, message):
        if (
            position_no_capillaire == "1" and
            message.data and
            message.data.extra_data and
            message.data.extra_data.programmable_field1
        ):
            return message.data.extra_data.programmable_field1
        elif(
            position_no_capillaire == "2" and
            message.data and
            message.data.extra_data and
            message.data.extra_data.programmable_field2
        ):
            return message.data.extra_data.programmable_field2
        else:
            return None

    def get_comments(self, raw_comment):
        comments = []
        for comment in raw_comment.split(RS):
            if self.phoresys_comment_textecode:
                comment = comment.split("~", 1)[0]
            comments.append(comment)
        return comments

    def get_result_dict(self, message):
        results = {}
        for m in message:
            if (
                m.qc_sample_flag == "1" or
                'CONTROL' in m.patient_id or
                (self.qc_prefix and m.patient_id.startswith(self.qc_prefix))
            ):
                return {}

            analyseur = self.get_analyseur(self.position_analyseur, m)
            tube = m.patient_id
            type_test = ""
            if self.immunotyping:
                patient_id_list = m.patient_id.split('-')
                if len(patient_id_list) > 1 and patient_id_list[1][:2] in ('EL', 'K', 'Ig', 'Re', 'L'):
                    tube = patient_id_list[0]
                    type_test = patient_id_list[1]

            results.setdefault(tube, {"resultats": []})
            resultats = self.get_resultats(m)
            alarmes = [m.pathological_flag]
            for i in range(1, 5):
                name, value, conc = [
                    getattr(m, "area{}_{}".format(i, k)) for k in (
                        "name", "value", "conc")]
                if name and value:
                    resultats.append({
                        "test": "{}{}%".format(type_test, name,),
                        "resu": value,
                    })
                if name and conc:
                    resultats.append({
                        "test": "{}{}#".format(type_test, name,),
                        "resu": conc,
                    })
            for r in resultats:
                results[tube]["resultats"].append({
                    "test": r["test"],
                    "resu": r["resu"],
                    "date": m.date,
                    "alarmes": alarmes,
                    "analyseur": analyseur,
                    "operator": m.operator_id.strip()
                })
            if m.data and m.data.points:
                points = []
                p = m.data.points
                while len(p) >= 4:
                    points.append(int(p[1:4], 16))
                    p = p[4:]
                if points:
                    results[tube]["resultats"].append({
                        "test": "{}{}GRAPHE".format(m.code, type_test),
                        "resu": "GRAPHE",
                        "date": m.date,
                        "alarmes": alarmes,
                        "graph": PhoresysGraphContainer(points),
                        "analyseur": analyseur,
                        "operator": m.operator_id.strip()
                    })
            if m.comment:
                tcomment = m.comment.split("~", 1)
                if len(tcomment) > 1:
                    comment = tcomment[1]
                else:
                    comment = m.comment
                results[tube]["resultats"].append({
                    "test": "{}{}COMMENT".format(m.code, type_test),
                    "resu": comment,
                    "date": m.date,
                    "alarmes": alarmes,
                    "analyseur": analyseur,
                    "operator": m.operator_id.strip()
                })

            if m.data:
                for code, raw_comment in (
                    ("COMMENT_EXTENDED", m.data.extended_comment),
                    ("CARD_COMMENT", m.data.extra_data.card_comment),
                ):
                    if not raw_comment:
                        continue

                    comments = self.get_comments(raw_comment)

                    if self.phoresys_comment_textecode:
                        value = ";".join(comments)
                    else:
                        value = "\n".join(comments)
                    results[tube]["resultats"].append({
                        "test": "{}{}{}".format(m.code, type_test, code),
                        "resu": value,
                        "date": m.date,
                        "alarmes": alarmes,
                        "analyseur": analyseur,
                    })

                    for idx, comment in enumerate(comments, 1):
                        results[tube]["resultats"].append({
                            "test": "{}{}{}{}".format(m.code, type_test, code, idx),
                            "resu": comment,
                            "date": m.date,
                            "alarmes": alarmes,
                            "analyseur": analyseur,
                        })

            if m.ratio1:
                results[tube]["resultats"].append({
                    "test": "{}{}RATIO1".format(m.code, type_test),
                    "resu": m.ratio1,
                    "date": m.date,
                    "alarmes": alarmes,
                    "analyseur": analyseur,
                    "operator": m.operator_id.strip()
                })
            if m.ratio2:
                results[tube]["resultats"].append({
                    "test": "{}{}RATIO2".format(m.code, type_test),
                    "resu": m.ratio2,
                    "date": m.date,
                    "alarmes": alarmes,
                    "analyseur": analyseur,
                    "operator": m.operator_id.strip()
                })
        return results

    def get_qc_result_list(self, message):
        for m in message:
            if (
                m.qc_sample_flag == "0" and
                'CONTROL' not in m.patient_id and (
                    not self.qc_prefix or
                    not m.patient_id.startswith(self.qc_prefix)
                )
            ):
                return []
            results = []
            now = datetime.datetime.now()
            date = datetime.datetime(*(
                m.date.timetuple()[:3] + now.timetuple()[3:5]))
            analyseur = self.get_analyseur(self.position_analyseur, m)
            if self.qc_legacy:
                lot = m.patient_id
            elif self.qc_capillaire:
                # le numéro de capillaire de 1 à 9 sur les capillarys réalisant
                # les A1C est dans le champ analyseur après |** et avant le
                # commentaire étendu
                capillaire = self.get_no_capillaire(self.position_no_capillaire, m)
                lot = "{}_{}".format(
                    m.patient_name.rsplit("-", 1)[1],
                    capillaire,
                )
            else:
                lot = m.patient_name.rsplit("-", 1)[1]
            for r in self.get_resultats(m):
                results.append({
                    "lot": lot,
                    "analyseur": analyseur,
                    "test": r["test"],
                    "date": date,
                    "resu": r["resu"],
                })
            return results
