# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class PianoSilSegmentH(HeaderSegment):
    sender_name = HprimString(5, default="TOSOH-G7")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PianoSilSegmentH2(HeaderSegment):
    sender_name = HprimString(5, default="TOSOH-G7")
    date = HprimDate(14)
    patients = HprimForeign("P", "header2")


class PianoSilSegmentP(Segment):
    sequence = HprimString(2)
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8)
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class PianoSilSegmentO(Segment):
    specimen_id = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        analyte=HprimString(4, required=True),
        dilution=HprimString(5),
    ))
    priority = HprimString(6)
    action = HprimString(12)
    specimen_type = HprimString(16, default="Sp.1")
    results = HprimForeign("R", "order")


class PianoSilSegmentR(Segment):
    code = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    reference_range = HprimString(6)
    result_abnormal_flags = HprimString(7)
    result_status = HprimString(9)
    operator = HprimString(11)
    date_result = HprimDate(13)
    instrument_name = HprimString(14)

    header2 = HprimForeign("H2", "result")
    end2 = HprimForeign("L", "result")


class PianoSilSegmentL(Segment):
    terminaison = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "piano_sil",
        "label": "Piano SIL EVO4 TOSOH",
    }
    SEGMENT_CLASS_MAP = {
        "H": PianoSilSegmentH,
        "H2": PianoSilSegmentH2,
        "P": PianoSilSegmentP,
        "O": PianoSilSegmentO,
        "R": PianoSilSegmentR,
        "L": PianoSilSegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "astm_piano_etendu",
            label="Protocole Piano Étendu",
            default=False),
        AstmMessageV2.Option(
            "piano_sil_code_emetteur",
            label="Code emetteur",
            default="LIS"),
        AstmMessageV2.Option(
            "piano_sil_code_recepteur",
            label="Code emetteur",
            default="HBDRIVE"),
    ]

    def __init__(
        self,
        astm_piano_etendu=False,
        piano_sil_code_emetteur="LIS",
        piano_sil_code_recepteur="HBDRIVE", **kwargs
    ):
        self.astm_piano_etendu = astm_piano_etendu
        self.piano_sil_code_emetteur = piano_sil_code_emetteur
        self.piano_sil_code_recepteur = piano_sil_code_recepteur
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = PianoSilSegmentH(type_segment="H", **kwargs)
        message.end = PianoSilSegmentL(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(
            context="order",
            sender_id=self.piano_sil_code_emetteur,
            receiver_id=self.piano_sil_code_recepteur,
        )
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_o.specimen_id,
                    request_id=seg_o.specimen_id,
                    patient_id=seg_p.patient_id,
                    last_name=seg_p.last_name,
                    first_name=seg_p.first_name,
                    birthdate=seg_p.birthdate,
                    sex=seg_p.sex,
                    priority=seg_o.priority,
                    specimen=seg_o.specimen_type,
                )
                for test in seg_o.tests:
                    sample.orders.get_or_create(
                        code=test.analyte,
                        dilution=test.dilution,
                        action="add",
                    )

        return worklist

    def encode_worklist(self, worklist, context, **kwargs):
        if worklist.context != "result":
            raise NotImplementedError("context %s" % (worklist.context,))

        message = self.create(
            date=context["now"],
        )
        worklist_sample_types = list(set([s.type for s in worklist.samples]))
        resultat_patient = 0
        if worklist_sample_types == ["patient"]:
            code_action = "N"
            resultat_patient = 1
        elif worklist_sample_types == ["control"]:
            code_action = "Q"
            resultat_patient = 0
        else:
            raise NotImplementedError("Multiple message types {}".format(worklist_sample_types))

        if resultat_patient:
            message = self.create(
                date=context["now"]
            )
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            priority={"S": "S", "A": "A", worklist.ELSE: "R"},
        ):
            if resultat_patient:
                seg_p = message.patients.add(
                    patient_id=patient.lab_patient_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                )
            list_orders = []
            nb = 1
            seg_r = None
            for sample, priority in patient.samples:
                for result in sample.results:
                    if result.code == "":
                        continue
                    if not resultat_patient:
                        if nb > 1:
                            seg_r.header2.add(
                                type_segment="H",
                                sender_name=result.instrument,
                                date=context["now"]
                            )
                            seg_r.end2.add()
                        seg_p = message.patients.add(sequence=1)
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        specimen_type="Sp.1",
                    )
                    code = result.code
                    test = seg_o.tests.add()
                    test.analyte = code
                    # La dilution est toujours 1
                    test.dilution = "1"
                    for lot in result.lots:
                        seg_o.specimen_id = lot.lot

                    if isinstance(result.value, basestring):
                        values = [{"value": result.value}]
                    elif isinstance(result.value, dict):
                        values = [result.value]
                    elif isinstance(result.value, list):
                        values = result.value
                    else:
                        raise NotImplementedError
                    if len(values) != 1:
                        raise NotImplementedError
                    value = values[0]

                    if result.flags:
                        flag = result.flags
                    else:
                        flag = "N"

                    # code opérateur ayant validé le résultat
                    if result.operator and (not result.validator or result.validator == 'auto'):
                        operator = result.operator
                    elif result.validator:
                        operator = result.validator
                    else:
                        operator = "auto"
                    seg_r = seg_o.results.add(
                        code=code,
                        value=value["value"],
                        units=result.unit,
                        reference_range=result.reference_ranges,
                        result_status="F",
                        result_abnormal_flags=flag[0],
                        operator=operator,
                        date_result=result.date,
                    )
                    if resultat_patient and result.instrument and result.instrument != "Manuel":
                        message.sender_name = result.instrument
                    elif nb == 1 and not resultat_patient and result.instrument:
                        message.sender_name = result.instrument
                    nb += 1
                    if self.astm_piano_etendu:
                        seg_o.action = code_action
                        seg_o.priority = priority
                        seg_r.instrument_name = result.instrument
                    list_orders.append(seg_o)

        if len(list_orders) > 0:
            return message
