# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    NoRangSegment,
    HeaderSegment,
    HprimString,
    HprimDate,
    HprimForeign,
)

logger = logging.getLogger("driverng.poccelarator")


class PoccelaratorSegmentMSH(HeaderSegment):
    model_name = HprimString((3, 1), default="MCA")
    device_id = HprimString((3, 2))
    sending_facility = HprimString(4)
    receiving_application = HprimString(5)
    receiving_facility = HprimString(6)
    date = HprimDate(7)
    message_type = HprimString((9, 1), default='ADR')
    message_code = HprimString((9, 2), default='A19')
    control_id = HprimString(10)
    processing_id = HprimString(11, default='P')
    version_id = HprimString(12, default='2.4')
    accept_type = HprimString(15, default='AL')
    application_type = HprimString(16, default='NE')
    character_set = HprimString(18, default='8859/1')

    ack = HprimForeign("MSA", "header")
    queries = HprimForeign("QRD", "header")
    patients = HprimForeign("PID", "header")
    orders = HprimForeign("OBR", "header")
    fin = HprimForeign("QRF", "header")

    #  Définit l'ordre des segments fils quand on genère le message
    SORT_CHILD_SEGMENT = ["ack", "queries", "commentaires", "patients", "orders", "fin"]


class PoccelaratorSegmentQRD(NoRangSegment):
    date = HprimDate(2)
    format_code = HprimString(3)
    priority = HprimString(4)
    query_id = HprimString(5)
    quantity = HprimString(8)
    barcode = HprimString(9)
    what_subject = HprimString(10)
    what_departement = HprimString(11)


class PoccelaratorSegmentMSA(NoRangSegment):
    code = HprimString(2, default='AA')
    control_id = HprimString(3)
    message = HprimString(4)
    error_code = HprimString(6, default='F')


class PoccelaratorSegmentNTE(Segment):
    comment_source = HprimString(3, default='D')
    comment_text = HprimString(4)


class PoccelaratorSegmentPID(Segment):
    patient_id = HprimString(3)
    lab_patient_id = HprimString(4)
    alternate_patient_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    street = HprimString((12, 1))
    town = HprimString((12, 3))
    zip_code = HprimString((12, 5))
    country_code = HprimString((12, 6))

    visits = HprimForeign("PV1", "patient")
    orders = HprimForeign("OBR", "patient")
    infos = HprimForeign("NTE", "patient")


class PoccelaratorSegmentPV1(Segment):
    location_ward = HprimString((4, 1))
    location_facility = HprimString((4, 4))
    attending_doctor = HprimString(8)
    referring_doctor = HprimString(9)
    consulting_doctor = HprimString(10)
    visit_number = HprimString(20)
    admin_date = HprimDate(46)

    orders = HprimForeign("OBR", "visit")
    orcs = HprimForeign("ORC", "visit")


class PoccelaratorSegmentORC(Segment):
    placer_order = HprimString(3)
    order_status = HprimString(6)


class PoccelaratorSegmentOBR(Segment):
    placer_order = HprimString(3)
    filler_order = HprimString(4)
    service_id = HprimString(5)
    date_request = HprimDate(7)
    date_debut = HprimDate(8)
    date_fin = HprimDate(9)
    action_code = HprimString(12, default='N')
    specimen_date = HprimDate(15)
    specimen_source = HprimString(16)
    date_entry = HprimString((28, 4))
    priority = HprimString((28, 6))
    technician = HprimString(35)

    results = HprimForeign("OBX", "order")
    common_order = HprimForeign("ORC", "order")
    comments = HprimForeign("NTE", "order")


class PoccelaratorSegmentOBX(Segment):
    value_type = HprimString(3)
    observation_identifier = HprimString((4, 1))
    observation_name = HprimString((4, 2))
    value = HprimString(6)
    units = HprimString(7)
    reference_range = HprimString(8)
    flag = HprimString(9)
    result_status = HprimString(12)  # F
    observer = HprimString(17)
    analyzer = HprimString(19)
    date = HprimDate(20)

    comments = HprimForeign("NTE", "result")


class PoccelaratorSegmentQRF(Segment):
    pass


class Driver(AstmMessageV2):
    SPEC = {
        "code": "poccelerator",
        "label": "Poccelarator Siemens"
    }
    SEGMENT_CLASS_MAP = {
        "MSH": PoccelaratorSegmentMSH,
        "MSA": PoccelaratorSegmentMSA,
        "NTE": PoccelaratorSegmentNTE,
        "PID": PoccelaratorSegmentPID,
        "PV1": PoccelaratorSegmentPV1,
        "OBR": PoccelaratorSegmentOBR,
        "OBX": PoccelaratorSegmentOBX,
        "ORC": PoccelaratorSegmentORC,
        "QRD": PoccelaratorSegmentQRD,
        "QRF": PoccelaratorSegmentQRF,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"
    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "poc_traitement_delocalise",
            label="Traitement délocalisé (auto création de demande avec le N° de séjour patient)"),
        AstmMessageV2.Option("qc_prefix", label="Préfixes QC (séparés par une virgule)"),
    ]

    def __init__(self, poc_traitement_delocalise=True, qc_prefix='', **kwargs):
        self.poc_traitement_delocalise = poc_traitement_delocalise
        self.qc_prefix = [e for e in qc_prefix.split(",") if e]
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return PoccelaratorSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        if not context["query"]:
            raise NotImplementedError
        msg = self.create(
            date=context["now"],
            control_id=context["query"].control_id
        )
        msg.ack.add(control_id=context["query"].control_id)
        query = context["query"].queries[0]
        msg.queries.add(
            date=query.date,
            format_code=query.format_code,
            priority=query.priority,
            query_id=query.query_id,
            quantity=query.quantity,
            barcode=query.barcode,
            what_subject=query.what_subject,
            what_departement=query.what_departement,
        )
        for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                priority={"A": "A", "S": "S", worklist.ELSE: "R"},
                action={"del": worklist.IGNORE, worklist.ELSE: "N"},
        ):
            seg_pid = msg.patients.add(
                patient_id=patient.patient_id,
                lab_patient_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex if patient.sex else "U",
                street=patient.address,
                zip_code=patient.postal_code,
                town=patient.city,
                country_code=patient.country,
            )
            doctor = ""
            for physician in patient.physicians:
                if physician.type == "attending":
                    doctor = physician.code
            seg_pid.visits.add(
                location_ward=patient.location,
                consulting_doctor=doctor,
                visit_number=patient.alternate_patient_id,
                admin_date=patient.admission_date,
            )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_qpd in message.queries:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_qpd.barcode,
                )
        else:
            worklist = Worklist(context="result")
            for patient in message.patients:
                if patient.visits:
                    seg_pv1 = patient.visits[0]
                else:
                    seg_pv1 = patient
                control = False
                for seg_obr in seg_pv1.orders:
                    for p in self.qc_prefix:
                        if seg_obr.placer_order.startswith(p):
                            control = True

                for seg_obr in seg_pv1.orders:
                        if not control:
                            if seg_obr.placer_order:
                                sample = worklist.get_or_create(
                                    sample_id=seg_obr.placer_order,
                                    type="patient",
                                )
                            elif self.poc_traitement_delocalise:
                                if patient.lab_patient_id:
                                    codebarre = patient.lab_patient_id
                                else:
                                    continue
                                sample = worklist.get_or_create(
                                    sample_id=codebarre,
                                    patient_id=patient.lab_patient_id,
                                    type="patient",
                                    request_info="Délocalisé",
                                    last_name=patient.last_name,
                                    first_name=patient.first_name,
                                    birthdate=patient.birthdate,
                                    sex=patient.sex,
                                    postal_code=patient.zip_code,
                                    city=patient.town,
                                    address=patient.street,
                                    country=patient.country_code,
                                    specimen=seg_obr.specimen_source,
                                    request_date=seg_obr.date_request if seg_obr.date_request else seg_obr.date_debut,
                                    collection_date=seg_obr.specimen_date,
                                )
                                sample.physicians.add(
                                    type="attending",
                                    code=seg_pv1.attending_doctor,
                                )
                                sample.services.add(
                                    code=seg_pv1.location_ward,
                                )
                            else:
                                continue
                        else:
                            sample = worklist.get_or_create(
                                sample_id=seg_obr.placer_order,
                                type="control",
                                specimen=seg_obr.specimen_source,
                            )
                        for seg_obx in seg_obr.results:
                            comments = []
                            for seg_nte in seg_obx.comments:
                                comments.append(seg_nte.comment_text)
                            sample.results.add(
                                code=seg_obx.observation_identifier,
                                type=seg_obx.value_type,
                                value=seg_obx.value,
                                flags=seg_obx.flag,
                                comments=comments,
                                date=seg_obx.date,
                                operator=seg_obx.observer,
                                instrument=seg_obx.analyzer,
                                unit=seg_obx.units
                            )
        return worklist
