# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimForeign,
)

logger = logging.getLogger("driverng.qiagen")


class QiagenSegmentMSH(HeaderSegment):
    sending_application = HprimString(3, default="MCA", required=True)
    sending_facility = HprimString(4)
    receiving_application = HprimString(5, default="QIAlink", required=True)
    date = HprimDate(7)
    message_type = HprimMultiple(
        9,
        m_type=HprimString(1, default="OML"),
        m_code=HprimString(2, default="O21"),
    )
    message_control_id = HprimString(10)
    processing_id = HprimString(11, required=True, default="P")
    version_id = HprimString(12, required=True, default="2.4")
    character_set = HprimString(18)

    patients = HprimForeign("SAC", "header")


class QiagenSegmentSAC(Segment):
    # tube
    container_identifier = HprimString(4)
    # Prelevement du tube
    specimen_source = HprimString(7)
    orders = HprimForeign("OBR", "patients")
    infos = HprimForeign("ORC", "patients")


class QiagenSegmentOBR(Segment):
    universal_service_identifier = HprimString(5)
    # Date when result has been released
    # from the instrument software
    date = HprimDate(8)
    # F: Analysis has been successful, valid results are available
    # X: A problem or error occured during analysis
    result_status = HprimString(26)
    technician = HprimString(35)
    results = HprimForeign("OBX", "order")


class QiagenSegmentOBX(Segment):
    # NM: Quantitative analyses
    # ST: Qualitative analyses
    value_type = HprimString(2)
    observation_identifier = HprimString(4)
    # Result of the analysis
    value = HprimString(6)
    units = HprimString(7)
    # Always F
    status = HprimString(12)


class QiagenSegmentORC(Segment):
    place_order_number = HprimString(3)
    filler_order_numer = HprimString(4)
    orders = HprimForeign("OBR", "infos")

    def to_string(self, separators=None):
        # order control
        # Only NW (New order) is supported
        # all others will be ignored
        self.rang_segment = "NW"
        return super(QiagenSegmentORC, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "qiagen",
        "label": "Qiagen",
    }
    DEFAULT_SEPARATORS = "|~^\&"

    SEGMENT_CLASS_MAP = {
        "MSH": QiagenSegmentMSH,
        "SAC": QiagenSegmentSAC,
        "OBR": QiagenSegmentOBR,
        "OBX": QiagenSegmentOBX,
        "ORC": QiagenSegmentORC,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    def create(self, **kwargs):
        return QiagenSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
                unique_together=["action"],
                action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            for sample, action in patient.samples:
                seg_sac = msg.patients.add(
                    container_identifier=sample.sample_id,
                    specimen_source=sample.specimen,
                )
                for order in sample["orders"]:
                    seg_orc = seg_sac.infos.add()
                    seg_o = seg_orc.orders.add()
                    seg_o.universal_service_identifier = order.code
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.message_type.m_type != "OUL":
            raise NotImplementedError

        worklist = Worklist(context="result")
        for seg_sac in message.patients:
            for seg_obr in seg_sac.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_sac.container_identifier,
                    type="patient",
                )
                for seg_obx in seg_obr.results:
                    if seg_obx.units:
                        test = "%s%s" % (
                            seg_obx.observation_identifier,
                            seg_obx.units,
                        )
                    else:
                        test = seg_obx.observation_identifier
                    sample.results.add(
                        code=test,
                        value=seg_obx.value,
                        date=seg_obr.date,
                    )
        return worklist
