# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class QuantalizerSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    receiver_name = HprimString(10)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1.0")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class QuantalizerSegmentP(Segment):
    patient_id = HprimString(3)
    laboratory_id = HprimString(4)
    patient_id_no3 = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    mother_maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    attending_physician = HprimString(14)
    status_patient = HprimString(16)
    diagnosis_code = HprimString((19, 1))
    diagnosis_label = HprimString((19, 2))
    location = HprimString(26)
    service_code = HprimString((33, 1))
    service_label = HprimString((33, 2))
    institution = HprimString(34)

    orders = HprimForeign("O", "patient")


class QuantalizerSegmentO(Segment):
    specimen_id = HprimString((3, 1))
    specimen_id_type = HprimString((3, 2))
    test = HprimString((5, 4))
    # R: Routine
    # S: Urgent
    priority = HprimString(6)
    request_date = HprimDate(7)
    collection_date = HprimDate(8)
    # C: Cancel
    # A: Add test
    action_code = HprimString(12)
    physician = HprimString(17)
    user_field1 = HprimString(19)
    # R: Rerun
    laboratory_field_no2 = HprimString(22)
    # C: Rerun if Rerun flag = R
    report_type = HprimString(26)
    specimen_service = HprimString(30)

    results = HprimForeign("R", "order")


class QuantalizerSegmentR(Segment):
    test = HprimMultiple(
        3,
        test_code=HprimString(4),
        test_label=HprimString(5),
        dilution=HprimString(7),
    )
    data_code = HprimString((4, 1))
    data_label = HprimString((4, 2))
    units = HprimString(5)
    reference_range = HprimString(6)
    abnormal_flag = HprimString(7)
    status = HprimString(9)
    date_change = HprimDate(10)


class QuantalizerSegmentC(Segment):
    comment_text = HprimString(4)


class QuantalizerSegmentL(Segment):
    terminator_code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "quantalizer",
        "label": "Quantalizer"
    }
    SEGMENT_CLASS_MAP = {
        "H": QuantalizerSegmentH,
        "P": QuantalizerSegmentP,
        "O": QuantalizerSegmentO,
        "C": QuantalizerSegmentC,
        "R": QuantalizerSegmentR,
        "L": QuantalizerSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = QuantalizerSegmentH(type_segment="H", **kwargs)
        message.end = QuantalizerSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):
            seg_p = msg.patients.add(
                laboratory_id=patient.lab_patient_id,
                first_name=patient.first_name,
                last_name=patient.last_name,
                mother_maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test=order.code,
                        priority=priority,
                        request_date=sample.request_date,
                        collection_date=sample.collection_date,
                        action_code=action,
                    )
                    if order.action == "add":
                        seg_o.report_type = "C"
                        seg_o.laboratory_field_no2 = "R"
                    else:
                        seg_o.report_type = "F"
                    seg_p.patient_id = sample.sample_id
                    for physician in sample.physicians:
                        seg_o.physician = physician.code

        if msg.patients:
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.specimen_id,
                    type="patient",
                )
                indice = 0
                for seg_r in seg_o.results:
                    if seg_r.status[0] != "F":
                        continue
                    commentaires = []
                    for seg_c in seg_r.commentaires:
                            commentaires.append(seg_c.comment_text)
                    if seg_r.abnormal_flag:
                        flags = [seg_r.abnormal_flag]
                    else:
                        flags = []
                    # le code test du segment O est concaténé avec le code test
                    # du segment R
                    if indice == 0:
                        test = "%s_%s" % (seg_o.test, seg_r.test.test_code,)
                        resu = seg_r.data_code
                    else:
                        # lorsqu'il y a des résultats multiples, le test est le
                        # résultat de l'aspect du prélèvement
                        test = "%s_ASPECT%d" % (seg_o.test, indice,)
                        resu = seg_r.test.test_code
                    sample.results.add(
                        code=test,
                        value=resu,
                        flags=flags,
                        comments=commentaires,
                        date=seg_r.date_change,
                    )
                    indice += 1
        return worklist
