# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class QwalysSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1), default="MCA")
    sender_name = HprimString((5, 2), default="MCA")
    sender_adress = HprimString(6)
    reserved_field = HprimString(7)
    receiver_id = HprimString((10, 1), default="QWALYS")
    receiver_name = HprimString((10, 2), default="QWALYS")
    version = HprimString(13)
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class QwalysSegmentP(Segment):
    practice_patient_id = HprimString(3)
    laboratory_patient_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    patient_address_part1 = HprimString((11, 1))
    patient_address_part2 = HprimString((11, 2))
    attending_physician_id = HprimString((14, 1))
    attending_physician_function = HprimString((14, 2))
    collection_date = HprimDate(24)
    service = HprimString(33)
    orders = HprimForeign("O", "patient")


class QwalysSegmentO(Segment):
    instrument_specimen_id = HprimString(4)
    universal_test_id = HprimMultiple(
        5,
        test=HprimString(4),
    )
    # S/R
    priority = HprimString(6)
    action_code = HprimString(12)
    lab_field_no1 = HprimString(21)
    date_result_reported = HprimDate(23)
    results = HprimForeign("R", "order")


class QwalysSegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        # G: interpretation des groupes
        # W: unité du résultat
        # A: auto-control result
        type_test=HprimString(4),
        num=HprimString(5),
        reagent=HprimString(6),
        test_name=HprimString(7),
        level=HprimString(8),
    )
    data = HprimString((4, 7))
    units = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimString(7)
    nature_abnormality = HprimString(8)
    result_status = HprimString(9)
    operator_id = HprimString(11)
    date_started = HprimDate(12)


class QwalysSegmentC(Segment):
    comment_text = HprimMultiple(
        3,
        comment1=HprimString(1),
        comment2=HprimString(2),
        comment3=HprimString(3),
        comment4=HprimString(4),
        comment5=HprimString(5),
    )


class QwalysSegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "qwalys",
        "label": "Qwalys",
    }
    SEGMENT_CLASS_MAP = {
        "H": QwalysSegmentH,
        "P": QwalysSegmentP,
        "O": QwalysSegmentO,
        "R": QwalysSegmentR,
        "C": QwalysSegmentC,
        "L": QwalysSegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    def __init__(self, qc_prefix=None, **kwargs):
        self.qc_prefix = qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = QwalysSegmentH(type_segment="H", **kwargs)
        message.end = QwalysSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "N", "new": "N"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                last_name=patient.last_name,
                first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        instrument_specimen_id=sample.sample_id,
                        priority=priority,
                        action_code=action,
                    )
                    for physician in sample.physicians:
                        seg_p.attending_physician_id = physician.code
                    seg_p.collection_date = sample.collection_date
                    seg_p.laboratory_patient_id = "%s%s" % (
                        sample.collection_date.strftime("%Y%m%d"),
                        patient.patient_id,
                    )
                    seg_o.universal_test_id.test = order.code
                    seg_p.practice_patient_id = sample.sample_id
        if len(msg.patients):
            return msg

    @staticmethod
    def get_resultats(seg_o):
        resultats = []
        resu = ''
        for seg_r in seg_o.results:
            # status du test Gn pour l'interprétation des groupes
            if seg_r.universal_test_id.type_test[0] == "G":
                if seg_r.universal_test_id.type_test == 'G0':
                    resu = seg_r.data
                elif seg_r.universal_test_id.type_test == 'G1':
                    resultats.append((
                        'G0G1',
                        "%s%s" % (resu, seg_r.data,),
                        seg_r.date_started,
                        seg_r.operator_id,
                    ))
                resultats.append((
                    seg_r.universal_test_id.type_test,
                    seg_r.data,
                    seg_r.date_started,
                    seg_r.operator_id,
                ))
            # si status W et presence du champ reagent R.3.6
            # récupération du résultat du reagent
            elif (
                seg_r.universal_test_id.type_test == "W" and
                seg_r.universal_test_id.reagent is not None
            ):
                resultats.append((
                    seg_r.universal_test_id.reagent,
                    seg_r.data,
                    seg_r.date_started,
                    seg_r.operator_id,
                ))
        return resultats

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                tube = seg_o.instrument_specimen_id
                for test, resu, date, operator in self.get_resultats(seg_o):
                    if tube.endswith('CQI'):
                        # QC
                        sample = worklist.get_or_create(
                            control_lot=tube,
                            sample_id=tube,
                            type="control",
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=tube,
                            type="patient",
                        )
                    r = sample.results.add(
                        code=test,
                        value=resu,
                        date=date,
                    )
                    if sample.type == 'patient' and operator:
                        r.operator = operator
        return worklist
