# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
from collections import OrderedDict

from driverng.utils import get_date, format_date
from driverng import Message

RS = "\x1e"  # Record Separator
FS = "\x1c"  # Field Separator
GS = "\x1d"  # Group Separator


class RapidlabField(object):

    def __init__(
        self, prefix=None, name=None, value=None, units=None, exc_group=None, rang=0
    ):
        self.prefix = prefix or "i"
        self.name = name or ""
        self.value = value or ""
        self.units = units or ""
        self.exc_group = exc_group or ""
        self.rang = rang
        super(RapidlabField, self).__init__()


class BaseRapidlabMessage(object):
    PREFIX = {
    }
    NON_TEST_FIELDS = (
        "PID", "FNAME", "LNAME", "SEX", "DOB", "DATE", "TIME",
        "MOD", "IID", "DEVICE", "TYPE", "SEQ", "RCartID",
        "WCartID", "QLEV", "QID", "AQCID", "STATUS",
    )

    DATE_FIELDS = {
        "DOB": "%d%b%Y",
        "DATE": "%d%b%Y",
        "TIME": "%H:%M:%S",
    }

    def __init__(self, message_type=None, **kwargs):
        self.message_type = message_type
        prefix = self.PREFIX.get(message_type, "i")
        self.fields = OrderedDict()
        for name, value in kwargs.items():
            self.fields[prefix + name] = RapidlabField(prefix, name, value)
        super(BaseRapidlabMessage, self).__init__()

    def __getitem__(self, key):
        return self.fields[key].value

    def __setitem__(self, key, value):
        if key not in self.fields:
            self.fields[key] = RapidlabField(name=key, value=value)
        else:
            self.fields[key].value = value
        if key == "DATE":
            self["TIME"] = value

    def parse(self, data):
        self.message_type, fields = data.split(FS, 1)
        rang = 0
        for line in fields[1:].split(FS):
            if line == RS:
                continue
            key, value, units, exc_group, _ = line.split(GS)
            prefix, name = key[0], key[1:]
            if name in self.DATE_FIELDS:
                value = get_date(value, fmt=self.DATE_FIELDS[name])
            if name in self.NON_TEST_FIELDS:
                fname = name
            else:
                fname = key
            self.fields[fname] = RapidlabField(
                prefix, name, value, units, exc_group, rang)
            rang += 1
        if "TIME" and "DATE" in self.fields:
            date = datetime.datetime(*(
                self.fields["DATE"].value.timetuple()[:3] +
                self.fields["TIME"].value.timetuple()[3:5]))
            self.fields["DATE"].value = self.fields["TIME"].value = date
        return self

    def to_string(self):
        fields = []
        for field in self.fields.values():
            if field.name in self.DATE_FIELDS:
                value = format_date(
                    field.value, self.DATE_FIELDS[field.name])
            else:
                value = field.value
            fields.append("%c%s%c%s%c%s%c%s%c" % (
                field.prefix, field.name, GS, value, GS,
                field.units, GS, field.exc_group, GS,
            ))
        return "%s%c%c%s%c%c" % (
            self.message_type,
            FS,
            RS,
            FS.join(fields),
            FS,
            RS,
        )


class Driver(Message):
    SPEC = {
        "code": "rapidlab",
        "label": "Rapidlab 1200",
    }

    OPTIONS = [
        Message.BooleanOption("prefix_code_test", label="Tenir compte du prefixe du code résultat", default=False),
    ]

    def __init__(self, prefix_code_test=False, **kwargs):
        self.prefix_code_test = prefix_code_test
        super(Driver, self).__init__(**kwargs)

    def create(self, message_type, **kwargs):
        return BaseRapidlabMessage(message_type, **kwargs)

    def get_message_type(self, message):
        if message.message_type == "PAT_DEMOG_REQ":
            return self.QUERY
        else:
            return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        return BaseRapidlabMessage().parse(data)

    def get_queries(self, message):
        queries = []
        if message.message_type != "PAT_DEMOG_REQ":
            return queries
        for key, field in message.fields.items():
            if key == "PID":
                queries.append(field.value)
        return queries

    def get_order_message(self, worklist, context):
        assert len(worklist.samples) <= 1, "Multitube non géré"
        iPID, iIID, aMOD = ('', '', '')
        if context["query"]:
            for key, field in context["query"].fields.items():
                if key == "PID":
                    iPID = field.value
                if key == "IID":
                    iIID = field.value
                if key == "MOD":
                    aMOD = field.value
            if len(worklist.samples) == 0:
                msg = self.create("PAT_DEMOG_NOT_AV")
                msg.fields['aMOD'] = RapidlabField('a', 'MOD', aMOD)
                msg.fields['iIID'] = RapidlabField('i', 'IID', iIID)
                msg.fields['iPID'] = RapidlabField('i', 'PID', iPID)

                return msg
        if worklist.samples:
            sample = worklist.samples[0]
            msg = self.create("PAT_DEMOG_DATA")
            if aMOD:
                msg.fields['aMOD'] = RapidlabField('a', 'MOD', aMOD)
            if iIID:
                msg.fields['iIID'] = RapidlabField('i', 'IID', iIID)
            if iPID:
                msg.fields['iPID'] = RapidlabField('i', 'PID', iPID)
            msg["PID"] = sample["sample_id"]
            msg["FNAME"] = sample["first_name"]
            msg["LNAME"] = sample["last_name"]
            msg["SEX"] = sample["sex"]
            msg["DOB"] = sample["birthdate"]
            msg["DATE"] = sample["collection_date"]
            return msg

    def get_results(self, message):
        results = []
        for key, field in message.fields.items():
            if field.name not in message.NON_TEST_FIELDS:
                if self.prefix_code_test:
                    results.append((key, field.value, field.exc_group))
                else:
                    results.append((field.name, field.value, field.exc_group))
        return results

    def get_result_dict(self, message):
        if message.message_type != "SMP_NEW_DATA":
            return {}
        results = {}
        for test, resu, flag in self.get_results(message):
            r = {
                "test": test,
                "resu": resu,
                "date": message["DATE"],
            }
            if flag.strip():
                r['alarmes'] = [flag, ]
            results.setdefault(message["PID"], {
                "resultats": []})["resultats"].append(r)
        return results

    def get_qc_result_list(self, message):
        if (
            message.message_type != "QC_NEW_DATA" or
            message["STATUS"] != "ACCEPTED"
        ):
            return []
        results = []
        for test, resu, flag in self.get_results(message):
            results.append({
                "lot": message["QID"] + message["QLEV"],
                "test": test,
                "resu": resu,
                "date": message["DATE"],
            })
        return results
