# -*- coding: utf8 -*-
from __future__ import unicode_literals

from collections import OrderedDict
from driverng import Worklist
from driverng.astm import (
    HprimString,
    HprimMultiple,
)
from driverng.drivers.hprim_spec import (
    HprimSegmentH,
    Driver as Hprim,
)


class REMISOLSegmentH(HprimSegmentH):
    emetteur = HprimMultiple(
        5,
        code=HprimString(1, default="MCA", obligatoire=True),
    )
    recepteur = HprimMultiple(
        10,
        code=HprimString(1, default="REMISOL", obligatoire=True),
    )


class Driver(Hprim):
    SPEC = {
        "code": "remisol",
        "label": "Remisol",
    }
    DEFAULT_SEPARATORS = "|^~\&"
    SEGMENT_CLASS_MAP = dict(Hprim.SEGMENT_CLASS_MAP.items() + {
        "H": REMISOLSegmentH,
    }.items())

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"],
            type_message="ORM",
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "N"},
        ):
                seg_p = msg.patients.add(
                    patient_demandeur=patient.patient_id,
                    nomjf=patient.maiden_name,
                    date_nai=patient.birthdate,
                    sexe=patient.sex,
                )
                seg_p.nom.nom = patient.last_name
                seg_p.nom.prenom = patient.first_name
                for sample, action, priority in patient.samples:
                    seg_o = seg_p.orders.add(
                        tube_demandeur=sample.sample_id,
                        no_demandeur=sample.request_id,
                        priorite=priority,
                        action=action,
                        date_prel=sample.collection_date,
                    )
                    presc = seg_o.prescripteur.add()
                    for physician in sample.physicians:
                        presc.medecin.nom = physician.code
                        presc.type_medecin = "L"
                    for service in sample.services:
                        presc.service.nom = sample.services[0].code
                    presc.type_service = "L"
                    for code in OrderedDict.fromkeys(
                            [o.code for o in sample.orders]):
                        test = seg_o.prescription.add()
                        test.code1 = code

                    # Envoi des résultats accueils et des antérios
                    for result in sample.results:
                        if not result.anterior:
                            # Pas de date si ce n'est pas une antério
                            result_date = None
                        else:
                            result_date = result.date
                        seg_r = seg_o.results.add(
                            code=result.code,
                            date=result_date,
                        )
                        for value in result.value.split(";"):
                            resu = seg_r.resu.add()
                            try:
                                float(value)
                                type_resultat = "NM"
                            except ValueError:
                                type_resultat = "CE"
                            resu.code1 = value
                        seg_r.type_resu = type_resultat

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.type_message != "ORU":
            raise NotImplementedError("type message %s" % (message.type_message,))
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_obr in seg_p.orders:
                if seg_obr.action == "Q":
                    sample = worklist.get_or_create(
                        control_lot=seg_obr.tube_demandeur,
                        sample_id=seg_obr.tube_demandeur,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.tube_demandeur,
                        type="patient",
                    )
                for seg_obx in seg_obr.results:
                    commentaires = []
                    for seg_c in seg_obx.commentaires:
                        commentaires.extend(seg_c.texte)
                    sample.results.add(
                        code=seg_obx.code,
                        date=seg_obr.date_resu,
                        value=";".join([r.code1 for r in seg_obx.resu]),
                        comments=commentaires,
                        flags=seg_obx.indic_norm,
                        instrument=seg_obx.droit_acces,
                    )
        return worklist
