# -*- coding: utf8 -*-

from __future__ import unicode_literals

from collections import OrderedDict
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
    HprimCE,
    HprimList,
    HprimCNA,
)


class ScanbacSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    sender_name = HprimString((5, 2))
    message_type = HprimString(7)
    receiver_id = HprimString((10, 1))
    receiver_name = HprimString((10, 2))
    processing_id = HprimString(12, default="P")
    version = HprimString((13, 1), default="A.2.")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True)


class ScanbacSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    orders = HprimForeign("OBR", "patient")


class ScanbacSegmentOBR(Segment):
    sample_id = HprimString((3, 2))
    tests = HprimList(5, basefield=HprimCE(0))
    priority = HprimString((6, 1))
    action = HprimString(12)
    prescriber = HprimCNA(17)

    results = HprimForeign("OBX", "order")


class ScanbacSegmentOBX(Segment):
    value_type = HprimString(3)
    test_id = HprimCE(4)
    sub_test_id = HprimString(5)
    value = HprimList(6, basefield=HprimCE(0))


class ScanbacSegmentC(Segment):
    comment_origine = HprimString(3)
    comment_text = HprimString(4)


class ScanbacSegmentL(Segment):
    patient_count = HprimString(4)
    line_count = HprimString(5)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "scanbac",
        "label": "Scanbac 3SI ASTM",
    }

    DEFAULT_SEPARATORS = r"|~^\&"
    SEGMENT_CLASS_MAP = {
        "H": ScanbacSegmentH,
        "P": ScanbacSegmentP,
        "OBR": ScanbacSegmentOBR,
        "OBX": ScanbacSegmentOBX,
        "C": ScanbacSegmentC,
        "L": ScanbacSegmentL,
    }

    OPTIONS = [
        AstmMessageV2.Option("code_emetteur", label="Code emetteur"),
        AstmMessageV2.Option("code_recepteur", label="Code recepteur"),
    ]

    def __init__(self, code_emetteur="", code_recepteur="", **kwargs):
        self.code_emetteur = code_emetteur or "MCA"
        self.code_recepteur = code_recepteur or "CERDISCAN"
        super(Driver, self).__init__(**kwargs)

    def to_string(self, message, *args, **kwargs):
        message.end.patient_count = len(message.patients)
        message.end.line_count = len(message.get_segments())
        return super(Driver, self).to_string(message, *args, **kwargs)

    def create(self, **kwargs):
        message = self.SEGMENT_CLASS_MAP["H"](type_segment="H", **kwargs)
        message.end = self.SEGMENT_CLASS_MAP["L"](type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        pass

    def decode_message(self, message):
        if message.message_type == "ORM":
            worklist = Worklist(
                context="order",
                sender_id=self.code_recepteur or message.sender_id,
                receiver_id=self.code_emetteur or message.receiver_id,
            )
        else:
            raise NotImplementedError("type message %s" % (message.message_type,))

        for seg_p in message.patients:
            for seg_obr in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.sample_id,
                    patient_id=seg_p.patient_id,
                    type="patient",
                )
                sample.update(
                    request_id=seg_obr.sample_id,
                    request_date=message.date,
                    collection_date=message.date,
                    last_name=seg_p.last_name,
                    first_name=seg_p.first_name,
                    birthdate=seg_p.birthdate,
                    sex=seg_p.sex if seg_p.sex in ("M", "F") else "U",
                )
                if seg_obr.prescriber:
                    sample.physicians.get_or_create(
                        code=seg_obr.prescriber.code,
                        last_name=seg_obr.prescriber.nom.nom,
                        type="prescriber",
                    )
                # Traitement des commentaires du patient et de l'obr
                for type, segments in (
                    ("patient", seg_p.commentaires),
                    ("sample", seg_obr.commentaires),
                ):
                    for seg_c in segments:
                        sample.comments.get_or_create(
                            type=type,
                            text=seg_c.comment_text.strip()
                        )
                for code in OrderedDict.fromkeys([
                    t.code1 for t in seg_obr.tests
                ] + [r.test_id.code1 for r in seg_obr.results]):
                    sample.orders.get_or_create(
                        code=code,
                        priority=seg_obr.priority or "R",
                        action="add" if seg_obr.action != "C" else "del",
                    )

                for seg_obx in seg_obr.results:
                    if seg_obx.value_type == "FIC":
                        value = ";".join([r.label1 for r in seg_obx.value])
                    else:
                        value = ";".join([r.code1 for r in seg_obx.value])

                    sample.results.get_or_create(
                        code=seg_obx.test_id.code1,
                        value=value,
                        sub_code=seg_obx.sub_test_id,
                        anterior=False,
                    )

        return worklist
