# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class ScanlabSegmentH(HeaderSegment):
    emetteur = HprimString(5, max_length=15)
    recepteur = HprimString(10, max_length=15)
    id_traitement = HprimString(12, default="P")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class ScanlabSegmentP(Segment):
    no_patient = HprimString(3)
    nom = HprimString((6, 1))
    prenom = HprimString((6, 2))
    nomjf = HprimString(7)
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")

    # M/F/U
    sexe = HprimString(9)
    service = HprimString(26)
    physician_id = HprimString(14, max_length=6)

    orders = HprimForeign("OBR", "patient")


class ScanlabSegmentOBR(Segment):
    # Pour le message de résultat ça dépend de la version du SCANLAB
    old_sample_id = HprimString((3, 1))
    sample_id = HprimString((3, 2), max_length=14)

    tests = HprimList(5, basefield=HprimMultiple(
        0, code=HprimString(1)))

    # S: Stat
    # R: Routine
    urgence = HprimString(6, max_length=1)

    collection_date = HprimDate(8)
    specimen_code = HprimString(16, max_length=3)

    results = HprimForeign("OBX", "order")


class ScanlabSegmentOBX(Segment):

    # code profile
    test = HprimString(4)

    # id_scanlab
    id_scanlab = HprimString(6)

    # flag serotheque
    flag = HprimString(9)

    # Contenant
    # selon le format suivant :
    # Congelat.:CXX;Rayon:RXX;Portoir:XXXXX;Repere:X;
    # le congelat est un identifiant unique
    contenant = HprimString(13)

    # date du contenant
    date = HprimDate(14)


class ScanlabSegmentL(Segment):

    # terminaison
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "scanlab",
        "label": "Scanlab",
    }

    SEGMENT_CLASS_MAP = {
        "H": ScanlabSegmentH,
        "P": ScanlabSegmentP,
        "OBR": ScanlabSegmentOBR,
        "OBX": ScanlabSegmentOBX,
        "L": ScanlabSegmentL,
    }
    DEFAULT_SEPARATORS = "|^~\\&"

    def create(self, **kwargs):
        message = ScanlabSegmentH(type_segment="H", **kwargs)
        message.end = ScanlabSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", "A": "A", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                no_patient=patient.patient_id,
                nom=patient.last_name,
                prenom=patient.first_name,
                nomjf=patient.maiden_name,
                date_naissance=patient.birthdate,
                sexe=patient.sex,
                service=patient.location,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    urgence=priority,
                    specimen_code=sample.specimen,
                    collection_date=sample.collection_date,
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code

        if len(msg.patients):
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.sample_id or seg_o.old_sample_id,
                    )
                    for seg_r in seg_o.results:
                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.contenant,
                            date=seg_r.date,
                        )
        return worklist
