# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class SofiaSegmentH(HeaderSegment):
    analyzer_name = HprimString((5, 1))
    serial_number = HprimString((5, 2))
    processing_id = HprimString(12)
    firmware_version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class SofiaSegmentP(Segment):
    patient_id = HprimString(3)
    location = HprimString(26)
    orders = HprimForeign("O", "patient")


class SofiaSegmentO(Segment):
    order_id = HprimString(3)
    test_type_name = HprimString(5)
    operator_id = HprimString(11)
    sample_type = HprimString(16)

    results = HprimForeign("R", "order")


class SofiaSegmentC(Segment):
    sample_comment = HprimString(4)


class SofiaSegmentR(Segment):
    test_name = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    reference_range = HprimString(6)
    flag = HprimString(7)
    result_type = HprimString(9)
    date_result = HprimDate(13)


class SofiaSegmentL(Segment):
    terminaison = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "sofia",
        "label": "Sofia Quidel"
    }
    SEGMENT_CLASS_MAP = {
        "H": SofiaSegmentH,
        "P": SofiaSegmentP,
        "O": SofiaSegmentO,
        "R": SofiaSegmentR,
        "C": SofiaSegmentC,
        "L": SofiaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = SofiaSegmentH(type_segment="H", **kwargs)
        message.end = SofiaSegmentL(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.sample_type == "Q":
                    sample = worklist.get_or_create(
                        sample_id=seg_o.order_id,
                        control_lot=seg_o.order_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.order_id,
                        type="patient",
                    )
                for seg_r in seg_o.results:
                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        commentaires.append(seg_c.sample_comment)
                    sample.results.add(
                        code=seg_r.test_name,
                        value=seg_r.value,
                        flags=[seg_r.flag],
                        comments=commentaires,
                        operator=seg_o.operator_id,
                        date=seg_r.date_result,
                        instrument=message.analyzer_name,
                    )
        return worklist
