# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import MessageV2, Worklist
from driverng.fields import String


class SortingDriveBase(object):
    FIELDS = ()

    def __init__(self, **kwargs):
        for field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        super(SortingDriveBase, self).__init__()

    def parse(self, data=None):
        fields = self.FIELDS
        for idx, elt in enumerate(data.split(";")):
            if not elt:
                continue
            try:
                field = fields[idx]
            except KeyError:
                pass
            else:
                value = field.to_python(elt)
                setattr(self, field.name, value)

    def to_string(self):
        ligne = []
        for field in self.FIELDS:
            ligne.append("%s" % field.to_string(getattr(self, field.name)))
        return "%s" % (";".join(ligne))


class SortingDriveOrderFile1(SortingDriveBase):
    # Protocole Version File 1
    FIELDS = (
        String("sample_id", 50),
        # Can be empty
        String("lab_sid", 50),
        String("emergency", 1),
        String("lis_action", 1),
        String("sex", 1),
        # Can be empty
        String("age"),
        String("patient_info", 50),
        String("tests"),
    )


class SortingDriveResult(SortingDriveBase):
    # Protocole Version File 1
    FIELDS = (
        String("SubRule"),
        String("sample_id", 30),
        String("lab_sid", 50),
        String("tube"),
        String("wpflag"),
        String("volume"),
        String("archive_id"),
        String("rack_id"),
        String("row"),
        String("col"),
        String("timestamp"),
    )


class SortingDriveMultipleMessage(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: SortingDriveOrderFile1,
        MessageV2.RESULT: SortingDriveResult,
    }

    def add(self, message_type=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[message_type](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not len(line):
                continue
            splitted = line.split(";")
            if len(splitted) == 8 and splitted[7].strip():
                message_type = MessageV2.ORDER
            else:
                message_type = MessageV2.RESULT
            obj = self.add(message_type=message_type)
            obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "sorting_drive_file1",
        "label": "Sorting Drive File Version-1 (Version Obsolète) Beckman-Coulter",
    }

    OPTIONS = [
        MessageV2.BooleanOption(
            "validation_preanalytique",
            label="Validation pré-analytique des tubes"
        ), ]
    MESSAGE_TYPE_MAP = {
        MessageV2.ORDER: SortingDriveOrderFile1,
        MessageV2.RESULT: SortingDriveResult,
    }

    def create(self, message_type, **kwargs):
        return SortingDriveMultipleMessage()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = SortingDriveMultipleMessage()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        msg = self.create(MessageV2.ORDER)
        for sample, action, priority, rerun in worklist.get_worklist(
            unique_together=["action", "priority", "rerun"],
            action={"del": "2", "add": "1", "new": "1"},
            priority={"A": "1", "S": "1", worklist.ELSE: "0"},
        ):
            # Ne pas transmettre les reruns au Sorting-Drive
            # Le tube n'est plus présent sur le trieur lors des reruns
            if rerun:
                continue
            patient_info = "%s-%s" % (
                sample.last_name,
                sample.first_name,
            )
            # OrgID: Department or Laboratory Identifier
            # specimen type au format
            # <MatCode1>^<BarcodeExt1>~<MatCode2>^<BarcodeExt2>~...~<MatCodeN>^<BarcodeExtN>
            # attente précisions de la part de Beckman

            # Liste des tests
            codes_tests = "|".join([order["code"] for order in sample.orders])
            tests = "%s|" % (codes_tests,)
            msg.add(
                message_type=MessageV2.ORDER,
                sample_id=sample.sample_id,
                lab_sid=sample.sample_id,
                emergency=priority,
                lis_action=action,
                sex=sample.sex,
                patient_info=patient_info,
                tests=tests,
            )
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            # Message de type Result
            sample = worklist.get_or_create(
                sample_id=m.sample_id,
                rack=m.rack_id,
                type="patient",
            )
            commentaires = []
            # Message de type Tube Recognition lorsqu'il n'y a pas de tests
            # L'id du rack est toujours mis à 0 désignant l'entrée dans le
            # trieur
            if "_" in m.col:
                # Date d'acquitement du tube
                # au format YYYYMMDD_HHMMSS
                date_acquitement = None
                if m.col and len(m.col) == 15:
                    date_acquitement = datetime.datetime.strptime(m.col, "%Y%m%d_%H%M%S")
                if m.volume:
                    volume = "Volume: %s" % (m.volume,)
                    commentaires.append(volume)
                if m.tube:
                    tube_typ = "Cap color: %s" % (m.tube,)
                    commentaires.append(tube_typ)
                sample.results.add(
                    code="TRACKINGT",
                    value=m.rack_id,
                    date=date_acquitement,
                    comments=commentaires,
                )
            else:
                # message de type result contenant l'ID des racks des
                # différents modules composant le trieur
                commentaires = []
                if m.timestamp:
                    date = datetime.datetime.strptime(m.timestamp, "%y.%m.%d %I:%M:%S %p")
                if m.archive_id and m.archive_id != 'N/A':
                    commentaires.append(m.archive_id)
                sample.results.add(
                    code="TRACKING",
                    value=m.rack_id,
                    date=date,
                    comments=commentaires,
                    preanalytique=True,
                )
        return worklist
