# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class Sp1000iSegmentH(HeaderSegment):
    sender_id = HprimMultiple(
        5,
        analyser_name=HprimString(1),
        software_version=HprimString(2),
        analyzer_serial_no=HprimString(3),
        ps_code=HprimString(7),
    )
    version = HprimString(13, default="E1394-97")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class Sp1000iSegmentP(Segment):

    # Le segment P contient uniquement le numéro de séquence
    orders = HprimForeign("O", "patient")


class Sp1000iSegmentO(Segment):
    specimen_id = HprimMultiple(
        3,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_no=HprimString(3),
        sample_attribute=HprimString(4, default="C"),
    )
    instrument_specimen_id = HprimMultiple(
        4,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_no=HprimString(3),
        sample_attribute=HprimString(4),
    )
    # universal test_id
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        smear=HprimString(1, default="SMEAR"),
        # valeur du HCT
        hct_value=HprimString(2, default="0"),
        # nombre de WBC
        wbc_count=HprimString(3, default="0"),
        # nombre de RBC
        rbc_count=HprimString(4, default="0"),
        sheet_of_sildes=HprimString(5, default="1"),
        specify_slide_glass1=HprimString(6, default="0"),
        specify_slide_glass2=HprimString(7, default="0"),
    ))
    order_date = HprimDate(7)
    action_code = HprimString(12)
    report_type = HprimString(26)

    results = HprimForeign("R", "order")
    comments = HprimForeign("C", "order")


class Sp1000iSegmentQ(Segment):

    starting_range_id = HprimMultiple(
        3,
        rack_no=HprimString(1),
        tube_position=HprimString(2),
        sample_no=HprimString(3),
        sample_id_attribute=HprimString(4),
    )
    beginning_request = HprimDate(7)
    user_field = HprimString(11)
    # champ fixe avec 8 espaces
    information_status_code = HprimString(13, default="        ")


class Sp1000iSegmentR(Segment):
    universal_test_id = HprimMultiple(
        3,
        parameter=HprimString(5),
    )
    value = HprimString(4)
    date = HprimDate(13)


class Sp1000iSegmentC(Segment):
    comment_text = HprimMultiple(
        4,
        info1=HprimString(1),
        info2=HprimString(2),
        info3=HprimDate(3),
    )


class Sp1000iSegmentL(Segment):
    termination_code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "sp1000i",
        "label": "SP1000i ASTM",
    }
    SEGMENT_CLASS_MAP = {
        "H": Sp1000iSegmentH,
        "P": Sp1000iSegmentP,
        "O": Sp1000iSegmentO,
        "Q": Sp1000iSegmentQ,
        "R": Sp1000iSegmentR,
        "C": Sp1000iSegmentC,
        "L": Sp1000iSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = Sp1000iSegmentH(type_segment="H", **kwargs)
        message.end = Sp1000iSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.starting_range_id.sample_no.strip())
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add()
            for sample, action in patient["samples"]:
                seg_o = seg_p.orders.add(
                    order_date=sample["collection_date"],
                    action_code=action,
                    report_type="Q",
                )
                seg_o.specimen_id.sample_no = "%15s" % (sample["sample_id"])
                if context["query"] is not None:
                    for seg_q in context["query"].queries:
                        if seg_q.starting_range_id.sample_no.strip() == sample["sample_id"]:
                            for attr in (
                                "rack_no", "tube_position",
                                "sample_id_attribute",
                            ):
                                setattr(
                                    seg_o.specimen_id, attr, getattr(
                                        seg_q.starting_range_id, attr))
                test = seg_o.tests.add()
                for attr, fmt, code in (
                    ("hct_value", "%s", "HCT"),
                    ("rbc_count", "%.3s", "RBC"),
                    ("wbc_count", "%.3s", "WBC"),
                ):
                    for result in sample["results"]:
                        if result["code"] == code:
                            value = result["value"].replace(".", "").replace(",", "")
                            setattr(test, attr, fmt % value)

                seg_c = seg_o.comments.add()
                seg_c.comment_text.info1 = "%15s" % (patient["patient_id"],)
                seg_c.comment_text.info2 = "%15s" % (patient["last_name"],)
                seg_c.comment_text.info3 = sample["collection_date"]

        if msg.patients:
            return msg
        elif context["query"]:
            for seg_q in context["query"].queries:
                seg_p = msg.patients.add()
                seg_o = seg_p.orders.add(
                    order_date=context["now"],
                    action_code="N",
                    report_type="Q",
                )
                for attr in (
                    "rack_no", "tube_position", "sample_no",
                ):
                    setattr(seg_o.specimen_id, attr, getattr(
                        seg_q.starting_range_id, attr))
                test = seg_o.tests.add()
                test.sheet_of_sildes = "0"
                seg_o.comments.add()
            return msg

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if (
                    seg_o.instrument_specimen_id.rack_no and
                    seg_o.instrument_specimen_id.tube_position
                ):
                    rack = "%s-%s" % (
                        seg_o.instrument_specimen_id.rack_no.strip(),
                        seg_o.instrument_specimen_id.tube_position.strip(),
                    )
                else:
                    rack = None

                tube = seg_o.instrument_specimen_id.sample_no.strip()
                results.setdefault(tube, {
                    "resultats": [],
                    "rack": rack,
                })
                for seg_r in seg_o.results:
                    results[tube]["resultats"].append({
                        "test": seg_r.universal_test_id.parameter,
                        "resu": seg_r.value,
                        "date": seg_r.date,
                    })

        return results
