# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class SSASegmentH(HeaderSegment):
    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class SSASegmentP(Segment):
    patient_id = HprimString(3)
    external_id = HprimString(4)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    gender = HprimString(9)
    orders = HprimForeign("O", "patient")


class SSASegmentO(Segment):

    sample_id = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(
        0, code=HprimString(4)))
    priority = HprimString(6, default="R")
    order_date = HprimDate(7)
    action_code = HprimString(12, default="N")

    # Corresponds with the field "TYPE" in the
    # configuration of the automate.
    # Default: BABYSA
    automate_type = HprimString(25)

    # F: Tube processed correctly
    # X: Processing error
    # O: Order
    report_code = HprimString(26)
    results = HprimForeign("R", "order")


class SSASegmentR(Segment):

    code = HprimString((3, 3))
    test = HprimString((3, 4))
    plate_id = HprimString((4, 1))
    plate_position = HprimString((4, 2))
    units = HprimString(5)
    volume = HprimString(6)

    # N: Normal
    # L: Fluid level insufficient
    flag = HprimString(7)

    result_type = HprimString(8, default="F")
    operator = HprimString(9)
    date_start = HprimDate(10)
    date_end = HprimDate(11)

    # Serial number of the automate
    serial = HprimString(12)


class SSASegmentC(Segment):
    # I: Instrument
    source = HprimString(3, default="I")
    naliquots = HprimString(4)

    # I: All the aliquots have been distributed
    # P: Not alle the required aliquots where performed
    comment_type = HprimString(5)


class SSASegmentL(Segment):
    # N: Normal
    # E: Abnormal
    code = HprimString((3, 1))
    batch = HprimString((3, 2))


class Driver(AstmMessageV2):
    SPEC = {
        "code": "ssa",
        "label": "SSA",
    }
    SEGMENT_CLASS_MAP = {
        "H": SSASegmentH,
        "P": SSASegmentP,
        "O": SSASegmentO,
        "R": SSASegmentR,
        "L": SSASegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = SSASegmentH(type_segment="H", **kwargs)
        message.end = Segment(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create()
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = msg.patients.add(
                external_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                maiden_name=patient.maiden_name,
                birthdate=patient.birthdate,
                gender=patient.sex,
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    order_date=sample.request_date,
                    report_code="O",
                )
                t = seg_o.tests.add()
                t.code = "SERO"
                for order in sample.orders:
                    seg_o.results.add(
                        code=order.code,
                        test="SERO",
                    )

        if len(msg.patients):
            return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(type="patient", sample_id=seg_o.sample_id)
                for seg_r in seg_o.results:
                    if seg_r.plate_position:
                        value = "%s-%s" % (seg_r.plate_id, seg_r.plate_position)
                    else:
                        value = seg_r.plate_id
                    sample.results.add(
                        code=seg_r.code,
                        date=seg_r.date_end,
                        flags=[seg_r.flag] if seg_r.flag else [],
                        value=value,
                        operator=seg_r.operator,
                    )
        return worklist
