# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.fixed import FixedLengthFormat, String


class StarrsedQuery(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "ESRSR   ",
        String("instrument_id", 20),
        String("sample_id", 40),
        String("reserved", 55),
        # R: Request
        String("request", 1),
        String("space", 1),
    ]


class StarrsedOrder(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "ESRSR   ",
        String("instrument_id", 20),
        String("sample_id", 40),
        String("reserved", 55),
        # Y: ESR Yes
        # N: EST No
        String("workorder", 1),
        String("space", 1),
    ]


class StarrsedResult(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "ESRRE   ",
        String("instrument_id", 20),
        String("sample_id", 40),
        String("reserved", 15),
        # date d'aspiration mais pas date du résultat
        # ddmmyyyy
        String("aspiration_date", 10),
        # hhmm
        String("aspiration_time", 5),
        String("esr30", 5),
        String("esr60mm", 5),
        String("esr60", 5),
        String("esr120", 5),
        String("reserved", 10),
        # 0: Patient sample
        # 1: QC normal
        # 2: QC abnormal
        String("sample_code", 5),
        String("aspect_code", 5),
        String("manually_added_code", 5),
        String("pipette_number", 5),
        String("sedimentation_time", 5),
        String("temperature", 5),
        String("dilution_rate", 5),
        String("column_height", 5),
        # 0: No Error
        # 1: No cells/ plasma found
        # 2: ESR Probably >140 mm
        # 3: Too many borders found
        # 4: Column height
        # 5: Measure error
        # 6: Bubbles on top
        # 7: Limit error
        String("error_code", 5),
        String("limit_error", 30),
        String("reserved", 50),
    ]


class Driver(MessageV2):
    SPEC = {
        "code": "starrsed",
        "label": "Starrsed Mechatronics-01 bidirectional",
    }
    TYPE_MESSAGE_MAP = {
        "REQ": StarrsedQuery,
        "ESRSR": StarrsedOrder,
        "ESRRE": StarrsedResult,
    }

    def create(self, message_type, **kwargs):
        return self.TYPE_MESSAGE_MAP[message_type](**kwargs)

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        message_type = data[0:5]
        if message_type == "ESRSR" and data[123] == "R":
            message_type = "REQ"
        obj = self.TYPE_MESSAGE_MAP[message_type]()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        )
        if len(samples) > 1:
            raise RuntimeError("Multi-sample not implemented")
        elif len(samples) == 1:
            sample = samples[0][0]
            return self.create(
                "ESRSR",
                sample_id=sample.sample_id,
                workorder="Y",
            )
        elif context["query"]:
            return self.create(
                "ESRSR",
                sample_id=context["query"].sample_id,
                workorder="N",
            )

    def decode_message(self, message):
        if isinstance(message, StarrsedQuery):
            worklist = Worklist(context="query")
            worklist.get_or_create(type="patient", sample_id=message.sample_id.strip())
            return worklist

        worklist = Worklist(context="result")
        if message.sample_code == "0":
            sample = worklist.get_or_create(
                sample_id=message.sample_id.strip(),
                type="patient",
            )
        else:
            sample = worklist.get_or_create(
                control_lot=message.sample_id.strip(),
                sample_id=message.sample_id.strip(),
                type="control",
            )

        # ESR 30 mn
        if message.esr30:
            sample.results.add(
                code="ESR30",
                value=message.esr30,
                flags=[message.error_code],
            )
        else:
            sample.results.add(
                code="ESR30",
                value="",
                flags=[message.error_code],
            )
        # ESR 60 mn
        if message.esr60:
            sample.results.add(
                code="ESR60",
                value=message.esr60,
                flags=[message.error_code],
            )
        else:
            sample.results.add(
                code="ESR60",
                value="",
                flags=[message.error_code],
            )
        # ESR 120 mn
        if message.esr120:
            sample.results.add(
                code="ESR120",
                value=message.esr120,
                flags=[message.error_code],
            )
        else:
            sample.results.add(
                code="ESR120",
                value="",
                flags=[message.error_code],
            )
        return worklist
