# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class TangoSegmentH(HeaderSegment):
    sender_id = HprimString(5, max_length=19)
    receiver_id = HprimString(10, max_length=19)
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class TangoSegmentP(Segment):

    # If the TANGO system operate in the mode
    # PatIdUnique=yes (see tango.ini file), then P.4
    # is a mandatory field.
    # else if PatIdUnique=no P.6.1/P.6.2 and P.8 are mandatory
    # Additionally the host can use a patient id (field 4) to
    # distinguish patients that have the same name and
    # date of birth.
    patient_id = HprimString(4)
    patient_name = HprimString((6, 1))
    patient_firstname = HprimString((6, 2))
    mothers_maiden_name = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")

    # M/F
    sex = HprimString(9, max_length=1)

    physician_id = HprimString(14)

    orders = HprimForeign("O", "patient")


class TangoSegmentO(Segment):
    specimen_id = HprimString(3)

    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
    ))

    # S: STAT
    # N: Normal
    priority = HprimString(6, max_length=1)

    requested_date = HprimDate(7)

    # S: Serum
    # P: Plasma
    # C: Cells
    specimen_descriptor = HprimString(16, max_length=1)

    # O: Order record
    # F: Mark final results
    report_type = HprimString(26, max_length=1)

    results = HprimForeign("R", "order")


class TangoSegmentR(Segment):
    test_abbrev = HprimString((3, 4))
    test_id = HprimString((3, 5))

    # AB0: AB0 system
    # RhF: Rhesus factor
    # RhS: Rh system
    # Kell: Kell system
    # AS: Antibody search
    # CDE: anti CDE pool test
    # DAT: Direct anti globulin test
    # AI: Antibody identification
    # Actl: Auto control for SSII
    # ASE: Antibody screening enzyme
    # ASED: Antibody screening enzyme details
    # DVI: DVI test
    # Rem: Validator Remarks
    # Strip: Strip status followed by a comment record containing the well
    # results
    test_category = HprimString((3, 6))

    value = HprimString(4)
    # F: Final
    status = HprimString(9)
    # Tango login name of operator (validator)
    operator = HprimString(11)
    date = HprimDate(13)
    instrument_id = HprimString(14)


class TangoSegmentC(Segment):
    # I: Tango is Sender
    source = HprimString(3)
    text = HprimString(4)


class TangoSegmentQ(Segment):
    # ALL or a list of sample_id
    range_id = HprimList(3)
    test_id = HprimString(5, default="ALL")
    request_status = HprimString(13, default="O")


class TangoSegmentL(Segment):
    # N: Normal
    # I: No info available
    code = HprimString(3, default="N")


class Driver(AstmMessage):
    SPEC = {
        "code": "tango",
        "label": "Tango",
    }
    SEGMENT_CLASS_MAP = {
        "H": TangoSegmentH,
        "P": TangoSegmentP,
        "O": TangoSegmentO,
        "R": TangoSegmentR,
        "Q": TangoSegmentQ,
        "C": TangoSegmentC,
        "L": TangoSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = TangoSegmentH(type_segment="H", **kwargs)
        message.end = TangoSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        return self.ORDER

    def get_queries(self, message):
        samples = []
        for seg_q in message.queries:
            samples.extend(seg_q.range_id)
        return list(set(samples))

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient["patient_id"],
                patient_name=patient["last_name"],
                patient_firstname=patient["first_name"],
                mothers_maiden_name=patient["maiden_name"],
                birthdate=patient["birthdate"],
                sex=patient["sex"] if patient["sex"] in ("M", "F") else "",
            )
            for sample, action, priority in patient["samples"]:
                if sample["specimen"] in ("S", "P", "C"):
                    specimen_descriptor = sample["specimen"]
                else:
                    specimen_descriptor = ""
                seg_o = seg_p.orders.add(
                    specimen_id=sample["sample_id"],
                    priority=priority,
                    report_type="O",
                    specimen_descriptor=specimen_descriptor,
                )
                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.code = order["code"]

        if len(msg.patients):
            return msg

    def get_result_dict(self, message):
        result = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                result.setdefault(seg_o.specimen_id, {"resultats": []})
                for seg_r in seg_o.results:

                    if seg_r.status != "F":
                        continue

                    if seg_r.test_category == "RhS":
                        for i, value in enumerate(seg_r.value):
                            result[seg_o.specimen_id]["resultats"].append({
                                "test": "%s%s%d" % (
                                    seg_r.test_abbrev,
                                    seg_r.test_category,
                                    i,
                                ),
                                "resu": value,
                                "date": seg_r.date,
                                "operator": seg_r.operator,
                            })
                    else:
                        result[seg_o.specimen_id]["resultats"].append({
                            "test": "%s%s" % (
                                seg_r.test_abbrev,
                                seg_r.test_category,
                            ),
                            "resu": seg_r.value,
                            "date": seg_r.date,
                            "operator": seg_r.operator,
                        })

        return result

    def get_qc_result_list(self, message):
        return []
