# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import (
    MessageV2,
    Worklist,
)
from driverng.fields import String, Date
import logging
logger = logging.getLogger("driverng.drivers.Tecan")


class TecanBase(object):
    FIELDS = ()

    def __init__(self, **kwargs):
        for field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        super(TecanBase, self).__init__()

    def parse(self, data):
        fields = self.FIELDS
        nbmaxelem = len(fields)
        for idx, elt in enumerate(data.split(";")):
            if idx >= nbmaxelem:
                break
            field = fields[idx]
            value = field.to_python(elt)
            setattr(self, field.name, value)

    def to_string(self):
        ligne = []
        for field in self.FIELDS:
            ligne.append("%s" % field.to_string(getattr(self, field.name)))
        return "%s;" % (";".join(ligne))


ALICOT, SEROTHEQUE = list(range(2))
# Serotheque
# Barcode_Rack;Position_Rack;Barcode_Tube2D;Barcode_Source;Volume;DateTime;Warehouse;Cabinet;Shelf;Drawer


class Serotheque(TecanBase):
    tecan_type = SEROTHEQUE
    FIELDS = (
        String("rack"),
        String("position"),
        String("bar_code2d"),
        String("bar_code"),
        String("volume"),
        Date("datetime", fmt="%H:%M:%S %d/%m/%Y"),
        String("warehouse"),
        String("cabinet"),
        String("shelf"),
        String("drawer"),
    )

# Alicot
# PrimaryBarcode;SecondaryBarcode;Volume;DateTime


class Alicot(TecanBase):
    tecan_type = ALICOT
    FIELDS = (
        String("no_echantillon"),
        String("no_echantillon2"),
        String("volume"),
        Date("datetime", fmt="%H:%M:%S %d/%m/%Y"),  # hh:mm:ss dd/MM/yyyy"
    )


class TecanMultiple(list):

    _MESSAGE_TYPE_MAP = {
        ALICOT: Alicot,
        SEROTHEQUE: Serotheque,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not len(line):
                continue
            lg = len(line.split(";"))
            if lg == 10:
                type_message = SEROTHEQUE
            elif lg == 4:
                type_message = ALICOT
            else:
                return
            obj = self.add(type_message=type_message)
            obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "tecan",
        "label": "Tecan H-RISLAB",
    }
    ENCODING = "utf8"

    MessageV2.Option("encoding", "Encodage"),

    def create(self):
        return TecanMultiple()

    def get_message_type(self, message):
        return self.ORDEALICOTR

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = TecanMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        pass

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for r in message:
            if r.tecan_type == SEROTHEQUE:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=r.bar_code,
                )
                sample.rack = "%s-%s" % (r.rack, r.position)
                commentaires = ["Rack: %s-%s" % (r.rack, r.position)]
                commentaires.append(u"Serotheque %s, Volume: %s" % (r.bar_code2d, r.volume))
                commentaires.append(u"Warehouse: %s, Cabinet: %s, Shelf: %s, Drawer: %s" % (
                    r.warehouse,
                    r.cabinet,
                    r.shelf,
                    r.drawer))

                sample.results.add({
                    "code": 'TRACKINGS',
                    "value": r.bar_code2d,
                    "date": r.datetime,
                    "comments": commentaires,
                })
            if r.tecan_type == ALICOT:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=r.no_echantillon
                )
                commentaires = [u"Alicot %s, Volume: %s" % (r.no_echantillon2, r.volume)]
                sample.results.add({
                    "code": 'TRACKINGA',
                    "value": r.no_echantillon2,
                    "date": r.datetime,
                    "comments": commentaires,
                    "date_plateau": r.datetime,
                })
        return worklist
