# -*- coding: utf8 -*-

from __future__ import unicode_literals

from unidecode import unidecode
from collections import OrderedDict
from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import String, Date


class Boolean(String):
    def to_python(self, value):
        assert(value in ("Y", "N"))
        return value == "Y"

    def to_string(self, value):
        return "Y" if value else "N"


class AsciiString(String):
    def to_string(self, value):
        return unidecode(
            super(AsciiString, self).to_string(value)).encode('ascii')


class ValabBase(object):
    FIELDS = []
    SEPARATOR = "|"
    SEGMENT_MAP = OrderedDict()
    _raw_len = 0

    def __init__(self, *args, **kwargs):
        self._locked = False
        self._fields = kwargs.get("fields", self.FIELDS)
        self._separator = kwargs.get("separator", self.SEPARATOR)
        for field in self._fields:
            setattr(self, field.name, kwargs.get(field.name, field.get_default()))
        for subfield in self.SEGMENT_MAP.values():
            setattr(self, subfield, [])
        super(ValabBase, self).__init__()
        self._locked = True

    def parse(self, data):
        splitted_data = data.split(self._separator)
        for i, field in enumerate(self._fields):
            value = splitted_data[i]
            setattr(self, field.name, field.to_python(value))
            self._raw_len += (len(value) + 1)
        return self

    def input_read_len(self):
        return self._raw_len

    def to_string(self):
        raise NotImplementedError


class AbstractCommand(ValabBase):
    def add_segment(self, segment):
        l = getattr(self, self.SEGMENT_MAP[segment.__class__])
        l.append(segment)

    def to_string(self):
        first_field = self._fields[0]
        rlist = ["{}{}".format(
            self.SEGMENT,
            first_field.to_string(
                getattr(self, first_field.name))
        )]
        rlist.extend([
            f.to_string(getattr(self, f.name)) for f in self._fields[1:]])
        for fieldlist in [getattr(self, l) for l in self.SEGMENT_MAP.values()]:
            rlist.extend([i.to_string() for i in fieldlist])
        # Commands are always finishing with trailing separator
        return self.SEPARATOR.join(rlist) + self.SEPARATOR


class AbstractSegment(ValabBase):
    """Must define SEGMENT"""
    def to_string(self):
        rlist = [self.SEGMENT]
        rlist.extend([
            f.to_string(getattr(self, f.name)) for f in self._fields])
        return self.SEPARATOR.join(rlist)


class SegmentO(AbstractSegment):
    """Micro-commande O: origine de la demande"""
    FIELDS = [
        AsciiString("label"),
    ]
    SEGMENT = "O"


class SegmentC(AbstractSegment):
    """Micro-commande C: RCT et RCO avec ou sans antériorité"""
    FIELDS = [
        AsciiString("result_actual"),
        AsciiString("result_previous"),
        Date("result_previous_date", fmt_date="%d/%m/%Y %H:%M"),
    ]
    SEGMENT = "C"


class SegmentA(AbstractSegment):
    FIELDS = [
        String("sample_id"),
        AsciiString("result_actual"),
        AsciiString("result_previous"),
        Date("result_previous_date", fmt_date="%d/%m/%Y %H:%M"),
    ]
    SEGMENT = "A"

    def to_string(self):
        first_field = self._fields[0]
        rlist = ["{}{}".format(
            self.SEGMENT,
            first_field.to_string(getattr(self, first_field.name)))
        ]
        rlist.extend([
            f.to_string(getattr(self, f.name)) for f in self._fields[1:]])
        return self.SEPARATOR.join(rlist)


class SegmentVA(AbstractSegment):
    FIELDS = [
        String("sample_id"),
        String("flag"),
    ]
    SEGMENT = "A"

    def to_string(self):
        first_field = self._fields[0]
        rlist = ["{}{}".format(
            self.SEGMENT,
            first_field.to_string(getattr(self, first_field.name)))
        ]
        rlist.extend([
            f.to_string(getattr(self, f.name)) for f in self._fields[1:]])
        return self.SEPARATOR.join(rlist)


class CommandN(AbstractCommand):
    FIELDS = [
        String("demande_number"),
        Date("prelevement_date", fmt_date="%d/%m/%Y %H:%M"),
        AsciiString("patient_name"),
        Date("patient_birthday", fmt_date="%d/%m/%Y"),
        String("patient_sex"),
        Boolean("emergency"),
        Boolean("clinical"),
    ]
    SEGMENT = "N"
    SEGMENT_MAP = OrderedDict((
        (SegmentO, "origins"),
        (SegmentC, "renseignements"),
        (SegmentA, "results"),
    ))


class CommandV(AbstractCommand):
    FIELDS = [
        String("demande_number"),
        String("result"),
        String("pad"),  # Empty segment.. Thank you valab..
    ]
    SEGMENT = "V"
    SEGMENT_MAP = OrderedDict((
        (SegmentVA, "results"),
    ))


class ValabDriver(MessageV2):
    SPEC = {
        "code": "valab",
        "label": "Valab",
    }
    COMMAND_MAP = {
        "N": (CommandN, {
            "A": SegmentA,
            "C": SegmentC,
            "O": SegmentO,
        }),
        "V": (CommandV, {
            "A": SegmentVA,
        }),
    }

    def to_string(self, message):
        return message.to_string()

    def create(self, message_type, **kwargs):
        return self.COMMAND_MAP[message_type][0](**kwargs)

    def parse(self, data):
        """Le protocole peut envoyer les commandes:
          - N: ordre du SIL -> VALAB
          - V: ordre de VALAB -> SIL
        """
        command_name, command_data = data[0], data[1:]
        command_klass, segment_map = self.COMMAND_MAP[command_name]
        command = command_klass().parse(command_data)
        total_read_len = command.input_read_len()
        while True:
            rest_data = command_data[total_read_len:]
            if not rest_data:
                break
            segment_name = rest_data[0]
            # Les segments O et C ne contiennent pas de suffixe.
            # On veut donc supprimer C| ou O|
            if segment_name in ("O", "C"):
                segment_data = rest_data[2:]
                total_read_len += 2
            elif segment_name == "A":
                # Les segments A contiennent un suffixe qu'on veut récupérer
                # A25 => on veut récupérer 25
                segment_data = rest_data[1:]
                total_read_len += 1
            else:
                raise NotImplementedError("segment: {}".format(segment_name))
            segment = segment_map[segment_name]()
            segment.parse(segment_data)
            command.add_segment(segment)
            total_read_len += segment.input_read_len()
        return command

    @staticmethod
    def _join_results(results, rtype):
        ret = []
        previous = None
        for r in results:
            if not r.type == rtype:
                continue
            if r.anterior and previous:
                ret.append((previous, r))
                previous = None
            else:
                if previous:
                    ret.append((previous, None))
                previous = r
        if previous:
            ret.append((previous, None))
        return ret

    def encode_worklist(self, worklist, context):
        sample = worklist.samples[0]
        if worklist.context == "query":
            message = self.create(
                "N",
                demande_number=sample.sample_id,
                prelevement_date=sample.collection_date,
                patient_name=sample.last_name,
                patient_birthday=sample.birthdate,
                patient_sex=sample.sex,
                emergency=sample.priority == "U",
                clinical=False)
            for physician in sample.physicians:
                message.origins.append(
                    SegmentO(label=physician.code)
                )
            for (r, ra) in self._join_results(sample.results, "medical_context"):
                segment = SegmentC(result_actual=r.value)
                if ra:
                    segment.result_previous = ra.value
                    segment.result_previous_date = ra.date
                message.renseignements.append(segment)
            for (r, ra) in self._join_results(sample.results, None):
                segment = SegmentA(sample_id=r.code, result_actual=r.value)
                if ra:
                    segment.result_previous = ra.value
                    segment.result_previous_date = ra.date
                message.results.append(segment)
            if (
                not message.renseignements and
                not message.origins and
                not message.results
            ):
                # Empty message, forbidden by valab, do not send
                return None
        elif worklist.context == "result":
            message = self.create(
                "V",
                demande_number=sample.sample_id,
                result=sample.flags[0])
            for r in sample.results:
                message.results.append(
                    SegmentVA(sample_id=r.code, flag=r.value))
        else:
            raise NotImplementedError
        return message

    def decode_message(self, message):
        if isinstance(message, CommandN):
            worklist = Worklist(context="query")
            patient = worklist.get_or_create(
                type="patient",
                sample_id=message.demande_number,
                last_name=message.patient_name,
                birthdate=message.patient_birthday,
                sex=message.patient_sex,
                collection_date=message.prelevement_date,
            )
            if message.emergency:
                patient.priority = "U"
            for seg_o in message.origins:
                patient.physicians.get_or_create(
                    code=seg_o.label)
            for seg_c in message.renseignements:
                patient.results.get_or_create(
                    type="medical_context",
                    code="CTX",
                    value=seg_c.result_actual,
                )
                if seg_c.result_previous:
                    patient.results.get_or_create(
                        type="medical_context",
                        code="CTX",
                        value=seg_c.result_previous,
                        anterior=True,
                        date=seg_c.result_previous_date,
                    )
            for seg_r in message.results:
                patient.results.get_or_create(
                    code=seg_r.sample_id,
                    value=seg_r.result_actual,
                )
                if seg_r.result_previous:
                    patient.results.get_or_create(
                        code=seg_r.sample_id,
                        value=seg_r.result_previous,
                        anterior=True,
                        date=seg_r.result_previous_date)
        elif isinstance(message, CommandV):
            worklist = Worklist(context="result")
            sample = worklist.get_or_create(
                type="patient",
                sample_id=message.demande_number,
                flags=[message.result])
            for seg_r in message.results:
                sample.results.get_or_create(
                    value=seg_r.flag,
                    code=seg_r.sample_id,
                )
        else:
            raise NotImplementedError
        return worklist
