# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
from unidecode import unidecode

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class VersacellSegmentH(HeaderSegment):

    password = HprimString(4)
    sender = HprimString(5, default="MCA")

    # 8 bit no parity 1 stop bit
    sender_characteristics = HprimString(9, default="N81")

    receiver = HprimString(10, default="SIEMENS")

    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class VersacellSegmentP(Segment):

    # NOTE: This field contains the QC lot number
    # for IMMULITE 2000/25000/XPi and ADVIA Centaur systems
    # control results
    patient_id = HprimString(3, max_length=20, required=True)

    # Patient names starting with ~C are regonized as
    # IMMULITE 2000/2500/XPi systems control samples
    # NOTE: Le centaur peut remplacer les espaces par des répétiteurs ^
    # NOTE: This field contains the QC lot number for advia 1800
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    initial = HprimString((6, 3))

    birthdate = HprimDate(8, fmt_date="%Y%m%d")

    # M/F/U
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class VersacellSegmentO(Segment):

    # sample_id starting with ~C are regonized as IMMULITE 2000/2500/XPi
    # systems control samples
    sample_id = HprimString(3, max_length=20, required=True)

    tests = HprimList(
        5,
        basefield=HprimMultiple(
            0,
            id=HprimString(4, max_length=30),
            dilution=HprimString(5),
            dilution_protocol=HprimString(6),
        ),
    )

    # S: Stat
    # A: ASAP
    # R: Routine
    priority = HprimString(6)

    request_date = HprimDate(7)
    collection_date = HprimDate(8)

    # Q: QC sample
    action_code = HprimString(12)

    # 1: Serum
    # 2: Urine
    # 3: Plasma
    # 4: CSF
    # 5: Whole blood
    # 6: Other
    # Required for systems that include
    # a Dimension RxL/EXL or ADVIA 1800
    specimen_descriptor = HprimString(16)

    # NOTE: This field contains the QC lot level for IMMULITE 2000/2500/Xpi and
    # Dimension RxL/EXL systems control results
    ordering_physician_lastname = HprimString((17, 1))
    ordering_physician_firstname = HprimString((17, 2))

    instrument_section_id = HprimString(25)
    report_type = HprimList(26)

    results = HprimForeign("R", "order")


class VersacellSegmentR(Segment):

    test = HprimMultiple(3,
                         id=HprimString(4),
                         dilution=HprimString(5),
                         dilution_protocol=HprimString(6),
                         replicate_number=HprimString(7),
                         aspect=HprimString(8)
                         )

    # NOTE: This field can contain non-numeric data or
    # symbols indicating out of range conditions
    value = HprimString(4, max_length=20, required=True)

    units = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimString(7)

    # N: Normal generic range
    nature_of_abnormality = HprimString(8, default="N")

    # F: Final results
    # R: This result was previously transmitted
    status = HprimString(9)

    date_start = HprimDate(12)
    date_end = HprimDate(13)

    instrument_id = HprimString(14)


class VersacellSegmentQ(Segment):

    sample_id = HprimString((3, 2))
    request = HprimString(5, default="ALL")

    # A: Abort/cancel last request
    # O: Requesting orders dans demographics
    status = HprimString(13)


class VersacellSegmentC(Segment):

    # I: Instrument
    # Peut être vide
    comment_source = HprimString(3)

    comment_code = HprimString((4, 1))
    comment_text = HprimString((4, 2))

    # I: Result flag
    # G: Operator entered comments
    comment_type = HprimString(5)


class VersacellSegmentL(Segment):

    # N = Terminaison normale
    # Q = Erreur dans la dernière demande d'informations
    # I = Pas d'informations disponibles à partir de la dernière demande
    # F = Dernière demande d'information traitée
    code = HprimString(3, default="N", max_length=1)


class Driver(AstmMessage):
    SPEC = {
        "code": "versacell",
        "label": "Versacell",
    }
    SEGMENT_CLASS_MAP = {
        "H": VersacellSegmentH,
        "P": VersacellSegmentP,
        "O": VersacellSegmentO,
        "Q": VersacellSegmentQ,
        "R": VersacellSegmentR,
        "C": VersacellSegmentC,
        "L": VersacellSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessage.BooleanOption("mode_ascii", label="Mode ascii (Dimension)", default=False),
        AstmMessage.BooleanOption(
            "ref_qc_centaur_tronquee",
            label="Ignorer le premier caractère 'K' des lots de QC Centaur",
            default=True),
    ]

    def __init__(self, mode_ascii=False, ref_qc_centaur_tronquee=True, **kwargs):
        self.mode_ascii = mode_ascii
        self.ref_qc_centaur_tronquee = ref_qc_centaur_tronquee
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = VersacellSegmentH(type_segment="H", **kwargs)
        message.end = VersacellSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        if message.queries:
            return self.QUERY
        else:
            return self.ORDER

    def get_queries(self, message):
        queries = []
        for seg_q in message.queries:
            queries.append(seg_q.sample_id)
        return queries

    def get_order_message(self, worklist, context):
        msg = self.create(date=context["now"])

        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient["patient_id"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
            )
            if self.mode_ascii:
                seg_p.last_name = unidecode(patient["last_name"])
                seg_p.first_name = unidecode(patient["first_name"])
            else:
                seg_p.last_name = patient["last_name"]
                seg_p.first_name = patient["first_name"]
            for sample, action, priority in patient["samples"]:
                if sample["specimen"] in ("1", "2", "3", "4", "5", "6"):
                    specimen_descriptor = sample["specimen"]
                else:
                    specimen_descriptor = ""
                seg_o = seg_p.orders.add(
                    sample_id=sample["sample_id"],
                    priority=priority,
                    request_date=sample["request_date"],
                    collection_date=sample["collection_date"],
                    specimen_descriptor=specimen_descriptor,
                )
                for order in sample["orders"]:
                    t = seg_o.tests.add()
                    t.id = order["code"]

        if msg.patients:
            return msg
        elif context["query"]:
            msg.end.code = "I"
            for sample_id in self.get_queries(context["query"]):
                msg.queries.add(sample_id=sample_id, status="X")
            return msg

    def get_result_dict(self, message):
        results = {}

        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                if seg_o.action_code == "Q":
                    continue

                results.setdefault(seg_o.sample_id, {"resultats": []})
                for seg_r in seg_o.results:

                    if seg_r.abnormal_flags and seg_r.abnormal_flags != "*":
                        # Le flag '*' veut dire qu'il y a des commentaires
                        # avec d'autres flags...
                        alarmes = [seg_r.abnormal_flags]
                    else:
                        alarmes = []

                    commentaires = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.comment_type == "I":
                            alarmes.append(seg_c.comment_code)
                        elif seg_c.comment_type == "G":
                            commentaires.append(seg_c.comment_code)

                    results[seg_o.sample_id]["resultats"].append({
                        "test": "%s%s" % (seg_r.test.id, seg_r.test.aspect),
                        "resu": seg_r.value,
                        "date": seg_r.date_end,
                        "analyseur": seg_r.instrument_id,
                        "alarmes": alarmes,
                        "commentaires": commentaires,
                    })

        return results

    def get_qc_result_list(self, message):
        results = []

        for seg_p in message.patients:
            for seg_o in seg_p.orders:

                if seg_o.action_code != "Q":
                    continue

                lot = seg_o.sample_id
                lot = None

                type_prlv = seg_o.specimen_descriptor or None

                if seg_p.last_name:
                    splitted = seg_p.last_name.split("/")
                    if len(splitted) == 2:
                        for fmt in ("%Y%m%d", "%d%m%Y", "%y%m"):
                            try:
                                datetime.datetime.strptime(splitted[1], fmt)
                                lot = splitted[0].strip()  # Advia 1800
                                break
                            except ValueError:
                                pass

                if lot is None:
                    if (
                        seg_o.sample_id[0] == "K" and
                        self.ref_qc_centaur_tronquee
                    ):
                        # Concordance QC centaurs et versacel
                        lot = seg_o.sample_id[1:]
                    else:
                        lot = seg_o.sample_id

                for seg_r in seg_o.results:
                    results.append({
                        "lot": lot,
                        "test": "%s%s" % (seg_r.test.id, seg_r.test.aspect),
                        "resu": seg_r.value,
                        "date": seg_r.date_end,
                        "analyseur": seg_r.instrument_id,
                        "type_prlv": type_prlv,
                    })

        return results
