# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import xml.etree.cElementTree as ET

from driverng import Message
from driverng import utils


class Driver(Message):
    SPEC = {
        "code": "vidasxml",
        "label": "Vidas XML",
    }
    ENCODING = "utf8"
    OPTIONS = [
        Message.Option(
            "qc_prefix",
            label="Préfixe QC",),
        Message.BooleanOption(
            "envoie_sexe",
            label="Envoie le sexe du patient",
            default=True)
    ]

    def __init__(self, qc_prefix=None, envoie_sexe=True, **kwargs):
        self.qc_prefix = qc_prefix
        self.envoie_sexe = envoie_sexe

    def parse(self, data):
        return ET.fromstring(data.encode(self.ENCODING))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, 'utf-8'))

    def get_message_type(self, message):
        return self.ORDER

    def get_order_message(self, worklist, context):
        root = ET.Element("lisMessage")
        root.set("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance")
        root.set("xsi:noNamespaceSchemaLocation", "ASTM.xsd")
        header = ET.SubElement(root, "header")
        ET.SubElement(header, "version").text = "1394-97"
        ET.SubElement(header, "dateTime").text = utils.format_date(context["now"], fmt="%Y%m%d%H%M%S")
        empty = True
        for sample in worklist.samples:
            request = ET.SubElement(root, "request")
            patient_information = ET.SubElement(request, "patientInformation")
            ET.SubElement(
                patient_information, "patientIdentifier"
            ).text = sample["patient_id"]
            ET.SubElement(
                patient_information, "lastName"
            ).text = "%s %s" % (sample["last_name"], sample["first_name"])
            if self.envoie_sexe:
                ET.SubElement(
                    patient_information, "sex"
                ).text = sample["sex"]
            test_order = ET.SubElement(request, "testOrder")
            if sample["birthdate"]:
                ET.SubElement(
                    patient_information, "birthdate"
                ).text = utils.format_date(sample["birthdate"], fmt="%Y%m%d")
            ET.SubElement(
                ET.SubElement(test_order, "specimen"),
                "specimenIdentifier",
            ).text = sample["sample_id"]
            for order in sample["orders"]:
                if order["action"] == "del":
                    continue
                test = ET.SubElement(test_order, "test")
                universal_identifier = ET.SubElement(test, "universalIdentifier")
                ET.SubElement(
                    universal_identifier,
                    "testIdentifier",
                ).text = order["code"]
                if order["dilution"]:
                    ET.SubElement(
                        universal_identifier,
                        "dilution",
                    ).text = order["dilution"]
                empty = False

        if not empty:
            return root

    def get_result_dict(self, message):
        results = {}
        if message.find("header").find("serialNumber") is None:
            analyseur = ""
        else:
            analyseur = message.find("header").find("serialNumber").text
        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.find(
                    "specimen").find("specimenIdentifier").text
                # Exclusion des CQ
                if self.qc_prefix and sample_id.startswith(self.qc_prefix):
                    continue
                if sample_id[0] == 'C':
                    continue
                for test in test_order.findall("test"):
                    testid = test.find(
                        "universalIdentifier").find("testIdentifier").text
                    result = test.find("result")
                    resu = result.find("value").text
                    if "|" in resu:
                        resu, resui = resu.split("|", 1)
                        # resultat interprété
                        resultats = [
                            (testid, resu),
                            ("%sI" % testid, resui)
                        ]
                    else:
                        resultats = [(testid, resu)]
                    date = datetime.datetime.strptime(
                        result.find("resultDateTime").text,
                        "%Y%m%d%H%M%S",
                    )
                    alarmes = []
                    for abnormalflags in result.findall("abnormalFlags"):
                        alarmes.append(abnormalflags.text)

                    lots_reactifs = []
                    for bmxResultData in result.findall("bmxResultData"):
                        for disposable in bmxResultData.findall("disposable"):
                            lot = disposable.findtext("lotNumber")
                            if lot:
                                lot = {"ref": lot}
                                expire = disposable.findtext("expirationDate")
                                if expire:
                                    lot.update(
                                        peremption=datetime.datetime.strptime(
                                            expire, "%Y%m%d"))
                                lots_reactifs.append(lot)

                    for code, resu in resultats:
                        results.setdefault(
                            sample_id, {"resultats": []},
                        )["resultats"].append({
                            "test": code,
                            "resu": resu,
                            "date": date,
                            "alarmes": alarmes,
                            "analyseur": analyseur,
                            "lots_reactifs": lots_reactifs,
                        })
        return results

    def get_qc_result_list(self, message):
        results = []
        if message.find("header").find("serialNumber") is None:
            analyseur = ""
        else:
            analyseur = message.find("header").find("serialNumber").text
        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.find(
                    "specimen").find("specimenIdentifier").text
                # Uniquement les CQ
                if not self.qc_prefix or not sample_id.startswith(self.qc_prefix):
                    continue
                for test in test_order.findall("test"):
                    testid = test.find(
                        "universalIdentifier").find("testIdentifier").text
                    result = test.find("result")
                    resu = result.find("value").text
                    if "|" in resu:
                        resu, resui = resu.split("|", 1)
                        # resultat interprété
                        resultats = [
                            (testid, resu),
                            ("%sI" % testid, resui)
                        ]
                    else:
                        resultats = [(testid, resu)]
                    date = datetime.datetime.strptime(
                        result.find("resultDateTime").text,
                        "%Y%m%d%H%M%S",
                    )
                    for code, resu in resultats:
                        results.append({
                            "lot": sample_id,
                            "test": code,
                            "resu": resu,
                            "date": date,
                            "analyseur": analyseur,
                        })
        return results
