# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)


class VircliaSegmentH(HeaderSegment):
    sender_id = HprimString((5, 1))
    type_message = HprimString(12, default="P")
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class VircliaSegmentQ(Segment):
    sample_id = HprimString((3, 1))
    test_order = HprimString(5)
    report_type = HprimString(13)


class VircliaSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    orders = HprimForeign("O", "patient")


class VircliaSegmentO(Segment):
    specimen_id = HprimString(3)
    test_id = HprimString((5, 4))
    priority = HprimString(6)
    results = HprimForeign("R", "order")


class VircliaSegmentR(Segment):
    test_id = HprimString((3, 4))
    test_type = HprimString((3, 5))
    value = HprimString((4, 1))
    conclusion = HprimString((4, 2))
    units = HprimString(5)
    status = HprimString(9)


class VircliaSegmentC(Segment):
    texte = HprimString(4)


class VircliaSegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "virclia",
        "label": "Virclia Astm",
    }
    SEGMENT_CLASS_MAP = {
        "H": VircliaSegmentH,
        "Q": VircliaSegmentQ,
        "P": VircliaSegmentP,
        "O": VircliaSegmentO,
        "R": VircliaSegmentR,
        "C": VircliaSegmentC,
        "L": VircliaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    def create(self, **kwargs):
        message = VircliaSegmentH(type_segment="H", **kwargs)
        message.end = VircliaSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["priority", ],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test_id=order["code"],
                        priority=priority,
                    )
        if len(msg.patients):
            return msg
        elif context["query"]:
            for sample_id in [seg_q.sample_id for seg_q in context["query"].queries]:
                msg.patients.add(
                    patient_id=sample_id
                )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.specimen_id:
                        tube = seg_o.specimen_id
                    else:
                        tube = seg_p.patient_id
                    for seg_r in seg_o.results:
                        if seg_r.status and seg_r.status != 'F':
                            continue
                        sample = worklist.get_or_create(
                            sample_id=tube,
                            type="patient",
                        )
                        test = seg_r.test_id
                        if seg_r.test_type:
                            test += "_%s" % seg_r.test_type
                        sample.results.add(
                            code=test,
                            value=seg_r.value,
                            unit=seg_r.units,
                        )
                        if seg_r.conclusion:
                            sample.results.add(
                                code=test + 'I',
                                value=seg_r.conclusion,
                            )
        return worklist
