# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)
from driverng.utils import get_date


class VistaSegmentH(HeaderSegment):

    sender = HprimString(5, max_length=40, default="MCA")
    receiver = HprimString(10, max_length=40, default="VISTA")
    # P - Production
    # Q - Quality Control
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="DB 100102")
    date = HprimDate(14)

    queries = HprimForeign("Q", "header")
    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class VistaSegmentP(Segment):
    FILL_SEGMENT = 35

    practice_assigned_id = HprimString(3, max_length=20)
    lab_assigned_id = HprimString(4, max_length=20)
    last_name = HprimString((6, 1), max_length=20)
    first_name = HprimString((6, 2), max_length=20)
    initial = HprimString((6, 3), max_length=1)
    suffix = HprimString((6, 4), max_length=5)

    birth_date = HprimDate(8, fmt="%Y%m%d")
    # M: Male
    # F: Female
    # U: Unknown
    sex = HprimString(9)

    height = HprimString(17)
    weight = HprimString(18)

    # Admission date et discharge date
    date = HprimList(24, basefield=HprimDate(0, fmt="%Y%m%d"))

    location_code = HprimString((26, 1), max_length=10)
    location_text = HprimString((26, 2), max_length=40)

    orders = HprimForeign("O", "patient")


class VistaTestField(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "code": HprimString(4, max_length=10, required=True),
            # 0401: Dimension RxL, ExL
            # 2202: Dimension Vista
            "instrument_type": HprimString(5, max_length=10),
            # Ignored from LIS
            "description": HprimString(6, max_length=10),
        })
        super(VistaTestField, self).__init__(*position, **kwargs)


class VistaSegmentO(Segment):
    FILL_SEGMENT = 32

    specimen_id = HprimString(3, max_length=30, required=True)
    instrument_specimen_id = HprimString(4)

    # Liste des codes tests
    # ^^^Test^^Description
    tests = HprimList(5, basefield=VistaTestField(0))

    # Priority
    # S: stat
    # A: ASAP
    # R: Routine
    priority = HprimString(6, max_length=1)

    # Requested/Ordered Date and time
    date_requested = HprimDate(7)

    collection_date = HprimDate(8)
    collection_end_time = HprimDate(9)
    collection_volume = HprimString(10)

    # N, A: Both values are processed by adding tests to the order
    action_code = HprimString(12, required=True)

    date_specimen_recevied = HprimDate(15)

    # S: Serum
    # C: CSF
    # P: Plasma
    # U: Urine
    # W: Whole blood
    # WB: Whole blood
    # O: Other
    # Note: O is not supported on downloads but will be returned
    # on uploads when not otherwise specified by the instrument
    sample_type = HprimString((16, 1), required=True)

    # Empty/Nil: Patient Result
    # QCn: Where n represents QC level
    # CALn: Where n represent Calibrator level
    sample_source = HprimString((16, 2))

    # Defaults to 1 (No dilution)
    sample_dilution = HprimString(19, max_length=6)

    total_number_of_tests = HprimString(20, max_length=3)

    results_reported_date = HprimDate(23)

    # O: Order (supported from host)
    # F: Final
    # P: Preliminaire
    # Q: Reponse to query (supported from host)
    report_type = HprimString(26, required=True)

    location_or_ward = HprimString(28)

    results = HprimForeign("R", "order")

    def to_string(self, separators=None):
        self.total_number_of_tests = len(self.tests)
        return super(VistaSegmentO, self).to_string(separators=separators)


class VistaSegmentR(Segment):

    test = VistaTestField(3)
    value = HprimString(4, max_length=20)
    units = HprimString(5, max_length=20)
    reference_ranges = HprimString(6, max_length=20)

    # Pour le vista c'est envoyé dans les segments C
    # abnormal_flags = HprimString(7, max_length=20)

    status = HprimString(9)
    operator = HprimString(11, max_length=20)
    date_started = HprimDate(12)
    date_completed = HprimDate(13)
    instrument_id = HprimString(14)


VISTA_COMMENT_CODE = {
    # Error codes
    "E116": "Inappropriate Fluid Type",
    "E121": "Never Calibrated",
    "E130": "Math Error",
    "E131": "Calculation Error",
    "E132": "No Reagent",
    "E133": "Manually Cancelled Test",
    "E134": "Process Error",
    "E141": "Absorbance",
    "E142": "Measurement Error",
    "E144": "Software Error",
    "E152": "Decode Error",
    "E161": "Result not valid",
    "E172": "No Sample",
    "E175": "No Test Module",
    "E178": "Reading Saturated",
    "E335": "Clog Detected",
    "E521": "Above Assay Range",
    "E522": "Below Assay Range",
    "E541": "Above Cut off Value",
    "E549": "Aliquot Expire",
    "E552": "Track Tube Not Aspirated",
    "E563": "Short Sample",
    "E700 ": "Bulk Consumable Need",
    "E701": "Mechanism Halted",
    "E702": "All Cuvettes Dirty",
    "E703": "IMT System Unavailable",
    "E704": "Waiting for calibration",
    "E705": "Waiting for QC",
    "E706": "Diluent need",
    "E099": "For Investigational Use Only",
    "E100": "Temperature Error",
    "E111": "H Interference",
    "E112": "L Interference",
    "E113": "I Interference",
    "E143": "Abnormal Assay",
    "E145": "Abnormal Reaction",
    "E317": "General Vista Error",
    "A726": "In Simulation Mod",
    "A001": "Autodilution factor",
    "E115": "Diluted",
    "E511": "Above Reference Range",
    "E512": "Below Reference Range",
    "E527": "Above Manufacturer’s Assay Range",
    "E528": "Below Manufacturer’s Assay Rang",
    "E531": "Above Panic Range",
    "E532": "Below Panic Range",
    "E550": "Correlated",
    # Calibration information
    "C001": "(Calibrator Name)",
    "C002": "(Calibrator Lot Number)",
    "C003": "(Calibrator Expiration Date)",
    "C010": "(Coefficient 1)",
    "C010": "(Coefficient 1)",
    "C011": "(Coefficient 2)",
    "C012": "(Coefficient 3)",
    "C013": "(Coefficient 4)",
    "C014": "(Coefficient 5)",
    "C015": "(Coefficient 6)",
    "C016": "(Coefficient 7)",
    "C017": "(Coefficient 8)",
    "C018": "(Coefficient 9)",
    "C020": "(Bottle Value 1)",
    "C021": "(Bottle Value 2)",
    "C022": "(Bottle Value 3)",
    "C023": "(Bottle Value 4)",
    "C024": "(Bottle Value 5)",
    "C025": "(Bottle Value 6)",
    "C026": "(Bottle Value 7)",
    "C027": "(Bottle Value 8)",
    "C028": "(Bottle Value 9)",
    "C030": "(Slope)",
    "C031": "(Intercept",
    # Reagent information
    "R001": "(Reagent Lot Number)",
    "R002": "(Reagent Expiration Date)",
    "R003": "(Reagent Lot Sequence Number)",
    # Quality control information
    "Q001": "Quality Control Lot Number",
    "Q002": "Quality Control Expiration Date",
    "Q003": "Quality Control Lot Sequence Number",
    "Q004": "Quality Control Name – The control name is derived from the",
    "Q005": "(P) Precision (A) Accuracy",
    "G001": "Rack ID for Front Loaded samples",
    "G002": "User entered Sample Comment",
}


class VistaSegmentC(Segment):
    # I: Instrument
    source = HprimString(3, default="I")
    code = HprimString((4, 1), max_length=6)
    text = HprimString((4, 2), max_length=35)
    comment_type = HprimString(5)


class VistaSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    # O: Patient and orders
    # X: No data found
    status = HprimString(13)


class VistaSegmentL(Segment):
    # Termination code
    # N: Normal
    # R: Receiver requested abort
    # E: Unknown Error
    # Q: Error in request information
    # I: No information available
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "vista",
        "label": "Vista Direct",
    }
    SEGMENT_CLASS_MAP = {
        "H": VistaSegmentH,
        "P": VistaSegmentP,
        "O": VistaSegmentO,
        "C": VistaSegmentC,
        "Q": VistaSegmentQ,
        "R": VistaSegmentR,
        "L": VistaSegmentL,
    }
    DEFAULT_SEPARATORS = "|\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "send_priority", label="Envoyer la priorité",
            default=True,
        ),
    ]

    def __init__(self, send_priority=True, **kwargs):
        self.send_priority = send_priority
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = VistaSegmentH(type_segment="H", **kwargs)
        message.end = VistaSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(date=context["now"])

        actions = {"del": worklist.IGNORE, worklist.ELSE: "N"}
        if not context["query"]:
            # En download on envoie avec "A"
            actions.update(add="A", new="A")

        if self.send_priority:
            priority_map = {"A": "A", "S": "S", worklist.ELSE: "R"}
        else:
            priority_map = {worklist.ELSE: ""}

        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action=actions,
            priority=priority_map,
        ):
            seg_p = message.patients.add(
                practice_assigned_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birth_date=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                if sample.specimen in ("S", "C", "P", "U", "W", "WB"):
                    sample_type = sample.specimen
                else:
                    sample_type = ""

                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    collection_date=sample.collection_date,
                    action_code=action,
                    sample_type=sample_type,
                    report_type="O",
                    sample_dilution="1",
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code

        if message.patients:
            return message
        elif context["query"]:
            for sample_id in [seg_q.sample_id for seg_q in context["query"].queries]:
                message.queries.add(sample_id=sample_id, status="X")
            message.end.code = "I"
            return message

    @staticmethod
    def get_lots(seg_r):
        lots_type = {
            "R": [],
            "C": [],
            "Q": [],
        }
        for seg_c in seg_r.commentaires:
            if seg_c.code in (
                    "R001", "C002", "Q001"):
                lots_type[seg_c.code[0]].append({
                    "lot": seg_c.text,
                    "expiration": None,
                })
            elif seg_c.code in (
                    "R002", "C003", "Q002"):
                lots_type[seg_c.code[0]][-1]["expiration"] = get_date(seg_c.text)
            elif seg_c.code in (
                    "C001", "Q004"):
                lots_type[seg_c.code[0]][-1]["name"] = seg_c.text

        lot_map = {"R": "reagent", "C": "calibration", "Q": "control"}
        lots = []
        for lot_type, lot_list in sorted(lots_type.items()):
            for lot in lot_list:
                lot["type"] = lot_map[lot_type]
                lots.append(lot)
        return lots

    def decode_message(self, message):

        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id,
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if message.processing_id == "P":
                    sample_type = "patient"
                elif message.processing_id == "Q":
                    sample_type = "control"
                else:
                    continue

                sample = worklist.get_or_create(
                    type=sample_type,
                    sample_id=seg_o.specimen_id,
                    specimen=seg_o.sample_type,
                )

                if sample.type == "control":
                    sample.control_lot = seg_o.specimen_id

                    if (
                        len(seg_o.sample_source) == 3 and
                        seg_o.sample_source[:2] == "QC"
                    ):
                        sample.control_level = seg_o.sample_source[2]

                for seg_c in seg_o.commentaires:
                    if seg_c.code and seg_c.code == "G001":
                        sample.rack = seg_c.text

                for seg_r in seg_o.results:
                    flags = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.code and seg_c.code[0] == "E":
                            flags.append(seg_c.code)

                    sample.results.add(
                        code=seg_r.test.code,
                        value=seg_r.value,
                        date=seg_r.date_completed,
                        instrument=seg_r.instrument_id,
                        flags=flags,
                        lots=self.get_lots(seg_r),
                        unit=seg_r.units,
                        operator=seg_r.operator,
                        reference_ranges=seg_r.reference_ranges,
                    )

        return worklist
