# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Message
from driverng.splitted import Splitted, String, Date
from driverng.exceptions import ValidationError


class WalkawayContainer(list):

    def __init__(self, separator=','):
        self.separator = separator
        super(WalkawayContainer, self).__init__()

    def add(self, *args, **kwargs):
        obj = Splitted(fields=self.fields, separator=self.separator, *args, **kwargs)
        self.append(obj)
        return obj

    def to_python(self, data):
        obj = self.add()
        obj.parse(data)
        return obj

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class WalkawayBase(Splitted):
    SEPARATOR = ","


class WalkawaySegmentH(WalkawayBase):
    FIELDS = [
        String("type", default='H'),
        String("site_name"),
        String("not_used"),
        String("interface_name"),
        String("interface_version_number"),
        String("field_delimiter"),
        String("string_delimiter"),
        String("delimit_flag"),
        String("checksum_type"),
        String("protocol_type"),
        # Timeout period in seconds
        String("timeout_period"),
        String("unknow"),
        String("flag"),
        String("not_used1"),
        String("reserved1"),
        String("reserved2"),
    ]

    def __init__(self, *arg, **kwargs):
        self._locked = False
        for attr in ['patients', 'request', 'end']:
            setattr(self, attr, [])
        super(WalkawayBase, self).__init__(*arg, **kwargs)


class WalkawaySegmentP(WalkawayBase):
    FIELDS = [
        String("type", default='P'),
        String("sequence"),
        String("patient_id"),
        String("last_name"),
        String("first_name"),
        Date("birthdate", fmt_date="%Y%m%d"),
        String("sex"),
        String("not_used1"),
        Date("admit_date", fmt_date="%Y%m%d"),
        Date("discharge_date", fmt_date="%Y%m%d"),
        String("room_number"),
        String("attending_physician_code"),
        String("attending_physician_description"),
        String("patient_ward_code"),
        String("patient_ward_description"),
        String("service_code"),
        String("service_description"),
        String("institution_code"),
        String("institution_description"),
        String("unknow1"),
        String("unknow2"),
        String("patient_free_text"),
        String("dob_status"),
        String("admit_status"),
        String("not_used2"),
        String("number_patient_comment"),
        String("admission_number"),
        String("unknow3"),
    ]

    def __init__(self, *args, **kwargs):
        self._locked = False
        self.specimen = []
        super(WalkawaySegmentP, self).__init__(*args, **kwargs)

    def to_string(self):
        lines = [super(WalkawaySegmentP, self).to_string()] +\
            [e.to_string() for e in self.specimen]
        return "\r\n".join(lines)


class WalkawaySegmentB(WalkawayBase):
    FIELDS = [
        String("type", default='B'),
        String("sequence"),
        String("specimen_number"),
        String("patient_id"),
        String("requesting_physician_code"),
        String("requesting_physician_description"),
        String("source_code"),
        String("source_description"),
        String("urine_source_flag"),
        Date("date_specimen_collected", fmt_date="%Y%m%d"),
        Date("time_specimen_collected", fmt_date="%H%M%S"),
        Date("request_date", fmt_date="%Y%m%d"),
        Date("receive_date", fmt_date="%Y%m%d"),
        Date("receive_time", fmt_date="%H%M%S"),
        String("reserved1"),
        String("reserved2"),
        String("specimen_status"),
        String("number_specimen_comment_records"),
        String("specimen_free_text"),
        String("ward_isolation_code"),
        String("ward_isolation_description"),
        String("tech_id_code"),
        String("tech_description"),
        String("reserved3"),
        String("unknow"),
    ]

    def __init__(self, *arg, **kwargs):
        self._locked = False
        for attr in ['resultats', 'commentaires', 'free_text']:
            setattr(self, attr, [])
        super(WalkawaySegmentB, self).__init__(*arg, **kwargs)


class WalkawaySegmentR(WalkawayBase):
    FIELDS = [
        String("type", default='R'),
        String("sequence"),
        String("isola_number"),
        String("specimen_number"),
        String("test_group_code"),
        String("test_group_description"),
        Date("test_group_status_date", fmt_date="%Y%m%d"),
        String("reserved1"),
        String("nosocomial_infection"),
        String("reserved2"),
        String("reserved3"),
        String("organism_code"),
        String("organism_description"),
        String("biotype_number"),
        String("beta_lactamase_test_result"),
        String("oxidase_reaction_result"),
        String("steptomycin_result"),
        String("gentamicin_result"),
        String("thymidine_result"),
        String("reserved4"),
        String("reserved5"),
        String("beta_hemolydic_flag"),
        String("not_used1"),
        String("not_used2"),
        String("exclude_result_flag"),
        String("not_used3"),
        String("not_used4"),
        String("ebsl_result"),
        String("not_used5"),
        String("number_test_mic"),
        String("not_used6"),
        String("isolate_free_text"),
        String("not_used7"),
        String("organism_class"),
        String("organism_set"),
        String("not_used8"),
        String("number_isolate_comments"),
        String("isolate_status"),
        String("isolate_tech_id_code"),
        String("isolate_tech_description")
    ]

    def __init__(self, *args, **kwargs):
        self._locked = False
        self.antibiotics = []
        super(WalkawaySegmentR, self).__init__(*args, **kwargs)


class WalkawaySegmentM(WalkawayBase):
    FIELDS = [
        String("type", default='M'),
        String("sequence"),
        String("drug_test_code"),
        String("drug_test_description"),
        String("mic_value"),
        String("drug_test_flag"),
        String("drug_test_interpretation"),
        String("clsi_systemic_interpretation"),
        String("clsi_urine_interpretation")
    ]
    FIELDS += [String("not_used%s" % i) for i in range(1, 17)]
    FIELDS += [
        String("result_suppressed_flag"),
    ]


class WalkawaySegmentC(WalkawayBase):
    FIELDS = [
        String("type", default='C'),
        String("sequence"),
        String("comment_code"),
        String("not_used1"),
        String("comment_description"),
        String("not_used2"),
        String("comment_association_flag"),
        String("comment_flag"),
        String("unknow")
    ]


class WalkawaySegmentF(WalkawayBase):
    FIELDS = [
        String("type", default='F'),
        String("text_association"),
        String("free_text"),
    ]


class WalkawaySegmentA(WalkawayBase):
    FIELDS = [
        String("type", default='A'),
        String("reserved1"),
        String("range_type_definition"),
        String("range_type_definition"),
        String("not_used1"),
        String("not_used2"),
        String("test_type_qualifier"),
        String("repeat_transmission_flag"),
        String("reserved2"),
        String("not_used3"),
    ]


class WalkawaySegmentL(WalkawayBase):
    # La fin de message L est optionnelle et est désactivée par défaut sur
    # l'automate
    FIELDS = [
        String("type", default='L'),
        String("sequence"),
        String("end_transmission"),
        String("error_return_code"),
        String("not_used"),
    ]


class Driver(Message):
    SPEC = {
        "code": "walkaway",
        "label": "Walkaway Bactério"
    }
    MESSAGE_TYPE_MAP = {
        "H": WalkawaySegmentH,
        "P": WalkawaySegmentP,
        "B": WalkawaySegmentB,
        "R": WalkawaySegmentR,
        "M": WalkawaySegmentM,
        "C": WalkawaySegmentC,
        "F": WalkawaySegmentF,
        "A": WalkawaySegmentA,
        "L": WalkawaySegmentL,
    }
    OPTIONS = [
        Message.Option(
            "atb_order_classtcode",
            label=u"Ordonner les antibiotiques selon la classe de texte codé"),
        Message.Option(
            "walkaway_prefixe_germe",
            label="Préfixe des textes codés des germes (par défaut G)",
            default="G"),
        Message.Option(
            "walkaway_prefixe_antibio",
            label="Préfixe des textes codés des antibiotiques (par défaut A)",
            default="A"),
    ]

    def __init__(self, atb_order_classtcode=None, walkaway_prefixe_germe="G", walkaway_prefixe_antibio="A", **kwargs):
        self.atb_order_classtcode = atb_order_classtcode
        self.prefixe_germe = walkaway_prefixe_germe
        self.prefixe_antibio = walkaway_prefixe_antibio
        super(Driver, self).__init__(**kwargs)

    def create(self, message_type, **kwargs):
        return self.MESSAGE_TYPE_MAP[message_type](**kwargs)

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        lines = [l for l in data.splitlines() if l]

        WalkawayMsg = WalkawayContainer()
        header = None
        patient = None
        specimen = None
        resultat = None
        lines[0] = lines[0].replace('"', '')
        for line in lines:
            message_type = line.split(",", 1)[0]
            try:
                obj = self.MESSAGE_TYPE_MAP[message_type]()
            except KeyError:
                # TO DO à gérer un champ du commentaire est de type csv
                if line[0] == 'C':
                    continue

            obj.parse(line)
            if message_type == 'H':
                header = obj
                WalkawayMsg.append(header)
            elif message_type == 'P':
                header.patients.append(obj)
                patient = obj
                specimen = None
            elif message_type == 'B':
                patient.specimen.append(obj)
                specimen = obj
                resultat = None
            elif message_type == 'R':
                specimen.resultats.append(obj)
                resultat = obj
            elif message_type == 'M':
                resultat.antibiotics.append(obj)
            elif message_type == 'C':
                specimen.commentaires.append(obj)
            elif message_type == 'F':
                specimen.free_text.append(obj)
            elif message_type == 'L':
                header.end.append(obj)
                patient = None
                resultat = None
                specimen = None
            else:
                raise ValidationError(
                    "Segment '%s' no mapped" % (message_type,)
                )

        return WalkawayMsg

    def get_order_message(self, worklist, context):
        if len(worklist.samples) == 0:
            return None
        samples = worklist.get_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        )
        msg = WalkawayContainer()
        for sample, action in samples:
            patient = self.create(
                "P",
                sequence="1",
                patient_id=sample.patient_id,
                last_name=sample.last_name,
                first_name=sample.first_name,
                birthdate=sample.birthdate,
                sex=sample.sex,
                attending_physician_code=sample.physician,
                attending_physician_description=sample.physician_name,
                service_code=sample.service,
                service_description=sample.service_name,
            )
            for seq, order in enumerate(sample.orders, 1):
                if "PLAQ_" in order.code:
                    code_test = order.code[5:]
                else:
                    code_test = order.code
                patient.specimen.append(
                    WalkawaySegmentB(
                        sequence_number=str(seq),
                        specimen_number=sample.sample_id,
                        patient_id=patient.patient_id,
                        source_code=code_test,
                    )
                )
                for cpt, result in enumerate(sample["results"], 1):
                    plaque = None
                    id_germe = None
                    if result["resu_json"] and result["resu_json"].get("germes"):
                        try:
                            plaque = result["resu_json"]["germes"][0].get("Plaques")
                            if isinstance(plaque, list):
                                plaque = ""
                        except KeyError:
                            plaque = ""
                        try:
                            id_germe = result["resu_json"]["germes"][0].get("Id")
                            if isinstance(id_germe, list):
                                id_germe = ""
                        except KeyError:
                            id_germe = ""
                        if id_germe.startswith("{}_".format(self.prefixe_germe)):
                            id_germe = id_germe[2:]
                        if plaque or id_germe:
                            patient.specimen.append(
                                WalkawaySegmentR(
                                    sequence=str(cpt),
                                    isola_number="1",
                                    specimen_number=sample.sample_id,
                                    test_group_code=plaque,
                                    organism_code=id_germe,
                                    nosocomial_infection="N",
                                )
                            )
            msg.append(patient)
        if len(msg):
            return msg

    def get_result_dict(self, message):
        samples_results = {}
        for header in message:
            for patient in header.patients:
                if patient is None:
                    continue
                specimen = patient.specimen
                for order in specimen:
                    specimen_type = order.source_description
                    sample_results = samples_results.setdefault(order.specimen_number, {})
                    resus = sample_results.setdefault('resultats', [])
                    for result in order.resultats:
                        germe = "{}_{}".format(self.prefixe_germe, result.organism_code)
                        resu = {
                            "atb": [],
                            "seq": result.isola_number,
                            "test": specimen_type,
                            "date": result.test_group_status_date,
                            "resu": germe,
                        }
                        for antibio in result.antibiotics:
                            code = "{}_{}".format(self.prefixe_antibio, antibio.drug_test_code)
                            cmi = antibio.mic_value
                            sensi = antibio.clsi_systemic_interpretation
                            if sensi == 'R*':
                                sensi = 'R'
                            elif sensi == 'S*':
                                sensi = 'S'
                            elif sensi == 'I*':
                                sensi = 'I'
                            elif sensi == 'ESBL':
                                sensi = 'R'
                            elif (
                                sensi == 'N/R' or
                                sensi == 'N/A' or
                                sensi == "" or
                                sensi == "POS" or
                                sensi == "NEG"
                            ):
                                continue

                            resu["atb"].append((
                                ("ANTIBIO", code),
                                ("CMI", cmi),
                                ("SENSI", sensi)))

                        resus.append(resu)

                        # identification de germes méthode JSON
                        code_test = "IDENT_{}".format(specimen_type)
                        resus.append({
                            "test": code_test,
                            "resu": {
                                "type_resultat": "identification_germes",
                                "germes": [{
                                    "id": germe,
                                    "Id": germe,
                                    "rang": int(result.isola_number),
                                }],
                            },
                            "atb": [],
                            "seq": result.isola_number,
                            "date": result.test_group_status_date,
                        })

                        # autre méthode de traitement des identification des germes
                        code_test = "IDENT_%s_%s" % (resu["seq"], resu["test"],)
                        resus.append({
                            "test": code_test,
                            "resu": germe,
                        })
        return samples_results
