# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class WaspSegmentH(HeaderSegment):
    sender_id = HprimString(5, default="LIS")
    receiver_id = HprimString(10, default="WASP")
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="1")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class WaspSegmentP(Segment):
    patient_id = HprimString(3)
    laboratoire_id = HprimString(4)
    other_patient_id = HprimString(5)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    middle_name = HprimString((6, 3))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # F/M
    sex = HprimString(9)
    age = HprimString(15)
    height = HprimString(17)
    weight = HprimString(18)
    admission_date = HprimDate(24)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class WaspSegmentO(Segment):
    specimen_id = HprimString(3)
    tray = HprimString((4, 1))
    cup = HprimString((4, 2))
    seq_format = HprimString((4, 3))
    tests = HprimList(5, basefield=HprimMultiple(
        0, test_id=HprimString(4)))
    # S = STAT
    # A = ASAP
    # R = Routine
    # C = Callback
    # P = Preoperative
    priority = HprimString(6)
    collection_date = HprimDate(8)
    # C/A/N/P/R/L/X/Q
    action_code = HprimString(12)
    specimen_descriptor = HprimString(16)
    additional_descriptor = HprimString(17)
    report_types = HprimString(26, default="Q")
    location_specimen_collection = HprimString(28)

    results = HprimForeign("R", "order")


class WaspSegmentR(Segment):
    test = HprimString(3)
    # Gélose
    plate_id = HprimString(4)
    # Lot
    lot_id = HprimString(6)
    streaking_time = HprimString(8)
    # C: correction of previously transmitted results
    # P: preliminary results
    # F: final results
    # X: order cannot be done
    # I: in instrument, results pending
    # S: partial results
    # M: this result is a MIC level
    # R: this result was previously transmitted
    # N: this result record contains necessary information to run a new order
    status = HprimString(9)


class WaspSegmentQ(Segment):
    specimen_id = HprimString(3)
    universal_test_id = HprimString(5)
    request_status = HprimString(13)


class WaspSegmentL(Segment):
    terminaison_code = HprimString(3, default="F")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "wasp",
        "label": "Wasp"
    }
    SEGMENT_CLASS_MAP = {
        "H": WaspSegmentH,
        "P": WaspSegmentP,
        "O": WaspSegmentO,
        "R": WaspSegmentR,
        "Q": WaspSegmentQ,
        "L": WaspSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = WaspSegmentH(type_segment="H", **kwargs)
        message.end = WaspSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={'del': "C", worklist.ELSE: "N"},
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                laboratoire_id=patient.lab_patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                height=patient.height,
                weight=patient.weight,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    collection_date=sample.request_date,
                    specimen_descriptor=sample.specimen,
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.test_id = order["code"]
        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_o in message.orders:
                sample = worklist.get_or_create(
                    type="patient",
                    sample_id=seg_o.specimen_id,
                )
                for seg_r in seg_o.results:
                    # ignorer les résultats avec le status à "X" ou "R" dans le champ
                    # R.9
                    # Le status "X" signifie que l'encemensement a été annulé
                    # sur le WASP
                    # "R": this result was previously transmitted
                    if seg_r.status == "X" or seg_r.status == "R":
                        continue
                    sample.results.add(
                        code="WASP",
                        value=seg_r.status,
                    )
                    if seg_r.plate_id:
                        sample.results.add(
                            code="%s_GELOSE" % (seg_r.test,),
                            value=seg_r.plate_id,
                        )
                    if seg_r.lot_id:
                        sample.results.add(
                            code="%s_LOT" % (seg_r.test,),
                            value=seg_r.lot_id,
                        )
                    if seg_r.streaking_time:
                        sample.results.add(
                            code="%s_STREAKING" % (seg_r.test,),
                            value=seg_r.streaking_time,
                        )
        return worklist
