# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)
from driverng.utils import num_years


class WasplabSegmentH(HeaderSegment):
    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class WasplabSegmentP(Segment):
    patient_id = HprimString(4)
    first_name = HprimString((6, 1))
    last_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # F or M
    sex = HprimString(9)
    # /!\ Non documenté dans le protocole, c'est le fournisseur COPAN qui a
    # donné cette information... /!\
    age = HprimString(14)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class WasplabSegmentO(Segment):
    specimen_id = HprimString(3)
    tests = HprimList(5, basefield=HprimMultiple(
        0, material_codes=HprimString(4)))
    status_lis = HprimString(6)
    collection_date = HprimDate(8)
    clinical_details = HprimString(14)
    specimen_descriptor = HprimString(16)
    media_code1 = HprimString((21, 1))
    media_code2 = HprimString((21, 2))
    media_code3 = HprimString((21, 3))
    media_code4 = HprimString((21, 4))
    media_code5 = HprimString((21, 5))
    media_code6 = HprimString((21, 6))
    media_code7 = HprimString((21, 7))
    media_code8 = HprimString((21, 8))
    media_code9 = HprimString((21, 9))
    media_code10 = HprimString((21, 10))
    media_suffix_code1 = HprimString((22, 1))
    media_suffix_code2 = HprimString((22, 2))
    media_suffix_code3 = HprimString((22, 3))
    media_suffix_code4 = HprimString((22, 4))
    media_suffix_code5 = HprimString((22, 5))
    media_suffix_code6 = HprimString((22, 6))
    media_suffix_code7 = HprimString((22, 7))
    media_suffix_code8 = HprimString((22, 8))
    media_suffix_code9 = HprimString((22, 9))
    media_suffix_code10 = HprimString((22, 10))
    location_group = HprimString(28)

    results = HprimForeign("R", "order")


class WasplabSegmentR(Segment):
    test_id = HprimString((3, 1))
    test_media = HprimString((3, 2))
    specimen_id = HprimString((4, 1))
    suffix = HprimString((4, 2))
    isolate_number = HprimString((4, 3))
    medias_codes = HprimString((5, 1))
    lot_number = HprimString((5, 2))
    expiration_date = HprimString((5, 3))
    # X, R, S, U
    inoculation_status = HprimString((5, 4))
    # T: Test
    # G: Growth
    # C: Comment
    # W: Workup
    result_type = HprimString(6)
    result_code = HprimString((7, 1))
    result_status = HprimString(9)
    instrument = HprimString(11)
    date_test_ordered = HprimDate(12)
    date_test_completed = HprimDate(13)


class WasplabSegmentQ(Segment):
    specimen_id = HprimString(3)
    sender_id = HprimString((11, 1))
    # S: Specimen
    # M: Media
    query_type = HprimString(12)
    # O: Order
    # D: Demographic
    # I: Isolate
    request_status = HprimString(13)


class WasplabSegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "wasplab",
        "label": "WaspLab Biomérieux"
    }
    SEGMENT_CLASS_MAP = {
        "H": WasplabSegmentH,
        "P": WasplabSegmentP,
        "O": WasplabSegmentO,
        "R": WasplabSegmentR,
        "Q": WasplabSegmentQ,
        "L": WasplabSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option("wasplab_sender", label="Code de l'émetteur (champ H.5)", default="LIS"),
        AstmMessageV2.Option("wasplab_receiver", label="Code du récepteur (champ H.10)", default="WASPLAB"),
    ]

    def __init__(self, wasplab_sender="LIS", wasplab_receiver="WASPLAB", **kwargs):
        self.sender_id = wasplab_sender
        self.receiver_id = wasplab_receiver
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = WasplabSegmentH(type_segment="H", **kwargs)
        message.end = WasplabSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            sender_id=self.sender_id,
            receiver_id=self.receiver_id,
            date=context["now"]
        )
        media_code1 = media_code2 = media_code3 = media_code4 = ''
        media_code5 = media_code6 = media_code7 = media_code8 = ''
        media_code9 = media_code10 = ''
        media_suffix_code1 = media_suffix_code2 = media_suffix_code3 = media_suffix_code4 = ''
        media_suffix_code5 = media_suffix_code6 = media_suffix_code7 = media_suffix_code8 = ''
        media_suffix_code9 = media_suffix_code10 = ''
        dict_media = {
            'media_code1': '',
            'media_code2': '',
            'media_code3': '',
            'media_code4': '',
            'media_code5': '',
            'media_code6': '',
            'media_code7': '',
            'media_code8': '',
            'media_code9': '',
            'media_code10': '',
        }
        dict_media_suffix = {
            'media_suffix_code1': '',
            'media_suffix_code2': '',
            'media_suffix_code3': '',
            'media_suffix_code4': '',
            'media_suffix_code5': '',
            'media_suffix_code6': '',
            'media_suffix_code7': '',
            'media_suffix_code8': '',
            'media_suffix_code9': '',
            'media_suffix_code10': '',
        }
        for patient in worklist.get_astm_worklist(
            unique_together=["priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            age = ""
            age = num_years(patient["birthdate"], patient["collection_date"])
            if age is None:
                age = ""
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                age=age,
                location=patient.location,
            )
            for sample, priority in patient.samples:
                if sample.media_code:
                    liste_media_code = sample.media_code.split("_")
                else:
                    liste_media_code = []
                for i, media_code in enumerate(liste_media_code, 1):
                    dict_media['media_code{}'.format(i)] = media_code
                    if i == 1:
                        media_code1 = media_code
                    elif i == 2:
                        media_code2 = media_code
                    elif i == 3:
                        media_code3 = media_code
                    elif i == 4:
                        media_code4 = media_code
                    elif i == 5:
                        media_code5 = media_code
                    elif i == 6:
                        media_code6 = media_code
                    elif i == 7:
                        media_code7 = media_code
                    elif i == 8:
                        media_code8 = media_code
                    elif i == 9:
                        media_code9 = media_code
                    elif i == 10:
                        media_code10 = media_code
                if sample.media_suffix_code:
                    liste_media_suffix_code = sample.media_suffix_code.split("_")
                else:
                    liste_media_suffix_code = []
                for i, media_suffix_code in enumerate(liste_media_suffix_code, 1):
                    dict_media_suffix['media_suffix_code{}'.format(i)] = media_suffix_code
                    if i == 1:
                        media_suffix_code1 = media_suffix_code
                    elif i == 2:
                        media_suffix_code2 = media_suffix_code
                    elif i == 3:
                        media_suffix_code3 = media_suffix_code
                    elif i == 4:
                        media_suffix_code4 = media_suffix_code
                    elif i == 5:
                        media_suffix_code5 = media_suffix_code
                    elif i == 6:
                        media_suffix_code6 = media_suffix_code
                    elif i == 7:
                        media_suffix_code7 = media_suffix_code
                    elif i == 8:
                        media_suffix_code8 = media_suffix_code
                    elif i == 9:
                        media_suffix_code9 = media_suffix_code
                    elif i == 10:
                        media_suffix_code10 = media_suffix_code
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    status_lis=priority,
                    collection_date=sample.request_date,
                    specimen_descriptor=sample.specimen,
                    media_code1=media_code1,
                    media_code2=media_code2,
                    media_code3=media_code3,
                    media_code4=media_code4,
                    media_code5=media_code5,
                    media_code6=media_code6,
                    media_code7=media_code7,
                    media_code8=media_code8,
                    media_code9=media_code9,
                    media_code10=media_code10,
                    media_suffix_code1=media_suffix_code1,
                    media_suffix_code2=media_suffix_code2,
                    media_suffix_code3=media_suffix_code3,
                    media_suffix_code4=media_suffix_code4,
                    media_suffix_code5=media_suffix_code5,
                    media_suffix_code6=media_suffix_code6,
                    media_suffix_code7=media_suffix_code7,
                    media_suffix_code8=media_suffix_code8,
                    media_suffix_code9=media_suffix_code9,
                    media_suffix_code10=media_suffix_code10,
                )
                list_code_results = []
                for result in sample.results:
                    list_code_results.append(result.code)
                for order in sample.orders:
                    if order["code"] in list_code_results:
                        continue
                    # pas d'envoi de code demande pour les leucocytes de
                    # cytologie urinaire et les identifications de germes
                    elif order["code"] == "LEUCO":
                        continue
                    elif order["code"].startswith("IDENT_"):
                        continue
                    elif order["code"].startswith("GERME_"):
                        continue
                    elif order["code"].startswith("="):
                        continue
                    t = seg_o.tests.add()
                    t.material_codes = order["code"]
                # Envoi des résultats accueils de la cytologie
                for result in sample.results:
                    if result.code.startswith("IDENT_"):
                        continue
                    seg_o.results.add(
                        medias_codes=result.code,
                        lot_number=result.value,
                    )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.specimen_id,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.specimen_id,
                    )
                    time = ""
                    liste_identification_germes = []
                    code_identification = ""
                    inoculation_code = ""
                    for seg_r in seg_o.results:
                        # résultats de boîtes et de traçabilité
                        # Résultat d'une boîte
                        # Hors Workgroup
                        if seg_r.suffix:
                            if seg_r.result_type == "W":
                                if seg_r.isolate_number:
                                    rang = seg_r.isolate_number
                                else:
                                    rang = ""
                                sample.results.add(
                                    code="{}_W{}".format(seg_r.test_id, rang),
                                    value=seg_r.medias_codes,
                                    instrument=seg_r.instrument,
                                )
                                # Le résultat codé POLY1B n'est pas un germe
                                if seg_r.medias_codes == "POLY1B":
                                    continue
                            elif seg_r.result_type == "M":
                                # Ignorer les résultats d'ensemencement
                                # incomplet avec le status U
                                if seg_r.inoculation_status == "U":
                                    continue
                                if seg_r.inoculation_status == "S":
                                    inoculation_code = seg_r.medias_codes
                                sample.results.add(
                                    code="{}ENS_{}".format(seg_r.test_id, time),
                                    value=seg_r.medias_codes,
                                    flags=seg_r.inoculation_status,
                                    instrument=seg_r.instrument,
                                )
                            elif seg_r.result_type == "G":
                                if seg_r.isolate_number:
                                    rang = seg_r.isolate_number
                                else:
                                    rang = ""
                                sample.results.add(
                                    code="{}_G{}".format(seg_r.test_id, rang),
                                    value=seg_r.medias_codes,
                                    instrument=seg_r.instrument,
                                )
                                if inoculation_code:
                                    sample.results.add(
                                        code="{}_{}".format(seg_r.test_id, inoculation_code),
                                        value=seg_r.medias_codes,
                                        instrument=seg_r.instrument,
                                    )
                                # Pas d'identification pour les résultats de
                                # type NUMER
                                if seg_r.medias_codes == "NUMER" and seg_r.lot_number:
                                    sample.results.add(
                                        code="{}_{}".format(seg_r.test_id, seg_r.medias_codes),
                                        value=seg_r.lot_number,
                                        instrument=seg_r.instrument,
                                    )
                                    continue
                                elif seg_r.medias_codes == "NUMER":
                                    continue

                                # Identification des germes
                                # Ne traiter en résultat complexe uniquement les
                                # résultats ayant un rang de germe
                                if seg_r.isolate_number == "":
                                    continue
                                code_identification = "IDENT_{}".format(seg_r.test_id)
                                liste_identification_germes.append({
                                    "rang": seg_r.isolate_number,
                                    "id": seg_r.medias_codes,
                                    "num": seg_r.lot_number
                                })
                            else:
                                sample.results.add(
                                    code="{}_{}".format(seg_r.test_id, seg_r.suffix),
                                    value=seg_r.medias_codes,
                                    instrument=seg_r.instrument,
                                )
                        else:
                            # Temps d'incubation
                            if seg_r.result_type == "T":
                                sample.results.add(
                                    code="{}_TIME".format(seg_r.test_id),
                                    value=seg_r.medias_codes,
                                    instrument=seg_r.instrument,
                                )
                                time = seg_r.medias_codes
                            sample.results.add(
                                code=seg_r.test_id,
                                value=seg_r.medias_codes,
                                instrument=seg_r.instrument,
                            )
                            if seg_r.result_type == "T" and seg_r.result_code:
                                sample.results.add(
                                    code="{}_SAMPLE".format(seg_r.test_id),
                                    value=seg_r.result_code,
                                    instrument=seg_r.instrument,
                                )
                    if code_identification and liste_identification_germes:
                        sample.results.add(
                            code=code_identification,
                            value={
                                "seq": "0",
                                "type_resultat": "identification_germes",
                                "germes": liste_identification_germes,
                            },
                        )
        return worklist
