# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimForeign,
    HprimCE,
)


class ZenitSegmentMSH(HeaderSegment):

    # LIS^HOST
    sending_application = HprimMultiple(3,
                                        code=HprimString(1, default="MCA"),
                                        name=HprimString(2, default="HOST"),
                                        )
    sending_facility = HprimString(4)

    # S02^ZENIT
    receiving_application = HprimMultiple(5,
                                          code=HprimString(1, default="S02"),
                                          name=HprimString(2, default="ZENIT"),
                                          )
    receiving_facility = HprimString(6)

    date = HprimDate(7, required=True)
    # OML^021^QML_021
    # ORL^022^ORL_022
    # ORU^R01^ORU_R01
    # ACK^R01^ACK
    # Les champs m_{type,code,name} ne sont pas nommés dans la doc.
    message_type = HprimMultiple(9,
                                 m_type=HprimString(1),
                                 m_code=HprimString(2),
                                 m_name=HprimString(3),
                                 )
    message_control_id = HprimString(10, required=True)
    processing_id = HprimString(11, required=True, default="P")
    version_id = HprimString(12, required=True, default="2.5")

    patients = HprimForeign("PID", "header")


class ZenitSegmentPID(Segment):

    patient_id = HprimString(3, required=True)
    patient_name = HprimMultiple(6,
                                 family_name=HprimString(1),
                                 given_name=HprimString(2),
                                 second_name=HprimString(3),
                                 suffix=HprimString(4),
                                 prefix=HprimString(5),
                                 degree=HprimString(6),
                                 name_type_code=HprimString(7),
                                 name_representation_code=HprimString(8),
                                 name_context=HprimString(9),
                                 name_validity_range=HprimString(10),
                                 name_assembly_order=HprimString(11),
                                 effective_date=HprimDate(12),
                                 expiration_date=HprimDate(13),
                                 professional_suffix=HprimString(14),
                                 )

    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)

    visits = HprimForeign("PV1", "patient")
    orders = HprimForeign("OBR", "patient")
    common_order = HprimForeign("ORC", "patient")


class ZenitSegmentPV1(Segment):

    # ????
    patient_class = HprimString(3, default="N")
    department_code = HprimString(4, default="LAB")
    common_order = HprimForeign("ORC", "patient")


class ZenitSegmentORC(Segment):

    response_flag = HprimString(7)
    transaction_date = HprimDate(10)
    order_date = HprimDate(16)
    orders = HprimForeign("OBR", "patient")

    def to_string(self, separators=None):
        # order control
        self.rang_segment = "NW"
        return super(ZenitSegmentORC, self).to_string(separators=separators)


class ZenitSegmentOBR(Segment):

    sample = HprimMultiple(3,
                           id=HprimString(1),
                           namespace_id=HprimString(2),
                           universal_id=HprimString(3),
                           universal_type_id=HprimString(4),
                           )

    test = HprimCE(5)

    request_date = HprimDate(7)

    # Only in result message
    obervation_date = HprimDate(8)

    results = HprimForeign("OBX", "order")


class ZenitSegmentOBX(Segment):

    # CE: Coded entry
    # NM: Numeric
    # ST: String data
    value_type = HprimString(3)

    observation = HprimCE(4)

    # ???
    observation_sub_id = HprimString(5)

    value = HprimString((6, 1))
    units = HprimString(7)
    references_range = HprimString(8)
    abnormal_flags = HprimString(9)

    # F: Final
    status = HprimString(12)

    date = HprimDate(15)

    observer = HprimString(17)


class Driver(AstmMessage):
    SPEC = {
        "code": "zenit",
        "label": "Zen-IT",
    }

    SEGMENT_CLASS_MAP = {
        "MSH": ZenitSegmentMSH,
        "PID": ZenitSegmentPID,
        "PV1": ZenitSegmentPV1,
        "ORC": ZenitSegmentORC,
        "OBR": ZenitSegmentOBR,
        "OBX": ZenitSegmentOBX,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3

    def create(self, **kwargs):
        return ZenitSegmentMSH(type_segment="MSH", **kwargs)

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message, separators=None, **kwargs):
        lines = super(Driver, self).to_string(
            message, separators=separators, **kwargs)
        return "\r".join(lines.splitlines()) + "\r"

    def get_order_message(self, worklist, context):
        message = self.create(
            date=context["now"],
            message_control_id=context.get(
                "control_id", int(time.time() * 1000)),
        )
        message.message_type.m_type = "OML"
        message.message_type.m_code = "021"
        message.message_type.m_name = "OML_021"

        for patient in worklist.get_astm_worklist(
                unique_together=["action"],
                action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            seg_p = message.patients.add(
                patient_id=patient["patient_id"],
                birthdate=patient["birthdate"],
                sex=patient["sex"],
            )
            seg_p.patient_name.family_name = patient["last_name"]
            seg_p.patient_name.given_name = patient["first_name"]
            seg_pv1 = seg_p.visits.add(patient_class="N")
            seg_orc = seg_pv1.common_order.add(
                response_flag="N",
                transaction_date=context["now"],
                order_date=context["now"],
            )
            for sample, action in patient["samples"]:
                for order in sample["orders"]:
                    seg_o = seg_orc.orders.add()
                    seg_o.sample.id = sample["sample_id"]
                    seg_o.request_date = context["now"]
                    seg_o.test.code1 = order["code"]

        if message.patients:
            return message

    def get_result_dict(self, message):

        result = {}
        for seg_p in message.patients:
            orders = seg_p.orders
            if seg_p.common_order:
                orders.extend(seg_p.common_order[0].orders)
            for seg_o in orders:

                result.setdefault(seg_o.sample.id, {"resultats": []})

                for seg_r in seg_o.results:

                    if seg_r.status != "F":
                        continue

                    alarmes = []
                    if seg_r.abnormal_flags:
                        alarmes.append(seg_r.abnormal_flags)
                    result[seg_o.sample.id]["resultats"].append({
                        "test": seg_r.observation.code1,
                        "resu": seg_r.value,
                        "alarmes": alarmes,
                        "date": seg_r.date,
                    })

        return result
