# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random
import time

from driverng.utils import STX, ETX, SOH, ENQ, byte2log
from driverng.protocol.acknack import ACKNACKProtocol


class AbxProtocol(ACKNACKProtocol):
    MAX_TRIES = 3

    def checksum(self, data):
        # Sum modulo 65536 of all characters except ETX, STX and all information
        # about checksum.
        return sum(ord(c) for c in data) % (1 << 16)

    def send_trames(self, trame):
        self.write(SOH)
        data = self.read(timeout=10)
        if not data:
            return False
        if data[0] != ENQ:
            self.tries += 1
            self.logger.warning("<enq> expected in %s", byte2log(data))

            if self.tries < self.MAX_TRIES:
                time.sleep(1)
                return self.send_trames(trame)
            else:
                return False
        self.tries = 0
        if super(AbxProtocol, self).send_trames(trame):
            return super(AbxProtocol, self).send_trames(
                self.build_trames(b"\xff END     \r"),
            )
        else:
            return False

    def recv_trames(self):
        data = self.read()

        if not data or data[0] != SOH:
            self.logger.warning("<soh> expected in %s", byte2log(data))
            return False

        if self.is_automate and not random.randint(0, 5):
            self.write(SOH)
            return self.recv_trames()

        self.write(ENQ)
        trames = []
        while True:
            payload = super(AbxProtocol, self).recv_trames()
            if not payload:
                payload = ""
            for line in payload.split(b"\r"):
                if line[:6] == b"\xff END ":
                    return trames
            trames.append(payload)
        return trames

    def get_payload(self, data):
        # <stx><packet_length><cr><payload><0xFD><checksum><cr><etx>
        return data[7:-9]

    def build_trames(self, message):
        # <stx><packet_length><cr><payload><0xFD><checksum><cr><etx>
        # (packet length + payload + checksum) length
        length = len(message) + 13
        msg = b"%.5d\r%s" % (
            length,
            message,
        )
        return b"%c%s\xfd %04X\r%c" % (
            STX,
            msg,
            self.checksum(msg),
            ETX,
        )

    def check_trame(self, trame):
        try:
            checksum = int(trame[-6:-2], 16)
            checksum_expected = self.checksum(trame[1:-8])
        except ValueError:
            self.logger.warning("Missing checksum for %s", byte2log(trame))

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%04X != %04X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False
        return True
