# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.utils import ACK, NACK, STX, ETX, byte2log
from driverng.protocol.base import BaseProtocol


class ACKNACKProtocol(BaseProtocol):
    CHECKSUM_NONE, CHECKSUM_ASTM, CHECKSUM_XOR, CHECKSUM_BCC, CHECKSUM_STARRSED_XOR = [None, "astm", "xor", "bcc", "xor_starrsed"]
    ACCEPTED_END_CHAR = [ETX]

    def __init__(self,
                 checksum_type=CHECKSUM_NONE,
                 max_tries=4,
                 ack=True,
                 timeout=10,
                 **kwargs):
        self.checksum_type = checksum_type
        self.ack = ack
        self.max_tries = max_tries
        self.timeout = timeout
        super(ACKNACKProtocol, self).__init__(**kwargs)

    def send_trames(self, trame):
        while self.tries < self.max_tries:
            self.tries += 1
            self.write(trame)
            if not self.ack:
                return True
            data = self.read(timeout=self.timeout)
            if not data or data[0] not in (ACK, NACK):
                self.rtimeout = random.randint(20, 30)
                self.logger.info(
                    "<stx> expected in %s: wait %ds before re-sending <stx>",
                    byte2log(data), self.rtimeout,
                )
                return False

            if data[0] == ACK:
                return True

        return False

    def recv_trames(self):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False

        if self.checksum_type == self.CHECKSUM_BCC:
            data += self.readline(break_func=lambda s: s and s[-2] in self.ACCEPTED_END_CHAR)
        else:
            data += self.readline(break_func=lambda s: s and s[-1] in self.ACCEPTED_END_CHAR)
        if self.check_trame(data):
            if self.ack:
                self.write(ACK)
            return self.get_payload(data)
        else:
            self.write(NACK)
            return False

    def check_trame(self, trame):
        if self.checksum_type == self.CHECKSUM_NONE:
            return True
        elif self.checksum_type == self.CHECKSUM_ASTM:
            try:
                checksum = int(trame[-3:-1], 16)
                checksum_expected = self.checksum_astm(trame[1:-3])
            except ValueError:
                self.logger.warning(
                    "Missing checksum for %s",
                    byte2log(trame),
                )
                return False
        elif self.checksum_type == self.CHECKSUM_XOR:
            try:
                checksum = ord(trame[-2])
                checksum_expected = self.checksum_xor(trame[1:-2])
            except ValueError:
                self.logger.warning(
                    "Missing checksum for %s",
                    byte2log(trame),
                )
                return False
        elif self.checksum_type == self.CHECKSUM_BCC:
            try:
                checksum = ord(trame[-1])
                checksum_expected = self.checksum_bcc(trame[1:-1])
            except ValueError:
                self.logger.warning(
                    "Missing checksum for %s",
                    byte2log(trame),
                )
                return False
        elif self.checksum_type == self.CHECKSUM_STARRSED_XOR:
            try:
                checksum = ord(trame[-2:-1])
                checksum_expected = self.checksum_another_xor(trame[1:-2])
            except ValueError:
                self.logger.warning(
                    "Missing checksum for %s",
                    byte2log(trame),
                )
        else:
            raise NotImplementedError

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False

        return True

    def get_payload(self, data):
        # <stx><payload><checksum><etx>
        if self.checksum_type == self.CHECKSUM_NONE:
            return data[1:-1]
        elif self.checksum_type == self.CHECKSUM_ASTM:
            return data[1:-3]
        elif self.checksum_type == self.CHECKSUM_XOR:
            return data[1:-2]
        elif self.checksum_type == self.CHECKSUM_BCC:
            return data[1:-2]
        elif self.checksum_type == self.CHECKSUM_STARRSED_XOR:
            return data[1:-2]
        else:
            raise NotImplementedError

    def build_trames(self, message):
        # <stx><payload><checksum><etx>
        if self.checksum_type == self.CHECKSUM_NONE:
            return b"%c%s%c" % (STX, message, ETX)
        elif self.checksum_type == self.CHECKSUM_ASTM:
            return b"%c%s%02X%c" % (
                STX,
                message,
                self.checksum_astm(message),
                ETX,
            )
        elif self.checksum_type == self.CHECKSUM_XOR:
            return b"%c%s%c%c" % (
                STX,
                message,
                self.checksum_xor(message),
                ETX,
            )
        elif self.checksum_type == self.CHECKSUM_BCC:
            return b"%c%s%c%c" % (
                STX,
                message,
                ETX,
                self.checksum_bcc(message + ETX),
            )
        elif self.checksum_type == self.CHECKSUM_STARRSED_XOR:
            return b"%c%s%c%s" % (
                STX,
                message,
                self.checksum_another_xor(message),
                ETX,
            )

        else:
            raise NotImplementedError

    @staticmethod
    def checksum_bcc(data):
        chk = 0
        for c in data:
            chk ^= ord(c)
        return chk

    @staticmethod
    def checksum_xor(data):
        # Exclusive XOR of all characters of the message
        # excluded <stx> and <etx> and <checksum>. If the checksum
        # calculated is 0x03 (ETX), or 0x02 (STX) or 0x00 (NULL), the
        # value will be automatically replaced with 0x7F hex value
        chk = 0
        for c in data:
            chk ^= ord(c)
        if chk in (0x02, 0x03, 0x00):
            chk = 0x7F
        return chk

    @staticmethod
    def checksum_astm(data):
        return (sum(ord(c) for c in data) & 0xFF) % 0x100

    @staticmethod
    def checksum_another_xor(data):
        # The CS byte verifies the accuracy of each transmitted message. Before
        # transmission, the value of the CS byte is calculated by the
        # "exclusice-or'ing" of all data bytes in the message, with the
        # exception of CS, STX, and ETX.
        # Since the CS byte precedes the ETX byte within the data stream,
        # the calculated value for CS cannot be equal to that of the ETX byte (03h).
        # Therefore, if the calculated CS value is 03h, the transmitted CS
        # byte is set to the substitute value 83h in order to avoid
        # erroneous action by the receiving device.
        chk = 0
        for c in data:
            chk ^= ord(c)
        if chk == 0x03:
            chk = 0x83
        return chk

    def checksum(self, data):
        if self.checksum_type == self.CHECKSUM_ASTM:
            return self.checksum_astm(data)
        elif self.checksum_type == self.CHECKSUM_XOR:
            return self.checksum_xor(data)
        elif self.checksum_type == self.CHECKSUM_BCC:
            return self.checksum_bcc(data)
        elif self.checksum_type == self.CHECKSUM_STARRSED_XOR:
            return self.checksum_another_xor(data)
        else:
            raise NotImplementedError
