# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import random

from driverng.utils import byte2log, ETX, NACK, STX
from driverng.protocol.base import BaseProtocol


class AdviaProtocol(BaseProtocol):
    SEND_TIMEOUT = 2  # TLS param "Host_Tmt"
    RECV_TIMEOUT = 5
    LEAVE_MASTER_DELAY = 10  # Param tS "Host_Tws"
    MAX_SLAVE_DELAY = 60

    def __init__(self, repertoire_ftp=None, testing=False, mode_query=False, **kwargs):
        # Le conteur "MT" (cf build_trames)
        self.mt_counter = 0x30  # '0'

        # Mode query ou download Advia ?
        self.mode_query = mode_query

        # Etat de la connexion maitre ou esclave.
        self.master = False

        # Etat voulu de la connexion.
        self.wanted_master = False

        # Temps au bout duquel on rend le mode master à l'automate
        self.leave_master_timer = None

        # Temps (MAX_SLAVE_DELAY) au bout duquel on reinitialise la connexion
        # si aucun message n'est echangé et qu'on est slave
        self.last_send_recv = None

        if repertoire_ftp:
            self.repertoire_ftp = repertoire_ftp.encode("ascii")
        else:
            self.repertoire_ftp = None
        self.testing = testing

        super(AdviaProtocol, self).__init__(**kwargs)

    def mt_next(self):
        if self.mt_counter >= 0x5a:
            self.mt_counter = 0x30
        else:
            self.mt_counter += 1

    def connect(self, transport):
        super(AdviaProtocol, self).connect(transport)
        if not self.testing:
            self.handle_reinit()

    def handle_reinit(self):

        if not self.is_automate:
            self.transport.flush()

        self.mt_counter = 0x30
        self.connected = True

        if self.is_automate:
            self.master = self.wanted_master = False
            self.transport.poll(None, None)
            self.recv_trames()  # On attend le I
        else:
            if self.repertoire_ftp:
                self.send_trames(self.build_trames(b"I|%03d|/%s/|" % (
                    random.randint(0, 999),
                    self.repertoire_ftp,
                )))
            else:
                self.send_trames(self.build_trames(b"I "))
            self.master = self.wanted_master = True

        return self.connected

    def can_send_trames(self, wqueue):

        empty = wqueue.empty()

        if self.master and not empty:
            self.leave_master_timer = None

        if self.master and empty:
            # S'il n'y a rien à demander on laisse la main au bout d'un moment
            if self.leave_master_timer is None:
                self.leave_master_timer = time.time()
            elif abs(time.time() - self.leave_master_timer) > self.LEAVE_MASTER_DELAY:
                self.logger.info(
                    "Nothing to send since %d seconds: leave master mode",
                    self.LEAVE_MASTER_DELAY,
                )
                self.send_trames(self.build_trames(b"S%10.10s" % (b"",)))
                self.master = self.wanted_master = False
                self.leave_master_timer = None
            return self.master

        if (
            not empty and
            not self.master and
            not self.wanted_master and
            not self.is_automate
        ):

            if self.mode_query:
                # Si on est en mode query et qu'on arrive là, c'est parce
                # qu'il faut envoyer une trame d'order en réponse à une
                # trame de query dans ce cas (et UNIQUEMENT dans ce cas),
                # on doit envoyer cette trame d'order alors qu'on est pas
                # maitre.
                return True

            # S'il y a quelque chose à demander on veut prendre la main
            self.logger.info("Trying to get master for next result message")
            self.wanted_master = True

        if (
            not self.is_automate and
            not self.master and
            self.last_send_recv is not None
        ):
            delay = abs(time.time() - self.last_send_recv)
            if delay > self.MAX_SLAVE_DELAY:
                # On est pas master et l'automate n'a rien envoyé depuis plus
                # de MAX_SLAVE_DELAY Dans ce cas on renvoie un message
                # d'initialisation
                self.logger.info("No token S received for %d seconds, reset connection.", delay)
                self.connected = False
                self.last_send_recv = None

        return self.master

    def send_trames(self, trames):
        self.last_send_recv = time.time()

        # On transmet la trame et on check le MT
        self.write(trames[0])
        data = self.read(timeout=self.SEND_TIMEOUT)

        if not data:
            return False

        if ord(data[0]) != self.mt_counter:
            self.logger.warning(
                "Wrong message toggle (MT) recevied %02X != %02X tries %d",
                ord(data[0]),
                self.mt_counter,
                self.tries,
            )

            if self.tries >= 1:
                self.connected = False
                return False

            self.tries += 1
            return self.send_trames(trames)

        self.mt_next()

        if trames[0][2] in (b"Y", b"R"):
            # On attend le message de validation E ou Z
            self.recv_trames()

        return True

    @staticmethod
    def break_func(buf):
        # Une ligne de l'advia fini par <cr><lf><lrc><etx>

        try:
            return (buf[-4], buf[-3], buf[-1]) == (b"\r", b"\n", ETX)
        except IndexError:
            return False

    def recv_trames(self):
        self.last_send_recv = time.time()
        trame = b""
        while True:
            c = self.readline(timeout=self.RECV_TIMEOUT,
                              break_func=self.break_func)
            if not c:
                self.connected = False
                return False

            trame += c
            if trame[-1] == ETX:
                break

        error = False

        if ord(trame[1]) != self.mt_counter:
            self.logger.warning(
                "Unexpected message toggle (MT) recevied %02X != %02X",
                ord(trame[1]),
                self.mt_counter,
            )
            error = True

        if not self.check_trame(trame):
            error = True

        if error:
            self.write(NACK)

            if self.tries >= 1:
                self.connected = False
                return False

            self.tries += 1
            return self.recv_trames()

        self.mt_counter = ord(trame[1])
        self.mt_next()

        self.write(trame[1])  # On envoie le MT recu
        payload = self.get_payload(trame)

        # Le protocole n'est pas asynchrone... Tout ceci devrait être géré
        # par service_auto avec une notion d'état.
        if payload[0] == b"Y":
            self.send_trames(self.build_trames(b"E%8.8s 0" % (b"",)))
        elif payload[0] == b"R":
            if not self.master and self.wanted_master:
                self.logger.info("Send Z result validation to take master")
                self.send_trames(self.build_trames(b"Z%17.17s 2" % (b"",)))
                self.master = self.wanted_master = True
            else:
                self.send_trames(self.build_trames(b"Z%17.17s 0" % (b"",)))
        elif payload[0] == b"S":
            assert not self.master
            self.logger.info("S token recevied: become master")
            self.master = self.wanted_master = True
        elif payload[0] == b"Z" and payload[-1] == b"2":
            assert self.master
            assert self.is_automate
            self.logger.info(
                "Get Z result validation with 2 token: leave master")
            self.master = self.wanted_master = False

        if payload[0] in (b"Q", b"Y", b"R"):
            return payload
        else:
            return False

    def build_trames(self, message):
        # <stx><MT><ID CODE><DATA><CRLF><LRC><etx>
        # MT boucle entre 0x30 ('0') et 0x5A ('Z')
        data = b"%c%s\r\n" % (
            self.mt_counter,
            b"\r\n".join(message.splitlines()),
        )
        lrc = self.checksum(data)
        trame = b"%c%s%c%c" % (STX, data, lrc, ETX)
        return [trame]

    @staticmethod
    def checksum(data):
        # LRC (Longitudinal Redundancy Check)
        # Since the LRC byte precedes the ETX byte within the data stream,
        # the calculated value for LRC cannot be qeual to that of the ETX
        # byte (0x03). Therefore if the caculated LRC value is 0x03, the
        # transmitted LRC byte is set to the substitute value 0x7F in order
        # to avoir erroneous action by the receiving device.
        # :D

        value = 0
        for c in data:
            value ^= ord(c)

        if value == 0x03:
            value = 0x7F

        return value

    def check_trame(self, trame):

        if trame[-4:-2] != b"\r\n":
            self.logger.warning("CRLF expected in %s", byte2log(trame))
            # return False

        try:
            lrc = self.checksum(trame[-2])
            lrc_expected = self.checksum(trame[1:-2])
        except ValueError:
            self.logger.warning(
                "Missing lrc checksum for %s", byte2log(trame))
            return False

        if lrc != lrc_expected:
            self.logger.warning(
                "Bad lrc checksum for %s: (%02X != %02X)",
                byte2log(trame),
                lrc,
                lrc_expected,
            )
            return False

        return True

    @staticmethod
    def get_payload(trame):
        # <stx><mt><PAYLOAD><cr><lf><lrc><etx>
        return trame[2:-4]
