# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.utils import byte2log, ENQ, EOT, ACK, ETB, NACK, STX, ETX
from driverng.protocol.base import BaseProtocol


class AstmProtocol(BaseProtocol):

    def __init__(self,
                 mode_bloc=False,
                 max_car_bloc=240,
                 end_payload_with_cr=True,
                 mode_iq200=False,
                 mode_etb=False,
                 checksum_ignore_end=False,
                 **kwargs):
        """:param mode_bloc: True pour forcer l'envoi en mode bloc
        :param max_car_bloc: Longueur max du payload en mode bloc
        """
        super(AstmProtocol, self).__init__(**kwargs)
        self.mode_bloc = mode_bloc
        self.max_car_bloc = max_car_bloc
        self.end_payload_with_cr = end_payload_with_cr
        self.mode_etb = mode_etb
        self.mode_iq200 = mode_iq200

        # Compter le dernier caractère (etx ou etb) dans le checksum ?
        self.checksum_ignore_end = checksum_ignore_end

    def send_trames(self, trames):
        """Envoie une liste de trame (message) sur la connexion.
        Les trames sont construites avec `build_trames` puis envoyés
        suivant le protocole automate::

            -> <enq>
            <- <ack>
            -> DATA
            <- <ack>
            -> <eot>

        :return: False si erreur sinon True (TODO: exceptions)
        :param trames: liste de trames à envoyer
        """

        self.write(ENQ)
        data = self.read(timeout=15)

        if not data:
            self.write(EOT)
            return False

        if data[0] == ENQ:
            # L'automate a la priorité, le driver attends un temps
            # aléatoire entre 10 et 20 secondes
            self.rtimeout = random.randint(10, 20)
            self.logger.info("ENQ clash: wait %ds before re-sending <enq>", self.rtimeout)
            return False

        if data[0] != ACK:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data), self.rtimeout,
            )
            return False

        index = 0

        while True:
            try:
                trame = trames[index]
            except IndexError:
                break

            self.write(trame)
            data = self.read(timeout=15)

            if not data:
                self.write(EOT)
                return False

            if data[0] not in (ACK, EOT):
                self.rtimeout = random.randint(10, 20)
                self.logger.info(
                    "<ack> or <eot> expected in %s: wait %ds before re-sending <enq>",
                    byte2log(data), self.rtimeout,
                )
                self.write(EOT)
                return False

            index += 1

        self.write(EOT)

        return True

    def build_trames(self, message):
        """Rajoute des informations de transport sur un message:

            <stx><indice><payload><cr (optionel)><etx><checksum><cr><lf>

        Le checksum se fait sur <indice><payload><cr><etx>

        :param message: Le message à envoyer
        :return: Liste de trame avec les informations de transport.

        """
        trames = []
        ind = 1

        if self.mode_bloc:
            payloads = [b"\r".join(message.splitlines())]
        else:
            payloads = message.splitlines()

        for payload in payloads:
            if self.mode_iq200:
                payload = b"\x00".join(payload) + b"\x00"
                payload = payload.encode("hex").upper()

            if self.end_payload_with_cr:
                payload += b"\r"

            while payload:

                if self.max_car_bloc:
                    trame = payload[:self.max_car_bloc]
                    payload = payload[self.max_car_bloc:]
                else:
                    trame = payload
                    payload = b""

                if payload or (self.mode_etb and trame[0] != b"L"):
                    end = ETB
                else:
                    end = ETX

                trame = b"%d%s" % (ind, trame)

                if self.checksum_ignore_end:
                    checksum = self.checksum(trame)
                else:
                    checksum = self.checksum(trame + end)

                trame = b"%c%s%c%02X\r\n" % (STX, trame, end, checksum)

                trames.append(trame)
                ind += 1
                if ind == 8:
                    ind = 0
        return trames

    @staticmethod
    def checksum(data):
        """Fonction de calcul du checksum, appelé par `build_trames`

        :param data: les data à checksumer
        :return: le checksum des data (int)
        """
        return (sum(ord(c) for c in data) & 0xFF) % 0x100

    def recv_trames(self):
        """Reçoit une liste de trames (message) sur la connexion::

            <- <enq>
            -> <ack>
            <- DATA
            -> <ack>
            <- <eot>

        :return: False si erreur sinon la liste des messages reçus
        """

        # TODO: timeout non spécifié dans le protocole
        data = self.read(timeout=1)

        if not data or data[0] != ENQ:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<enq> expected in %s: wait %ds",
                byte2log(data),
                self.rtimeout,
            )
            return False

        self.write(ACK)

        trames = b""

        while True:
            # TODO il faut faire un read avec un timeout 15
            # pour l'ensemble de la trame
            ctrl = self.read(timeout=15)

            if not ctrl:
                return False

            if ctrl[0] == EOT:
                break

            if ctrl[0] != STX:
                self.logger.warning("<stx> expexted in %s", byte2log(ctrl))
                return False

            data = self.readline(timeout=15)

            if self.check_trame(data):
                self.write(ACK)
                trames += self.get_payload(data)
            else:
                self.write(NACK)

        messages = b"\n".join(trames.splitlines())
        return self.get_messages(messages.splitlines())

    @staticmethod
    def get_messages(trames):
        """Retourne la liste de messages '\n' joined."""

        # ASTM over ASTM
        if trames and trames[0][0] == b"H":

            messages = []
            current_message = []

            for line in trames:
                if line and line[0] == b"H":
                    if current_message:
                        messages.append(b"\n".join(current_message))
                        current_message = []

                current_message.append(line)

            if current_message:
                messages.append(b"\n".join(current_message))

            return messages
        else:
            return [b"\n".join(trames)]

    def check_trame(self, trame):
        """Vérifie qu'une trame est valide (checksum)

        :return: True si la trame est valide, sinon False
        :param trame: La trame à vérifier.
        """
        try:
            checksum = int(trame[-4:-2], 16)
            if self.checksum_ignore_end:
                checksum_expected = self.checksum(trame[:-5])
            else:
                checksum_expected = self.checksum(trame[:-4])
        except ValueError:
            self.logger.warning("Missing checksum for %s", byte2log(trame))
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False

        return True

    def get_payload(self, trame):
        """Retourne les informations utiles de la trame.
        Appelé par `recv_trames`
        <index><PAYLOAD><cr><etx|etb><checksum><cr><lf>
        """
        payload = trame[1:-5]
        if not self.mode_iq200:
            if trame[-5] == ETX and payload and payload[-1] != b"\r":
                payload += b"\r"

            # TODO: Advia 1200/1800...
            if (
                trame[-5] == ETB and
                payload[-1] != b"\r" and
                not self.end_payload_with_cr
            ):
                payload += b"\r"
        else:
            payload = b"".join(payload.decode("hex").split(b"\x00"))

        return payload
