# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
import Queue
import threading

import six

from driverng.transport.base import TransportException


class BaseProtocol(object):

    def __init__(
        self, is_automate=False, logger=None, encoding=None, **kwargs
    ):
        """:param is_automate: True si utilisé pour simuler l'automate."""
        self.transport = None
        self.is_automate = is_automate
        # Nombre d'essais d'envoi de trames
        self.tries = 0
        self.rtimeout = 0.05
        self.connected = False
        if logger is None:
            logger = logging.getLogger("driverng.protocol")
        self.logger = logger
        self.encoding = encoding
        super(BaseProtocol, self).__init__()

    def send_trames(self, trames):
        raise NotImplementedError

    def build_trames(self, message):
        raise NotImplementedError

    def recv_trames(self):
        raise NotImplementedError

    def connect(self, transport):
        self.transport = transport
        self.connected = True

    def write(self, data):
        return self.transport.write(data)

    def read(self, size=1, timeout=None):
        if timeout is not None:
            self.transport.set_timeout(timeout)
        return self.transport.read(size)

    def readline(self, timeout=None, **kwargs):
        if timeout is not None:
            self.transport.set_timeout(timeout)
        return self.transport.readline(**kwargs)

    def can_send_trames(self, wqueue):
        """Méthode appellée avant d'appeller send_trames
        Utile pour les protocoles à états.
        """
        return True

    def handle_reinit(self):
        """Méthode appellée pour reinitialiser la communication
        quand self.connected est False
        """
        self.connected = True

    def _start(self, rqueue, wqueue):
        data = trames = uuid = None
        while True:
            try:
                if not self.connected:
                    self.handle_reinit()

                # On reset le read timeout à chaque bloucle de poll()
                rtimeout = self.rtimeout
                self.rtimeout = 0.05

                if self.transport.poll(rtimeout, wqueue):

                    # On a lu quelque chose, il est probable qu'on ait aussi
                    # quelque chose à lire la prochaine boucle (permet en
                    # particulier d'eviter les ENQ clash sur le protocole ASTM)
                    # NB: Le rtimeout peut aussi être modifié dans recv_trames()
                    self.rtimeout = 1

                    self.transport.pre_recv()
                    recv_trames = self.recv_trames()
                    filename = self.transport.get_recv_filename()
                    self.transport.post_recv()
                    if recv_trames:
                        # On peut recevoir plusieurs messages
                        # dans la même session
                        if isinstance(recv_trames, list):
                            messages = recv_trames
                        else:
                            messages = [recv_trames]
                        for message in messages:
                            rqueue.put({
                                "data": message.decode(self.encoding),
                                "filename": filename,
                                "uuid": None,
                            })
                    continue

                if trames is None:
                    if not self.can_send_trames(wqueue):
                        continue
                    try:
                        data = wqueue.get(True, 0.05)
                        uuid = data["uuid"]
                        if isinstance(data["data"], six.text_type):
                            data_bytes = data["data"].encode(self.encoding)
                        else:
                            assert isinstance(data["data"], six.binary_type)
                            data_bytes = data["data"]
                        trames = self.build_trames(data_bytes)
                        meta = data["meta"]
                    except Queue.Empty:
                        continue

                self.tries = 0
                self.transport.pre_send(meta=meta)
                send_trames_ok = self.send_trames(trames)
                self.transport.post_send()
                if send_trames_ok:
                    rqueue.put({"data": None, "uuid": uuid})
                    uuid = trames = data = None
            except TransportException:
                if data is not None:
                    self.logger.warning(
                        "TransportException for uuid %s", uuid)
                    # Retransmission
                    wqueue.put(data)
                raise

    def start(self, rqueue, wqueue, background=True):
        if background:
            t = threading.Thread(target=self._start, args=(rqueue, wqueue))
            t.daemon = True
            t.start()
            return t
        else:
            self._start(rqueue, wqueue)
