# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from driverng.protocol.base import BaseProtocol
from driverng.utils import (
    RS, GS, ACK, NACK, STX, ETX,
    ENQ, EOT, byte2log,
)


class BCIProtocol(BaseProtocol):
    MAX_RECORD_LENGTH = 79
    MAX_TRY = 3

    def wait_for_ctrl(self, ctrl, timeout=10):
        data = self.read(timeout=timeout)
        if ctrl == ACK:
            expected = (ACK, NACK)
        else:
            expected = (ctrl,)
        if not data or data[0] not in expected:
            self.logger.info(
                "%s expected in data '%s'",
                byte2log(ctrl),
                byte2log(data),
            )
        return data[0] == ctrl

    def send_trames(self, trames):
        # -> <enq>
        # <- <ack>
        # -> <stx><rs>foo<gs><checksum><etx>
        # <- <ack>
        # -> <eot>
        self.write(ENQ)
        if self.wait_for_ctrl(ACK):
            for trame in trames:
                for i in range(self.MAX_TRY):
                    self.write(trame)
                    if self.wait_for_ctrl(ACK):
                        break
        self.write(EOT)

        return True

    def build_trames(self, message):
        # <stx><rs><payload><rs><payload><gs><checksum><etx>
        # Chaque record peut être terminé par CRLF (optionnel).
        # le couple <rs><payload> peut se répéter plusieurs fois
        # jusqu'au <gs>
        # Le checksum se calcule sur <rs><payload><gs>
        trames = []
        for line in message.splitlines():
            # On split line tous les MAX_RECORD_LENGTH caracteres
            splitted = [
                line[i:i + self.MAX_RECORD_LENGTH]
                for i in range(0, len(line), self.MAX_RECORD_LENGTH)
            ]
            line = b"%c%s%c" % (
                RS,
                RS.join(splitted),
                GS,
            )
            trames.append(b"%c%s%02X%c" % (
                STX,
                line,
                self.checksum(line),
                ETX,
            ))
        return trames

    def recv_trames(self):
        # <- <enq>
        # -> <ack>
        # <- <stx><rs>foo<gs><checksum><etx>
        # -> <ack>
        # <- <eot>

        if not self.wait_for_ctrl(ENQ):
            return False

        self.write(ACK)
        messages = []

        while True:
            data = self.read()

            # Fin de session
            if not data or data[0] == EOT:
                break

            # Debut de communication message
            if data[0] != STX:
                self.logger.warning("<stx> expected in %s", byte2log(data))
                return False

            data += self.readline(
                break_func=lambda x: len(x) and x[-1] == ETX)

            if self.check_trame(data):
                self.write(ACK)
                messages.append(self.get_payload(data))
            else:
                self.write(NACK)

        return messages

    def check_trame(self, trame):
        # <stx><payload><gs><checksum><etx>
        try:
            checksum = int(trame[-3:-1], 16)
        except ValueError:
            self.logger.warning("Missing checksum for %s", byte2log(trame))
            return False

        expected = self.checksum(trame[1:-3])

        if checksum != expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                expected,
            )
            return False

        return True

    def checksum(self, data):
        return (sum(ord(c) for c in data) & 0xFF) % 0x100

    def get_payload(self, data):
        # <stx><payload><gs><checksum><etx>
        return data[1:-4].replace(RS, b"")
