# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.utils import byte2log, STX, ETX, ACK, NACK
from driverng.protocol.base import BaseProtocol


class BaseBN2Protocol(BaseProtocol):

    def check_trame(self, trame):
        try:
            checksum = ord(trame[-3])
            checksum_expected = self.checksum(trame[1:-3])
        except ValueError:
            self.logger.warning("Missing checksum for %s", byte2log(trame))
            return False
        else:
            if checksum != checksum_expected:
                self.logger.warning(
                    "Bad checksum for %s: (%02X != %02X)",
                    byte2log(trame),
                    checksum,
                    checksum_expected,
                )
                return False
            else:
                return True

    @staticmethod
    def get_payload(data):
        return data[1:-3]

    def build_trames(self, message):
        return b"%c%s%c\r%c" % (
            STX,
            message,
            self.checksum(message),
            ETX,
        )

    @staticmethod
    def checksum(data):
        bcc = 64 - (sum((ord(c) for c in data)) % 64)
        if bcc < 32:
            return bcc + 64
        else:
            return bcc


class BN2SerHostProtocol(BaseBN2Protocol):
    POLL_IDLE = 15
    POLL_NEXT = 1
    EMPTY_RESULT = b"D" + 29 * b" "
    EMPTY_QC = b"R" + 4 * b" "

    def __init__(self, *args, **kwargs):
        self.poll_timing = self.POLL_IDLE
        self.last_poll = None
        self.irequest = 0
        super(BN2SerHostProtocol, self).__init__(*args, **kwargs)

    def send_trames(self, trame):
        self.write(trame)
        ack = self.recv_trames(ack=False)
        if ack == "A":
            return True
        else:
            self.logger.error("Message rejected")
            return False

    def can_send_trames(self, wqueue):
        if self.is_automate:
            return True
        else:
            if not wqueue.empty():
                return True
            else:
                if self.last_poll is None or time.time() - self.last_poll > self.poll_timing:
                    if self.irequest % 5 == 0:
                        self.write(self.build_trames(b"R"))
                        self.irequest = 0
                    else:
                        self.write(self.build_trames(b"D"))
                    self.irequest += 1
                    self.last_poll = time.time()
        return False

    def recv_trames(self, ack=True):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False
        data += self.readline(break_func=lambda s: s and s[-1] == ETX)
        if self.check_trame(data):
            if ack:
                self.write(self.build_trames(b"A"))
            payload = self.get_payload(data)
            if not self.is_automate:
                if payload.startswith(self.EMPTY_RESULT) or payload.startswith(self.EMPTY_QC):
                    # Fin des resultats/QC
                    self.poll_timing = self.POLL_IDLE
                    return False
                else:
                    # Il reste potentiellement des resultats
                    # on renvoie une demande de resultats
                    self.poll_timing = self.POLL_NEXT
            return payload
        else:
            if ack:
                self.write(self.build_trames(b"E"))
            return False


class BN2AutoHostProtocol(BaseBN2Protocol):

    def send_trames(self, trame):
        self.write(trame)
        data = self.read(timeout=10)
        if data == ACK:
            return True
        else:
            self.logger.error("Message rejected")
            return False

    def recv_trames(self, ack=True):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False
        data += self.readline(break_func=lambda s: s and s[-1] == ETX)
        if self.check_trame(data):
            if ack:
                self.write(ACK)
            payload = self.get_payload(data)
            return payload
        else:
            self.write(NACK)
            return False
