# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.base import BaseProtocol
from driverng.utils import SOH


def tochar(value):
    return chr(value + 32)


def unchar(value):
    return ord(value) - 32


class KermitProtocol(BaseProtocol):
    MAXL = 94
    TIME = 50
    PAD = 0

    def __init__(self, *args, **kwargs):
        self.seq = 0
        super(KermitProtocol, self).__init__(*args, **kwargs)

    def build_trames(self, messages):
        "<soh><len><seq><type><data><checksum><eol>."
        messages = messages.replace(b"#", b"##")
        message = b"".join(messages.splitlines())
        trames = []
        for trame in [message[i:i + self.MAXL]
                      for i in range(0, len(message), self.MAXL)]:
            trames.append(b"D%s" % trame)
        return trames

    def get_seq(self):
        seq = self.seq
        self.seq += 1
        if self.seq >= 64:
            self.seq = 0
        return seq

    def _build_trame(self, data, seq=None):
        if seq is None:
            seq = self.get_seq()
        line = b"%c%c%s" % (
            tochar(len(data) + 2),  # len + seq + data,
            tochar(seq),
            data,
        )
        return b"%c%s%s\r" % (
            SOH,
            line,
            tochar(self.checksum(line)),
        )

    @staticmethod
    def checksum(data):
        # <len><seq><type><data>
        s = sum([ord(c) for c in data])
        return ((s + (s & 0xC0) / 0x40) & 0x3F)

    @staticmethod
    def get_payload(data):
        return data[3:-2]

    def send_trame(self, data):
        self.write(data)
        ack = self.readline(break_func=lambda s: s and s[-1] == b"\r")
        return ack

    def send_trames(self, trames):
        self.seq = 0
        self.send_trame(self._build_trame(b"S%c%c%c@%c#N1" % (
            tochar(self.MAXL),
            tochar(self.TIME),
            tochar(self.PAD),
            tochar(ord(b"\r")),
        )))
        self.send_trame(self._build_trame(b"FS001"))
        for trame in trames:
            self.send_trame(self._build_trame(trame))
        self.send_trame(self._build_trame(b"Z"))
        self.send_trame(self._build_trame(b"B"))
        return True

    def recv_trame(self, header=False):
        data = self.readline(break_func=lambda s: s and s[-1] == b"\r")
        payload = self.get_payload(data)
        if payload[0] == b"S":
            trame = b"Y%s" % payload[1:]
        else:
            trame = b"Y"
        self.write(self._build_trame(trame, seq=unchar(data[2])))
        return payload

    def recv_trames(self):
        # header
        self.recv_trame()
        messages = []
        payload = b""
        while True:
            data = self.recv_trame()
            if data[0] == b"F" and payload:
                messages.append(payload)
                payload = b""
            elif data[0] == b"D":
                payload += data[1:]
            elif data[0] == b"B":
                if payload:
                    messages.append(payload)
                    payload = b""
                break
            elif data[0] != b"Z":
                if payload:
                    messages.append(payload)
                    payload = b""

        # The analyzer always uses the # character to quote
        # control characters in the data stream (currently none
        # are expected in the data). The laboratory computer
        # must convey the character it will use for this purpose;
        # the character is sent literally and is not encoded. The
        # quote character itself must be quoted in the data
        # stream; that is, if a # character is found in the data, it
        # must be preceded by a # character in the packet
        return [p.replace(b"]", b"]\n").replace(b"##", b"#") for p in messages]
