# -*- coding: utf8 -*-
from __future__ import unicode_literals

import errno
import hashlib
import select
import struct
import traceback

from driverng.protocol.message_pb2 import MCAMessage


class MCAProtocolError(Exception):
    pass


class MCAProtocol(object):
    """Gestion du procotole MCA. C'est le protocole utilisé
    entre ServiceAuto et le Driver.
    C'est un échange asynchrone de messages MCAMessage (protobuf).

    Les données sont envoyées sur le réseau dans le format suivant:
    <size sha1 + data sur 4 bytes (unsigned long)>
    <SHA1 du bloc de DATA sur 40 bytes>
    <DATA sur size - 40 bytes>
    """

    RECV_BUFFER_LENGTH = 1024
    SELECT_TIMEOUT = 0.5

    def __init__(self, sock, logger=None):
        """:param sock: Peut être n'importe quel objet qui implémente fileno()
        recv(), send() et close().
        """

        self.sock = sock
        self.rbuf = b""
        self.wbuf = b""
        self.logger = logger
        self.global_exit = False
        super(MCAProtocol, self).__init__()

    def run_protocol(self):
        """Lance le protocole et gère les buffers de lecture/écriture."""
        fd = self.sock.fileno()
        while not self.global_exit:
            try:
                wfds = [fd] if self.wbuf else []
                rfds, wfds, _ = select.select([fd], wfds, [],
                                              self.SELECT_TIMEOUT)
                if fd in rfds:
                    self.recv()
                if fd in wfds:
                    self.send()
                else:
                    self.do_something()
            except select.error as exc:
                if exc[0] == errno.EINTR:
                    break
                else:
                    self.handle_traceback()
                    self.close()
                    raise
            except Exception:
                self.handle_traceback()
                self.close()
                raise

        self.close()

    def close(self):
        try:
            self.sock.close()
        except Exception:
            self.handle_traceback()
            raise

    def do_something(self):
        """Callback à implémenter pour effectuer des actions régulières.
        Elle est appelée tous les SELECT_TIMEOUT si le buffer de sortie
        est vide.
        """
        pass

    def recv(self):
        data = self.sock.recv(self.RECV_BUFFER_LENGTH)
        if not data:
            raise MCAProtocolError("Read 0 bytes")
        self.rbuf += data
        self.check_for_message()

    def write(self, data):
        self.wbuf += data

    def send(self):
        sent = self.sock.send(self.wbuf)
        if sent > 0:
            self.wbuf = self.wbuf[sent:]
        else:
            raise MCAProtocolError("Write 0 bytes")

    def send_message(self, msg):
        """Envoi d'un MCAMessage suivant le protocole MCA."""
        self.send_data(msg.SerializeToString())

    def send_data(self, data):
        sha1 = hashlib.sha1(data).hexdigest()
        buf = b"%s%s" % (sha1, data)
        length = struct.pack(b">L", len(buf))
        self.write(length + buf)

    def check_for_message(self):
        """On extrait les MCAMessage du buffer de lecture.
        Appel à handle_message() pour chaque message reçu.
        """

        if len(self.rbuf) < 4:
            return

        datalen = struct.unpack(b">L", self.rbuf[:4])[0]

        if datalen == 0:
            self.rbuf = self.rbuf[4:]
            return

        if len(self.rbuf[4:]) >= datalen:
            sha1 = self.rbuf[4:][:40]
            data = self.rbuf[44:][:datalen - 40]

            if hashlib.sha1(data).hexdigest() != sha1:
                raise MCAProtocolError("Wrong checksum !")

            self.rbuf = self.rbuf[4 + datalen:]
            msg = MCAMessage.FromString(data)
            self.handle_message(msg)
            return self.check_for_message()

    def handle_message(self, msg):
        """Cette méthode est à implémenter pour gérer les messages d'entrée."""
        raise NotImplementedError("handle_message() must by implemented "
                                  "by child classes")

    def handle_traceback(self):
        if self.logger:
            for line in traceback.format_exc().decode("ISO-8859-15").splitlines():
                self.logger.error(line)
        else:
            traceback.print_exc()
