# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import STX, ETX, byte2log
from driverng.protocol.base import BaseProtocol


class PvsProtocol(BaseProtocol):

    def __init__(self, **kwargs):
        self.seq = 0
        super(PvsProtocol, self).__init__(**kwargs)

    # la sequence débute à 0 et s'incrémente jusqu'à 63
    def get_seq(self):
        seq = self.seq
        if self.seq >= 63:
            self.seq = 0
        return seq

    def build_trames(self, message):
        # <stx>FN:<sequence>|<payload><cr><lf><checksum><etx>
        seq = self.get_seq()
        message_final = b"FN:%.02d|%s\r\n" % (
            seq,
            message,
        )
        self.seq += 1
        return b"%c%s%02X%s" % (
            STX,
            message_final,
            self.checksum_xor(message_final),
            ETX,
        )

    def send_trames(self, trame):
        self.write(trame)
        data = self.read(timeout=10)
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False
        data += self.readline(break_func=lambda s: s and s[-1] == ETX)

        if not data or data[11:14] not in (b'ACK', b'NAK'):
            self.logger.info(
                "message type inattendu %s", byte2log(data[11:14]))
            return False

        if data[11:14] == b'ACK':
            return True

        return False

    def recv_trames(self):

        data = self.read()

        if not data or data[0] != STX:
            self.logger.warning("<stx> expexted in %s", byte2log(data))
            return False

        data += self.readline(break_func=lambda s: s and s[-1] == ETX)
        if self.check_trame(data):
            ack = b'FN:%02d|TYP:ACK|CHK:%s|\r\n' % (
                self.get_seq(),
                data[-3:-1],
            )
            message_ack = b'%c%s%c' % (
                STX,
                ack,
                ETX,
            )
            self.write(message_ack)
            if data[11:14] != b'SYN':
                return self.get_payload(data)
        else:
            nak = b'FN:%02d|TYP:NAK|ERR:CS|CHK:%s|\r\n' % (
                self.get_seq(),
                data[-3:-1],
            )
            message_nak = b'%c%s%c' % (
                STX,
                nak,
                ETX,
            )
            self.write(message_nak)
            return False

    def check_trame(self, trame):
        try:
            checksum = int(trame[-3:-1], 16)
            checksum_expected = self.checksum_xor(trame[1:-3])
        except ValueError:
            self.logger.warning(
                "Missing checksum for %s", byte2log(trame),
            )
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False
        return True

    def get_payload(self, data):
        # <stx>FN:<sequence>|<payload><cr><lf><checksum><etx>
        return data[7:-5]

    @staticmethod
    def checksum_xor(data):
        # The checksum is calculated by the following procedure:
        # Exclusive XOR of all characters after characters <stx>
        # but including characters <CR><LF> at the end and excluded
        # <stx> and <etx> and <checksum>.
        # Binary OR of the remaining value and 0xFF
        # OR of the remaining value increased by 1 and 0xFF
        # The remaining checksum is represented as two-digit
        # hexadecimal number.
        chk = 0
        for c in data:
            chk ^= ord(c)
        chk ^= 0xFF
        chk += 1
        chk &= 0xFF
        return chk
