# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import byte2log, ACK, NACK, STX, ETX
from driverng.protocol.base import BaseProtocol


class SedimaxProtocol(BaseProtocol):

    def send_trames(self, trame):
        self.write(trame)
        data = self.read(timeout=10)
        if not data or data[0] != ACK:
            if data[0] != NACK:
                self.logger.info(
                    "Unexpected data '%s', wait 10 seconds and retry",
                    byte2log(data),
                )
            self.rtimeout = 10
            return False
        else:
            return True

    def build_trames(self, message):
        # <stx><payload><etx><checksum><cr><lf>
        return b"%c%s%c%02X\r\n" % (
            STX,
            message,
            ETX,
            self.checksum(message + ETX),
        )

    def recv_trames(self):
        data = self.read()
        if not data or data[0] != STX:
            self.logger.warning("<stx> expected in %s", byte2log(data))
            return False

        data += self.readline()
        if self.check_trame(data):
            self.write(ACK)
            if data[1] == b"S" or data[1:3] == b"CH" or data[1:4] == b"QS2":
                return self.get_payload(data)
        else:
            self.write(NACK)

    def get_payload(self, data):
        # <stx><payload><etx><checksum><cr><lf>
        return data[1:-5]

    def check_trame(self, trame):
        try:
            checksum = int(trame[-4:-2], 16)
            checksum_expected = self.checksum(trame[1:-4])
        except ValueError:
            self.logger.warning(
                "Missing checksum for %s", byte2log(trame),
            )
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False
        return True

    @staticmethod
    def checksum(data):
        """Fonction de calcul du checksum, appelé par `build_trames`
        Somme de tous les caractères en excluant les caractères <stx> et <etx>,
        convertir la somme en hexa, puis faire modulo 256 (0x100).
        :param data: les data à checksumer
        :return: le checksum des data (int)
        """
        return (sum(ord(c) for c in data) & 0xFF) % 0x100
