# -*- coding: utf8 -*-

from __future__ import unicode_literals
import time
from driverng.protocol.base import BaseProtocol
from driverng.utils import ACK, NACK, STX, ETX, byte2log


class ValabProtocol(BaseProtocol):
    CHECKSUM_NONE, CHECKSUM_VALAB = [None, "valab"]
    # Durée maximum d'attente après avoir envoyé une question à valab
    # par exemple une question "N". (On attend ACK, puis le V ou NACK)
    TIMEOUT = 20
    MAX_BUFFER_SIZE = (1024 * 1024)

    def __init__(self, **kwargs):
        self.checksum_type = kwargs.pop("checksum_type", self.CHECKSUM_VALAB)
        self.can_send = True
        self.recv_expired = 0
        self._recvbuffer = ""
        super(ValabProtocol, self).__init__(**kwargs)

    def checksum(self, data):
        chk = 0
        for c in data:
            chk = (chk + ord(c)) % 100
        return chk

    def send_trames(self, trames):
        trame = trames[0]
        self.write(trame)
        if trame[1] in ('R', 'P', 'V', 'N'):
            self.can_send = False
            self.recv_expired = time.time() + self.TIMEOUT
        return True

    def can_send_trames(self, wqueue):
        if not self.can_send:
            if self.recv_expired < time.time():
                self.logger.warning("protocol receive timeout")
                self.can_send = True
        return self.can_send

    def recv_buffer_split(self, start=STX, stop=ETX):
        trames = []
        while self._recvbuffer:
            try:
                stx_index = self._recvbuffer.index(STX)
            except ValueError:
                break
            if stx_index != 0:
                self.logger.warning("received junk data before <stx>: %s",
                                    byte2log(self._recvbuffer[:stx_index]))
            try:
                etx_index = self._recvbuffer.index(ETX)
            except ValueError:
                break
            trames.append(self._recvbuffer[stx_index:etx_index + 1])
            # Truncate the buffer
            self._recvbuffer = self._recvbuffer[etx_index + 1:]
        return trames

    def recv_trames(self):
        """Il y a quelque chose à lire. Cependant, on ne sait pas si l'information
        sera complète.
        """
        self.rtimeout = 0.01
        payloads = []
        self._recvbuffer += self.read(1024 * 512)
        if len(self._recvbuffer) > self.MAX_BUFFER_SIZE:
            self.logger.fatal(
                "receive buffer capacity exhausted (message too long)")
            self._recvbuffer = ""
            return False
        # Split and return payload if present
        # In valab, we expect the following payload:
        # <STX>Message<ETX>
        trames = self.recv_buffer_split()
        if not trames:
            return False
        for trame in trames:
            if trame[1] == NACK:
                self.can_send = True
                self.recv_expired = time.time() + self.TIMEOUT
                self.logger.warning("received <nack>%1s" % (trame[2]),)
                continue
            elif trame[1] == ACK:
                # Don't set can_send to True because
                # receiving an ACK is not sufficient with VALAB.
                # We must not send another "N" command before receiving
                # "R", "P", "V" or a NACK
                self.recv_expired = time.time() + self.TIMEOUT
                continue
            elif trame[1] in ('R', 'P', 'V', 'N'):
                self.can_send = True
                self.recv_expired = time.time() + self.TIMEOUT
                self.send_ack()
            payloads.append(self.get_payload(trame))
        return payloads

    def send_ack(self):
        self.send_trames([b"%c%c%c" % (STX, ACK, ETX)])

    def send_nack(self, n):
        self.send_trames([b"%c%c%1d%c" % (STX, NACK, n, ETX)])

    def build_trames(self, message):
        # <stx><cmd><payload_len><payload><checksum><etx>
        cmd = message[0]
        message_data = message[1:]
        payload = b"%c%04d%s" % (
            cmd,
            len(message_data),
            message_data)
        if self.checksum_type == self.CHECKSUM_NONE:
            return [b"%c%s??%c" % (
                STX,
                payload,
                ETX)]
        elif self.checksum_type == self.CHECKSUM_VALAB:
            return [b"%c%s%02d%c" % (
                STX,
                payload,
                self.checksum(payload),
                ETX,
            )]
        else:
            raise NotImplementedError

    def get_payload(self, data):
        # <stx><cmd[1]><payload_len[4]><payload><checksum[2]><etx>
        # rebuild to get: <cmd[1]><payload[payload_len]>
        cmd = data[1]
        if self.checksum_type == self.CHECKSUM_NONE:
            payload = data[6:-1]
        elif self.checksum_type == self.CHECKSUM_VALAB:
            payload = data[6:-3]
        else:
            raise NotImplementedError
        return "%(cmd)s%(payload)s" % {"cmd": cmd, "payload": payload}
