# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import utils

__all__ = ["Splitted", "String", "Date", "Multiple"]


class Field(object):
    def __init__(self, name, default=None):
        self.name = name
        self.default = default
        super(Field, self).__init__()

    def get_default(self):
        raise NotImplementedError

    def to_python(self, value):
        raise NotImplementedError

    def to_string(self, value):
        raise NotImplementedError


class String(Field):
    def get_default(self):
        return self.default or ""

    def to_python(self, value):
        return value

    def to_string(self, value):
        if value is None:
            value = ""
        return value


class Date(String):
    def __init__(self, *args, **kwargs):

        self.fmt_map = kwargs.pop("fmt_map", utils.FMT_MAP)
        self.fmt_date = kwargs.pop("fmt_date", "%Y%m%d%H%M%S")
        self.auto_now = kwargs.pop("auto_now", False)
        super(Date, self).__init__(*args, **kwargs)

    def get_default(self):
        if self.auto_now:
            return datetime.datetime.now()
        else:
            return self.default

    def to_python(self, value):
        return utils.get_date(value, fmt_map=self.fmt_map, fmt=self.fmt_date)

    def to_string(self, value):
        return super(Date, self).to_string(
            utils.format_date(value, fmt=self.fmt_date))


class MultipleContainer(list):

    def __init__(self, fields, separator):
        self.fields = fields
        self.separator = separator
        super(MultipleContainer, self).__init__()

    def add(self, *args, **kwargs):
        obj = Splitted(fields=self.fields, separator=self.separator, *args, **kwargs)
        self.append(obj)
        return obj

    def to_python(self, data):
        obj = self.add()
        obj.parse(data)
        return obj

    def to_string(self):
        return [e.to_string() for e in self]


class Multiple(object):

    def __init__(self, name, fields, separator):

        self.name = name
        self.fields = fields
        self.separator = separator
        super(Multiple, self).__init__()

    def get_default(self):
        return MultipleContainer(self.fields, self.separator)

    def to_python(self, value):
        return MultipleContainer(self.fields, self.separator).to_python(value)

    def to_string(self, values):
        return [v.to_string() for v in values]


class Splitted(object):
    FIELDS = []
    SEPARATOR = None

    def __init__(self, *args, **kwargs):
        self._locked = False
        self._fields = kwargs.get("fields", self.FIELDS)
        self._separator = kwargs.get("separator", self.SEPARATOR)
        for field in self._fields:
            setattr(self, field.name, kwargs.get(field.name, field.get_default()))
        super(Splitted, self).__init__()
        self._locked = True

    def __setattr__(self, attr, value):
        # On empèche la création d'attributs en dehors de __init__
        if attr != "_locked" and self._locked and not hasattr(self, attr):
            raise AttributeError("Object %s has not attribute '%s'" % (
                self, attr,))
        else:
            return super(Splitted, self).__setattr__(attr, value)

    def parse(self, data):
        i = 0
        for value in data.split(self._separator):
            field = self._fields[i]
            value = field.to_python(value)
            if not isinstance(field, Multiple):
                setattr(self, field.name, value)
                i += 1
            else:
                getattr(self, field.name).append(value)
        return self

    def to_string(self):
        values = []
        for field in self._fields:
            value = field.to_string(getattr(self, field.name))
            if isinstance(field, Multiple):
                values.extend(value)
            else:
                values.append(unicode(value))
        return self._separator.join(values)
