# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import fnmatch
import io
import os
import select
import time

from driverng.utils import byte2log


class TransportException(Exception):
    pass


class BaseTransport(object):

    def __init__(self, logger=None):
        super(BaseTransport, self).__init__()
        self.logger = logger
        self.meta = {}

    def write(self, data):
        self.logger.debug("send %s", byte2log(data, truncate=True))
        return self._write(data)

    def _write(self, data):
        raise NotImplementedError

    def read(self, size):
        data = self._read(size)
        self.logger.debug("recv %s", byte2log(data, truncate=True))
        return data

    def _read(self, size):
        raise NotImplementedError

    def readline(self, break_func=lambda buf: buf[-2:] == b"\r\n"):
        """Lecture d'un bloc de données

        @param break_func: Fonction appelée à chaque bloc lu pour savoir
        s'il faut continuer de lire ou non. Par défaut c'est les caractères
        <cr><lf> en fin de buffer (protocole ASTM)
        """

        data = b""
        while not break_func(data):
            try:
                data += self._read(4096)
            except Exception:
                if data:
                    self.logger.debug("(before error) recv %s", byte2log(data, truncate=True))
                raise

        self.logger.debug("recv %s", byte2log(data, truncate=True))
        return data

    def poll(self, timeout, wqueue):
        r, _, _ = select.select([self.fileno(), ], [], [], timeout)
        if r:
            return True
        else:
            return False

    def flush(self):
        while True:
            if self.poll(2, None):
                self.read(1024)
            else:
                break

    def fileno(self):
        raise NotImplementedError

    def set_timeout(self, timeout):
        pass

    def close(self):
        pass

    def post_send(self):
        self.meta = {}

    def post_recv(self):
        pass

    def pre_send(self, meta):
        self.meta = meta or {}

    def pre_recv(self):
        pass

    def get_recv_filename(self):
        """Retourne le nom du fichier reçu

        Utilisé pour les transports fichiers (file, ftp, etc)
        """
        return None


class BaseFileTransport(BaseTransport):

    def __init__(
        self, backend, input_dir=None, output_dir=None, fmt=None,
        glob=None, name_ok=None, poll_interval=10, **kwargs
    ):
        super(BaseFileTransport, self).__init__(**kwargs)
        self.backend = backend
        self.poll_interval = poll_interval
        self.input_dir = input_dir
        self.output_dir = output_dir
        self.fmt = fmt
        self.glob = glob
        self.name_ok = name_ok
        self._rpath = None
        self._rpath_ok = None
        self._wfile = None
        self._count = 0
        self._ttl = None
        self._files = []

    def list_files(self):
        if self.input_dir and (
            self._ttl is None or (time.time() - self._ttl) > self.poll_interval
        ):
            self._files = self.backend.list(self.input_dir)
            self._ttl = time.time()
        return self._files

    def fileno(self):
        return None

    def _write(self, data):
        self._wfile.write(data)
        return len(data)

    def _read(self, size=None):
        self.logger.info("Read from %s", self._rpath)
        data = self.backend.get(self._rpath)
        return data

    def readline(self, break_func=None):
        return self._read()

    def post_recv(self):
        if self._rpath is not None:
            self.logger.info("Remove %s", self._rpath)
            self.backend.delete(self._rpath)
            if self._rpath_ok is not None:
                self.logger.info("Remove %s", self._rpath_ok)
                self.backend.delete(self._rpath_ok)
            self._rpath = self._rpath_ok = None

    def get_recv_filename(self):
        if self._rpath is not None:
            return os.path.basename(self._rpath)
        else:
            return None

    def get_filename(self):
        self._count = (self._count + 1) % 1000
        return self.fmt.format(
            now=datetime.datetime.now(),
            count=self._count,
        )

    def pre_send(self, meta):
        self._wfile = io.BytesIO()
        super(BaseFileTransport, self).pre_send(meta)

    def post_send(self):
        if self._wfile is not None:
            if "filename" in self.meta:
                path = os.path.join(self.output_dir, self.meta["filename"])
            else:
                filename = self.get_filename()
                path = os.path.join(self.output_dir, filename)
            self.logger.info("Write to %s", path)
            self.backend.put(self._wfile.getvalue(), path)
            if self.name_ok:
                ok_path = "%s.%s" % (
                    os.path.splitext(path)[0],
                    self.name_ok,
                )
                self.logger.info("Write to %s", ok_path)
                self.backend.put(b"", ok_path)
        super(BaseFileTransport, self).post_send()

    def poll(self, timeout, wqueue):
        filenames = sorted(self.list_files())
        for filename in filenames:
            if fnmatch.fnmatch(filename, self.glob):
                if self.name_ok:
                    filename_ok = "%s.%s" % (
                        os.path.splitext(filename)[0],
                        self.name_ok,
                    )
                    if filename_ok in filenames:
                        self._rpath_ok = os.path.join(self.input_dir, filename_ok)
                        self._files.remove(filename_ok)
                    else:
                        # No .ok file
                        continue
                self._rpath = os.path.join(self.input_dir, filename)
                self._files.remove(filename)
                return True
        return False


class BaseFileBackend(object):

    def list(self, path):
        raise NotImplementedError

    def get(self, path):
        raise NotImplementedError

    def delete(self, path):
        raise NotImplementedError

    def put(self, path):
        raise NotImplementedError
