# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import os
import shutil
import fcntl
from tempfile import NamedTemporaryFile

import watchdog.observers
import watchdog.events

from driverng.transport.base import (
    BaseFileTransport,
    BaseFileBackend,
)


class EventHandler(watchdog.events.FileSystemEventHandler):

    def __init__(self, *args, **kwargs):
        self._has_event = False
        super(EventHandler, self).__init__(*args, **kwargs)

    def on_any_event(self, event):
        if (
            not event.is_directory and
            event.event_type in ("created", "modified")
        ):
            self._has_event = True

    def has_event(self):
        return self._has_event

    def clear(self):
        self._has_event = False


class FileBackend(BaseFileBackend):

    def list(self, path):
        return os.listdir(path)

    def get(self, path):
        with open(path, "r") as rfile:
            fcntl.lockf(rfile, fcntl.LOCK_SH)
            data = rfile.read()
            fcntl.lockf(rfile, fcntl.LOCK_UN)
        return data

    def put(self, data, path):
        wfile = NamedTemporaryFile(delete=False)
        wfile.write(data)
        wfile.close()
        with open(wfile.name, "r") as src, open(path, "a") as dst:
            shutil.copyfileobj(src, dst)
        os.unlink(wfile.name)
        os.chmod(path, 0o660)

    def delete(self, path):
        os.unlink(path)


class FileTransport(BaseFileTransport):

    def __init__(self, append=False, **kwargs):
        self.append = append
        backend = FileBackend()
        kwargs.setdefault("poll_interval", 0)
        super(FileTransport, self).__init__(backend, **kwargs)

        for path in [self.input_dir, self.output_dir]:
            if path and not os.path.exists(path):
                os.makedirs(path)

        if self.input_dir:
            self.observer = watchdog.observers.Observer()
            self.handler = EventHandler()
            self.observer.schedule(self.handler, self.input_dir, recursive=False)
            self.observer.start()
        else:
            self.observer = self.handler = None

    def get_filename(self):
        while True:
            filename = super(FileTransport, self).get_filename()
            path = os.path.join(self.output_dir, filename)
            if self.append or not os.path.exists(path):
                break
            self.logger.warning("%s already exists, sleep 1 second", filename)
            time.sleep(1)
        return filename

    def poll(self, timeout, wqueue):
        if self.handler is not None:
            self.handler.clear()

        result = super(FileTransport, self).poll(timeout, wqueue)

        if result:
            return result

        start_time = time.time()
        while abs(start_time - time.time()) < 10:
            if not wqueue.empty() or (
                self.handler is not None and
                self.handler.has_event()
            ):
                break
            else:
                time.sleep(.5)
        return False
