# -*- coding: utf8 -*-
from __future__ import unicode_literals

import io
import os
import time
import ftplib

from driverng.transport.base import (
    BaseFileTransport,
    BaseFileBackend,
    TransportException,
)


def catch_ftp_exception(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except ftplib.all_errors as e:
            raise TransportException(e)
    return wrapper


class FTP(BaseFileBackend):

    def __init__(
        self, hostname=None, port=None, username=None, password=None,
        mode="active", transfert_mode="binary", logger=None,
        del_command_disable=False,
    ):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.port = port
        self.mode = mode
        self.transfert_mode = transfert_mode
        self._ttl = time.time()
        self._ftp = None
        self.logger = logger
        self.del_command_disable = del_command_disable

    @property
    def _expired(self):
        return (time.time() - self._ttl) > 120

    @catch_ftp_exception
    def _reinit_connection(self):
        if self._ftp is not None:
            self._ftp.close()
        self._ftp = ftplib.FTP()
        self._ftp.connect(self.hostname, self.port)
        self._ftp.login(self.username, self.password)
        self._ftp.set_pasv(self.mode == "passive")
        self.logger.info(
            "Connected to ftp://%s@%s:%s",
            self.username, self.hostname, self.port,
        )

    @property
    def ftp(self):
        if self._ftp is None or self._expired:
            self._reinit_connection()
        self._ttl = time.time()
        return self._ftp

    @catch_ftp_exception
    def cwd(self, path):
        self.ftp.cwd(path)

    @catch_ftp_exception
    def list(self, path=None):
        files = []
        for filepath in self.ftp.nlst(path):
            files.append(os.path.basename(filepath))
        return files

    @catch_ftp_exception
    def put(self, data, filename):
        if self.transfert_mode == "ascii":
            func = self.ftp.storlines
        else:
            func = self.ftp.storbinary
        return func('STOR ' + filename, io.BytesIO(data))

    @catch_ftp_exception
    def get(self, filename):
        if self.transfert_mode == "ascii":
            func = self.ftp.retrlines
        else:
            func = self.ftp.retrbinary
        buf = io.BytesIO()
        func('RETR ' + filename, buf.write)
        return buf.getvalue()

    @catch_ftp_exception
    def delete(self, filename):
        if not self.del_command_disable:
            return self.ftp.delete(filename)


class FTPTransport(BaseFileTransport):
    def __init__(
        self, hostname=None, username=None, port=None, password=None,
        mode="active", transfert_mode="binary", host=None, **kwargs
    ):
        del_command_disable = kwargs.pop("del_command_disable", False)
        if host is not None and hostname is None and port is None:
            if ":" in host:
                hostname, port = host.rsplit(":", 1)
                port = int(port)
            else:
                hostname = host
        backend = FTP(
            hostname or "localhost",
            port or 21,
            username or "anonymous",
            password, mode, transfert_mode, kwargs.get("logger"),
            del_command_disable)
        super(FTPTransport, self).__init__(backend, **kwargs)
