# -*- coding: utf8 -*-
from __future__ import unicode_literals

import serial
import select
import os

from serial.serialutil import SerialException, portNotOpenError
from driverng.transport.base import BaseTransport, TransportException


class MySerial(serial.Serial):

    def may_read(self, size=1):
        # Le serial.Serial.read() (posixserial.py) ne retourne pas tand que
        # exactement `size` bytes a été lu ou s'il y a un timeout. Donc si on ne
        # connait pas la taille des données mais qu'on veut lire un bloc jusqu'à
        # une condition d'arrêt sur un protocole synchrone il faut utiliser
        # may_read()
        # Ce may_read() permet de lire jusqu'à `size` bytes et retourne ce qu'il
        # a lu imédiatement
        if self.fd is None:
            raise portNotOpenError
        read = bytearray()
        ready, _, _ = select.select([self.fd], [], [], self._timeout)
        # If select was used with a timeout, and the timeout occurs, it
        # returns with empty lists -> thus abort read operation.
        # For timeout == 0 (non-blocking operation) also abort when there
        # is nothing to read.
        if not ready:
            raise TransportException("Timeout")
        buf = os.read(self.fd, size)
        # read should always return some data as select reported it was
        # ready to read when we get to this point.
        if not buf:
            # Disconnected devices, at least on Linux, show the
            # behavior that they are always ready to read immediately
            # but reading returns nothing.
            raise SerialException(
                'device reports readiness to read '
                'but returned no data (device disconnected?)')
        read.extend(buf)
        return bytes(read)


class SerialTransport(BaseTransport):

    def __init__(self, *args, **kwargs):
        serial_kwargs = {}
        for arg in ("port", "baudrate", "parity",
                    "bytesize", "stopbits", "xonxoff"):
            if arg in kwargs:
                serial_kwargs[arg] = kwargs.pop(arg)

        super(SerialTransport, self).__init__(**kwargs)
        self.transport = MySerial(**serial_kwargs)

    def fileno(self):
        return self.transport.fileno()

    def set_timeout(self, timeout):
        self.transport.timeout = timeout

    def _read(self, size):
        return self.transport.may_read(size)

    def _write(self, data):
        return self.transport.write(data)
