# -*- coding: utf8 -*-
from __future__ import unicode_literals

import errno
import socket
import select

from driverng.transport.base import BaseTransport, TransportException


class TCPTransport(BaseTransport):

    def __init__(self, host=None, port=None, mode="server", **kwargs):
        super(TCPTransport, self).__init__(**kwargs)

        if port is None:
            host, port = host.rsplit(":", 1)
            port = int(port)

        if mode == "server":
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.set_keepalive(s)
            s.bind((host, port))
            s.listen(1)
            self.logger.info("Waiting for connection on %s:%s", host, port)
            conn, addr = s.accept()
            self.logger.info("Connected by %s", addr)
            self.socket = s
            self.transport = conn
        else:  # client
            self.logger.info("Try to connect to %s:%d", host, port)
            try:
                self.transport = socket.create_connection(
                    (host, port),
                    10,
                )
                self.set_keepalive(self.transport)
            except socket.gaierror as e:
                raise TransportException("Disconnected %s" % e)
            except socket.timeout:
                raise TransportException("Timeout")
            except socket.error as e:
                if e.args[0] in (errno.ECONNREFUSED,):
                    raise TransportException("Disconnected (errno=%d)" % (
                        e.args[0],
                    ))
                else:
                    raise
            else:
                self.logger.info("Connected to %s:%d", host, port)
            self.socket = None

    @staticmethod
    def set_keepalive(s):
        s.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)

        # Activate after 5 second of idle
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 5)

        # ping every 60 second
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPINTVL, 60)

        # Timeout after 2 failed ping
        s.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPCNT, 2)

    def fileno(self):
        return self.transport.fileno()

    def set_timeout(self, timeout):
        self.transport.settimeout(timeout)

    def poll(self, timeout, wqueue):
        rfds = [self.transport]

        if self.socket is not None:  # serveur
            rfds.append(self.socket)

        r, _, _ = select.select(rfds, [], [], timeout)

        if self.socket is not None and self.socket in r:
            conn, addr = self.socket.accept()
            self.transport = conn
            self.logger.info("Connection remplaced by %s", addr)
            return False

        if self.transport in r:
            return True
        else:
            return False

    def _read(self, size):
        data = None

        r, _, _ = select.select([self.fileno(), ], [], [],
                                self.transport.gettimeout())

        if not r:
            raise TransportException("Timeout")

        try:
            data = self.transport.recv(size)
        except socket.error as e:
            if e.args[0] in (
                errno.EBADF,
                errno.ECONNRESET,
                errno.ENOTCONN,
                errno.ESHUTDOWN,
                errno.ECONNABORTED,
                errno.ETIMEDOUT,
            ):
                raise TransportException("Disconnected (errno=%d)" % e.args[0])
            else:
                raise

        if not data:
            raise TransportException("Disconnected (recv 0 bytes)")

        return data

    def _write(self, data):

        _, w, _ = select.select([], [self.fileno(), ], [],
                                self.transport.gettimeout())

        if not w:
            raise TransportException("Timeout")

        totalsent = 0
        msglen = len(data)

        while totalsent < msglen:

            try:
                sent = self.transport.send(data[totalsent:])
            except socket.error as e:
                if e.args[0] in (
                    errno.EBADF,
                    errno.ECONNRESET,
                    errno.ENOTCONN,
                    errno.ESHUTDOWN,
                    errno.ECONNABORTED,
                    errno.ETIMEDOUT,
                ):
                    raise TransportException("Disconnected (errno=%d)" % (
                        e.args[0],
                    ))
                else:
                    raise

            if sent == 0:
                raise TransportException("Disconnected (sent 0 bytes)")

            totalsent += sent

    def close(self):
        try:
            self.transport.close()
            if self.socket is not None:
                self.socket.close()
        except socket.error as e:
            if e.args[0] not in (errno.ENOTCONN, errno.EBADF):
                raise
