# -*- coding: utf8 -*-

from __future__ import unicode_literals

import collections
import logging
import datetime
import json
import decimal
import dateutil.parser
from xml.dom import minidom

logger = logging.getLogger("driverng")


NULL = b"\x00"
SOH = b"\x01"
STX = b"\x02"
ETX = b"\x03"
EOT = b"\x04"
ENQ = b"\x05"
ACK = b"\x06"
BEL = b"\x07"
BS = b"\x08"
HT = b"\x09"
LF = b"\x0A"
VT = b"\x0B"
FF = b"\x0C"
CR = b"\x0D"
SO = b"\x0E"
SI = b"\x0F"
DLE = b"\x10"
DC1 = b"\x11"
DC2 = b"\x12"
DC3 = b"\x13"
DC4 = b"\x14"
NACK = b"\x15"
SYN = b"\x16"
ETB = b"\x17"
CAN = b"\x18"
EM = b"\x19"
SUB = b"\x1A"
ESC = b"\x1B"
FS = b"\x1C"
GS = b"\x1D"
RS = b"\x1E"
US = b"\x1F"
DEL = b"\x7F"

_PRETTY_MAP = {
    "null": NULL,
    "soh": SOH,
    "stx": STX,
    "etx": ETX,
    "eot": EOT,
    "enq": ENQ,
    "ack": ACK,
    "bel": BEL,
    "bs": BS,
    "ht": HT,
    "vt": VT,
    "ff": FF,
    "so": SO,
    "si": SI,
    "dle": DLE,
    "dc1": DC1,
    "dc2": DC2,
    "dc3": DC3,
    "dc4": DC4,
    "nack": NACK,
    "syn": SYN,
    "etb": ETB,
    "can": CAN,
    "em": EM,
    "sub": SUB,
    "esc": ESC,
    "fs": FS,
    "gs": GS,
    "rs": RS,
    "us": US,
    "del": DEL,
    "cr": CR,
    "lf": LF,
}

FMT_MAP = {
    14: "%Y%m%d%H%M%S",
    12: "%Y%m%d%H%M",
    8: "%Y%m%d",
}


def get_date(raw_value, fmt_map=FMT_MAP, fmt=None):

    if raw_value is None:
        return None

    value = raw_value.strip()

    if not value:
        return None

    date = None
    if fmt is not None:
        try:
            date = datetime.datetime.strptime(value, fmt)
        except ValueError:
            pass

    if date is None:
        if (
            len(value) == 19 and
            value[14] in ("+", "-") and
            value[15:].isdigit()
        ):
            # DateTime avec timezone
            # http://www.hl7standards.com/blog/2008/07/25/hl7-time-zone-qualification/
            # Ignoré pour le moment
            value = value[:14]
        try:
            date = datetime.datetime.strptime(value, fmt_map[len(value)])
        except (ValueError, KeyError):
            pass

    if date is None:
        logger.warning("Impossible de parser la date '%s'", value)
        return None
    else:
        return date


def format_date(date, fmt=None):
    if fmt is None:
        fmt = FMT_MAP[14]
    try:
        if date is not None and date.year >= 1900:
                return date.strftime(fmt)
        elif date is not None:
            # ValueError: the datetime strftime() methods require year >= 1900
            logger.warning("Date %s antérieure à 1900 non transmise", date)
    except Exception:
        logger.warning("Date %s invalide non transmise", date)
    return ""


def pretty_data(data, crlf=True):
    for k, v in _PRETTY_MAP.items():
        if not crlf and v in (CR, LF):
            continue
        data = data.replace(v, "<{0}>".format(k))
    return data


def byte2log(data, truncate=False):
    if truncate and len(data) > 2048:
        data = data[:20] + b" [ ... %s bytes ... ] " % (len(data),) + data[-20:]
    for k, v in _PRETTY_MAP.items():
        data = data.replace(v, "<{0}>".format(k).encode())
    data = data.decode("latin1").encode("unicode_escape").decode("ascii")
    data = data.replace("\\\\", "\\")
    return data


def log2byte(data):
    m = _PRETTY_MAP.copy()
    m.update(**{"cr": CR, "lf": LF})
    data = data.encode("ascii").decode("unicode_escape").encode("latin1")
    for k, v in m.items():
        data = data.replace(b"<" + k.encode() + b">", v)
    return data


def splitlines(data):
    # Only split on \r\n, \r and \n
    splitted = []
    cur = ""
    i = 0
    while i < len(data):
        if data[i:][:2] == "\r\n":
            splitted.append(cur)
            cur = ""
            i += 2
        elif data[i] in ("\r", "\n"):
            splitted.append(cur)
            cur = ""
            i += 1
        else:
            cur += data[i]
            i += 1
    if cur:
        splitted.append(cur)
    return splitted


def num_years(from_date, to_date):
    if from_date is not None and to_date is not None:
        return int((to_date - from_date).days / 365.2425)


def num_month(from_date, to_date):
    if from_date is not None and to_date is not None:
        return int((to_date - from_date).days / 30)


def num_days(from_date, to_date):
    if from_date is not None and to_date is not None:
        return (to_date - from_date).days


def is_aware(value):
    """Determines if a given datetime.datetime is aware.

    The logic is described in Python's docs:
    http://docs.python.org/library/datetime.html#datetime.tzinfo
    """
    return value.tzinfo is not None and value.tzinfo.utcoffset(value) is not None


class JSONEncoder(json.JSONEncoder):
    """JSONEncoder subclass that knows how to encode date/time and decimal types.
    Taken from django.core.serializers.json.DjangoJSONEncoder
    """
    def default(self, o):
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(o, datetime.date):
            return o.isoformat()
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError("JSON can't represent timezone-aware times.")
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return r
        elif isinstance(o, decimal.Decimal):
            return str(o)
        else:
            return super(JSONEncoder, self).default(o)


def decode_isoformat(s):
    if s is not None:
        return dateutil.parser.parse(s)
    else:
        return None


def prettify_xml(data, encoding="UTF-8"):
    return minidom.parseString(data).toprettyxml(indent="  ", encoding=encoding).decode(encoding)


class OrderedSet(collections.MutableSet):
    # http://code.activestate.com/recipes/576694/

    def __init__(self, iterable=None):
        self.end = end = []
        end += [None, end, end]
        self.map = {}
        if iterable is not None:
            self |= iterable

    def __len__(self):
        return len(self.map)

    def __contains__(self, key):
        return key in self.map

    def add(self, key):
        if key not in self.map:
            end = self.end
            curr = end[1]
            curr[2] = end[1] = self.map[key] = [key, curr, end]

    def discard(self, key):
        if key in self.map:
            key, prev, next = self.map.pop(key)
            prev[2] = next
            next[1] = prev

    def __iter__(self):
        end = self.end
        curr = end[2]
        while curr is not end:
            yield curr[0]
            curr = curr[2]

    def __reversed__(self):
        end = self.end
        curr = end[1]
        while curr is not end:
            yield curr[0]
            curr = curr[1]

    def pop(self, last=True):
        if not self:
            raise KeyError('set is empty')
        key = self.end[1][0] if last else self.end[2][0]
        self.discard(key)
        return key

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, list(self))

    def __eq__(self, other):
        if isinstance(other, OrderedSet):
            return len(self) == len(other) and list(self) == list(other)
        return set(self) == set(other)
