# -*- coding: utf8 -*-
from __future__ import unicode_literals

import json
import types
import datetime
import jsonschema
import jsonschema_model
import driverng.utils as utils
from collections import OrderedDict


ORDER_SCHEMA = {
    "name": "Order",
    "type": "object",
    "properties": {
        "code": {
            "title": "Code",
            "type": "string"
        },
        "label": {
            "title": "Label",
            "type": "string"
        },
        "alt_code": {
            "title": "Code alternatif",
            "type": "string",
        },
        "alt_label": {
            "title": "Libelle alternatif",
            "type": "string",
        },
        "priority": {
            "title": "Priority",
            "type": "string"
        },
        "dilution": {
            "title": "Dilution",
            "type": "string"
        },
        "analyseur_repasse": {
            "title": "Analyseur demandé",
            "type": "string"
        },
        "action": {
            "title": "Action",
            "type": "string",
            "enum": ["new", "add", "del"],
        },
        "rerun": {
            "title": "rerun",
            "type": "boolean",
        },
        "ordre": {"type": "number"},
        "order_id": {"type": "string"},
    },
    "required": ["code", "action"],
    "additionalProperties": False,
}

LOT_SCHEMA = {
    "name": "Lot",
    "type": "object",
    "properties": {
        "type": {
            "title": "Type",
            "type": "string",
            "enum": ["reagent", "calibration", "control"]
        },
        "lot": {
            "title": "N°",
            "type": "string"
        },
        "name": {
            "title": "Name",
            "type": "string"
        },
        "expiration": {
            "title": "Expiration",
            "type": "date-time",
        },
    },
    "required": ["type", "lot"],
    "additionalProperties": False,
}

RESULT_SCHEMA = {
    "name": "Result",
    "type": "object",
    "properties": {
        "code": {
            "title": "Code",
            "type": "string"
        },
        "label": {
            "title": "Label",
            "type": "string"
        },
        # Attention sub_code ce n'est pas la meme chose, c'est utilisé
        # dans des scénarios différents pour les résultats de bactériologie.
        "alt_code": {
            "title": "Code alternatif",
            "type": "string",
        },
        "alt_label": {
            "title": "Libelle alternatif",
            "type": "string",
        },
        "type": {
            "title": "Type",
            "type": "string",
        },
        "sub_code": {
            "title": "Sub code",
            "type": "string",
        },
        "value": {
            "title": "Value",
            "type": ["string", "array", "object"],
        },
        "value_label": {
            "title": "Value labels",
            "type": ["string", "array", "object"],
        },
        "resu_json": {
            "title": "Resu json",
            "type": "object",
            "properties": {
                "germes": {
                    "title": "Germes",
                    "type": "array",
                },
                "resultat": {
                    "title": "Resultat",
                    "type": "string",
                },
                "type_resultat": {
                    "title": "Type resultat",
                    "type": "string",
                }
            }
        },
        "graph": {
            "type": "object",
        },
        "date": {
            "title": "Date",
            "type": "date-time",
        },
        "anterior": {
            "title": "Anterior",
            "type": "boolean",
        },
        "request_id": {
            "title": "Anterior request ID",
            "type": "string",
        },
        "request_date": {
            "title": "Anterior request date",
            "type": "date-time",
        },
        "unit": {
            "title": "Unit",
            "type": "string",
        },
        "flags": {
            "title": "Flags",
            "type": "array", "items": {"title": "Flag", "type": "string"},
        },
        "comments": {
            "title": "Comments",
            "type": "array", "items": {"title": "Comment", "type": "string"},
        },
        "instrument": {
            "title": "Instrument",
            "type": "string"
        },
        "status": {
            "title": "Status",
            "type": "string",
        },
        "validator": {
            "title": "Validator",
            "type": "string",
        },
        "operator": {
            "title": "Operator",
            "type": "string",
        },
        "dilution": {
            "title": "Dilution",
            "type": "string",
        },
        "date_plateau": {
            "title": "Date plateau",
            "type": "date-time",
        },
        "nom_antibiogramme": {
            "title": "Nom antibiogramme",
            "type": "string",
        },
        "ligne": {
            "title": "Ligne",
            "type": "string",
        },
        # TODO: LEGACY biorad
        "reagent": {"title": "Reagent", "type": "string"},
        "lots": {"type": "array", "items": LOT_SCHEMA},

        # Valide pour MCA/HPRIM uniquement
        "resultat_controle": {"type": "string"},
        "order_id": {"type": "string"},
        "reference_ranges": {"type": "string"},
        "preanalytique": {
            "title": "Pré-anlytique",
            "type": "boolean",
        },
    },
    "required": ["code", "value"],
    "additionalProperties": False,
}

BILLING_SCHEMA = {
    "name": "Billing",
    "type": "object",
    "properties": {
        "lettre_cle": {
            "title": "Lettre cle",
            "type": "string"
        },
        "coefficient": {
            "title": "Coefficient",
            "type": "number"
        },
        "denombrement": {
            "title": "Denombrement",
            "type": "number"
        },
        "quantite": {
            "title": "Quantite",
            "type": "number"
        },
        "libelle_acte": {
            "title": "Libelle de l'acte",
            "type": "string"
        },
        "type_majoration": {
            "title": "Type de majoration",
            "type": "string"
        },
        "no_executant": {
            "title": "Numero executant",
            "type": "string"
        },
        "code_executant": {
            "title": "Code executant",
            "type": "string"
        },
        "nom_executant": {
            "title": "Nom executant",
            "type": "string"
        },
        "code_nomenclature": {
            "title": "Code de nomenclature",
            "type": "string"
        },
        "catalogue": {
            "title": "Catalogue",
            "type": "string"
        },
        "code_rmo": {
            "title": "Code RMO",
            "type": "string"
        }
    },
    "additionalProperties": False,
}

SAMPLE_SCHEMA = {
    "name": "Sample",
    "type": "object",
    "properties": {
        "sample_id": {
            "title": "Sample ID",
            "type": "string"
        },
        "type": {
            "title": "Sample type",
            "type": "string",
            "enum": ["patient", "control"],
        },
        "request_id": {
            "title": "Request ID",
            "type": "string",
        },
        "request_info": {
            "title": "Request Info",
            "type": "string",
        },
        "exe_sample_id": {
            "title": "Sample ID (executant)",
            "type": "string",
        },
        "exe_request_id": {
            "title": "Request ID (executant)",
            "type": "string",
        },
        "receiver_request_id": {
            "title": "Receiver Request ID",
            "type": "string",
        },
        "control_lot": {
            "title": "Lot ID",
            "type": "string"
        },
        "control_name": {
            "title": "Lot Name",
            "type": "string"
        },
        "control_level": {
            "title": "Lot Level",
            "type": "string"
        },
        "rack": {
            "title": "Rack",
            "type": "string"
        },
        "collection_date": {
            "title": "Collection date",
            "type": "date-time",
        },
        "collection_end_date": {
            "title": "Collection end date",
            "type": "date-time",
        },
        "request_date": {
            "title": "Request date",
            "type": "date-time"
        },
        "admission_date": {
            "title": "Admission date",
            "type": "date-time",
        },
        "admission_end_date": {
            "title": "Admission end date",
            "type": "date-time",
        },
        "admission_status": {
            "title": "Admission status",
            "type": "string",
        },
        "services": {
            "title": "Services",
            "type": "array",
            "name": "Services",
            "items": {
                "title": "Service",
                "name": "Service",
                "type": "object",
                "required": ["code"],
                "properties": {
                    "code": {
                        "title": "Code",
                        "type": "string",
                    },
                    "name": {
                        "title": "Name",
                        "type": "string",
                    },
                },
            },
        },
        "physicians": {
            "title": "Physicians",
            "name": "Physicians",
            "type": "array",
            "items": {
                "title": "Physician",
                "name": "Physician",
                "type": "object",
                "required": ["code"],
                "properties": {
                    "code": {
                        "title": "Code",
                        "type": "string",
                    },
                    "type": {
                        "title": "Type",
                        "type": "string",
                        "enum": ["attending", "sampler", "prescriber"],
                    },
                    "last_name": {
                        "title": "Last name",
                        "type": "string",
                    },
                    "first_name": {
                        "title": "First name",
                        "type": "string",
                    },
                    "phone": {
                        "title": "Phone",
                        "type": "string",
                    },
                    "codification": {
                        "title": "Codification",
                        "type": "string",
                    },
                    "alias": {
                        "title": "Alias",
                        "type": "string",
                    },
                    "title": {
                        "title": "Title",
                        "type": "string",
                    },
                },
            },
        },
        # LEGACY
        "service": {"type": "string"},
        "service_name": {"type": "string"},
        "physician_name": {"type": "string"},
        "physician_first_name": {"type": "string"},
        "medecin2": {"type": "string"},
        # END LEGACY
        "transcriptionist": {
            "title": "Transcriptionist",
            "type": "string",
        },
        "location": {
            "title": "Location",
            "type": "string",
        },
        "location_name": {
            "title": "Location name",
            "type": "string",
        },
        "room": {
            "title": "Room",
            "type": "string",
        },
        "bed": {
            "title": "Bed",
            "type": "string",
        },
        "hospital_service": {
            "title": "Hospital Service",
            "type": "string",
        },
        "last_name": {
            "title": "Last name",
            "type": "string",
        },
        "first_name": {
            "title": "First name",
            "type": "string"
        },
        "title": {
            "title": "Title",
            "type": "string",
        },
        "birthdate": {
            "title": "Birth date",
            "type": "date-time"
        },
        "deathdate": {
            "title": "Date of death",
            "type": "date-time",
        },
        "birth_row": {
            "title": "Birth row",
            "type": "string",
        },
        "maiden_name": {
            "title": "Maiden name",
            "type": "string",
        },
        "postal_code": {
            "title": "Postal code",
            "type": "string",
        },
        "city": {
            "title": "City",
            "type": "string",
        },
        "country": {
            "title": "Country",
            "type": "string",
        },
        "address": {
            "title": "Address",
            "type": "string",
        },
        "lab": {
            "title": "Laboratory",
            "type": "string",
        },
        "sex": {
            "title": "Sex",
            "type": "string",
        },
        "patient_id": {
            "title": "Patient ID",
            "type": "string"
        },
        "lab_patient_id": {
            "title": "Laboratory patient ID",
            "type": "string",
        },
        "alternate_patient_id": {
            "title": "Alternate patient ID",
            "type": "string",
        },
        "specimen": {
            "title": "Specimen code",
            "type": "string",
        },
        "diagnostic": {
            "title": "Diagnostic",
            "type": "string",
        },
        "height": {
            "title": "Height",
            "type": "string",
        },
        "weight": {
            "title": "Weight",
            "type": "string",
        },
        "phone": {
            "title": "Phone",
            "type": "string",
        },
        "medications": {
            "title": "Medications",
            "type": "array",
            "items": {"type": "string"},
        },
        "priority": {
            "title": "Priority",
            "type": "string"
        },
        "physiologie": {
            "title": "Physiologie",
            "type": "string",
        },
        "patient_comment1": {
            "title": "Patient comment1",
            "type": "string",
        },
        "patient_comment2": {
            "title": "Patient comment2",
            "type": "string",
        },
        "media_code": {
            "title": "Media code",
            "type": "string",
        },
        "media_suffix_code": {
            "title": "Media suffix code",
            "type": "string",
        },
        "comments": {
            "title": "Comments",
            "type": "array",
            "items": {
                "title": "Comment",
                "type": "object",
                "name": "Comment",
                "properties": {
                    "text": {
                        "title": "Text",
                        "type": "string",
                    },
                    "type": {
                        "title": "Type",
                        "type": "string",
                        "enum": ["patient", "sample"],
                    },
                },
            },
        },
        "flags": {
            "title": "Flags",
            "type": "array", "items": {"title": "Flag", "type": "string"},
        },
        "orders": {"type": "array", "items": ORDER_SCHEMA},
        "results": {"type": "array", "items": RESULT_SCHEMA},
        "billings": {"type": "array", "items": BILLING_SCHEMA},
        "info_serotheque": {
            "title": "Serotheque",
            "type": "object",
            "properties": {
                "code": {
                    "title": "code serotheque",
                    "type": "string"
                },
                "portoir": {
                    "title": "code portoir",
                    "type": "string"
                },
                "sample_id": {
                    "title": "Sample ID",
                    "type": "string"
                },
                "numero": {
                    "title": "numero serotheque",
                    "type": "string"
                },
                "destination": {
                    "title": "destination serotheque",
                    "type": "string"
                },
                "position": {
                    "title": "position serotheque",
                    "type": "string"
                },
                "etiquette": {
                    "title": "etiquette serotheque",
                    "type": "string"
                },
            },
        },
    },
    "required": ["sample_id", "type"],
    "additionalProperties": False,
}

DICTIONARY_SCHEMA = {
    "name": "Dictionary",
    "type": "object",
    "properties": {
        "emetteur": {
            "title": "emetteur du dictionnaire",
            "type": "string",
        },
        "production": {
            "title": "message production ou mode test",
            "type": "boolean",
        },
        "unique_identifier": {
            "title": "identifiant unique du message",
            "type": "string",
        },
        "version": {
            "title": "version du catalogue (ne pas parser)",
            "type": "string"
        }
    },
    "entries": {
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "pkey": {
                    "title": 'entrie pkey',
                    "type": "string"
                },
                "code": {
                    "title": "code examen",
                    "type": "string"
                },
                "libelle": {
                    "title": "libelle examen",
                    "type": "string"
                },
                "codification": {
                    "title": "codification (L=Local LN=LOINC)",
                    "type": "string",
                    "enum": ["L", "LN"],
                },
                "sample_required": {
                    "title": "echantillon requis",
                    "type": "boolean"
                },
                "method": {
                    "title": "technique",
                    "type": "string"
                },
                "battery": {
                    "title": "examen atomique ou batterie d'examens",
                    "type": "boolean"
                },
                "turnaroundtime": {
                    "title": "delai de realisation en minutes",
                    "type": "integer"
                },
                "frequency": {
                    "title": "frequence de realisation (OM1-40)",
                    "type": "string"
                },
                "comments": {
                    "title": "commentaires",
                    "type": "string"
                },
                "price_hn_eur_cents": {
                    "title": "prix hors nomenclature (en centime d'euros)",
                    "type": "integer"
                },
                "price_fixed": {
                    "title": "prix fixe",
                    "type": "boolean"
                },
                "entente_prealable": {
                    "title": "entente prealable necessaire",
                    "type": "boolean"
                },
                "consentement": {
                    "title": "consentement patient necessaire",
                    "type": "boolean"
                },
                "analyse_declenchee": {
                    "title": "code analyse à declencher obligatoirement",
                    "type": "string"
                },
                "nabms": {
                    "title": "codes nabm",
                    "type": "array",
                    "items": {
                        "title": "code nabm",
                        "type": "string"
                    }
                },
                "url_doc": {
                    "title": "url documentaire",
                    "type": "string"
                },
                "conditions_tarifaires": {
                    "title": "conditions tarifaires",
                    "type": "string"
                },
                "samples": {
                    "title": "echantillons",
                    "type": "array",
                    "items": {
                        "type": "object",
                        "properties": {
                            "container_description": {
                                "title": "description du contenant",
                                "type": "string"
                            },
                            "container_volume": {
                                "title": "volume du contenant",
                                "type": "integer"
                            },
                            "container_unit": {
                                "title": "unite de volume du contenant",
                                "type": "string"
                            },
                            "nature_code": {
                                "title": "code nature",
                                "type": "string"
                            },
                            "nature_libelle": {
                                "title": "libelle nature",
                                "type": "string"
                            },
                            "alt_nature_code": {
                                "title": "code nature (alternatif)",
                                "type": "string"
                            },
                            "alt_nature_libelle": {
                                "title": "label nature (alternatif)",
                                "type": "string"
                            },
                            "additive": {
                                "title": "additif ajoute au prelevement",
                                "type": "string"
                            },
                            "transport_conditions_code": {
                                "title": "conditions de transport (code)",
                                "type": "string"
                            },
                            "transport_conditions_libelle": {
                                "title": "conditions de transport (libelle)",
                                "type": "string"
                            },
                            "minimum_volume": {
                                "title": "volume minimum necessaire ([valeur] [unite])",
                                "type": "string"
                            },
                        },
                    },
                },
            },
            'required': ['code', 'libelle'],
        },
    },
    "required": ["entries"],
    "additionalProperties": False,
}

PATIENT_ATTRIBUTES = (
    "last_name", "first_name", "birthdate", "maiden_name", "sex",
    "patient_id", "lab", "lab_patient_id",
    "postal_code", "address", "city", "country", "collection_date",
    "services", "physicians", "location", "location_name", "bed", "room",
    "admission_date", "admission_end_date", "admission_status",
    "height", "weight", "title", "alternate_patient_id", "phone", "birth_row",
    "deathdate",
    # Legacy
    "service", "service_name", "physician", "physician_name", "physician_first_name",
    "comments", "medications", "patient_comment1", "patient_comment2",
)

PATIENT_SCHEMA = {
    "name": "Patient",
    "type": "object",
    "properties": dict([
        (k, v) for k, v in SAMPLE_SCHEMA["properties"].items()
        if k in PATIENT_ATTRIBUTES]),
    "required": ["patient_id"],
    "additionalProperties": True,
}


SCHEMA = {
    "name": "Worklist",
    "type": "object",
    "required": ["context"],
    "properties": {
        "samples": {"type": "array", "items": SAMPLE_SCHEMA},
        "dictionary": {"type": "object", "properties": DICTIONARY_SCHEMA},
        "context": {
            "title": "Context",
            "type": "string",
            "enum": ["order", "query", "result", "billing", "dictionary"],
        },
        "filename": {
            "title": "Filename",
            "type": "string"
        },
        "sender_id": {
            "title": "Sender ID",
            "type": "string",
        },
        "sender_name": {
            "title": "Sender Name",
            "type": "string",
        },
        "receiver_id": {
            "title": "Receiver ID",
            "type": "string",
        },
        "receiver_name": {
            "title": "Receiver Name",
            "type": "string",
        },
        "orig_data": {"type": "string"},
        "data": {"type": "string"},
        "traceback": {"type": "string"},
    },
    "additionalProperties": False,
}


Model = jsonschema_model.model_factory(SCHEMA)
Patient = jsonschema_model.model_factory(PATIENT_SCHEMA)
Sample = jsonschema_model.model_factory(SAMPLE_SCHEMA)
Order = jsonschema_model.model_factory(ORDER_SCHEMA)
Result = jsonschema_model.model_factory(RESULT_SCHEMA)
Lot = jsonschema_model.model_factory(LOT_SCHEMA)
Billing = jsonschema_model.model_factory(BILLING_SCHEMA)
Dictionary = jsonschema_model.model_factory(DICTIONARY_SCHEMA)


class Worklist(Model):
    """Format natif pour représenter un message

    La classe est basée sur un schema json et construite avec
    `jsonschema-model <https://github.com/philpep/jsonschema-model>`_

    Exemple de message de demandes::

        {
            "context": "order",
            "samples": [{
                "type": "patient",
                "sample_id": "123456789",
                "first_name": "ÉTIENNE",
                "last_name": "Bézout,
                "birthdate": datetime.datetime(1730, 3, 31),
                "specimen": "Serum",
                "orders": [
                    {"code": "A", "action": "add", "priority": "S"},
                    {"code": "B", "action": "del", "priority": "S"},
                ],
                "results": [{
                    # Anteriorité
                    "code": "A",
                    "date": datetime.datetime(1750, 1, 1),
                    "value": "24",
                    "anterior": True,
                }, {
                    # Résultat Accueil
                    "code": "A",
                    "date": datetime.datetime(1750, 1, 1),
                    "value": "48",
                }],
            }],
        }


    Exemple de message de query::

        {
            "context": "query",
            "samples": [{
                "type": "patient",
                "sample_id": "123456789",
            }],
        }

    Exemple de message de résultat patient::

        {
            "context": "result",
            "samples": [{
                "type": "patient",
                "sample_id": "123456789",
                "specimen": "Urine",
                "results": [{
                    "code": "A",
                    "date": datetime.datetime(2016, 1, 27, 14, 54),
                    "value": "42",
                    "flags": ["HH"],
                    "comments": ["Je lui donne pas 3 jours"],
                }],
            }],
        }

    Exemple de message de résultat QC::

        {
            "context": "result",
            "samples": [{
                "type": "control",
                "sample_id": "45633VB",
                "control_lot": "45633VB",
                "control_level": "3",
                "specimen": "Whole blood",
                "results": [{
                    "code": "A",
                    "value": "42",
                    "date": datetime.datetime(2016, 1, 27, 14, 54),
                }],
            }],
        }

    """
    VALIDATOR = jsonschema.Draft4Validator(
        schema={"type": "date-time"},
        types={
            "string": (unicode, str, types.NoneType),
            "date-time": (datetime.datetime, types.NoneType),
        },
    )
    ELSE = "_else_"
    IGNORE = "_ignore_"

    class ValidationError(Exception):
        pass

    def __init__(self, samples=None, **kwargs):
        super(Worklist, self).__init__(samples=(samples or []), **kwargs)

    def get_worklist(self, unique_together=[], **kwargs):
        """Permet de découper un sample plusieurs samples

        En fonction des valeurs d'attributs sur les `orders`.

        Utile pour les drivers non ASTM (où le patient est mis au
        même niveau que le sample).

        .. code-block:: python

            for sample, action in worklist.get_worklist(
                unique_together=["action"],
                action={"del": worklist.IGNORE, worklist.ELSE: "ADD"},
            ):
                # [...]
        """
        samples = []
        for sample in self.samples:
            for couple, orders in self.get_orders(
                    sample["orders"], unique_together, **kwargs):
                new_sample = Sample(sample.copy())
                new_sample["orders"] = orders
                samples.append((new_sample,) + couple)
        return samples

    def get_astm_worklist(self, unique_together=[], samples="orders", **kwargs):
        """Permet de découper les sample et de regrouper les patients

        Les patients sont groupés par l'attribut `patient_id`.

        .. code-block:: python

            for patient in worklist.get_astm_worklist(
                unique_together=["action", "priority"],
                action={"del": "C", worklist.ELSE: "N"},
                priority={"S": "S", worklist.ELSE: "R"},
            ):
                # Ici on remplis un segment P
                for sample, action, priority in patient.samples:
                    # Ici on remplis un segment O
        """
        patients = OrderedDict()
        for sample in self.samples:
            patients.setdefault(
                sample["patient_id"], Patient(dict([
                    (k, sample[k])
                    for k in PATIENT_ATTRIBUTES if k in sample])))
            patients[sample["patient_id"]].setdefault("samples", [])
            orphan_results = sample.results[:]
            for couple, orders in self.get_orders(
                    sample[samples], unique_together, **kwargs):
                new_sample = Sample(sample.copy())
                new_sample[samples] = orders
                order_codes = [o.code for o in orders]
                new_sample["results"] = []
                for r in sample.results:
                    if r["code"] in order_codes:
                        orphan_results.remove(r)
                        new_sample["results"].append(Result(r))
                patients[sample["patient_id"]]["samples"].append(
                    ((new_sample,) + couple))
            if orphan_results and self.context != "result":
                for couple in patients[sample["patient_id"]]["samples"]:
                    couple[0].results.extend([Result(r) for r in orphan_results])

            if not sample.orders and self.context == "result":
                new_sample = Sample(sample.copy())
                couple = []
                for k in unique_together:
                    v = kwargs.get(k, {})
                    if k not in new_sample:
                        couple.append(None)
                        continue
                    key = v.get(new_sample[k], v.get(Worklist.ELSE, new_sample[k]))
                    couple.append(key)
                patients[sample["patient_id"]]["samples"].append(
                    ((new_sample,) + tuple(couple)))
        return [p for p in patients.values() if p.samples]

    @staticmethod
    def get_orders(orders, unique_together=[], **kwargs):
        r = OrderedDict()
        # key => [values to ignore]
        ignore = {}
        for key, values in kwargs.items():
            for k, v in values.items():
                if v == Worklist.IGNORE:
                    ignore.setdefault(key, []).append(k)
        for order in orders:
            for k, values in ignore.items():
                if order[k] in values:
                    break
            else:
                couple = []
                for k in unique_together:
                    v = kwargs.get(k, {})
                    key = v.get(order[k], v.get(Worklist.ELSE, order[k]))
                    couple.append(key)
                r.setdefault(tuple(couple), []).append(Order(order))
        return r.items()

    def get_or_create(self, **kwargs):
        """Permet d'ajouter un sample dans une worklist sans créer de doublon

        >>> worklist = Worklist(context="result")
        >>> sample = worklist.get_or_create(type="patient", sample_id="123456789")
        """
        return self.samples.get_or_create(**kwargs)

    def validate(self):
        """Valide la worklist avec jsonschema"""
        try:
            return self.VALIDATOR.validate(self, SCHEMA)
        except jsonschema.exceptions.ValidationError as error:
            raise self.ValidationError(error)

    def dumps(self):
        """Dumpe la Worklist en json (gestion du type datetime)"""
        return json.dumps(self, cls=utils.JSONEncoder)

    @classmethod
    def loads(cls, data):
        """Crée une Worklist à partir d'une chaine json"""
        doc = json.loads(data)
        return cls.from_json(doc)

    @classmethod
    def from_json(cls, doc):
        for sample in doc.get("samples", []):
            for k, v in sample.items():
                if k in (
                    "request_date", "admission_date", "birthdate",
                    "collection_date", "collection_end_date",
                ):
                    sample[k] = utils.decode_isoformat(v)
            for result in sample.get("results", []):
                for rk, rv in result.items():
                    if rk in ("date",):
                        result[rk] = utils.decode_isoformat(rv)
                for lot in result.get("lots", []):
                    for lk, lv in lot.items():
                        if lk == "expiration":
                            lot[lk] = utils.decode_isoformat(lv)

        return cls(**doc)
